@@ -1,3 +1,32 @@
+# -*- coding: utf-8 -*-
+"""
+
+    mslib.msui
+    ~~~~~~~~~~
+
+    This module provides the default configuration for the msui
+
+    This file is part of mss.
+
+    :copyright: Copyright 2008-2014 Deutsches Zentrum fuer Luft- und Raumfahrt e.V.
+    :copyright: Copyright 2011-2014 Marc Rautenhaus (mr)
+    :copyright: Copyright 2016-2017 by the mss team, see AUTHORS.
+    :license: APACHE-2.0, see LICENSE for details.
+
+    Licensed under the Apache License, Version 2.0 (the "License");
+    you may not use this file except in compliance with the License.
+    You may obtain a copy of the License at
+
+       http://www.apache.org/licenses/LICENSE-2.0
+
+    Unless required by applicable law or agreed to in writing, software
+    distributed under the License is distributed on an "AS IS" BASIS,
+    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+    See the License for the specific language governing permissions and
+    limitations under the License.
+"""
+
+
 import os
 import tempfile
 
@@ -88,6 +117,13 @@ class MissionSupportSystemDefaultConfig(object):
     # Maximum age of a cached file in seconds.
     wms_cache_max_age_seconds = 5 * 86400
 
+    wms_prefetch = {
+        "validtime_fwd": 0,
+        "validtime_bck": 0,
+        "level_up": 0,
+        "level_down": 0
+    }
+
     locations = {
         "EDMO": (48.08, 11.28),
         "Hannover": (52.37, 9.74),
