@@ -1,11 +1,31 @@
+# -*- coding: utf-8 -*-
 """
-AUTHORS:
-========
 
-* Joern Ungermann (ju)
+    mslib.msui.aircrafts
+    ~~~~~~~~~~~~~~~~~~~~
 
+    This module provides aircrafts definitions
+
+    This file is part of mss.
+
+    :copyright: Copyright 2016-2017 Joern Ungermann
+    :copyright: Copyright 2016-2017 by the mss team, see AUTHORS.
+    :license: APACHE-2.0, see LICENSE for details.
+
+    Licensed under the Apache License, Version 2.0 (the "License");
+    you may not use this file except in compliance with the License.
+    You may obtain a copy of the License at
+
+       http://www.apache.org/licenses/LICENSE-2.0
+
+    Unless required by applicable law or agreed to in writing, software
+    distributed under the License is distributed on an "AS IS" BASIS,
+    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+    See the License for the specific language governing permissions and
+    limitations under the License.
 """
 
+
 import bisect
 import numpy as np
 
