@@ -1,68 +1,56 @@
-"""Data model representing a flight track. The model is derived from
-QAbstractTableModel, so that it can directly be connected to any Qt view.
+# -*- coding: utf-8 -*-
+"""
 
-********************************************************************************
+    mslib.msui.flighttrack
+    ~~~~~~~~~~~~~~~~~~~~~~
 
-   Copyright 2008-2014 Deutsches Zentrum fuer Luft- und Raumfahrt e.V.
-   Copyright 2011-2014 Marc Rautenhaus
+    Data model representing a flight track. The model is derived from
+    QAbstractTableModel, so that it can directly be connected to any Qt view.
 
-   Licensed under the Apache License, Version 2.0 (the "License");
-   you may not use this file except in compliance with the License.
-   You may obtain a copy of the License at
+    For better understanding of the code, compare to the 'ships' example
+    from chapter 14/16 of 'Rapid GUI Programming with Python and Qt: The
+    Definitive Guide to PyQt Programming' (Mark Summerfield).
+
+    The model includes a method for computing the distance between waypoints
+    and for the entire flight track.
+
+    This file is part of mss.
+
+    :copyright: Copyright 2008-2014 Deutsches Zentrum fuer Luft- und Raumfahrt e.V.
+    :copyright: Copyright 2011-2014 Marc Rautenhaus (mr)
+    :copyright: Copyright 2016-2017 by the mss team, see AUTHORS.
+    :license: APACHE-2.0, see LICENSE for details.
+
+    Licensed under the Apache License, Version 2.0 (the "License");
+    you may not use this file except in compliance with the License.
+    You may obtain a copy of the License at
 
        http://www.apache.org/licenses/LICENSE-2.0
 
-   Unless required by applicable law or agreed to in writing, software
-   distributed under the License is distributed on an "AS IS" BASIS,
-   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-   See the License for the specific language governing permissions and
-   limitations under the License.
-
-********************************************************************************
-
-This file is part of the DLR/IPA Mission Support System User Interface (MSUI).
-
-For better understanding of the code, compare to the 'ships' example
-from chapter 14/16 of 'Rapid GUI Programming with Python and Qt: The
-Definitive Guide to PyQt Programming' (Mark Summerfield).
-
-The model includes a method for computing the distance between waypoints
-and for the entire flight track.
-
-AUTHORS:
-========
-
-* Marc Rautenhaus (mr)
-
+    Unless required by applicable law or agreed to in writing, software
+    distributed under the License is distributed on an "AS IS" BASIS,
+    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+    See the License for the specific language governing permissions and
+    limitations under the License.
 """
 
-# standard library imports
-from datetime import datetime, timedelta
 
-import os
-import pickle
-import csv
+import datetime
+import codecs
 import logging
 import xml.dom.minidom
 
-
 # related third party imports
-from PyQt4 import QtGui, QtCore  # Qt4 bindings
-from PyQt4.QtCore import QString, QVariant, Qt, QModelIndex, SIGNAL, QAbstractTableModel
-from PyQt4.QtGui import QItemDelegate, QComboBox
-import numpy as np
+from mslib.msui.mss_qt import QtGui, QtCore, QtWidgets, QString, USE_PYQT5
 
 # local application imports
-from mslib import mss_util
+from mslib import utils
 from mslib import thermolib
-from mslib.mss_util import config_loader
-from mslib.msui import constants
+from mslib.utils import config_loader, find_location, save_settings_pickle, load_settings_pickle
 from mslib.msui.performance_settings import DEFAULT_PERFORMANCE
 from mslib.msui import MissionSupportSystemDefaultConfig as mss_default
 
 
-# CONSTANTS (used in this module)
-#
 # Constants for identifying the table columns when the WaypointsTableModel is
 # used with a QTableWidget.
 LOCATION, LAT, LON, FLIGHTLEVEL, PRESSURE = range(5)
@@ -82,27 +70,22 @@ TABLE_FULL = [
     ("Lat\n(+-90)", lambda waypoint: waypoint.lat, True),
     ("Lon\n(+-180)", lambda waypoint: waypoint.lon, True),
     ("Flightlevel", lambda waypoint: waypoint.flightlevel, True),
-    ("Pressure\n(hPa)", lambda waypoint: "%.2f" % (waypoint.pressure / 100.), False),
-    ("Leg dist.\n(km [nm])", lambda waypoint: "%d [%d]" % (waypoint.distance_to_prev,
-                                                           waypoint.distance_to_prev / 1.852), False),
-    ("Cum. dist.\n(km [nm])", lambda waypoint: "%d [%d]" % (waypoint.distance_total,
-                                                            waypoint.distance_total / 1.852), False),
+    ("Pressure\n(hPa)", lambda waypoint: "{:.2f}".format(waypoint.pressure / 100.), True),
+    ("Leg dist.\n(km [nm])", lambda waypoint: "{:d} [{:d}]".format(
+        int(waypoint.distance_to_prev), int(waypoint.distance_to_prev / 1.852)), False),
+    ("Cum. dist.\n(km [nm])", lambda waypoint: "{:d} [{:d}]".format(
+        int(waypoint.distance_total), int(waypoint.distance_total / 1.852)), False),
     ("Leg time", lambda waypoint: seconds_to_string(waypoint.leg_time), False),
     ("Cum. time", lambda waypoint: seconds_to_string(waypoint.cum_time), False),
     ("Time (UTC)", lambda waypoint: waypoint.utc_time.strftime("%Y-%m-%d %H:%M:%S"), False),
-    ("Rem. fuel\n(lb)", lambda waypoint: ("%d" % waypoint.rem_fuel), False),
-    ("Aircraft\nweight (lb)", lambda waypoint: ("%d" % waypoint.weight), False),
+    ("Rem. fuel\n(lb)", lambda waypoint: ("{:d}".format(int(waypoint.rem_fuel))), False),
+    ("Aircraft\nweight (lb)", lambda waypoint: ("{:d}".format(int(waypoint.weight))), False),
     ("Comments                        ", lambda waypoint: waypoint.comments, True),
 ]
 
 TABLE_SHORT = [TABLE_FULL[_i] for _i in range(7)] + [TABLE_FULL[-1]] + [("", lambda _: "", False)] * 6
 
 
-"""
-CLASS Waypoint
-"""
-
-
 class Waypoint(object):
     """Represents a waypoint with position, altitude and further
        properties. Used internally by WaypointsTableModel.
@@ -149,11 +132,11 @@ class Waypoint(object):
         """String representation of the waypoint (e.g., when used with the print
            statement).
         """
-        return "WAYPOINT(LAT=%f, LON=%f, FL=%f)" % (self.lat, self.lon,
-                                                    self.flightlevel)
+        return "WAYPOINT(LAT={:f}, LON={:f}, FL={:f})".format(self.lat, self.lon,
+                                                              self.flightlevel)
 
 
-class WaypointsTableModel(QAbstractTableModel):
+class WaypointsTableModel(QtCore.QAbstractTableModel):
     """Qt-QAbstractTableModel-derived data structure representing a flight
        track composed of a number of waypoints.
 
@@ -165,6 +148,8 @@ class WaypointsTableModel(QAbstractTableModel):
     flight performance calculations.
     """
 
+    dataChanged = QtCore.pyqtSignal([QtCore.QModelIndex, QtCore.QModelIndex])
+
     def __init__(self, name="", filename=None, waypoints=None):
         super(WaypointsTableModel, self).__init__()
         self.name = name  # a name for this flight track
@@ -173,7 +158,7 @@ class WaypointsTableModel(QAbstractTableModel):
         self.waypoints = []  # user-defined waypoints
 
         # self.aircraft.setErrorHandling("permissive")
-        self.settingsfile = os.path.join(constants.MSS_CONFIG_PATH, "mss.performance.cfg")
+        self.settings_tag = "performance"
         self.loadSettings()
 
         # If a filename is passed to the constructor, load data from this file.
@@ -181,7 +166,7 @@ class WaypointsTableModel(QAbstractTableModel):
             if filename.endswith(".ftml"):
                 self.loadFromFTML(filename)
             else:
-                logging.debug("No known file extension! %s", filename)
+                logging.debug(u"No known file extension! '{}'".format(filename))
 
         if waypoints:
             self.replaceWaypoints(waypoints)
@@ -189,12 +174,7 @@ class WaypointsTableModel(QAbstractTableModel):
     def loadSettings(self):
         """Load settings from the file self.settingsfile.
         """
-        if os.path.exists(self.settingsfile):
-            logging.debug("loading settings from %s", self.settingsfile)
-            with open(self.settingsfile, "r") as fileobj:
-                self.performance_settings = pickle.load(fileobj)
-        else:
-            self.performance_settings = DEFAULT_PERFORMANCE
+        self.performance_settings = load_settings_pickle(self.settings_tag, DEFAULT_PERFORMANCE)
 
     def saveSettings(self):
         """Save the current settings (map appearance) to the file
@@ -202,34 +182,29 @@ class WaypointsTableModel(QAbstractTableModel):
         """
         # TODO: ConfigParser and a central configuration file might be the better solution than pickle.
         # http://stackoverflow.com/questions/200599/whats-the-best-way-to-store-simple-user-settings-in-python
-        logging.debug("storing settings to %s", self.settingsfile)
-        with open(self.settingsfile, "w") as fileobj:
-            pickle.dump(self.performance_settings, fileobj)
+        save_settings_pickle(self.settings_tag, self.performance_settings)
 
     def setName(self, name):
         self.name = name
         self.modified = True
 
-    def performanceValid(self):
-        return self.performance_valid
-
     def flags(self, index):
         """Used to specify which table columns can be edited by the user;
            overrides the corresponding QAbstractTableModel method.
 
         """
         if not index.isValid():
-            return Qt.ItemIsEnabled
+            return QtCore.Qt.ItemIsEnabled
         column = index.column()
         table = TABLE_SHORT
         if self.performance_settings["visible"]:
             table = TABLE_FULL
         if table[column][2]:
-            return Qt.ItemFlags(QAbstractTableModel.flags(self, index) | Qt.ItemIsEditable)
+            return QtCore.Qt.ItemFlags(QtCore.QAbstractTableModel.flags(self, index) | QtCore.Qt.ItemIsEditable)
         else:
-            return Qt.ItemFlags(QAbstractTableModel.flags(self, index))
+            return QtCore.Qt.ItemFlags(QtCore.QAbstractTableModel.flags(self, index))
 
-    def data(self, index, role=Qt.DisplayRole):
+    def data(self, index, role=QtCore.Qt.DisplayRole):
         """Return a data field at the given index (of type QModelIndex,
            specifying row and column); overrides the corresponding
            QAbstractTableModel method.
@@ -242,17 +217,17 @@ class WaypointsTableModel(QAbstractTableModel):
         waypoints = self.waypoints
 
         if not index.isValid() or not (0 <= index.row() < len(waypoints)):
-            return QVariant()
+            return QtCore.QVariant()
         waypoint = waypoints[index.row()]
         column = index.column()
-        if role == Qt.DisplayRole:
+        if role == QtCore.Qt.DisplayRole:
             if self.performance_settings["visible"]:
-                return QVariant(TABLE_FULL[column][1](waypoint))
+                return QtCore.QVariant(TABLE_FULL[column][1](waypoint))
             else:
-                return QVariant(TABLE_SHORT[column][1](waypoint))
-        elif role == Qt.TextAlignmentRole:
-            return QVariant(int(Qt.AlignLeft | Qt.AlignVCenter))
-        return QVariant()
+                return QtCore.QVariant(TABLE_SHORT[column][1](waypoint))
+        elif role == QtCore.Qt.TextAlignmentRole:
+            return QtCore.QVariant(int(QtCore.Qt.AlignLeft | QtCore.Qt.AlignVCenter))
+        return QtCore.QVariant()
 
     def waypointData(self, row):
         """Get the waypoint object defining the given row.
@@ -272,61 +247,66 @@ class WaypointsTableModel(QAbstractTableModel):
         Returns lats, lons.
         """
         path = [[wp.lat, wp.lon] for wp in self.waypoints]
-        lats, lons = mss_util.path_points(
+        lats, lons = utils.path_points(
             path, numpoints=numpoints, connection=connection)
         return lats, lons
 
-    def headerData(self, section, orientation, role=Qt.DisplayRole):
+    def headerData(self, section, orientation, role=QtCore.Qt.DisplayRole):
         """Return data describing the table header; overrides the
            corresponding QAbstractTableModel method.
         """
-        if role == Qt.TextAlignmentRole:
-            if orientation == Qt.Horizontal:
-                return QVariant(int(Qt.AlignLeft | Qt.AlignVCenter))
-            return QVariant(int(Qt.AlignRight | Qt.AlignVCenter))
-        if role != Qt.DisplayRole:
-            return QVariant()
+        if role == QtCore.Qt.TextAlignmentRole:
+            if orientation == QtCore.Qt.Horizontal:
+                return QtCore.QVariant(int(QtCore.Qt.AlignLeft | QtCore.Qt.AlignVCenter))
+            return QtCore.QVariant(int(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter))
+        if role != QtCore.Qt.DisplayRole:
+            return QtCore.QVariant()
         # Return the names of the table columns.
-        if orientation == Qt.Horizontal:
+        if orientation == QtCore.Qt.Horizontal:
             if self.performance_settings["visible"]:
-                return QVariant(TABLE_FULL[section][0])
+                return QtCore.QVariant(TABLE_FULL[section][0])
             else:
-                return QVariant(TABLE_SHORT[section][0])
+                return QtCore.QVariant(TABLE_SHORT[section][0])
         # Table rows (waypoints) are labelled with their number (= number of
         # waypoint).
-        return QVariant(int(section))
+        return QtCore.QVariant(int(section))
 
-    def rowCount(self, index=QModelIndex()):
+    def rowCount(self, index=QtCore.QModelIndex()):
         """Number of waypoints in the model.
         """
         return len(self.waypoints)
 
-    def columnCount(self, index=QModelIndex()):
+    def columnCount(self, index=QtCore.QModelIndex()):
         return len(TABLE_FULL)
 
-    def setData(self, index, value, role=Qt.EditRole, update=True):
+    def setData(self, index, value, role=QtCore.Qt.EditRole, update=True):
         """Change a data element of the flight track; overrides the
            corresponding QAbstractTableModel method.
 
         NOTE: Performance computations loose their validity if a change is made.
         """
         if index.isValid() and 0 <= index.row() < len(self.waypoints):
-            # print "\n>> SetData()"
+            if not USE_PYQT5:
+                value = value.toString()
+            elif isinstance(value, QtCore.QVariant):
+                value = value.value()
             waypoint = self.waypoints[index.row()]
             column = index.column()
             index2 = index  # in most cases only one field is being changed
             if column == LOCATION:
-                waypoint.location = value.toString()
+                waypoint.location = value
             elif column == LAT:
-                # In Qt4.6 and higher, value.toFloat() is available:
-                # value, ok = value.toFloat()
-                # For lower versions, we need to use try..except.
                 try:
-                    value, ok = float(value.toString()), True
-                except:
-                    ok = False
-                if ok:
+                    value = float(value)
+                except ValueError:
+                    pass
+                else:
                     waypoint.lat = value
+                    waypoint.location = u""
+                    loc = find_location(waypoint.lat, waypoint.lon)
+                    if loc is not None:
+                        waypoint.lat, waypoint.lon = loc[0]
+                        waypoint.location = loc[1]
                     # A change of position requires an update of the distances.
                     if update:
                         self.update_distances(index.row())
@@ -335,27 +315,31 @@ class WaypointsTableModel(QAbstractTableModel):
                     # changed.
                     # Delete the location name -- it won't be valid anymore
                     # after its coordinates have been changed.
-                    waypoint.location = ""
                     index2 = self.createIndex(index.row(), LOCATION)
             elif column == LON:
                 try:
-                    value, ok = float(value.toString()), True
-                except:
-                    ok = False
-                if ok:
+                    value = float(value)
+                except ValueError:
+                    pass
+                else:
                     waypoint.lon = value
+                    waypoint.location = u""
+                    loc = find_location(waypoint.lat, waypoint.lon)
+                    if loc is not None:
+                        waypoint.lat, waypoint.lon = loc[0]
+                        waypoint.location = loc[1]
                     if update:
                         self.update_distances(index.row())
-                    waypoint.location = ""
                     index2 = self.createIndex(index.row(), LOCATION)
             elif column == FLIGHTLEVEL:
                 try:
-                    value, ok = float(value.toString()), True
-                except:
-                    ok = False
-                if ok:
-                    waypoint.flightlevel = value
-                    waypoint.pressure = thermolib.flightlevel2pressure(value)
+                    flightlevel = float(value)
+                    pressure = thermolib.flightlevel2pressure(flightlevel)
+                except ValueError:
+                    pass
+                else:
+                    waypoint.flightlevel = flightlevel
+                    waypoint.pressure = pressure
                     if update:
                         self.update_distances(index.row())
                     # need to notify view of the second item that has been
@@ -363,34 +347,40 @@ class WaypointsTableModel(QAbstractTableModel):
                     index2 = self.createIndex(index.row(), PRESSURE)
             elif column == PRESSURE:
                 try:
-                    value, ok = float(value.toString()), True
-                except:
-                    ok = False
-                if ok:
-                    waypoint.pressure = value
-                    waypoint.flightlevel = int(thermolib.pressure2flightlevel(value))
+                    pressure = float(value) * 100  # convert hPa to Pa
+                    if pressure > 200000:
+                        raise ValueError
+                    flightlevel = round(thermolib.pressure2flightlevel(pressure))
+                    pressure = thermolib.flightlevel2pressure(flightlevel)
+                except ValueError:
+                    pass
+                else:
+                    waypoint.pressure = pressure
+                    waypoint.flightlevel = flightlevel
                     if update:
                         self.update_distances(index.row())
                     index2 = self.createIndex(index.row(), FLIGHTLEVEL)
             else:
-                waypoint.comments = value.toString()
+                waypoint.comments = value
             self.modified = True
             # Performance computations loose their validity if a change is made.
-            self.performance_valid = False
-            self.emit(SIGNAL("dataChanged(QModelIndex,QModelIndex)"),
-                      index, index2)
+            self.dataChanged.emit(index, index2)
             return True
         return False
 
-    def insertRows(self, position, rows=1, index=QModelIndex(),
-                   waypoints=[None]):
+    def insertRows(self, position, rows=1, index=QtCore.QModelIndex(),
+                   waypoints=None):
         """Insert waypoint; overrides the corresponding QAbstractTableModel
            method.
         """
-        self.beginInsertRows(QModelIndex(), position,
+        if not waypoints:
+            waypoints = [Waypoint(0, 0, 0)] * rows
+
+        assert len(waypoints) == rows, (waypoints, rows)
+
+        self.beginInsertRows(QtCore.QModelIndex(), position,
                              position + rows - 1)
-        for row in range(rows):
-            wp = waypoints[row] if waypoints[row] else Waypoint(0, 0, 0)
+        for row, wp in enumerate(waypoints):
             self.waypoints.insert(position + row, wp)
 
         self.update_distances(position, rows=rows)
@@ -398,12 +388,12 @@ class WaypointsTableModel(QAbstractTableModel):
         self.modified = True
         return True
 
-    def removeRows(self, position, rows=1, index=QModelIndex()):
+    def removeRows(self, position, rows=1, index=QtCore.QModelIndex()):
         """Remove waypoint; overrides the corresponding QAbstractTableModel
            method.
         """
         # beginRemoveRows emits rowsAboutToBeRemoved(index, first, last).
-        self.beginRemoveRows(QModelIndex(), position,
+        self.beginRemoveRows(QtCore.QModelIndex(), position,
                              position + rows - 1)
         self.waypoints = self.waypoints[:position] + self.waypoints[position + rows:]
         if position < len(self.waypoints):
@@ -467,13 +457,13 @@ class WaypointsTableModel(QAbstractTableModel):
                 wp1.rem_fuel = self.performance_settings["fuel"]
             else:
                 wp0 = waypoints[pos - 1]
-                wp1.distance_to_prev = mss_util.get_distance((wp0.lat, wp0.lon),
-                                                             (wp1.lat, wp1.lon))
+                wp1.distance_to_prev = utils.get_distance((wp0.lat, wp0.lon),
+                                                          (wp1.lat, wp1.lon))
 
                 time, fuel = get_duration_fuel(wp0.flightlevel, wp1.flightlevel, wp1.distance_to_prev, wp0.weight)
                 wp1.leg_time = time
                 wp1.cum_time = wp0.cum_time + wp1.leg_time
-                wp1.utc_time = wp0.utc_time + timedelta(seconds=wp1.leg_time)
+                wp1.utc_time = wp0.utc_time + datetime.timedelta(seconds=wp1.leg_time)
                 wp1.leg_fuel = fuel
                 wp1.rem_fuel = wp0.rem_fuel - wp1.leg_fuel
                 wp1.weight = wp0.weight - wp1.leg_fuel
@@ -481,8 +471,8 @@ class WaypointsTableModel(QAbstractTableModel):
         # Update the distance of the following waypoint as well.
         if pos < len(waypoints) - 1:
             wp2 = waypoints[pos + 1]
-            wp2.distance_to_prev = mss_util.get_distance((wp1.lat, wp1.lon),
-                                                         (wp2.lat, wp2.lon))
+            wp2.distance_to_prev = utils.get_distance((wp1.lat, wp1.lon),
+                                                      (wp2.lat, wp2.lon))
 
         # Update total distances of waypoint at index position and all
         # following waypoints.
@@ -495,7 +485,7 @@ class WaypointsTableModel(QAbstractTableModel):
 
             wp1.leg_time = time
             wp1.cum_time = wp0.cum_time + wp1.leg_time
-            wp1.utc_time = wp0.utc_time + timedelta(seconds=wp1.leg_time)
+            wp1.utc_time = wp0.utc_time + datetime.timedelta(seconds=wp1.leg_time)
             wp1.leg_fuel = fuel
             wp1.rem_fuel = wp0.rem_fuel - wp1.leg_fuel
             wp1.weight = wp0.weight - wp1.leg_fuel
@@ -506,16 +496,15 @@ class WaypointsTableModel(QAbstractTableModel):
             self.waypoints[0].distance_to_prev = 0
             self.waypoints[0].distance_total = 0
         for i in range(1, len(self.waypoints)):
-            wp_comm = str(self.waypoints[i].comments)
+            wp_comm = unicode(self.waypoints[i].comments)
             if len(wp_comm) == 9 and wp_comm.startswith("Hexagon "):
                 wp_comm = "Hexagon {:d}".format(8 - int(wp_comm[-1]))
                 self.waypoints[i].comments = QString(wp_comm)
         self.update_distances(position=0, rows=len(self.waypoints))
         index = self.index(0, 0)
 
-        self.emit(SIGNAL("layoutChanged()"))
-        self.emit(SIGNAL("dataChanged(QModelIndex,QModelIndex)"),
-                  index, index)
+        self.layoutChanged.emit()
+        self.dataChanged.emit(index, index)
 
     def replaceWaypoints(self, new_waypoints):
         self.waypoints = []
@@ -551,17 +540,16 @@ class WaypointsTableModel(QAbstractTableModel):
         for wp in self.waypoints:
             element = doc.createElement("Waypoint")
             wp_el.appendChild(element)
-            element.setAttribute("location", str(wp.location))
+            element.setAttribute("location", unicode(wp.location))
             element.setAttribute("lat", str(wp.lat))
             element.setAttribute("lon", str(wp.lon))
             element.setAttribute("flightlevel", str(wp.flightlevel))
             comments = doc.createElement("Comments")
-            comments.appendChild(doc.createTextNode(str(wp.comments)))
+            comments.appendChild(doc.createTextNode(unicode(wp.comments)))
             element.appendChild(comments)
 
-        file_object = open(self.filename, 'w')
-        doc.writexml(file_object, indent="  ", addindent="  ", newl="\n")
-        file_object.close()
+        with codecs.open(self.filename, 'w', encoding="utf-8") as file_object:
+            doc.writexml(file_object, indent="  ", addindent="  ", newl="\n", encoding="utf-8")
 
     def loadFromFTML(self, filename):
         """Load a flight track from an XML file at <filename>.
@@ -601,7 +589,7 @@ class WaypointsTableModel(QAbstractTableModel):
 #
 
 
-class WaypointDelegate(QItemDelegate):
+class WaypointDelegate(QtWidgets.QItemDelegate):
     """Qt delegate class for the appearance of the table view. Based on the
        'ships' example in chapter 14/16 of 'Rapid GUI Programming with Python
        and Qt: The Definitive Guide to PyQt Programming' (Mark Summerfield).
@@ -609,7 +597,6 @@ class WaypointDelegate(QItemDelegate):
 
     def __init__(self, parent=None):
         super(WaypointDelegate, self).__init__(parent)
-        self.viewParent = parent
 
     def paint(self, painter, option, index):
         """Colours waypoints with a minor waypoint number (i.e. intermediate
@@ -623,18 +610,18 @@ class WaypointDelegate(QItemDelegate):
             colour = QtGui.QColor(255, 255, 0)  # yellow
             newpalette.setColor(QtGui.QPalette.HighlightedText, colour)
             option.palette = newpalette
-        QItemDelegate.paint(self, painter, option, index)
+        QtWidgets.QItemDelegate.paint(self, painter, option, index)
 
     def createEditor(self, parent, option, index):
         """Create a combobox listing predefined locations in the LOCATION
            column.
         """
         if index.column() == LOCATION:
-            combobox = QComboBox(parent)
+            combobox = QtWidgets.QComboBox(parent)
             locations = config_loader(dataset='locations', default=mss_default.locations)
             adds = locations.keys()
-            if self.viewParent is not None:
-                for loc in [wp.location for wp in self.viewParent.waypoints_model.allWaypointData() if
+            if self.parent() is not None:
+                for loc in [wp.location for wp in self.parent().waypoints_model.allWaypointData() if
                             wp.location != ""]:
                     if loc not in adds:
                         adds.append(loc)
@@ -645,43 +632,45 @@ class WaypointDelegate(QItemDelegate):
             return combobox
         else:
             # All other columns get the standard editor.
-            return QItemDelegate.createEditor(self, parent, option,
-                                              index)
+            return QtWidgets.QItemDelegate.createEditor(self, parent, option, index)
 
     def setEditorData(self, editor, index):
-        text = index.model().data(index, Qt.DisplayRole).toString()
+        if USE_PYQT5:
+            text = index.model().data(index, QtCore.Qt.DisplayRole).value()
+        else:
+            text = index.model().data(index, QtCore.Qt.DisplayRole).toString()
         if index.column() in (LOCATION,):
             i = editor.findText(text)
             if i == -1:
                 i = 0
             editor.setCurrentIndex(i)
         else:
-            QItemDelegate.setEditorData(self, editor, index)
+            QtWidgets.QItemDelegate.setEditorData(self, editor, index)
 
     def setModelData(self, editor, model, index):
         """For the LOCATION column: If the user selects a location from the
            combobox, get the corresponding coordinates.
         """
         if index.column() == LOCATION:
-            loc = str(editor.currentText())
+            loc = unicode(editor.currentText())
             locations = config_loader(dataset='locations', default=mss_default.locations)
             if loc in locations.keys():
                 lat, lon = locations[loc]
                 # Don't update distances and flight performance twice, hence
                 # set update=False for LAT.
-                model.setData(index.sibling(index.row(), LAT), QVariant(lat),
+                model.setData(index.sibling(index.row(), LAT), QtCore.QVariant(lat),
                               update=False)
-                model.setData(index.sibling(index.row(), LON), QVariant(lon))
+                model.setData(index.sibling(index.row(), LON), QtCore.QVariant(lon))
             else:
-                for wp in self.viewParent.waypoints_model.allWaypointData():
+                for wp in self.parent().waypoints_model.allWaypointData():
                     if loc == wp.location:
                         lat, lon = wp.lat, wp.lon
                         # Don't update distances and flight performance twice, hence
                         # set update=False for LAT.
-                        model.setData(index.sibling(index.row(), LAT), QVariant(lat),
+                        model.setData(index.sibling(index.row(), LAT), QtCore.QVariant(lat),
                                       update=False)
-                        model.setData(index.sibling(index.row(), LON), QVariant(lon))
+                        model.setData(index.sibling(index.row(), LON), QtCore.QVariant(lon))
 
-            model.setData(index, QVariant(editor.currentText()))
+            model.setData(index, QtCore.QVariant(editor.currentText()))
         else:
-            QItemDelegate.setModelData(self, editor, model, index)
+            QtWidgets.QItemDelegate.setModelData(self, editor, model, index)
