@@ -1,29 +1,61 @@
-"""Control widget to configure remote sensing overlays.
-
-
-This file is part of the Mission Support System User Interface (MSUI).
-
-AUTHORS:
-========
-
-* Joern Ungermann
-* Stefan Ensmann
-
+# -*- coding: utf-8 -*-
 """
 
-from PyQt4 import QtGui, QtCore  # Qt4 bindings
-from mslib.msui import ui_hexagon_dockwidget as ui
+    mslib.msui.hexagon_dockwidget
+    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+
+    Control widget to configure remote sensing overlays.
+
+    This file is part of mss.
+
+    :copyright: Copyright 2016-2017 Joern Ungermann, Stefan Ensmann
+    :copyright: Copyright 2016-2017 by the mss team, see AUTHORS.
+    :license: APACHE-2.0, see LICENSE for details.
+
+    Licensed under the Apache License, Version 2.0 (the "License");
+    you may not use this file except in compliance with the License.
+    You may obtain a copy of the License at
+
+       http://www.apache.org/licenses/LICENSE-2.0
+
+    Unless required by applicable law or agreed to in writing, software
+    distributed under the License is distributed on an "AS IS" BASIS,
+    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+    See the License for the specific language governing permissions and
+    limitations under the License.
+"""
+
+
+import numpy as np
+
+from mslib.msui.mss_qt import QtWidgets
+from mslib.msui.mss_qt import ui_hexagon_dockwidget as ui
 from mslib.msui import flighttrack as ft
-from mslib.mss_util import create_hexagon
-from mslib.mss_util import config_loader
 from mslib.msui import MissionSupportSystemDefaultConfig as mss_default
+from mslib.utils import config_loader, rotate_point
 
 
 class HexagonException(Exception):
     pass
 
 
-class HexagonControlWidget(QtGui.QWidget, ui.Ui_HexagonDockWidget):
+def create_hexagon(center_lat, center_lon, radius, angle=0.):
+    coords_0 = (radius, 0.)
+    CoordsCart_0 = [rotate_point(coords_0, angle=0. + angle),
+                    rotate_point(coords_0, angle=60. + angle),
+                    rotate_point(coords_0, angle=120. + angle),
+                    rotate_point(coords_0, angle=180. + angle),
+                    rotate_point(coords_0, angle=240. + angle),
+                    rotate_point(coords_0, angle=300. + angle),
+                    rotate_point(coords_0, angle=360. + angle)]
+    CoordsSphere_rot = [(center_lat + vec[0] / 110.,
+                         center_lon + vec[1] / (110. *
+                                                np.cos(np.deg2rad(vec[0] / 110. + center_lat))))
+                        for vec in CoordsCart_0]
+    return CoordsSphere_rot
+
+
+class HexagonControlWidget(QtWidgets.QWidget, ui.Ui_HexagonDockWidget):
     """This class implements the remote sensing functionality as dockable widget.
     """
 
@@ -35,15 +67,12 @@ class HexagonControlWidget(QtGui.QWidget, ui.Ui_HexagonDockWidget):
         """
         super(HexagonControlWidget, self).__init__(parent)
         self.setupUi(self)
-        self.parent = parent
         self.view = view
 
         self.dsbHexgaonRadius.setValue(200)
 
-        self.connect(self.pbAddHexagon, QtCore.SIGNAL("clicked()"),
-                     self._add_hexagon)
-        self.connect(self.pbRemoveHexagon, QtCore.SIGNAL("clicked()"),
-                     self._remove_hexagon)
+        self.pbAddHexagon.clicked.connect(self._add_hexagon)
+        self.pbRemoveHexagon.clicked.connect(self._remove_hexagon)
 
     def _get_parameters(self):
         return {
@@ -59,9 +88,8 @@ class HexagonControlWidget(QtGui.QWidget, ui.Ui_HexagonDockWidget):
         params = self._get_parameters()
 
         if params["radius"] < 0.01:
-            QtGui.QMessageBox.warning(
-                self, "Add hexagon", "You cannot create a hexagon with zero radius!",
-                QtGui.QMessageBox.Ok)
+            QtWidgets.QMessageBox.warning(
+                self, "Add hexagon", "You cannot create a hexagon with zero radius!")
             return
         points = create_hexagon(params["center_lat"], params["center_lon"], params["radius"], params["angle"])
         index = table_view.currentIndex()
@@ -92,7 +120,7 @@ class HexagonControlWidget(QtGui.QWidget, ui.Ui_HexagonDockWidget):
             if not index.isValid():
                 raise HexagonException("A waypoint of the hexagon must be selected.")
             row = index.row()
-            comm = str(waypoints_model.waypointData(row).comments)
+            comm = unicode(waypoints_model.waypointData(row).comments)
             if len(comm) == 9 and comm.startswith("Hexagon "):
                 if (len(waypoints_model.allWaypointData()) - 7) < 2:  # = 3 waypoints + 7 hexagon points
                     raise HexagonException("Cannot remove hexagon, the flight track needs to consist "
@@ -106,21 +134,21 @@ class HexagonControlWidget(QtGui.QWidget, ui.Ui_HexagonDockWidget):
                 else:
                     found_one = False
                     for i in range(0, row_max - row_min):
-                        if str(waypoints_model.waypointData(row_min + i).comments) != "Hexagon {:d}".format(i + 1):
+                        if unicode(waypoints_model.waypointData(row_min + i).comments) != "Hexagon {:d}".format(i + 1):
                             found_one = True
                             break
                     if found_one:
                         raise HexagonException("Cannot remove hexagon, hexagon comments are not found in all "
                                                "points (min, max = {:d}, {:d})".format(row_min, row_max))
                     else:
-                        sel = QtGui.QMessageBox.question(
+                        sel = QtWidgets.QMessageBox.question(
                             None, "Remove hexagon",
                             "This will remove waypoints {:d}-{:d}. Continue?".format(row_min, row_max),
-                            QtGui.QMessageBox.Yes | QtGui.QMessageBox.No)
-                        if sel == QtGui.QMessageBox.Yes:
+                            QtWidgets.QMessageBox.Yes | QtWidgets.QMessageBox.No)
+                        if sel == QtWidgets.QMessageBox.Yes:
                             waypoints_model.removeRows(row_min, rows=7)
             else:
                 raise HexagonException("Cannot remove hexagon, please select a hexagon "
                                        "waypoint ('Hexagon x' in comments field)")
         except HexagonException, ex:
-            QtGui.QMessageBox.warning(self, "Remove hexagon", str(ex), QtGui.QMessageBox.Ok)
+            QtWidgets.QMessageBox.warning(self, "Remove hexagon", str(ex))
