@@ -1,46 +1,41 @@
-"""Widget to display batch-generated images.
+# -*- coding: utf-8 -*-
+"""
+    mslib.msui.loopviewer_widget
+    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 
-********************************************************************************
+    Widget to display batch-generated images.
 
-   Copyright 2008-2014 Deutsches Zentrum fuer Luft- und Raumfahrt e.V.
+    This module provides three classes used by "loopview.py":
+    1) ProductChooserDialog: To display a dialog in which the user can choose
+    a product to load into an image widget (e.g. ECMWF Europe relative humidity).
+    2) LoopLabel: Derived from QLabel. This is a label that is used to display
+    an image, extended by a method that observes mouse wheel events (for time
+    and level navigation).
+    3) ImageLoopWidget: Widget that includes a LoopLabel for an image. Additionally
+    provides GUI elements to control zoom and vertical level navigation. A double
+    click on the widget opens a ProductChooserDialog.
 
-   Licensed under the Apache License, Version 2.0 (the "License");
-   you may not use this file except in compliance with the License.
-   You may obtain a copy of the License at
+    This file is part of mss.
+
+    :copyright: Copyright 2008-2014 Deutsches Zentrum fuer Luft- und Raumfahrt e.V.
+    :copyright: Copyright 2011-2014 Marc Rautenhaus (mr)
+    :copyright: Copyright 2016-2017 by the mss team, see AUTHORS.
+    :license: APACHE-2.0, see LICENSE for details.
+
+    Licensed under the Apache License, Version 2.0 (the "License");
+    you may not use this file except in compliance with the License.
+    You may obtain a copy of the License at
 
        http://www.apache.org/licenses/LICENSE-2.0
 
-   Unless required by applicable law or agreed to in writing, software
-   distributed under the License is distributed on an "AS IS" BASIS,
-   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-   See the License for the specific language governing permissions and
-   limitations under the License.
-
-********************************************************************************
-
-This file is part of the Mission Support System User Interface (MSUI).
-
-DESCRIPTION:
-============
-
-This file provides three classes used by "loopview.py":
-1) ProductChooserDialog: To display a dialog in which the user can choose
-a product to load into an image widget (e.g. ECMWF Europe relative humidity).
-2) LoopLabel: Derived from QLabel. This is a label that is used to display
-an image, extended by a method that observes mouse wheel events (for time
-and level navigation).
-3) ImageLoopWidget: Widget that includes a LoopLabel for an image. Additionally
-provides GUI elements to control zoom and vertical level navigation. A double
-click on the widget opens a ProductChooserDialog.
-
-AUTHORS:
-========
-
-* Marc Rautenhaus (mr)
-
+    Unless required by applicable law or agreed to in writing, software
+    distributed under the License is distributed on an "AS IS" BASIS,
+    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+    See the License for the specific language governing permissions and
+    limitations under the License.
 """
 
-# standard library imports
+
 from datetime import datetime, timedelta
 
 import StringIO
@@ -50,19 +45,15 @@ import os
 import urllib2
 
 # related third party imports
-from PyQt4 import QtGui, QtCore  # Qt4 bindings
+from mslib.msui.mss_qt import QtGui, QtCore, QtWidgets
 
 # local application imports
-from mslib.msui import ui_imageloop_widget as ui
-from mslib.msui import ui_imageloop_load_dialog as uipc
+from mslib.msui.mss_qt import ui_imageloop_widget as ui
+from mslib.msui.mss_qt import ui_imageloop_load_dialog as uipc
 from mslib.msui.wms_control import MSS_WMS_AuthenticationDialog
 
-#
-# Product Chooser Dialog
-#
 
-
-class ProductChooserDialog(QtGui.QDialog, uipc.Ui_ProductChooserDialog):
+class ProductChooserDialog(QtWidgets.QDialog, uipc.Ui_ProductChooserDialog):
     """Dialog to let the user choose a product to load (e.g. ECMWF Europe
        relative humidity).
 
@@ -83,17 +74,11 @@ class ProductChooserDialog(QtGui.QDialog, uipc.Ui_ProductChooserDialog):
                                       microsecond=0)))
 
         # Connect slots and signals.
-        self.connect(self.cbType,
-                     QtCore.SIGNAL("currentIndexChanged (const QString&)"),
-                     self.loadProducts)
-        self.connect(self.cbProduct,
-                     QtCore.SIGNAL("currentIndexChanged (const QString&)"),
-                     self.loadRegions)
+        self.cbType.currentIndexChanged.connect(self.loadProducts)
+        self.cbProduct.currentIndexChanged.connect(self.loadRegions)
 
-        self.connect(self.tbInitTime_back, QtCore.SIGNAL("clicked()"),
-                     functools.partial(self.changeInitTime, False))
-        self.connect(self.tbInitTime_fwd, QtCore.SIGNAL("clicked()"),
-                     functools.partial(self.changeInitTime, True))
+        self.tbInitTime_back.clicked.connect(functools.partial(self.changeInitTime, False))
+        self.tbInitTime_fwd.clicked.connect(functools.partial(self.changeInitTime, True))
 
     def type(self):
         return str(self.cbType.currentText())
@@ -155,16 +140,14 @@ class ProductChooserDialog(QtGui.QDialog, uipc.Ui_ProductChooserDialog):
                                                timestep))
 
 
-#
-# CLASS LoopLabel
-#
-
-
-class LoopLabel(QtGui.QLabel):
+class LoopLabel(QtWidgets.QLabel):
     """This is a label that is used to display an image, extended by a method
        that observes mouse wheel events (for time and level navigation).
     """
 
+    wheelOnImage = QtCore.pyqtSignal([bool], name="wheelOnImage")
+    shiftWheelOnImage = QtCore.pyqtSignal([bool], name="shiftWheelOnImage")
+
     def wheelEvent(self, event):
         """Called when the mouse wheel has been moved above the label.
 
@@ -174,19 +157,13 @@ class LoopLabel(QtGui.QLabel):
         been moved upwards, False if it has been moved downwards.
         """
         if event.modifiers() == QtCore.Qt.ShiftModifier:
-            self.emit(QtCore.SIGNAL("shiftWheelOnImage(bool)"),
-                      event.delta() > 0)
+            self.shiftWheelOnImage.emit(event.delta() > 0)
         else:
-            self.emit(QtCore.SIGNAL("wheelOnImage(bool)"), event.delta() > 0)
+            self.wheelOnImage.emit(event.delta() > 0)
         event.accept()
 
 
-#
-# CLASS Image Viewer Widget
-#
-
-
-class ImageLoopWidget(QtGui.QWidget, ui.Ui_ImageLoopWidget):
+class ImageLoopWidget(QtWidgets.QWidget, ui.Ui_ImageLoopWidget):
     """Widget that includes a LoopLabel for an image. Additionally
        provides GUI elements to control zoom and vertical level navigation.
        A double click on the widget opens a ProductChooserDialog.
@@ -204,6 +181,7 @@ class ImageLoopWidget(QtGui.QWidget, ui.Ui_ImageLoopWidget):
     """
 
     num = 0
+    signalChangeValidTime = QtCore.pyqtSignal([bool, object])
 
     def __init__(self, config=None, *args):
         super(ImageLoopWidget, self).__init__(*args)
@@ -230,7 +208,7 @@ class ImageLoopWidget(QtGui.QWidget, ui.Ui_ImageLoopWidget):
         # central widget of the scroll area.
         self.imageLabel = LoopLabel()
         self.imageLabel.setBackgroundRole(QtGui.QPalette.Base)
-        self.imageLabel.setSizePolicy(QtGui.QSizePolicy.Ignored, QtGui.QSizePolicy.Ignored)
+        self.imageLabel.setSizePolicy(QtWidgets.QSizePolicy.Ignored, QtWidgets.QSizePolicy.Ignored)
         self.imageLabel.setScaledContents(True)
         self.imageLabel.setText("double click to select a product for retrieval.")
         self.imageLabel.adjustSize()
@@ -247,26 +225,19 @@ class ImageLoopWidget(QtGui.QWidget, ui.Ui_ImageLoopWidget):
         self.products_dialog = ProductChooserDialog(config=config)
 
         # Connect slots and signals.
-        self.connect(self.btZoomIn, QtCore.SIGNAL("clicked()"), self.zoomIn)
-        self.connect(self.btZoomOut, QtCore.SIGNAL("clicked()"), self.zoomOut)
-        self.connect(self.btZoomNormalSize, QtCore.SIGNAL("clicked()"),
-                     self.normalSize)
-        self.connect(self.btFitToWindow, QtCore.SIGNAL("clicked()"),
-                     self.fitToWindow)
+        self.btZoomIn.clicked.connect(self.zoomIn)
+        self.btZoomOut.clicked.connect(self.zoomOut)
+        self.btZoomNormalSize.clicked.connect(self.normalSize)
+        self.btFitToWindow.clicked.connect(self.fitToWindow)
 
-        self.connect(self.tbLevel_down, QtCore.SIGNAL("clicked()"),
-                     functools.partial(self.changeLevel, False))
-        self.connect(self.tbLevel_up, QtCore.SIGNAL("clicked()"),
-                     functools.partial(self.changeLevel, True))
+        self.tbLevel_down.clicked.connect(functools.partial(self.changeLevel, False))
+        self.tbLevel_up.clicked.connect(functools.partial(self.changeLevel, True))
 
-        self.connect(self.imageLabel, QtCore.SIGNAL("wheelOnImage(bool)"),
-                     self.changeValidTime)
-        self.connect(self.imageLabel, QtCore.SIGNAL("shiftWheelOnImage(bool)"),
-                     self.changeLevel)
+        self.imageLabel.wheelOnImage.connect(self.changeValidTime)
+        self.imageLabel.shiftWheelOnImage.connect(self.changeLevel)
 
         # Progress dialog to inform the user about image ongoing retrievals.
-        self.pdlg = QtGui.QProgressDialog("retrieving images...", "Cancel",
-                                          0, 100, self)
+        self.pdlg = QtWidgets.QProgressDialog("retrieving images...", "Cancel", 0, 100, self)
 
     def zoomIn(self):
         """Slot connected to the '+' button (zooms in).
@@ -358,13 +329,13 @@ class ImageLoopWidget(QtGui.QWidget, ui.Ui_ImageLoopWidget):
 
             pixmap_cache = []
 
-            filename = "%s_%s_%s" % (prod_abbrev, reg_abbrev, str(level))
-            url = os.path.join(base_url, init_time_str, filename) + "_%03i.png"
+            filename = u"{}_{}_{}".format(prod_abbrev, reg_abbrev, str(level))
+            url = u"{0}_%03i.png".format(os.path.join(base_url, init_time_str, filename))
 
             # LOOP over TIMESTEPS **********************************************
             for istep, step in enumerate(steps):
-                retrieve = url % step
-                logging.debug("attempting to retrieve image file:\n >>> %s", retrieve)
+                retrieve = url.format(step)
+                logging.debug(u"attempting to retrieve image file: '{}'".format(retrieve))
                 try:
                     auth_required = True
                     while auth_required:
@@ -390,18 +361,18 @@ class ImageLoopWidget(QtGui.QWidget, ui.Ui_ImageLoopWidget):
                                 # returned by the server and ask the user for username
                                 # and password.
                                 realm = ex.hdrs["WWW-Authenticate"].split('"')[1]
-                                logging.debug("'%s' asks for authentication.", realm)
+                                logging.debug(u"'{}' asks for authentication.".format(realm))
                                 dlg = MSS_WMS_AuthenticationDialog(parent=self)
                                 dlg.lblMessage.setText(realm)
                                 dlg.setModal(True)
-                                if dlg.exec_() == QtGui.QDialog.Accepted:
+                                if dlg.exec_() == QtWidgets.QDialog.Accepted:
                                     username, password = dlg.getAuthInfo()
                                     auth_installed = False
                                 else:
                                     cancel = True
                                     break
                             else:
-                                raise ex
+                                raise
                     if not cancel:
                         # Read the image file from the URL into a string (urlobject.read())
                         # and wrap this string into a StringIO object that behaves like a file.
@@ -412,12 +383,12 @@ class ImageLoopWidget(QtGui.QWidget, ui.Ui_ImageLoopWidget):
                         valid_time = init_time + timedelta(seconds=3600 * step)
                         pixmap_cache.append((qp, valid_time))
                 except urllib2.HTTPError as ex:
-                    logging.debug("timestep %03i not available (HTTP error %i)", step, ex.code)
+                    logging.debug("timestep {:03d} not available (HTTP error {d})".format(step, ex.code))
 
                 # Update progress dialog.
                 self.pdlg.setValue((float(ilevel) + float(istep) / num_steps) / num_levels * 100.)
                 self.pdlg.repaint()
-                QtGui.QApplication.processEvents()
+                QtWidgets.QApplication.processEvents()
                 if self.pdlg.wasCanceled() or cancel:
                     break
 
@@ -440,13 +411,12 @@ class ImageLoopWidget(QtGui.QWidget, ui.Ui_ImageLoopWidget):
             self.synchronized = True
             self.levels = levels
             self.init_time = init_time
-            self.lblInfo.setToolTip("Initialisation time: %s" %
-                                    self.init_time.strftime("%Y-%m-%d %H:%M UTC"))
+            self.lblInfo.setToolTip(u"Initialisation time: {}"
+                                    .format(self.init_time.strftime("%Y-%m-%d %H:%M UTC")))
             self.updateImage()
-            self.emit(QtCore.SIGNAL("changeValidTime(bool, PyQt_PyObject)"), True,
-                      self.valid_time)
-            self.emit(QtCore.SIGNAL("changeValidTime(bool, PyQt_PyObject)"), False,
-                      self.valid_time)
+            self.signalChangeValidTime.emit(True, self.valid_time)
+            self.signalChangeValidTime.emit(False, self.valid_time)
+
             # TODO: Resize the window so that the image fits exactly. Also make the image
             #      keep its aspect ratio on fit-to-window resizes (mr, 2010-09-02):
             #      http://lists.trolltech.com/qt-interest/2005-11/thread01034-0.html
@@ -454,8 +424,7 @@ class ImageLoopWidget(QtGui.QWidget, ui.Ui_ImageLoopWidget):
             pxm = self.imageLabel.pixmap()
             self.resize(pxm.width() + 50, pxm.height() + 150)
 
-            self.connect(self.viewer_parent, QtCore.SIGNAL("changeValidTime(bool, PyQt_PyObject)"),
-                         self.changeValidTime)
+            self.viewer_parent.signalChangeValidTime.connect(self.changeValidTime)
 
         else:
             self.imageLabel.clear()
@@ -479,10 +448,10 @@ class ImageLoopWidget(QtGui.QWidget, ui.Ui_ImageLoopWidget):
             else:
                 self.synchronized = True
         colour = "black" if self.synchronized else "red"
-        self.lblInfo.setText("<font style='color: %s;'><b>VT: %s, LVL: %s</b></font>" %
-                             (colour,
-                              self.valid_time.strftime("%Y-%m-%d %H:%M UTC"),
-                              self.levels[self.current_level]))
+        self.lblInfo.setText(u"<font style='color: {};'><b>VT: {}, LVL: {}</b></font>"
+                             .format(colour,
+                                     self.valid_time.strftime("%Y-%m-%d %H:%M UTC"),
+                                     self.levels[self.current_level]))
 
     def updateImage(self, time=None):
         """Update the displayed pixmap.
@@ -544,8 +513,7 @@ class ImageLoopWidget(QtGui.QWidget, ui.Ui_ImageLoopWidget):
         # Emit a changeValidTime signal to inform the other image views
         # of the time change.
         if time is None:
-            self.emit(QtCore.SIGNAL("changeValidTime(bool, PyQt_PyObject)"), forward,
-                      self.valid_time)
+            self.changeValidTime.emit(forward, self.valid_time)
 
     def changeLevel(self, up):
         """Change the current level. Called on either
@@ -567,33 +535,9 @@ class ImageLoopWidget(QtGui.QWidget, ui.Ui_ImageLoopWidget):
            and lets the user load new images.
         """
         result = self.products_dialog.exec_()
-        if result == QtGui.QDialog.Accepted:
+        if result == QtWidgets.QDialog.Accepted:
             self.loadDataFromHTTP(self.products_dialog.type(),
                                   self.products_dialog.product(),
                                   self.products_dialog.region(),
                                   self.products_dialog.initTime())
         event.accept()
-
-
-################################################################################
-################################################################################
-# Module test.
-
-def _main():
-    # Log everything, and send it to stderr.
-    # See http://docs.python.org/library/logging.html for more information
-    # on the Python logging module.
-    # NOTE: http://docs.python.org/library/logging.html#formatter-objects
-    logging.basicConfig(level=logging.DEBUG,
-                        format="%(asctime)s (%(module)s.%(funcName)s): %(message)s",
-                        datefmt="%Y-%m-%d %H:%M:%S")
-
-    import sys
-
-    application = QtGui.QApplication(sys.argv)
-    window = ImageLoopWidget(config=configuration)
-    window.show()
-    sys.exit(application.exec_())
-
-if __name__ == "__main__":
-    _main()
