@@ -1,63 +1,56 @@
-"""Map canvas for the top view.
+# -*- coding: utf-8 -*-
+"""
 
-********************************************************************************
+    mslib.msui.mpl_map
+    ~~~~~~~~~~~~~~~~~~
 
-   Copyright 2008-2014 Deutsches Zentrum fuer Luft- und Raumfahrt e.V.
+    Map canvas for the top view.
+    As Matplotlib's Basemap is primarily designed to produce static plots,
+    we derived a class MapCanvas to allow for a certain degree of
+    interactivity. MapCanvas extends Basemap by functionality to, for
+    instance, automatically draw a graticule. It also keeps references to
+    plotted map elements to allow the user to toggle their visibility, or
+    to redraw when map section (zoom/pan) or projection have been changed.
 
-   Licensed under the Apache License, Version 2.0 (the "License");
-   you may not use this file except in compliance with the License.
-   You may obtain a copy of the License at
+    This file is part of mss.
+
+    :copyright: Copyright 2008-2014 Deutsches Zentrum fuer Luft- und Raumfahrt e.V.
+    :copyright: Copyright 2011-2014 Marc Rautenhaus (mr)
+    :copyright: Copyright 2016-2017 by the mss team, see AUTHORS.
+    :license: APACHE-2.0, see LICENSE for details.
+
+    Licensed under the Apache License, Version 2.0 (the "License");
+    you may not use this file except in compliance with the License.
+    You may obtain a copy of the License at
 
        http://www.apache.org/licenses/LICENSE-2.0
 
-   Unless required by applicable law or agreed to in writing, software
-   distributed under the License is distributed on an "AS IS" BASIS,
-   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-   See the License for the specific language governing permissions and
-   limitations under the License.
-
-********************************************************************************
-
-This file is part of the Mission Support System User Interface (MSUI).
-
-As Matplotlib's Basemap is primarily designed to produce static plots,
-we derived a class MapCanvas to allow for a certain degree of
-interactivity. MapCanvas extends Basemap by functionality to, for
-instance, automatically draw a graticule. It also keeps references to
-plotted map elements to allow the user to toggle their visibility, or
-to redraw when map section (zoom/pan) or projection have been changed.
-
-AUTHORS:
-========
-
-* Marc Rautenhaus (mr)
-
+    Unless required by applicable law or agreed to in writing, software
+    distributed under the License is distributed on an "AS IS" BASIS,
+    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+    See the License for the specific language governing permissions and
+    limitations under the License.
 """
 
-# standard library imports
+
 import logging
 
 # related third party imports
-from PyQt4 import QtCore  # Qt4 bindings
 import numpy as np
 import matplotlib
 import matplotlib.path as mpath
 import matplotlib.patches as mpatches
 import matplotlib.pyplot as plt
+from matplotlib import patheffects
 import mpl_toolkits.basemap as basemap
 try:
     import mpl_toolkits.basemap.pyproj as pyproj
 except ImportError:
     import pyproj
 
-# local application imports
 from mslib.msui import mpl_pathinteractor as mpl_pi
 from mslib.msui import trajectory_item_tree as titree
 
-"""
-CLASS MapCanvas
-"""
-
 
 class MapCanvas(basemap.Basemap):
     """Derivative of mpl_toolkits.basemap, providing additional methods to
@@ -77,7 +70,7 @@ class MapCanvas(basemap.Basemap):
 
         """
         # Coordinate reference system identifier and coordinate system units.
-        self.crs = CRS if CRS else self.crs if hasattr(self, "crs") else None
+        self.crs = CRS if CRS is not None else self.crs if hasattr(self, "crs") else None
         self.bbox_units = BBOX_UNITS if BBOX_UNITS else self.bbox_units \
             if hasattr(self, "bbox_units") else None
 
@@ -133,7 +126,7 @@ class MapCanvas(basemap.Basemap):
         self.image = None
 
         # Print CRS identifier into the figure.
-        if self.crs:
+        if self.crs is not None:
             if hasattr(self, "crs_text"):
                 self.crs_text.set_text(self.crs)
             else:
@@ -143,7 +136,7 @@ class MapCanvas(basemap.Basemap):
             try:
                 self._draw_auto_graticule()
             except Exception, ex:
-                logging.error("ERROR: cannot plot graticule (message: %s)", ex)
+                logging.error(u"ERROR: cannot plot graticule (message: {} - '{}')".format(type(ex), ex))
         else:
             self.map_parallels = None
             self.map_meridians = None
@@ -152,11 +145,15 @@ class MapCanvas(basemap.Basemap):
         self.ax.set_autoscale_on(False)
 
         # Connect to the trajectory item tree, if defined.
-        self.traj_item_tree = traj_item_tree if traj_item_tree else self.traj_item_tree if hasattr(
+        self.traj_item_tree = traj_item_tree if traj_item_tree is not None else self.traj_item_tree if hasattr(
             self, "traj_item_tree") else None
-        if traj_item_tree:
+        if traj_item_tree is not None:
             self.set_trajectory_tree(traj_item_tree)
 
+        # The View may be destroyed and then this class is left dangling due to the connected
+        # trajectories unless we disconnect it.
+        self.ax.figure.canvas.destroyed.connect(self.disconnectTrajectories)
+
     def set_identifier(self, identifier):
         self.identifier = identifier
 
@@ -296,7 +293,7 @@ class MapCanvas(basemap.Basemap):
             self._draw_auto_graticule()
             # Update the figure canvas.
             self.ax.figure.canvas.draw()
-        elif not visible and self.map_parallels and self.map_meridians:
+        elif not visible and self.map_parallels is not None and self.map_meridians is not None:
             # If visible if False, remove current graticule if one exists.
             # Every item in self.map_parallels and self.map_meridians is
             # a tuple of a list of lines and a list of text labels.
@@ -334,14 +331,14 @@ class MapCanvas(basemap.Basemap):
                                                       lake_color=self.appearance["colour_water"],
                                                       zorder=0)
             self.ax.figure.canvas.draw()
-        elif not visible and self.map_continents:
+        elif not visible and self.map_continents is not None:
             # Remove current fills. They are stored as a list of polygon patches
             # in self.map_continents.
             for patch in self.map_continents:
                 patch.remove()
             self.map_continents = None
             self.ax.figure.canvas.draw()
-        elif visible and self.map_continents:
+        elif visible and self.map_continents is not None:
             # Colours have changed: Remove the old fill and redraw.
             for patch in self.map_continents:
                 patch.remove()
@@ -358,7 +355,7 @@ class MapCanvas(basemap.Basemap):
             self.map_coastlines = self.drawcoastlines()
             self.map_countries = self.drawcountries()
             self.ax.figure.canvas.draw()
-        elif not visible and self.map_coastlines and self.map_countries:
+        elif not visible and self.map_coastlines is not None and self.map_countries is not None:
             self.map_coastlines.remove()
             self.map_countries.remove()
             del self.cntrysegs
@@ -375,7 +372,7 @@ class MapCanvas(basemap.Basemap):
         self.appearance["fill_waterbodies"] = visible
         self.appearance["colour_water"] = bg_color
 
-        if not visible and self.map_boundary:
+        if not visible and self.map_boundary is not None:
             self.map_boundary.remove()
             self.map_boundary = None
             self.ax.figure.canvas.draw()
@@ -411,48 +408,44 @@ class MapCanvas(basemap.Basemap):
 
         if self.kwargs["projection"] in ["cyl"]:
             # Latitudes in cylindrical projection need to be within -90..90.
-            if self.kwargs['urcrnrlat'] > 90:
-                self.kwargs['urcrnrlat'] = 90
-            if self.kwargs['llcrnrlat'] < -90:
-                self.kwargs['llcrnrlat'] = -90
-            # Longitudes in cylindrical projection need to be within -360..360.
-            if self.kwargs['llcrnrlon'] < -360 \
-                    or self.kwargs['urcrnrlon'] < -360:
-                self.kwargs['llcrnrlon'] += 360.
-                self.kwargs['urcrnrlon'] += 360.
-            if self.kwargs['llcrnrlon'] > 360 \
-                    or self.kwargs['urcrnrlon'] > 360:
-                self.kwargs['llcrnrlon'] -= 360.
-                self.kwargs['urcrnrlon'] -= 360.
+            self.kwargs['llcrnrlat'] = max(self.kwargs['llcrnrlat'], -90)
+            self.kwargs['urcrnrlat'] = max(self.kwargs['urcrnrlat'], -89)
+            self.kwargs['llcrnrlat'] = min(self.kwargs['llcrnrlat'], 89)
+            self.kwargs['urcrnrlat'] = min(self.kwargs['urcrnrlat'], 90)
+            # Longitudes in cylindrical projection need to be within -360..540.
+            self.kwargs["llcrnrlon"] = max(self.kwargs["llcrnrlon"], -360)
+            self.kwargs["urcrnrlon"] = max(self.kwargs["urcrnrlon"], -359)
+            self.kwargs["llcrnrlon"] = min(self.kwargs["llcrnrlon"], 539)
+            self.kwargs["urcrnrlon"] = min(self.kwargs["urcrnrlon"], 540)
 
         # Remove the current map artists.
         grat_vis = self.appearance["draw_graticule"]
         self.set_graticule_visible(False)
         self.appearance["draw_graticule"] = grat_vis
-        if self.map_coastlines:
+        if self.map_coastlines is not None:
             self.map_coastlines.remove()
 
-        if self.image:
+        if self.image is not None:
             self.image.remove()
             self.image = None
 
         # Refer to Basemap.drawcountries() on how to remove country borders.
         # In addition to the matplotlib lines, the loaded country segment data
         # needs to be loaded. THE SAME NEEDS TO BE DONE WITH RIVERS ETC.
-        if self.map_countries:
+        if self.map_countries is not None:
             self.map_countries.remove()
             del self.cntrysegs
 
         # map_boundary is None for rectangular projections (basemap simply sets
         # the backgorund colour).
-        if self.map_boundary:
+        if self.map_boundary is not None:
             try:
                 # FIX (mr, 15Oct2012) -- something seems to have changed in
                 # newer Matplotlib versions: this causes an exception when
                 # the user zooms/pans..
                 self.map_boundary.remove()
-            except Exception, ex:
-                logging.debug("wild exception catched: %s", ex)
+            except NotImplementedError:
+                pass
 
         cont_vis = self.appearance["fill_continents"]
         self.set_fillcontinents_visible(False)
@@ -495,7 +488,7 @@ class MapCanvas(basemap.Basemap):
         """Overloads basemap.imshow(). Deletes any existing image and
            redraws the figure after the new image has been plotted.
         """
-        if self.image:
+        if self.image is not None:
             self.image.remove()
         self.image = super(MapCanvas, self).imshow(X, **kwargs)
         self.ax.figure.canvas.draw()
@@ -508,20 +501,18 @@ class MapCanvas(basemap.Basemap):
         """
         logging.debug("registering trajectory tree model")
         # Disconnect old tree, if defined.
-        if self.traj_item_tree:
-            self.traj_item_tree.disconnect(
-                self.traj_item_tree,
-                QtCore.SIGNAL("dataChanged(QModelIndex, QModelIndex)"),
-                self.update_from_trajectory_tree)
+        if self.traj_item_tree is not None:
+            self.traj_item_tree.dataChanged.disconnect(self.update_from_trajectory_tree)
         # Set and connect new tree.
         self.traj_item_tree = tree
-        self.traj_item_tree.connect(
-            self.traj_item_tree,
-            QtCore.SIGNAL("dataChanged(QModelIndex, QModelIndex)"),
-            self.update_from_trajectory_tree)
+        self.traj_item_tree.dataChanged.connect(self.update_from_trajectory_tree)
         # Draw tree items.
         self.update_trajectory_items()
 
+    def disconnectTrajectories(self):
+        if self.traj_item_tree is not None:
+            self.traj_item_tree.dataChanged.disconnect(self.update_from_trajectory_tree)
+
     def update_from_trajectory_tree(self, index1, index2):
         """This method should be connected to the 'dataChanged()' signal
            of the MapItemsTree.
@@ -530,7 +521,7 @@ class MapCanvas(basemap.Basemap):
               found out so far how to get all the items between the two
               indices).
         """
-        if not self.traj_item_tree:
+        if self.traj_item_tree is None:
             return
         # Update the map elements if the change that occured in traj_item_tree
         # affected a LagrantoMapItem.
@@ -574,14 +565,14 @@ class MapCanvas(basemap.Basemap):
                 VISIBILITY_CHANGE -- same action as for GXPROPERTY_CHANGE
         """
         # If no trajectory item tree is defined exit the method.
-        if not self.traj_item_tree:
+        if self.traj_item_tree is None:
             return
 
         logging.debug("updating trajectory items on map <%s>, mode %s",
                       self.identifier, mode)
 
         # If no item is given start at the root of the 'traj_item_tree'.
-        if not item:
+        if item is None:
             item = self.traj_item_tree.getRootItem()
             logging.debug("processing all trajectory items")
 
@@ -603,7 +594,7 @@ class MapCanvas(basemap.Basemap):
                              "linewidth": None,
                              "timeMarkerInterval": None}
         parent = item.parent()  # this would be None if item == rootItem
-        while parent:  # loop until root has been reached
+        while parent is not None:  # loop until root has been reached
             # Set visible to False if one item along the path to the root is
             # found to be set to invisible.
             parent_properties["visible"] = parent.isVisible(self.identifier) and parent_properties["visible"]
@@ -669,7 +660,7 @@ class MapCanvas(basemap.Basemap):
                     elif element == "lineplot":
                         plt.setp(
                             item.getGxElementProperty(element,
-                                                      "instance::%s" % self.identifier),
+                                                      u"instance::{}".format(self.identifier)),
                             visible=item_properties["visible"],
                             color=item_properties["colour"],
                             linestyle=item_properties["linestyle"],
@@ -677,7 +668,7 @@ class MapCanvas(basemap.Basemap):
                     elif element == "timemarker":
                         plt.setp(
                             item.getGxElementProperty(element,
-                                                      "instance::%s" % self.identifier),
+                                                      u"instance::{}".format(self.identifier)),
                             visible=item_properties["visible"],
                             color=item_properties["colour"])
 
@@ -695,7 +686,7 @@ class MapCanvas(basemap.Basemap):
                     # Remove old plot instances.
                     try:
                         instances = item.getGxElementProperty("lineplot",
-                                                              "instance::%s" % self.identifier)
+                                                              u"instance::{}".format(self.identifier))
                         for instance in instances:
                             instance.remove()
                     except (KeyError, ValueError):
@@ -707,7 +698,7 @@ class MapCanvas(basemap.Basemap):
                                               linestyle=item_properties["linestyle"],
                                               linewidth=item_properties["linewidth"])
                     item.setGxElementProperty("lineplot",
-                                              "instance::%s" % self.identifier,
+                                              u"instance::{}".format(self.identifier),
                                               plot_instance)
 
                 # Get the indexes for the time markers. If no time markers should
@@ -716,7 +707,7 @@ class MapCanvas(basemap.Basemap):
                     # Try to remove an existing time marker instance from
                     # the plot.
                     old_scatter_instance = item.getGxElementProperty("timemarker",
-                                                                     "instance::%s" % self.identifier)
+                                                                     u"instance::{}".format(self.identifier))
                     plt.setp(old_scatter_instance, visible=False)
                     old_scatter_instance.remove()
                 except (KeyError, ValueError):
@@ -727,7 +718,7 @@ class MapCanvas(basemap.Basemap):
                                                     s=20, c=item_properties["colour"],
                                                     visible=item_properties["visible"], linewidth=0)
                     item.setGxElementProperty("timemarker",
-                                              "instance::%s" % self.identifier,
+                                              u"instance::{}".format(self.identifier),
                                               scatter_instance)
 
         # Update the figure canvas.
@@ -778,9 +769,7 @@ class MapCanvas(basemap.Basemap):
             # projection, gc.npts() returns lons that connect lon1 and lat2, not lon1 and
             # lon2 ... I cannot figure out why, maybe this is an issue in certain versions
             # of pyproj?? (mr, 16Oct2012)
-            # print lons[i],lats[i],lons[i+1],lats[i+1],npoints
             lonlats = gc.npts(lons[i], lats[i], lons[i + 1], lats[i + 1], npoints)
-            # print lonlats
             for lon, lat in lonlats:
                 gclons.append(lon)
                 gclats.append(lat)
@@ -799,11 +788,6 @@ class MapCanvas(basemap.Basemap):
         return self.plot(x, y, **kwargs)
 
 
-"""
-CLASS SatelliteOverpassPatch
-"""
-
-
 class SatelliteOverpassPatch(object):
     """Represents a satellite overpass on the top view map (satellite
        track and, if available, swath).
@@ -882,11 +866,11 @@ class SatelliteOverpassPatch(object):
     def remove(self):
         """Remove this satellite patch from the map canvas.
         """
-        if self.trackline:
+        if self.trackline is not None:
             for element in self.trackline:
                 element.remove()
             self.trackline = None
-        if self.patch:
+        if self.patch is not None:
             self.patch.remove()
             self.patch = None
         for element in self.texts:
@@ -894,8 +878,8 @@ class SatelliteOverpassPatch(object):
             # the plots look fine nevertheless.
             try:
                 element.remove()
-            except:
-                pass
+            except Exception, ex:
+                logging.error("Wildcard exception caught: %s %s", type(ex), ex)
 
 
 class KMLPatch(object):
@@ -905,60 +889,128 @@ class KMLPatch(object):
     KML overlay implementation is currently very crude and basic and most features are not supported.
     """
 
-    def __init__(self, mapcanvas, kml):
+    def __init__(self, mapcanvas, kml, overwrite=False, color="red", linewidth=1):
         self.map = mapcanvas
         self.kml = kml
         self.patches = []
+        self.color = color
+        self.linewidth = linewidth
+        self.overwrite = overwrite
         self.draw()
 
-    def add_polygon(self, polygon):
-        coords = str(polygon.outerBoundaryIs.LinearRing.coordinates).split()
+    def computeXY(self, coordinates):
+        coords = str(coordinates).split()
         lons, lats = [[float(_x.split(",")[_i]) for _x in coords] for _i in range(2)]
-        x, y = self.map(lons, lats)
-        self.patches.append(self.map.plot(x, y, "-k"))
+        return self.map(lons, lats)
 
-    def add_point(self, point):
-        coords = str(point.coordinates).split()
-        lons, lats = [[float(_x.split(",")[_i]) for _x in coords] for _i in range(2)]
-        x, y = self.map(lons, lats)
-        self.patches.append(self.map.plot(x, y, "ok"))
+    def add_polygon(self, polygon, style, _):
+        """
+        Plot KML polygons
 
-    def parse_placemarks(self, placemarks):
-        for i in range(len(placemarks)):
-            placemark = placemarks[i]
-            if hasattr(placemark, "Polygon"):
-                self.add_polygon(placemark.Polygon)
-            if hasattr(placemark, "Point"):
-                self.add_point(placemark.Point)
-            if hasattr(placemark, "MultiGeometry"):
-                multigeometry = placemark.MultiGeometry
-                if hasattr(multigeometry, "Point"):
-                    for j in range(len(multigeometry.Point)):
-                        self.add_point(multigeometry.Point[j])
-                if hasattr(multigeometry, "Polygon"):
-                    for j in range(len(multigeometry.Polygon)):
-                        self.add_polygon(multigeometry.Polygon[j])
+        :param polygon: pykml object specifying a polygon
+        """
+        kwargs = self.styles.get(style, {}).get("PolyStyle", {"linewidth": self.linewidth, "color": self.color})
+        for boundary in ["outerBoundaryIs", "innerBoundaryIs"]:
+            if hasattr(polygon, boundary):
+                x, y = self.computeXY(getattr(polygon, boundary).LinearRing.coordinates)
+                self.patches.append(self.map.plot(x, y, "-", **kwargs))
 
-    def search_placemarks(self, document):
-        if hasattr(document, "Placemark"):
-            self.parse_placemarks(document.Placemark)
-        if hasattr(document, "Folder"):
-            for j in range(len(document.Folder)):
-                self.search_placemarks(document.Folder[j])
+    def add_point(self, point, style, name):
+        """
+        Plot KML point
+
+        :param point: pykml object specifying point
+        :param name: name of placemark for annotation
+        """
+        x, y = self.computeXY(point.coordinates)
+        self.patches.append(self.map.plot(x[0], y[0], "o", color=self.color))
+        if name is not None:
+            self.patches.append([self.map.ax.annotate(name, xy=(x[0], y[0]), xycoords="data",
+                                xytext=(5, 5), textcoords='offset points',
+                                path_effects=[patheffects.withStroke(linewidth=2, foreground='w')])])
+
+    def add_line(self, line, style, _):
+        """
+        Plot KML line
+
+        :param line: pykml LineString object
+        """
+        kwargs = self.styles.get(style, {}).get("LineStyle", {"linewidth": self.linewidth, "color": self.color})
+        x, y = self.computeXY(line.coordinates)
+        self.patches.append(self.map.plot(x, y, "-", **kwargs))
+
+    def parse_geometries(self, placemark, style=None, name=None):
+        if style.startswith("#"):
+            # Remove # at beginning of style marking a locally defined style.
+            # general urls for styles are not supported
+            style = style[1:]
+        for attr_name, method in (
+                ("Point", self.add_point),
+                ("Polygon", self.add_polygon),
+                ("LineString", self.add_line),
+                ("MultiGeometry", self.parse_geometries)):
+            for attr in getattr(placemark, attr_name, []):
+                logging.debug("Found %s", attr_name)
+                method(attr, style, name)
+
+    def parse_placemarks(self, level):
+        for placemark in getattr(level, "Placemark", []):
+            name = getattr(placemark, "name", None)
+            style = unicode(getattr(placemark, "styleUrl", ""))
+            logging.debug("Placemark: %s %s", style, name)
+            self.parse_geometries(placemark, style, name)
+        for folder in getattr(level, "Folder", []):
+            name = getattr(folder, "name", None)
+            logging.debug("Folder: %s", name)
+            self.parse_placemarks(folder)
+
+    def get_style_params(self, style):
+        result = {
+            "color": unicode(getattr(style, "color", "")),
+            "linewidth": float(getattr(style, "width", self.linewidth))
+        }
+        if not result["color"] or len(result["color"]) != 8:
+            result["color"] = self.color
+        else:
+            result["color"] = [int(result["color"][i:i + 2], 16) / 255. for i in range(0, 8, 2)]
+        return result
+
+    def parse_styles(self, level):
+        for style in getattr(level, "Style", []):
+            name = style.attrib.get("id")
+            if name is None:
+                continue
+            self.styles[name] = {
+                "LineStyle": self.get_style_params(getattr(style, "LineStyle", None)),
+                "PolyStyle": self.get_style_params(getattr(style, "PolyStyle", None))
+            }
+        for folder in getattr(level, "Folder", []):
+            name = getattr(folder, "name", None)
+            logging.debug("Folder: %s", name)
+            self.parse_styles(folder)
 
     def draw(self):
         """Do the actual plotting of the patch.
         """
         # Plot satellite track.
-        self.search_placemarks(self.kml.Document)
+        self.styles = {}
+        if not self.overwrite:
+            self.parse_styles(self.kml.Document)
+        self.parse_placemarks(self.kml.Document)
 
         self.map.ax.figure.canvas.draw()
 
-    def update(self):
+    def update(self, overwrite=None, color=None, linewidth=None):
         """Removes the current plot of the patch and redraws the patch.
            This is necessary, for instance, when the map projection and/or
            extent has been changed.
         """
+        if overwrite is not None:
+            self.overwrite = overwrite
+        if color is not None:
+            self.color = color
+        if linewidth is not None:
+            self.linewidth = linewidth
         self.remove()
         self.draw()
 
