@@ -1,72 +1,65 @@
-"""Interactive editing of Path objects on a Matplotlib canvas.
+# -*- coding: utf-8 -*-
+"""
 
-********************************************************************************
+    mslib.msui.mpl_pathinteractor
+    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 
-   Copyright 2008-2014 Deutsches Zentrum fuer Luft- und Raumfahrt e.V.
+    Interactive editing of Path objects on a Matplotlib canvas.
 
-   Licensed under the Apache License, Version 2.0 (the "License");
-   you may not use this file except in compliance with the License.
-   You may obtain a copy of the License at
+    This module provides the following classes:
+
+    a) WaypointsPath and subclasses PathV, PathH and PathH-GC:
+    Derivatives of matplotlib.Path, provide additional methods to
+    insert and delete vertices and to find best insertion points for new
+    vertices.
+
+    b) PathInteractor and subclasses VPathInteractor and HPathInteractor:
+    Classes that implement a path editor by binding matplotlib mouse events
+    to a WaypointsPath object. Support for moving, inserting and deleting vertices.
+
+    The code in this module is inspired by the matplotlib example 'path_editor.py'
+    (http://matplotlib.sourceforge.net/examples/event_handling/path_editor.html).
+
+    For more information on implementing animated graphics in matplotlib, see
+    http://www.scipy.org/Cookbook/Matplotlib/Animations.
+
+
+    This file is part of mss.
+
+    :copyright: Copyright 2008-2014 Deutsches Zentrum fuer Luft- und Raumfahrt e.V.
+    :copyright: Copyright 2011-2014 Marc Rautenhaus (mr)
+    :copyright: Copyright 2016-2017 by the mss team, see AUTHORS.
+    :license: APACHE-2.0, see LICENSE for details.
+
+    Licensed under the Apache License, Version 2.0 (the "License");
+    you may not use this file except in compliance with the License.
+    You may obtain a copy of the License at
 
        http://www.apache.org/licenses/LICENSE-2.0
 
-   Unless required by applicable law or agreed to in writing, software
-   distributed under the License is distributed on an "AS IS" BASIS,
-   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-   See the License for the specific language governing permissions and
-   limitations under the License.
-
-********************************************************************************
-
-This file is part of the Mission Support System User Interface (MSUI).
-
-This module provides the following classes:
-
-a) WaypointsPath and subclasses PathV, PathH and PathH-GC:
-Derivatives of matplotlib.Path, provide additional methods to
-insert and delete vertices and to find best insertion points for new
-vertices.
-
-b) PathInteractor and subclasses VPathInteractor and HPathInteractor:
-Classes that implement a path editor by binding matplotlib mouse events
-to a WaypointsPath object. Support for moving, inserting and deleting vertices.
-
-The code in this module is inspired by the matplotlib example 'path_editor.py'
-(http://matplotlib.sourceforge.net/examples/event_handling/path_editor.html).
-
-For more information on implementing animated graphics in matplotlib, see
-http://www.scipy.org/Cookbook/Matplotlib/Animations.
-
-AUTHORS:
-========
-
-* Marc Rautenhaus (mr)
-
+    Unless required by applicable law or agreed to in writing, software
+    distributed under the License is distributed on an "AS IS" BASIS,
+    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+    See the License for the specific language governing permissions and
+    limitations under the License.
 """
 
-# standard library imports
+
 import logging
+import math
 import numpy as np
 
 # related third party imports
 import matplotlib.path as mpath
 import matplotlib.patches as mpatches
-from PyQt4 import QtCore, QtGui
+from mslib.msui.mss_qt import QtCore, QtWidgets
+from mslib.utils import get_distance, find_location
 
 # local application imports
 from mslib.msui import flighttrack as ft
 
-"""
-CONSTANTS
-"""
-
-# Constants to specify the current editing mode.
 MOVE, DELETE, INSERT = range(3)
 
-"""
-HELPER ROUTINES
-"""
-
 
 def distance_point_linesegment(p, l1, l2):
     """Computes the distance between a point p and a line segment given by its
@@ -98,11 +91,6 @@ def distance_point_linesegment(p, l1, l2):
         return np.linalg.norm(p - p_on_line)
 
 
-"""
-CLASS WaypointsPath
-"""
-
-
 class WaypointsPath(mpath.Path):
     """Derivative of matplotlib.Path that provides methods to insert and
        delete vertices.
@@ -150,7 +138,6 @@ class WaypointsPath(mpath.Path):
             l1 = self.vertices[i]
             l2 = self.vertices[i + 1]
             distance = distance_point_linesegment(point, l1, l2)
-            # print i, l1, l2, distance
             if distance < eps and distance < min_distance:
                 min_index = i + 1
                 min_distance = distance
@@ -158,11 +145,9 @@ class WaypointsPath(mpath.Path):
         # Compute the distance between the given point and the end point of
         # the path. Is it smaller than the currently smallest distance?
         distance = np.linalg.norm(point - self.vertices[-1])
-        # print "last",distance
         if distance < min_distance:
             min_index = len(self.vertices)
 
-        # print "return", min_index, "\n"
         return min_index
 
     def transform_waypoint(self, wps_list, index):
@@ -183,7 +168,7 @@ class WaypointsPath(mpath.Path):
         wps = wps_model.allWaypointData()
         if len(wps) > 0:
             pathdata = [(Path.MOVETO, self.transform_waypoint(wps, 0))]
-            for i in range(len(wps[1:])):
+            for i, _ in enumerate(wps[1:]):
                 pathdata.append((Path.LINETO, self.transform_waypoint(wps, i + 1)))
 
         self.codes, self.vertices = zip(*pathdata)
@@ -191,9 +176,9 @@ class WaypointsPath(mpath.Path):
         self.vertices = np.array(self.vertices)
 
 
-"""
-CLASS PathV
-"""
+#
+# CLASS PathV
+#
 
 
 class PathV(WaypointsPath):
@@ -255,9 +240,9 @@ class PathV(WaypointsPath):
         return (self.intermediate_indexes[index], wps_list[index].pressure)
 
 
-"""
-CLASS PathH, PathH_GC
-"""
+#
+# CLASS PathH, PathH_GC
+#
 
 
 class PathH(WaypointsPath):
@@ -437,22 +422,16 @@ class PathInteractor:
         # If a model exists, disconnect from the old change() signals.
         wpm = self.waypoints_model
         if wpm:
-            wpm.disconnect(wpm, QtCore.SIGNAL("dataChanged(QModelIndex,QModelIndex)"),
-                           self.qt_data_changed_listener)
-            wpm.disconnect(wpm, QtCore.SIGNAL("rowsInserted(QModelIndex, int, int)"),
-                           self.qt_insert_remove_point_listener)
-            wpm.disconnect(wpm, QtCore.SIGNAL("rowsRemoved(QModelIndex, int, int)"),
-                           self.qt_insert_remove_point_listener)
+            wpm.dataChanged.disconnect(self.qt_data_changed_listener)
+            wpm.rowsInserted.disconnect(self.qt_insert_remove_point_listener)
+            wpm.rowsRemoved.disconnect(self.qt_insert_remove_point_listener)
         # Set the new waypoints model.
         self.waypoints_model = waypoints
         # Connect to the new model's signals.
         wpm = self.waypoints_model
-        wpm.connect(wpm, QtCore.SIGNAL("dataChanged(QModelIndex,QModelIndex)"),
-                    self.qt_data_changed_listener)
-        wpm.connect(wpm, QtCore.SIGNAL("rowsInserted(QModelIndex, int, int)"),
-                    self.qt_insert_remove_point_listener)
-        wpm.connect(wpm, QtCore.SIGNAL("rowsRemoved(QModelIndex, int, int)"),
-                    self.qt_insert_remove_point_listener)
+        wpm.dataChanged.connect(self.qt_data_changed_listener)
+        wpm.rowsInserted.connect(self.qt_insert_remove_point_listener)
+        wpm.rowsRemoved.connect(self.qt_insert_remove_point_listener)
         # Redraw.
         self.pathpatch.get_path().update_from_WaypointsTableModel(wpm)
         self.redraw_figure()
@@ -581,7 +560,7 @@ class PathInteractor:
         for i in range(len(wpd)):
             textlabel = str(i)
             if wpd[i].location != "":
-                textlabel = "{:}".format(wpd[i].location)
+                textlabel = u"{:}".format(wpd[i].location)
             t = self.ax.text(x[i] + label_offset,
                              y[i] + label_offset,
                              textlabel,
@@ -620,18 +599,18 @@ class PathInteractor:
         """
         wps = self.waypoints_model.allWaypointData()
         if len(wps) < 3:
-            QtGui.QMessageBox.warning(
+            QtWidgets.QMessageBox.warning(
                 None, "Remove waypoint",
                 "Cannot remove waypoint, the flight track needs to consist "
-                "of at least two points.", QtGui.QMessageBox.Ok)
+                "of at least two points.")
             return False
         else:
             wp = wps[row]
-            return (QtGui.QMessageBox.question(
+            return QtWidgets.QMessageBox.question(
                 None, "Remove waypoint",
-                "Remove waypoint no.%i at %.2f/%.2f, flightlevel %.2f?"
-                % (row, wp.lat, wp.lon, wp.flightlevel),
-                QtGui.QMessageBox.Yes | QtGui.QMessageBox.No) == QtGui.QMessageBox.Yes)
+                u"Remove waypoint no.{:d} at {:.2f}/{:.2f}, flightlevel {:.2f}?"
+                .format(row, wp.lat, wp.lon, wp.flightlevel),
+                QtWidgets.QMessageBox.Yes | QtWidgets.QMessageBox.No) == QtWidgets.QMessageBox.Yes
 
     def set_path_color(self, line_color=None, marker_facecolor=None,
                        patch_facecolor=None):
@@ -650,9 +629,9 @@ class PathInteractor:
             self.pathpatch.set_facecolor(patch_facecolor)
 
 
-"""
-CLASS VPathInteractor
-"""
+#
+# CLASS VPathInteractor
+#
 
 
 class VPathInteractor(PathInteractor):
@@ -713,12 +692,11 @@ class VPathInteractor(PathInteractor):
             # in the side view) to the data model.
             vertices = self.pathpatch.get_path().vertices
             pressure = vertices[self._ind][1]
-            # print "moved", self._ind, pressure
             # http://doc.trolltech.com/4.3/qabstractitemmodel.html#createIndex
             qt_index = self.waypoints_model.createIndex(self._ind, ft.PRESSURE)
             # NOTE: QVariant cannot handle numpy.float64 types, hence convert
             # to float().
-            self.waypoints_model.setData(qt_index, QtCore.QVariant(float(pressure)))
+            self.waypoints_model.setData(qt_index, QtCore.QVariant(float(pressure / 100.)))
 
         self._ind = None
 
@@ -755,10 +733,9 @@ class VPathInteractor(PathInteractor):
         # profile needs to be redrawn (redraw_path()). If the horizontal
         # position of a waypoint has changed, the entire figure needs to be
         # redrawn, as this affects the x-position of all points.
-        if index1.column() == ft.FLIGHTLEVEL:
+        if index1.column() == ft.FLIGHTLEVEL or index1.column() == ft.PRESSURE:
             i = index1.row()
             pres = self.waypoints_model.waypointData(i).pressure
-            # print "SideView: pressure of point %i has been changed to %f." % (i, pres)
             vertices = self.pathpatch.get_path().vertices
             vertices[i] = vertices[i][0], pres
             self.redraw_path(vertices)
@@ -820,13 +797,27 @@ class HPathInteractor(PathInteractor):
         width = int(round(ax_bounds[2]))
         map_delta_x = abs(self.map.llcrnrx - self.map.urcrnrx)
         map_coords_per_px_x = map_delta_x / width
-        # X and Y are always approximately the same, hence it's enough to only
-        # consider X.
-        # height = int(round(ax_bounds[3]))
-        # map_delta_y = abs(self.map.llcrnry - self.map.urcrnry)
-        # map_coords_per_px_y = map_delta_y / height
+
         return map_coords_per_px_x * px
 
+    def appropriateEpsilon2(self, px=5):
+        """Determine an epsilon value appropriate for the current projection and
+           figure size.
+
+        The epsilon value gives the distance required in map projection
+        coordinates that corresponds to approximately px Pixels in screen
+        coordinates. The value can be used to find the line/point that is
+        closest to a click while discarding clicks that are too far away
+        from any geometry feature.
+        """
+        # (bounds = left, bottom, width, height)
+        ax_bounds = self.ax.bbox.bounds
+        diagonal = math.hypot(round(ax_bounds[2]), round(ax_bounds[3]))
+        map_delta = get_distance((self.map.llcrnry, self.map.llcrnrx), (self.map.urcrnry, self.map.urcrnrx))
+        km_per_px = map_delta / diagonal
+
+        return km_per_px * px
+
     def button_release_callback(self, event):
         """Called whenever a mouse button is released.
         """
@@ -840,27 +831,33 @@ class HPathInteractor(PathInteractor):
                 # removeRows() will trigger a signal that will redraw the path.
                 self.waypoints_model.removeRows(self._ind)
 
-        elif self.editmode == INSERT:
+        elif self.editmode == INSERT and event.inaxes is not None:
             # Get position for new vertex.
             x, y = event.xdata, event.ydata
             best_index = self.pathpatch.get_path() \
                 .index_of_closest_segment(x, y, eps=self.appropriateEpsilon())
-            logging.debug("TopView insert point: clicked at (%f, %f), "
-                          "best index: %i", x, y, best_index)
+            logging.debug(u"TopView insert point: clicked at ({:f}, {:f}), "
+                          u"best index: {:d}".format(x, y, best_index))
             self.pathpatch.get_path().insert_vertex(best_index, [x, y],
                                                     WaypointsPath.LINETO)
 
             lon, lat = self.map(x, y, inverse=True)
+            loc = find_location(lat, lon, tolerance=self.appropriateEpsilon2(px=15))
+            if loc is not None:
+                (lat, lon), location = loc
+            else:
+                lat, lon = round(lat, 2), round(lon, 2)
+                location = u""
             wpm = self.waypoints_model
             if len(wpm.allWaypointData()) > 0 and 0 < best_index <= len(wpm.allWaypointData()):
                 flightlevel = wpm.waypointData(best_index - 1).flightlevel
             elif len(wpm.allWaypointData()) > 0 and best_index == 0:
                 flightlevel = wpm.waypointData(0).flightlevel
             else:
-                logging.error("Cannot copy flightlevel. best_index: %s, len: %s",
-                              best_index, len(wpm.allWaypointData()))
+                logging.error(u"Cannot copy flightlevel. best_index: {}, len: {}".format(
+                              best_index, len(wpm.allWaypointData())))
                 flightlevel = 0
-            new_wp = ft.Waypoint(round(lat, 2), round(lon, 2), flightlevel)
+            new_wp = ft.Waypoint(lat, lon, flightlevel, location=location)
             wpm.insertRows(best_index, rows=1, waypoints=[new_wp])
             self.redraw_path()
 
@@ -869,14 +866,18 @@ class HPathInteractor(PathInteractor):
             vertices = self.pathpatch.get_path().wp_vertices
             lon, lat = self.map(vertices[self._ind][0], vertices[self._ind][1],
                                 inverse=True)
-            # print "moved", self._ind, lon, lat
+            loc = find_location(lat, lon, tolerance=self.appropriateEpsilon2(px=15))
+            if loc is not None:
+                lat, lon = loc[0]
+            else:
+                lat, lon = round(lat, 2), round(lon, 2)
             # http://doc.trolltech.com/4.3/qabstractitemmodel.html#createIndex
             # TODO: can lat/lon be submitted together to avoid emitting dataChanged() signals
             #      twice?
             qt_index = self.waypoints_model.createIndex(self._ind, ft.LAT)
-            self.waypoints_model.setData(qt_index, QtCore.QVariant(round(float(lat), 2)))
+            self.waypoints_model.setData(qt_index, QtCore.QVariant(lat))
             qt_index = self.waypoints_model.createIndex(self._ind, ft.LON)
-            self.waypoints_model.setData(qt_index, QtCore.QVariant(round(float(lon), 2)))
+            self.waypoints_model.setData(qt_index, QtCore.QVariant(lon))
 
         self._ind = None
 
@@ -981,7 +982,7 @@ class HPathInteractor(PathInteractor):
         for i in range(len(wpd)):
             textlabel = str(i)
             if wpd[i].location != "":
-                textlabel = "{:}".format(wpd[i].location)
+                textlabel = u"{:}".format(wpd[i].location)
             t = self.ax.text(x[i] + label_offset,
                              y[i] + label_offset,
                              textlabel,
@@ -1023,6 +1024,10 @@ class HPathInteractor(PathInteractor):
         """
         PathInteractor.draw_callback(self, event)
         self.ax.draw_artist(self.wp_scatter)
+        if self.show_solar_angle:
+            self.ax.draw_artist(self.solar_lines)
+        if self.show_tangent_points:
+            self.ax.draw_artist(self.tangent_lines)
 
     def get_ind_under_point(self, event):
         """Get the index of the waypoint vertex under the point
