@@ -1,37 +1,38 @@
-"""Definitions of Matplotlib widgets for Qt4 Designer.
+# -*- coding: utf-8 -*-
+"""
 
-********************************************************************************
+    mslib.msui.mpl_qtwidget
+    ~~~~~~~~~~~~~~~~~~~~~~~
 
-   Copyright 2008-2014 Deutsches Zentrum fuer Luft- und Raumfahrt e.V.
+    Definitions of Matplotlib widgets for Qt4 Designer.
 
-   Licensed under the Apache License, Version 2.0 (the "License");
-   you may not use this file except in compliance with the License.
-   You may obtain a copy of the License at
+    This file is part of mss.
+
+    :copyright: Copyright 2008-2014 Deutsches Zentrum fuer Luft- und Raumfahrt e.V.
+    :copyright: Copyright 2011-2014 Marc Rautenhaus (mr)
+    :copyright: Copyright 2016-2017 by the mss team, see AUTHORS.
+    :license: APACHE-2.0, see LICENSE for details.
+
+    Licensed under the Apache License, Version 2.0 (the "License");
+    you may not use this file except in compliance with the License.
+    You may obtain a copy of the License at
 
        http://www.apache.org/licenses/LICENSE-2.0
 
-   Unless required by applicable law or agreed to in writing, software
-   distributed under the License is distributed on an "AS IS" BASIS,
-   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-   See the License for the specific language governing permissions and
-   limitations under the License.
-
-********************************************************************************
-
-Parts of the code have been adapted from Chapter 6 of Sandro Tosi,
-'Matplotlib for Python Developers'.
-
-AUTHORS:
-========
-
-* Marc Rautenhaus (mr)
-
+    Unless required by applicable law or agreed to in writing, software
+    distributed under the License is distributed on an "AS IS" BASIS,
+    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+    See the License for the specific language governing permissions and
+    limitations under the License.
 """
-# standard library imports
+
+# Parts of the code have been adapted from Chapter 6 of Sandro Tosi,
+# 'Matplotlib for Python Developers'.
+
 from datetime import datetime
 
 import logging
-from mslib.mss_util import config_loader
+from mslib.utils import config_loader
 from mslib.msui import MissionSupportSystemDefaultConfig as mss_default
 # related third party imports
 import numpy as np
@@ -45,16 +46,7 @@ from mslib import thermolib
 
 
 # Python Qt4 bindings for GUI objects
-from PyQt4 import QtGui, QtCore
-
-# import the Qt4Agg FigureCanvas object, that binds Figure to
-# Qt4Agg backend. It also inherits from QWidget
-from matplotlib.backends.backend_qt4agg \
-    import FigureCanvasQTAgg as FigureCanvas
-
-# import the NavigationToolbar Qt4Agg widget
-from matplotlib.backends.backend_qt4agg \
-    import NavigationToolbar2QT as NavigationToolbar
+from mslib.msui.mss_qt import QtCore, QtWidgets, FigureCanvas, NavigationToolbar
 
 # Matplotlib Figure object
 from matplotlib.figure import Figure
@@ -71,10 +63,6 @@ if _matplotlib_version >= '1.2':
 else:
     PIL_image_origin = "lower"
 
-"""
-Matplotlib Qt Canvas
-"""
-
 
 class MplCanvas(FigureCanvas):
     """Class to represent the FigureCanvas widget.
@@ -94,8 +82,8 @@ class MplCanvas(FigureCanvas):
 
         # we define the widget as expandable
         FigureCanvas.setSizePolicy(self,
-                                   QtGui.QSizePolicy.Expanding,
-                                   QtGui.QSizePolicy.Expanding)
+                                   QtWidgets.QSizePolicy.Expanding,
+                                   QtWidgets.QSizePolicy.Expanding)
 
         # notify the system of updated policy
         FigureCanvas.updateGeometry(self)
@@ -113,12 +101,12 @@ class MplCanvas(FigureCanvas):
         """
         self.default_filename = ""
         if title:
-            self.default_filename += "_%5s" % title.split()[0]
-        if isinstance(style, basestring):
-            title += ' (%s)' % style
+            self.default_filename += u"_{:>5}".format(title.split()[0])
+        if isinstance(style, basestring) and style:
+            title += u' ({})'.format(style)
         if isinstance(level, basestring):
-            title += ' at %s' % level
-            self.default_filename += "_%s" % level.split()[0]
+            title += u' at {}'.format(level)
+            self.default_filename += u"_{}".format(level.split()[0])
         if isinstance(valid_time, datetime) and isinstance(init_time, datetime):
             time_step = valid_time - init_time
         else:
@@ -132,15 +120,12 @@ class MplCanvas(FigureCanvas):
         if valid_time:
             if init_time:
                 if time_step is not None:
-                    title += '\nValid: %s (step %i hrs from %s)' % \
-                             (valid_time,
-                              (time_step.days * 86400 + time_step.seconds) / 3600,
-                              init_time)
+                    title += u"\nValid: {} (step {:d} hrs from {})".format(
+                        valid_time, (time_step.days * 86400 + time_step.seconds) / 3600, init_time)
                 else:
-                    title += '\nValid: %s (initialisation: %s)' % \
-                             (valid_time, init_time)
+                    title += u"\nValid: {} (initialisation: {})".format(valid_time, init_time)
             else:
-                title += '\nValid: %s' % valid_time
+                title += u"\nValid: {}".format(valid_time)
 
         # Set title.
         self.ax.set_title(title, horizontalalignment='left', x=0)
@@ -160,23 +145,18 @@ class MplCanvas(FigureCanvas):
         return width, height
 
 
-#
-# Matplotlib Qt Widget
-#
-
-
-class MplWidget(QtGui.QWidget):
+class MplWidget(QtWidgets.QWidget):
     """Matplotlib canvas widget defined in Qt Designer"""
 
     def __init__(self, parent=None):
         # initialization of Qt MainWindow widget
-        QtGui.QWidget.__init__(self, parent)
+        QtWidgets.QWidget.__init__(self, parent)
 
         # set the canvas to the Matplotlib widget
         self.canvas = MplCanvas()
 
         # create a vertical box layout
-        self.vbl = QtGui.QVBoxLayout()
+        self.vbl = QtWidgets.QVBoxLayout()
 
         # add mpl widget to vertical box
         self.vbl.addWidget(self.canvas)
@@ -185,12 +165,12 @@ class MplWidget(QtGui.QWidget):
         self.setLayout(self.vbl)
 
 
-class MplNavBarWidget(QtGui.QWidget):
+class MplNavBarWidget(QtWidgets.QWidget):
     """Matplotlib canvas widget with navigation toolbar defined in Qt Designer"""
 
     def __init__(self, parent=None, canvas=None):
         # initialization of Qt MainWindow widget
-        QtGui.QWidget.__init__(self, parent)
+        QtWidgets.QWidget.__init__(self, parent)
 
         # set the canvas to the Matplotlib widget
         if canvas:
@@ -202,7 +182,7 @@ class MplNavBarWidget(QtGui.QWidget):
         self.navbar = NavigationToolbar(self.canvas, self)
 
         # create a vertical box layout
-        self.vbl = QtGui.QVBoxLayout()
+        self.vbl = QtWidgets.QVBoxLayout()
 
         # add mpl widget to vertical box
         self.vbl.addWidget(self.navbar)
@@ -212,11 +192,6 @@ class MplNavBarWidget(QtGui.QWidget):
         self.setLayout(self.vbl)
 
 
-"""
-Side View Canvas and Widget
-"""
-
-
 class MplSideViewCanvas(MplCanvas):
     """Specialised MplCanvas that draws a side view (vertical section) of a
        flight track / list of waypoints.
@@ -317,7 +292,7 @@ class MplSideViewCanvas(MplCanvas):
                                     len_major_ticks + len(major_ticks2) - 1)
             major_ticks[len_major_ticks:] = major_ticks2[1:]
 
-        labels = ['%i' % (l / 100.) for l in major_ticks]
+        labels = ["{:d}".format(int(l / 100.)) for l in major_ticks]
 
         # .. the same for the minor ticks ..
         p_top_minor = max(label_distance, p_top)
@@ -367,10 +342,10 @@ class MplSideViewCanvas(MplCanvas):
         lat_inds = np.arange(len(lats))
         tick_index_step = len(lat_inds) / self.numlabels
         self.ax.set_xticks(lat_inds[::tick_index_step])
-        self.ax.set_xticklabels(["%2.1f, %2.1f" % (d[0], d[1])
+        self.ax.set_xticklabels(["{:2.1f}, {:2.1f}".format(d[0], d[1])
                                  for d in zip(lats[::tick_index_step],
                                               lons[::tick_index_step])],
-                                rotation=25, fontsize=10, horizontalalignment='right')
+                                rotation=25, fontsize=10, horizontalalignment="right")
 
     def setVerticalExtent(self, pbot, ptop):
         """Set the vertical extent of the view to the specified pressure
@@ -407,7 +382,7 @@ class MplSideViewCanvas(MplCanvas):
         for level in self.flightlevels:
             pressure = thermolib.flightlevel2pressure(level)
             self.fl_label_list.append(ax.axhline(pressure, color='k'))
-            self.fl_label_list.append(ax.text(0.1, pressure, "FL%i" % level))
+            self.fl_label_list.append(ax.text(0.1, pressure, u"FL{:d}".format(level)))
         self.draw()
 
     def getFlightLevels(self):
@@ -479,6 +454,11 @@ class MplSideViewCanvas(MplCanvas):
                 num_labels, axis[3] / 100)
         return bbox
 
+    def drawLegend(self, img):
+        if img is not None:
+            logging.error("Legends not supported in SideView mode!")
+            raise NotImplementedError
+
     def drawImage(self, img):
         """Draw the image img on the current plot.
 
@@ -488,12 +468,12 @@ class MplSideViewCanvas(MplCanvas):
         """
         logging.debug("plotting vertical section image..")
         ix, iy = img.size
-        logging.debug("  image size is %ix%i px, format is %s", ix, iy, img.format)
+        logging.debug(u"  image size is {:d}{:d} px, format is {}".format(ix, iy, img.format))
         # Test if the image axes exist. If not, create them.
         if self.imgax is None:
             # Disable old white figure background so that the new underlying
             # axes become visible.
-            self.ax.patch.set_facecolor('None')
+            self.ax.patch.set_facecolor("None")
             # self.mpl.canvas.ax.patch.set_alpha(0.5)
 
             # Add new axes to the plot (imshow doesn't work with logarithmic axes).
@@ -501,7 +481,7 @@ class MplSideViewCanvas(MplCanvas):
             # Main axes instance of mplwidget has zorder 99.
             self.imgax = self.fig.add_axes(ax_bbox, frameon=True,
                                            xticks=[], yticks=[],
-                                           label='ax2', zorder=0)
+                                           label="ax2", zorder=0)
 
         # If an image is currently displayed, remove it from the plot.
         if self.image is not None:
@@ -511,7 +491,7 @@ class MplSideViewCanvas(MplCanvas):
         self.image = self.imgax.imshow(img, interpolation="nearest", aspect="auto",
                                        origin=PIL_image_origin)
         self.imgax.set_xlim(0, ix - 1)
-        if _matplotlib_version >= '1.2':
+        if _matplotlib_version >= "1.2":
             self.imgax.set_ylim(iy - 1, 0)
         else:
             self.imgax.set_ylim(0, iy - 1)
@@ -537,16 +517,13 @@ class MplSideViewWidget(MplNavBarWidget):
                 action.setEnabled(False)
 
 
-"""
-Top View Canvas and Widget
-"""
-
-
 class MplTopViewCanvas(MplCanvas):
     """Specialised MplCanvas that draws a top view (map), together with a
        flight track, trajectories and other items.
     """
 
+    redrawn = QtCore.pyqtSignal(name="redrawn")
+
     def __init__(self, settings=None):
         """
         """
@@ -566,18 +543,18 @@ class MplTopViewCanvas(MplCanvas):
         self.setMapAppearance(settings)
 
         # Progress dialog to inform the user about map redraws.
-        self.pdlg = QtGui.QProgressDialog("redrawing map...", "Cancel",
-                                          0, 10, self)
+        self.pdlg = QtWidgets.QProgressDialog("redrawing map...", "Cancel", 0, 10, self)
+        self.pdlg.close()
 
     def initMap(self, model=None, **kwargs):
         """Set up the map view.
         """
         ax = self.ax
         self.map = mpl_map.MapCanvas(appearance=self.getMapAppearance(),
-                                     resolution='l', area_thresh=1000., ax=ax,
+                                     resolution="l", area_thresh=1000., ax=ax,
                                      **kwargs)
         ax.set_autoscale_on(False)
-        ax.set_title('top view')
+        ax.set_title("Top view", horizontalalignment="left", x=0)
         self.draw()  # necessary?
 
         if model:
@@ -614,10 +591,14 @@ class MplTopViewCanvas(MplCanvas):
         See MapCanvas.update_with_coordinate_change(). After the map redraw,
         coordinates of all objects overlain on the map have to be updated.
         """
+
+        # remove legend
+        self.drawLegend(None)
+
         # Show the progress dialog, since the retrieval can take a few seconds.
         self.pdlg.setValue(0)
         self.pdlg.show()
-        QtGui.QApplication.processEvents()
+        QtWidgets.QApplication.processEvents()
 
         logging.debug("redrawing map")
 
@@ -625,7 +606,7 @@ class MplTopViewCanvas(MplCanvas):
 
         # (Currently none.)
         self.pdlg.setValue(1)
-        QtGui.QApplication.processEvents()
+        QtWidgets.QApplication.processEvents()
 
         # 2) UPDATE MAP.
         self.map.update_with_coordinate_change(kwargs_update)
@@ -634,11 +615,11 @@ class MplTopViewCanvas(MplCanvas):
         # in waypoints_interactor()
 
         self.pdlg.setValue(5)
-        QtGui.QApplication.processEvents()
+        QtWidgets.QApplication.processEvents()
 
         # 3) UPDATE COORDINATES OF NON-MAP OBJECTS.
         self.pdlg.setValue(8)
-        QtGui.QApplication.processEvents()
+        QtWidgets.QApplication.processEvents()
 
         if self.satoverpasspatch:
             self.satoverpasspatch.update()
@@ -648,17 +629,17 @@ class MplTopViewCanvas(MplCanvas):
 
         self.drawMetadata("Top view")
 
-        # correctly. no idea why..
+        # Update in case of a projection change
         self.waypoints_interactor.update()
 
         self.pdlg.setValue(10)
-        QtGui.QApplication.processEvents()
+        QtWidgets.QApplication.processEvents()
 
         logging.debug("finished redrawing map")
         self.pdlg.close()
 
         # Emit signal so other parts of the module can react to a redraw event.
-        self.emit(QtCore.SIGNAL("redrawn()"))
+        self.redrawn.emit()
 
     def getCRS(self):
         """Get the coordinate reference system of the displayed map.
@@ -707,43 +688,39 @@ class MplTopViewCanvas(MplCanvas):
         """
         # If the method is called with a "None" image, the current legend
         # graphic should be removed (if one exists).
-        if img is None:
-            if self.legimg is not None:
-                self.legimg.remove()
-                self.legimg = None
-            return
+        if self.legimg is not None:
+            logging.debug("removing image %s", self.legimg)
+            self.legimg.remove()
+            self.legimg = None
 
-        # The size of the legend axes needs to be given in relative figure
-        # coordinates. To determine those from the legend graphics size in
-        # pixels, we need to determine the size of the currently displayed
-        # figure in pixels.
-        figsize_px = self.fig.get_size_inches() * self.fig.get_dpi()
-        ax_extent_x = img.size[0] / figsize_px[0]
-        ax_extent_y = img.size[1] / figsize_px[1]
+        if img is not None:
+            # The size of the legend axes needs to be given in relative figure
+            # coordinates. To determine those from the legend graphics size in
+            # pixels, we need to determine the size of the currently displayed
+            # figure in pixels.
+            figsize_px = self.fig.get_size_inches() * self.fig.get_dpi()
+            ax_extent_x = img.size[0] / figsize_px[0]
+            ax_extent_y = img.size[1] / figsize_px[1]
 
-        # If no legend axes have been created, do so now.
-        if self.legax is None:
-            # Add new axes to the plot.
-            ax_bbox = self.ax.get_position()
-            # Main axes instance of mplwidget has zorder 99.
-            self.legax = self.fig.add_axes([1 - ax_extent_x, 0.01, ax_extent_x, ax_extent_y],
-                                           frameon=False,
-                                           xticks=[], yticks=[],
-                                           label='ax2', zorder=0)
-            self.legax.patch.set_facecolor('None')
+            # If no legend axes have been created, do so now.
+            if self.legax is None:
+                # Main axes instance of mplwidget has zorder 99.
+                self.legax = self.fig.add_axes([1 - ax_extent_x, 0.01, ax_extent_x, ax_extent_y],
+                                               frameon=False,
+                                               xticks=[], yticks=[],
+                                               label="ax2", zorder=0)
+                self.legax.patch.set_facecolor("None")
 
-        # If axes exist, remove the current legend image and adjust their
-        # position.
-        else:
-            if self.legimg is not None:
-                self.legimg.remove()
-                self.legimg = None
-            self.legax.set_position([1 - ax_extent_x, 0.01, ax_extent_x, ax_extent_y])
+            # If axes exist, adjust their position.
+            else:
+                self.legax.set_position([1 - ax_extent_x, 0.01, ax_extent_x, ax_extent_y])
 
-        # Plot the new legimg in the legax axes.
-        self.legimg = self.legax.imshow(img, origin=PIL_image_origin, aspect="equal",
-                                        interpolation="nearest")
+            # Plot the new legimg in the legax axes.
+            self.legimg = self.legax.imshow(img, origin=PIL_image_origin, aspect="equal",
+                                            interpolation="nearest")
         self.draw()
+        # required so that it is actually drawn...
+        QtWidgets.QApplication.processEvents()
 
     def plotSatelliteOverpass(self, segment):
         """Plots a satellite track on top of the map.
@@ -756,8 +733,20 @@ class MplTopViewCanvas(MplCanvas):
                 self.draw()
         if segment:
             # Create a new patch.
-            self.satoverpasspatch = mpl_map.SatelliteOverpassPatch(self.map,
-                                                                   segment)
+            self.satoverpasspatch = mpl_map.SatelliteOverpassPatch(self.map, segment)
+
+    def plotKML(self, kmloverlay):
+        """Plots a satellite track on top of the map.
+        """
+        if self.kmloverlay:
+            # If track is currently plotted on the map, remove it.
+            self.kmloverlay.remove()
+            if not kmloverlay:
+                self.kmloverlay = None
+                self.draw()
+        if kmloverlay:
+            # Create a new patch.
+            self.kmloverlay = kmloverlay
 
     def setMapAppearance(self, settings_dict):
         """Apply settings from dictionary 'settings_dict' to the view.
@@ -822,8 +811,7 @@ class MplTopViewWidget(MplNavBarWidget):
             if action.text() in ["Subplots", "Customize"]:
                 action.setEnabled(False)
             elif action.text() in ["Home", "Back", "Forward"]:
-                self.connect(action, QtCore.SIGNAL("triggered()"),
-                             self.historyEvent)
+                action.triggered.connect(self.historyEvent)
         # Identify zoom events to redraw the map, if necessary.
         self.canvas.mpl_connect('button_release_event', self.zoomEvent)
 
@@ -841,11 +829,6 @@ class MplTopViewWidget(MplNavBarWidget):
         self.canvas.redrawMap()
 
 
-"""
-Time Series View Canvas and Widget
-"""
-
-
 class MplTimeSeriesViewCanvas(MplCanvas):
     """Specialised MplCanvas that draws time series of trajectory data.
     """
@@ -872,16 +855,10 @@ class MplTimeSeriesViewCanvas(MplCanvas):
         logging.debug("registering trajectory tree model")
         # Disconnect old tree, if defined.
         if self.traj_item_tree:
-            self.traj_item_tree.disconnect(
-                self.traj_item_tree,
-                QtCore.SIGNAL("dataChanged(QModelIndex, QModelIndex)"),
-                self.updateFromTrajectoryTree)
+            self.traj_item_tree.dataChanged.disconnect(self.updateFromTrajectoryTree)
         # Set and connect new tree.
         self.traj_item_tree = tree
-        self.traj_item_tree.connect(
-            self.traj_item_tree,
-            QtCore.SIGNAL("dataChanged(QModelIndex, QModelIndex)"),
-            self.updateFromTrajectoryTree)
+        self.traj_item_tree.dataChanged.connect(self.updateFromTrajectoryTree)
         # Draw tree items.
         self.updateTrajectoryItems()
 
@@ -920,7 +897,7 @@ class MplTimeSeriesViewCanvas(MplCanvas):
         if not self.traj_item_tree:
             return
 
-        logging.debug("updating trajectory items on view <%s>, mode %s",
+        logging.debug("updating trajectory items on view '%s', mode '%s'",
                       self.identifier, mode)
 
         # Determine which items have to be plotted (these are the items that
@@ -946,13 +923,12 @@ class MplTimeSeriesViewCanvas(MplCanvas):
             # below the items that are on the stack.
             item = stack.pop()
             if item.isVisible(self.identifier):
-                if isinstance(item, titree.FlightTrackItem) \
-                        or isinstance(item, titree.TrajectoryItem):
+                if isinstance(item, (titree.FlightTrackItem, titree.TrajectoryItem)):
                     itemsList.append(item)
                 else:
                     stack.extend(item.childItems)
 
-        logging.debug("Plotting elements %s", [i.getName() for i in itemsList])
+        logging.debug("Plotting elements '%s'", [i.getName() for i in itemsList])
 
         if mode in ["REDRAW", "MARKER_CHANGE"]:
             # Clear the figure -- it needs to be completely redrawn.
@@ -964,8 +940,7 @@ class MplTimeSeriesViewCanvas(MplCanvas):
             earliestStartTime = datetime(3000, 1, 1)
             self.subPlots = []
             for item in itemsList:
-                earliestStartTime = min(earliestStartTime,
-                                        item.getStartTime())
+                earliestStartTime = min(earliestStartTime, item.getStartTime())
                 for variable in item.childItems:
                     if variable.isVisible(self.identifier) and variable.getVariableName() not in self.subPlots:
                         self.subPlots.append(variable.getVariableName())
@@ -973,7 +948,7 @@ class MplTimeSeriesViewCanvas(MplCanvas):
         # Iterative list traversal no. 2: Draw / update plots.
         for item in itemsList:
 
-            logging.debug("plotting item %s", item.getName())
+            logging.debug(u"plotting item {}".format(item.getName()))
             # The variables that have to be plotted are children of self.mapItem.
             # Plot all variables whose visible-flag is set to True.
             variablesToPlot = [variable for variable in item.childItems
@@ -985,8 +960,8 @@ class MplTimeSeriesViewCanvas(MplCanvas):
                 # the new properties to all variables of trajectory 'item'.
                 for variable in variablesToPlot:
                     plt.setp(
-                        variable.getGxElementProperty("timeseries",
-                                                      "instance::%s" % self.identifier),
+                        variable.getGxElementProperty(
+                            "timeseries", u"instance::{}".format(self.identifier)),
                         visible=item.isVisible(self.identifier),
                         color=item.getGxElementProperty("general", "colour"),
                         linestyle=item.getGxElementProperty("general", "linestyle"),
@@ -1019,7 +994,7 @@ class MplTimeSeriesViewCanvas(MplCanvas):
                                            linewidth=item.getGxElementProperty("general",
                                                                                "linewidth"))
                     variable.setGxElementProperty("timeseries",
-                                                  "instance::%s" % self.identifier,
+                                                  u"instance::{}".format(self.identifier),
                                                   plotInstance)
                     #
                     # If we've just plotted the pressure variable flip the y-axis
@@ -1046,12 +1021,10 @@ class MplTimeSeriesViewCanvas(MplCanvas):
                     # The topmost subplot gets the title (the name of self.mapItem).
                     if index == 0:
                         if len(itemsList) > 1:
-                            ax.set_title("Ensemble: %s to %s" %
-                                         (itemsList[0].getName(),
-                                          itemsList[-1].getName()))
+                            ax.set_title(u"Ensemble: {} to {}".format(
+                                itemsList[0].getName(), itemsList[-1].getName()))
                         else:
-                            ax.set_title("Single item: %s" %
-                                         itemsList[0].getName())
+                            ax.set_title(u"Single item: {}".format(itemsList[0].getName()))
 
                     #
                     # The bottommost subplot gets the x-label: The name of the time
