@@ -1,18 +1,37 @@
+# -*- coding: utf-8 -*-
 """
-AUTHORS:
-========
 
-* Joern Ungermann (ju)
+    mslib.msui.performance_settings
+    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 
+    This module defines the performance settings dialog
+
+    This file is part of mss.
+
+    :copyright: Copyright 2017 Joern Ungermann
+    :copyright: Copyright 2017 by the mss team, see AUTHORS.
+    :license: APACHE-2.0, see LICENSE for details.
+
+    Licensed under the Apache License, Version 2.0 (the "License");
+    you may not use this file except in compliance with the License.
+    You may obtain a copy of the License at
+
+       http://www.apache.org/licenses/LICENSE-2.0
+
+    Unless required by applicable law or agreed to in writing, software
+    distributed under the License is distributed on an "AS IS" BASIS,
+    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+    See the License for the specific language governing permissions and
+    limitations under the License.
 """
-import logging
 
-from PyQt4 import QtGui, QtCore  # Qt4 bindings
 
-from mslib.mss_util import config_loader
+from mslib.msui.mss_qt import QtCore, QtWidgets, USE_PYQT5
+
+from mslib.utils import config_loader, FatalUserError
 from mslib.msui import aircrafts
 from mslib.msui import constants
-from mslib.msui import ui_performance_settings as ui_ps
+from mslib.msui.mss_qt import ui_performance_settings as ui_ps
 
 
 DEFAULT_PERFORMANCE = {
@@ -24,7 +43,7 @@ DEFAULT_PERFORMANCE = {
 }
 
 
-class MSS_PerformanceSettingsDialog(QtGui.QDialog, ui_ps.Ui_PerformanceSettingsDialog):
+class MSS_PerformanceSettingsDialog(QtWidgets.QDialog, ui_ps.Ui_PerformanceSettingsDialog):
     """Dialog to set map appearance parameters. User interface is
        defined in "ui_topview_mapappearance.py".
     """
@@ -47,8 +66,7 @@ class MSS_PerformanceSettingsDialog(QtGui.QDialog, ui_ps.Ui_PerformanceSettingsD
         self.dsbFuel.setValue(settings_dict["fuel"])
         self.dteTakeoffTime.setDateTime(settings_dict["takeoff_time"])
 
-        self.connect(self.pbLoadPerformance, QtCore.SIGNAL("clicked()"),
-                     self.load_performance)
+        self.pbLoadPerformance.clicked.connect(self.load_performance)
 
     def get_settings(self):
         """
@@ -70,24 +88,20 @@ class MSS_PerformanceSettingsDialog(QtGui.QDialog, ui_ps.Ui_PerformanceSettingsD
         """
         Gets a filename for a JSON file specifying aircraft performance and initializes an SimpleAircraft model.
         """
-        fname = QtGui.QFileDialog.getOpenFileName(self,
-                                                  "Open Aircraft Performance JSON File",
-                                                  constants.MSS_CONFIG_PATH, "(*.json)")
-        if fname.isEmpty():
-            return
+        filename = QtWidgets.QFileDialog.getOpenFileName(
+            self, "Open Aircraft Performance JSON File", constants.MSS_CONFIG_PATH, "(*.json)")
+        filename = filename[0] if USE_PYQT5 else unicode(filename)
 
         try:
-            performance = config_loader(config_file=str(fname))
+            performance = config_loader(config_file=filename)
             self.aircraft = aircrafts.SimpleAircraft(performance)
             self.lbAircraftName.setText(self.aircraft.name)
             self.dsbTakeoffWeight.setValue(self.aircraft.takeoff_weight)
             self.dsbFuel.setValue(self.aircraft.fuel)
 
         except KeyError, ex:
-            QtGui.QMessageBox.critical(self, self.tr("Performance JSON Load"),
-                                       self.tr("JSON File missing '{}' entry".format(ex)),
-                                       QtGui.QMessageBox.Ok)
-        except ValueError, ex:
-            QtGui.QMessageBox.critical(self, self.tr("Performance JSON Load"),
-                                       self.tr("JSON File has Syntax Problems:\n{}".format(ex)),
-                                       QtGui.QMessageBox.Ok)
+            QtWidgets.QMessageBox.critical(self, self.tr("Performance JSON Load"),
+                                           self.tr(u"JSON File missing '{}' entry".format(ex)))
+        except (FatalUserError, ValueError), ex:
+            QtWidgets.QMessageBox.critical(self, self.tr("Performance JSON Load"),
+                                           self.tr(u"JSON File has Syntax Problems:\n{}".format(ex)))
