@@ -1,29 +1,138 @@
-"""Control widget to configure remote sensing overlays.
+# -*- coding: utf-8 -*-
+"""
 
+    mslib.msui.remotesensing_dockwidget
+    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 
-This file is part of the Mission Support System User Interface (MSUI).
+    Control widget to configure remote sensing overlays.
 
-AUTHORS:
-========
+    This file is part of mss.
 
-* Joern Ungermann
+    :copyright: Copyright 2017 Joern Ungermann
+    :copyright: Copyright 2017 by the mss team, see AUTHORS.
+    :license: APACHE-2.0, see LICENSE for details.
 
+    Licensed under the Apache License, Version 2.0 (the "License");
+    you may not use this file except in compliance with the License.
+    You may obtain a copy of the License at
+
+       http://www.apache.org/licenses/LICENSE-2.0
+
+    Unless required by applicable law or agreed to in writing, software
+    distributed under the License is distributed on an "AS IS" BASIS,
+    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+    See the License for the specific language governing permissions and
+    limitations under the License.
 """
 
 import numpy as np
-from PyQt4 import QtGui, QtCore  # Qt4 bindings
-from mslib.msui import ui_remotesensing_dockwidget as ui
-from mslib.mss_util import datetime_to_jsec, compute_solar_angle, \
-    compute_view_angles, get_distance, rotate_point
+from mslib.msui.mss_qt import QtGui, QtWidgets
+from mslib.msui.mss_qt import ui_remotesensing_dockwidget as ui
+from mslib.utils import datetime_to_jsec, get_distance, rotate_point, fix_angle, compute_hour_of_day
 from matplotlib.collections import LineCollection
 from matplotlib.colors import BoundaryNorm, ListedColormap
 import collections
 
 
-R = 6371.
+EARTH_RADIUS = 6371.
 
 
-class RemoteSensingControlWidget(QtGui.QWidget, ui.Ui_RemoteSensingDockWidget):
+def compute_view_angles(lon0, lat0, h0, lon1, lat1, h1, angle):
+    mlat = (lat0 + lat1) / 2.
+    lon0 *= np.cos(np.deg2rad(mlat))
+    lon1 *= np.cos(np.deg2rad(mlat))
+    dlon = lon1 - lon0
+    dlat = lat1 - lat0
+    obs_azi2 = fix_angle(angle + np.rad2deg(np.arctan2(dlon, dlat)))
+    return obs_azi2, -1
+
+
+def compute_solar_angle(jsec, lon, lat):
+    # The input to the Astronomer's almanach is the difference between
+    # the Julian date and JD 2451545.0 (noon, 1 January 2000)
+    time = jsec / (60. * 60. * 24.) - 0.5
+
+    # Mean longitude
+    mnlong = 280.460 + .9856474 * time
+    mnlong %= 360.
+    if mnlong < 0:
+        mnlong += 360
+        assert mnlong >= 0
+
+    # Mean anomaly
+    mnanom = 357.528 + .9856003 * time
+    mnanom = np.deg2rad(mnanom % 360.)
+    if mnanom < 0:
+        mnanom += 2 * np.pi
+        assert mnanom >= 0
+
+    # Ecliptic longitude and obliquity of ecliptic
+    eclong = mnlong + 1.915 * np.sin(mnanom) + 0.020 * np.sin(2 * mnanom)
+    eclong = np.deg2rad(eclong % 360.)
+    if (eclong < 0):
+        eclong += 2 * np.pi
+        assert (eclong >= 0)
+
+    oblqec = np.deg2rad(23.439 - 0.0000004 * time)
+
+    # Celestial coordinates
+    # Right ascension and declination
+    num = np.cos(oblqec) * np.sin(eclong)
+    den = np.cos(eclong)
+    ra = np.arctan(num / den)
+    if den < 0:
+        ra += np.pi
+    elif den >= 0 and num < 0:
+        ra += 2 * np.pi
+
+    dec = np.arcsin(np.sin(oblqec) * np.sin(eclong))
+    # Local coordinates
+    # Greenwich mean sidereal time
+    gmst = 6.697375 + .0657098242 * time + compute_hour_of_day(jsec)
+
+    gmst = gmst % 24.
+    if gmst < 0:
+        gmst += 24
+        assert gmst >= 0
+
+    # Local mean sidereal time
+    if lon < 0:
+        lon += 360
+        assert 0 <= lon <= 360
+
+    lmst = gmst + lon / 15.
+    lmst = np.deg2rad(15. * (lmst % 24.))
+
+    # Hour angle
+    ha = lmst - ra
+    if ha < -np.pi:
+        ha += 2 * np.pi
+
+    if ha > np.pi:
+        ha -= 2 * np.pi
+
+    assert -np.pi < ha < 2 * np.pi
+
+    # Latitude to radians
+    lat = np.deg2rad(lat)
+
+    # Azimuth and elevation
+    zenithAngle = np.arccos(np.sin(lat) * np.sin(dec) + np.cos(lat) * np.cos(dec) * np.cos(ha))
+    azimuthAngle = np.arccos((np.sin(lat) * np.cos(zenithAngle) - np.sin(dec)) /
+                             (np.cos(lat) * np.sin(zenithAngle)))
+
+    if ha > 0:
+        azimuthAngle += np.pi
+    else:
+        azimuthAngle = 3 * np.pi - azimuthAngle % (2 * np.pi)
+
+    if azimuthAngle > np.pi:
+        azimuthAngle -= 2 * np.pi
+
+    return np.rad2deg(azimuthAngle), 90 - np.rad2deg(zenithAngle)
+
+
+class RemoteSensingControlWidget(QtWidgets.QWidget, ui.Ui_RemoteSensingDockWidget):
     """This class implements the remote sensing functionality as dockable widget.
     """
 
@@ -51,8 +160,7 @@ class RemoteSensingControlWidget(QtGui.QWidget, ui.Ui_RemoteSensingDockWidget):
         # update plot on every value change
         self.cbDrawTangents.stateChanged.connect(self.update_settings)
         self.cbShowSolarAngle.stateChanged.connect(self.update_settings)
-        self.connect(self.btTangentsColour, QtCore.SIGNAL("clicked()"),
-                     self.set_tangentpoint_colour)
+        self.btTangentsColour.clicked.connect(self.set_tangentpoint_colour)
         self.dsbTangentHeight.valueChanged.connect(self.update_settings)
         self.dsbObsAngle.valueChanged.connect(self.update_settings)
 
@@ -87,10 +195,9 @@ class RemoteSensingControlWidget(QtGui.QWidget, ui.Ui_RemoteSensingDockWidget):
            new button face colour.
         """
         button = self.btTangentsColour
-
         palette = QtGui.QPalette(button.palette())
         colour = palette.color(QtGui.QPalette.Button)
-        colour = QtGui.QColorDialog.getColor(colour)
+        colour = QtWidgets.QColorDialog.getColor(colour)
         if colour.isValid():
             palette.setColor(QtGui.QPalette.Button, colour)
             button.setPalette(palette)
@@ -119,8 +226,8 @@ class RemoteSensingControlWidget(QtGui.QWidget, ui.Ui_RemoteSensingDockWidget):
             fine_lines[i][0], fine_lines[i][1], line_heights[i],
             cut_height=self.dsbTangentHeight.value()) for i in range(len(fine_lines))]
         for i, line in enumerate(tplines):
-            for j in range(len(line)):
-                line[j] = bmap(line[j][0], line[j][1])
+            for j, (lon, lat) in enumerate(line):
+                line[j] = bmap(lon, lat)
             tplines[i] = line
         return LineCollection(
             tplines,
@@ -143,7 +250,6 @@ class RemoteSensingControlWidget(QtGui.QWidget, ui.Ui_RemoteSensingDockWidget):
                  angle
         """
         # calculate distances and times
-        speed = 850. / 3.6  # speed in km/s
         times = [datetime_to_jsec(_wp_time) for _wp_time in wp_times]
         x, y = zip(*wp_vertices)
         wp_lons, wp_lats = bmap(x, y, inverse=True)
@@ -225,10 +331,11 @@ class RemoteSensingControlWidget(QtGui.QWidget, ui.Ui_RemoteSensingDockWidget):
         los.append(los[-1])
 
         if isinstance(flight_alt, (collections.Sequence, np.ndarray)):
-            dist = [np.sqrt(max((R + a) ** 2 - (R + cut_height) ** 2, 0)) / 110. for a in flight_alt[:-1]]
+            dist = [np.sqrt(max((EARTH_RADIUS + a) ** 2 - (EARTH_RADIUS + cut_height) ** 2, 0)) / 110.
+                    for a in flight_alt[:-1]]
             dist.append(dist[-1])
         else:
-            dist = np.sqrt((R + flight_alt) ** 2 - (R + cut_height) ** 2) / 110.
+            dist = np.sqrt((EARTH_RADIUS + flight_alt) ** 2 - (EARTH_RADIUS + cut_height) ** 2) / 110.
 
         tp_dir = (np.array(los).T * dist).T
 
@@ -238,7 +345,8 @@ class RemoteSensingControlWidget(QtGui.QWidget, ui.Ui_RemoteSensingDockWidget):
                (x0, y0) in tps]
         return tps
 
-    def calc_view_rating(self, obs_azi, obs_ele, sol_azi, sol_ele, height):
+    @staticmethod
+    def calc_view_rating(obs_azi, obs_ele, sol_azi, sol_ele, height):
         """
         Calculates the angular distance between given directions under the
         condition that the sun is above the horizon.
@@ -252,7 +360,7 @@ class RemoteSensingControlWidget(QtGui.QWidget, ui.Ui_RemoteSensingDockWidget):
 
         Returns: angular distance or 180 degrees if sun is below horizon
         """
-        thresh = -np.rad2deg(np.arccos(R / (height + R))) - 3
+        thresh = -np.rad2deg(np.arccos(EARTH_RADIUS / (height + EARTH_RADIUS))) - 3
 
         delta_azi = obs_azi - sol_azi
         delta_ele = obs_ele + sol_ele
