@@ -1,65 +1,52 @@
-"""Side view module of the MSUI.
+# -*- coding: utf-8 -*-
+"""
 
-********************************************************************************
+    mslib.msui.sideview
+    ~~~~~~~~~~~~~~~~~~~
 
-   Copyright 2008-2014 Deutsches Zentrum fuer Luft- und Raumfahrt e.V.
+    Side view module of the msui
 
-   Licensed under the Apache License, Version 2.0 (the "License");
-   you may not use this file except in compliance with the License.
-   You may obtain a copy of the License at
+    This file is part of mss.
+
+    :copyright: Copyright 2008-2014 Deutsches Zentrum fuer Luft- und Raumfahrt e.V.
+    :copyright: Copyright 2011-2014 Marc Rautenhaus (mr)
+    :copyright: Copyright 2016-2017 by the mss team, see AUTHORS.
+    :license: APACHE-2.0, see LICENSE for details.
+
+    Licensed under the Apache License, Version 2.0 (the "License");
+    you may not use this file except in compliance with the License.
+    You may obtain a copy of the License at
 
        http://www.apache.org/licenses/LICENSE-2.0
 
-   Unless required by applicable law or agreed to in writing, software
-   distributed under the License is distributed on an "AS IS" BASIS,
-   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-   See the License for the specific language governing permissions and
-   limitations under the License.
-
-********************************************************************************
-
-This file is part of the Mission Support System User Interface (MSUI).
-
-See the reference documentation, Supplement, for details on the
-implementation.
-
-AUTHORS:
-========
-
-* Marc Rautenhaus (mr)
-
+    Unless required by applicable law or agreed to in writing, software
+    distributed under the License is distributed on an "AS IS" BASIS,
+    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+    See the License for the specific language governing permissions and
+    limitations under the License.
 """
 
-# standard library imports
 import functools
-import logging
-import os
-import pickle
-from mslib.mss_util import config_loader
+from mslib.utils import config_loader, save_settings_pickle, load_settings_pickle
 from mslib.msui import MissionSupportSystemDefaultConfig as mss_default
 
 # related third party imports
-from PyQt4 import QtGui, QtCore  # Qt4 bindings
+from mslib.msui.mss_qt import QtGui, QtWidgets
 
 # local application imports
-from mslib.msui import ui_sideview_window as ui
-from mslib.msui import ui_sideview_options as ui_opt
-from mslib.msui import mss_qt
-from mslib.msui import flighttrack as ft
+from mslib.msui.mss_qt import ui_sideview_window as ui
+from mslib.msui.mss_qt import ui_sideview_options as ui_opt
+from mslib.msui.viewwindows import MSSMplViewWindow
 from mslib.msui import mpl_pathinteractor as mpl_pi
 from mslib.msui import wms_control as wms
-from mslib.msui import constants
+from mslib.msui.icons import icons
 
 
 # Dock window indices.
 WMS = 0
 
-#
-# DIALOG for setting sideview options
-#
 
-
-class MSS_SV_OptionsDialog(QtGui.QDialog, ui_opt.Ui_SideViewOptionsDialog):
+class MSS_SV_OptionsDialog(QtWidgets.QDialog, ui_opt.Ui_SideViewOptionsDialog):
     """Dialog to specify sideview options. User interface is specified
        in "ui_sideview_options.py".
     """
@@ -82,7 +69,8 @@ class MSS_SV_OptionsDialog(QtGui.QDialog, ui_opt.Ui_SideViewOptionsDialog):
                                  "colour_ft_vertices": (0, 0, 0, 0),
                                  "colour_ft_waypoints": (0, 0, 0, 0),
                                  "colour_ft_fill": (0, 0, 0, 0)}
-        default_settings_dict.update(settings_dict)
+        if settings_dict is not None:
+            default_settings_dict.update(settings_dict)
         settings_dict = default_settings_dict
 
         self.sbPbot.setValue(settings_dict["vertical_extent"][0])
@@ -92,7 +80,7 @@ class MSS_SV_OptionsDialog(QtGui.QDialog, ui_opt.Ui_SideViewOptionsDialog):
         self.tableWidget.setRowCount(len(flightlevels))
         flightlevels.sort()
         for i, level in enumerate(flightlevels):
-            tableitem = QtGui.QTableWidgetItem("%3i" % level)
+            tableitem = QtWidgets.QTableWidgetItem(str(int(level)))
             self.tableWidget.setItem(i, 0, tableitem)
 
         self.cbDrawFlightLevels.setChecked(settings_dict["draw_flightlevels"])
@@ -110,20 +98,14 @@ class MSS_SV_OptionsDialog(QtGui.QDialog, ui_opt.Ui_SideViewOptionsDialog):
             button.setPalette(palette)
 
         # Connect colour button signals.
-        self.connect(self.btFillColour, QtCore.SIGNAL("clicked()"),
-                     functools.partial(self.setColour, "ft_fill"))
-        self.connect(self.btWaypointsColour, QtCore.SIGNAL("clicked()"),
-                     functools.partial(self.setColour, "ft_waypoints"))
-        self.connect(self.btVerticesColour, QtCore.SIGNAL("clicked()"),
-                     functools.partial(self.setColour, "ft_vertices"))
+        self.btFillColour.clicked.connect(functools.partial(self.setColour, "ft_fill"))
+        self.btWaypointsColour.clicked.connect(functools.partial(self.setColour, "ft_waypoints"))
+        self.btVerticesColour.clicked.connect(functools.partial(self.setColour, "ft_vertices"))
 
-        self.connect(self.btAdd, QtCore.SIGNAL("clicked()"),
-                     self.addItem)
-        self.connect(self.btDelete, QtCore.SIGNAL("clicked()"),
-                     self.deleteSelected)
+        self.btAdd.clicked.connect(self.addItem)
+        self.btDelete.clicked.connect(self.deleteSelected)
 
-        self.connect(self.tableWidget, QtCore.SIGNAL("itemChanged(QTableWidgetItem *)"),
-                     self.itemChanged)
+        self.tableWidget.itemChanged.connect(self.itemChanged)
 
     def setColour(self, which):
         """Slot for the colour buttons: Opens a QColorDialog and sets the
@@ -138,7 +120,7 @@ class MSS_SV_OptionsDialog(QtGui.QDialog, ui_opt.Ui_SideViewOptionsDialog):
 
         palette = QtGui.QPalette(button.palette())
         colour = palette.color(QtGui.QPalette.Button)
-        colour = QtGui.QColorDialog.getColor(colour)
+        colour = QtWidgets.QColorDialog.getColor(colour)
         if colour.isValid():
             if which == "ft_fill":
                 # Fill colour is transparent with an alpha value of 0.15. If
@@ -150,7 +132,9 @@ class MSS_SV_OptionsDialog(QtGui.QDialog, ui_opt.Ui_SideViewOptionsDialog):
     def addItem(self):
         """Add a new item (i.e. flight level) to the table.
         """
-        self.tableWidget.insertRow(self.tableWidget.rowCount())
+        self.tableWidget.insertRow(0)
+        self.tableWidget.setItem(0, 0, QtWidgets.QTableWidgetItem("0"))
+        self.tableWidget.sortItems(0)
 
     def deleteSelected(self):
         """Remove the selected items (i.e. flight levels) from the table.
@@ -165,20 +149,20 @@ class MSS_SV_OptionsDialog(QtGui.QDialog, ui_opt.Ui_SideViewOptionsDialog):
            values are corrected.
         """
         try:
-            flightlevel = int(float(str(item.text())))
+            flightlevel = int(float(unicode(item.text())))
         except:
             flightlevel = 0
         if flightlevel < 0:
             flightlevel = 0
         if flightlevel > 999:
             flightlevel = 999
-        item.setText("%3i" % flightlevel)
+        item.setText(str(int(flightlevel)))
         self.tableWidget.sortItems(0)
 
     def getFlightLevels(self):
         """Returns the flight level values contained in the table.
         """
-        return [int(str(self.tableWidget.item(row, 0).text()))
+        return [int(unicode(self.tableWidget.item(row, 0).text()))
                 for row in range(self.tableWidget.rowCount())]
 
     def getSettings(self):
@@ -201,12 +185,7 @@ class MSS_SV_OptionsDialog(QtGui.QDialog, ui_opt.Ui_SideViewOptionsDialog):
         return settings_dict
 
 
-#
-# CLASS MSSSideViewWindow
-#
-
-
-class MSSSideViewWindow(mss_qt.MSSMplViewWindow, ui.Ui_SideViewWindow):
+class MSSSideViewWindow(MSSMplViewWindow, ui.Ui_SideViewWindow):
     """PyQt4 window implementing a matplotlib canvas as an interactive
        side view flight track editor.
     """
@@ -217,6 +196,7 @@ class MSSSideViewWindow(mss_qt.MSSMplViewWindow, ui.Ui_SideViewWindow):
         """
         super(MSSSideViewWindow, self).__init__(parent)
         self.setupUi(self)
+        self.setWindowIcon(QtGui.QIcon(icons('64x64')))
 
         # Dock windows [WMS]:
         self.cbTools.clear()
@@ -225,31 +205,26 @@ class MSSSideViewWindow(mss_qt.MSSMplViewWindow, ui.Ui_SideViewWindow):
 
         self.setFlightTrackModel(model)
 
-        self.settingsfile = os.path.join(constants.MSS_CONFIG_PATH, "mss.sideview.cfg")
+        self.settings_tag = "sideview"
         self.loadSettings()
 
         # Connect slots and signals.
         # ==========================
 
         # Buttons to set sideview options.
-        self.connect(self.btOptions, QtCore.SIGNAL("clicked()"),
-                     self.setOptions)
+        self.btOptions.clicked.connect(self.setOptions)
 
         # Tool opener.
-        self.connect(self.cbTools, QtCore.SIGNAL("currentIndexChanged(int)"),
-                     self.openTool)
+        self.cbTools.currentIndexChanged.connect(self.openTool)
 
         # Controls to interact with the flight track.
         # (For usage of the functools.partial() function, see Chapter 4 (Section
         # Signals and Slots) of 'Rapid GUI Programming with Python and Qt: The
         # Definitive Guide to PyQt Programming' (Mark Summerfield).)
         wpi = self.mpl.canvas.waypoints_interactor
-        self.connect(self.btMvWaypoint, QtCore.SIGNAL("clicked()"),
-                     functools.partial(wpi.set_edit_mode, mpl_pi.MOVE))
-        self.connect(self.btInsWaypoint, QtCore.SIGNAL("clicked()"),
-                     functools.partial(wpi.set_edit_mode, mpl_pi.INSERT))
-        self.connect(self.btDelWaypoint, QtCore.SIGNAL("clicked()"),
-                     functools.partial(wpi.set_edit_mode, mpl_pi.DELETE))
+        self.btMvWaypoint.clicked.connect(functools.partial(wpi.set_edit_mode, mpl_pi.MOVE))
+        self.btInsWaypoint.clicked.connect(functools.partial(wpi.set_edit_mode, mpl_pi.INSERT))
+        self.btDelWaypoint.clicked.connect(functools.partial(wpi.set_edit_mode, mpl_pi.DELETE))
 
     def __del__(self):
         del self.mpl.canvas.waypoints_interactor
@@ -286,7 +261,7 @@ class MSSSideViewWindow(mss_qt.MSSMplViewWindow, ui.Ui_SideViewWindow):
         settings = self.getView().getSettings()
         dlg = MSS_SV_OptionsDialog(parent=self, settings_dict=settings)
         dlg.setModal(True)
-        if dlg.exec_() == QtGui.QDialog.Accepted:
+        if dlg.exec_() == QtWidgets.QDialog.Accepted:
             settings = dlg.getSettings()
             self.getView().setSettings(settings)
             self.saveSettings()
@@ -299,46 +274,10 @@ class MSSSideViewWindow(mss_qt.MSSMplViewWindow, ui.Ui_SideViewWindow):
         # TODO: ConfigParser and a central configuration file might be the better solution than pickle.
         # http://stackoverflow.com/questions/200599/whats-the-best-way-to-store-simple-user-settings-in-python
         settings = self.getView().getSettings()
-        logging.debug("storing settings to %s", self.settingsfile)
-        fileobj = open(self.settingsfile, "w")
-        pickle.dump(settings, fileobj)
-        fileobj.close()
+        save_settings_pickle(self.settings_tag, settings)
 
     def loadSettings(self):
         """Load settings from the file self.settingsfile.
         """
-        if os.path.exists(self.settingsfile):
-            logging.debug("loading settings from %s", self.settingsfile)
-            fileobj = open(self.settingsfile, "r")
-            settings = pickle.load(fileobj)
-            fileobj.close()
-            self.getView().setSettings(settings)
-
-
-def _main():
-    # Log everything, and send it to stderr.
-    # See http://docs.python.org/library/logging.html for more information
-    # on the Python logging module.
-    # NOTE: http://docs.python.org/library/logging.html#formatter-objects
-    logging.basicConfig(level=logging.DEBUG,
-                        format="%(asctime)s (%(module)s.%(funcName)s): %(message)s",
-                        datefmt="%Y-%m-%d %H:%M:%S")
-
-    import sys
-
-    # Create an initital flight track.
-    initial_waypoints = [ft.Waypoint(48.10, 11.27, 0, comments="take off"),
-                         ft.Waypoint(52.32, 09.21, 200),
-                         ft.Waypoint(48.10, 11.27, 0, comments="landing")]
-
-    waypoints_model = ft.WaypointsTableModel(QtCore.QString(""))
-    waypoints_model.insertRows(0, rows=len(initial_waypoints),
-                               waypoints=initial_waypoints)
-
-    application = QtGui.QApplication(sys.argv)
-    window = MSSSideViewWindow(model=waypoints_model)
-    window.show()
-    sys.exit(application.exec_())
-
-if __name__ == "__main__":
-    _main()
+        settings = load_settings_pickle(self.settings_tag)
+        self.getView().setSettings(settings)
