@@ -1,56 +1,50 @@
-"""Top view implementation for the MSUI.
+# -*- coding: utf-8 -*-
+"""
 
-********************************************************************************
+    mslib.msui.topview
+    ~~~~~~~~~~~~~~~~~~
 
-   Copyright 2008-2014 Deutsches Zentrum fuer Luft- und Raumfahrt e.V.
+    Top view implementation for the msui.
+    See the reference documentation, Supplement, for details on the
+    implementation.
 
-   Licensed under the Apache License, Version 2.0 (the "License");
-   you may not use this file except in compliance with the License.
-   You may obtain a copy of the License at
+    This file is part of mss.
+
+    :copyright: Copyright 2008-2014 Deutsches Zentrum fuer Luft- und Raumfahrt e.V.
+    :copyright: Copyright 2011-2014 Marc Rautenhaus (mr)
+    :copyright: Copyright 2016-2017 by the mss team, see AUTHORS.
+    :license: APACHE-2.0, see LICENSE for details.
+
+    Licensed under the Apache License, Version 2.0 (the "License");
+    you may not use this file except in compliance with the License.
+    You may obtain a copy of the License at
 
        http://www.apache.org/licenses/LICENSE-2.0
 
-   Unless required by applicable law or agreed to in writing, software
-   distributed under the License is distributed on an "AS IS" BASIS,
-   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-   See the License for the specific language governing permissions and
-   limitations under the License.
-
-********************************************************************************
-
-This file is part of the Mission Support System User Interface (MSUI).
-
-See the reference documentation, Supplement, for details on the
-implementation.
-
-AUTHORS:
-========
-
-* Marc Rautenhaus (mr)
-
+    Unless required by applicable law or agreed to in writing, software
+    distributed under the License is distributed on an "AS IS" BASIS,
+    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+    See the License for the specific language governing permissions and
+    limitations under the License.
 """
 
-# standard library imports
 import functools
 import logging
-import os
-import pickle
-from mslib.mss_util import config_loader
+from mslib.utils import config_loader, get_projection_params, save_settings_pickle, load_settings_pickle
 from mslib.msui import MissionSupportSystemDefaultConfig as mss_default
 # related third party imports
-from PyQt4 import QtGui, QtCore  # Qt4 bindings
+from mslib.msui.mss_qt import QtGui, QtWidgets
 
 # local application imports
-from mslib.msui import ui_topview_window as ui
-from mslib.msui import ui_topview_mapappearance as ui_ma
-from mslib.msui import mss_qt
+from mslib.msui.mss_qt import ui_topview_window as ui
+from mslib.msui.mss_qt import ui_topview_mapappearance as ui_ma
+from mslib.msui.viewwindows import MSSMplViewWindow
 from mslib.msui import mpl_pathinteractor as mpl_pi
-from mslib.msui import flighttrack as ft
 from mslib.msui import wms_control as wms
 from mslib.msui import satellite_dockwidget as sat
 from mslib.msui import remotesensing_dockwidget as rs
 from mslib.msui import kmloverlay_dockwidget as kml
-from mslib.msui import constants
+from mslib.msui.icons import icons
 
 # Dock window indices.
 WMS = 0
@@ -58,12 +52,8 @@ SATELLITE = 1
 REMOTESENSING = 2
 KMLOVERLAY = 3
 
-#
-# DIALOG for map appearance
-#
 
-
-class MSS_TV_MapAppearanceDialog(QtGui.QDialog, ui_ma.Ui_MapAppearanceDialog):
+class MSS_TV_MapAppearanceDialog(QtWidgets.QDialog, ui_ma.Ui_MapAppearanceDialog):
     """Dialog to set map appearance parameters. User interface is
        defined in "ui_topview_mapappearance.py".
     """
@@ -106,14 +96,10 @@ class MSS_TV_MapAppearanceDialog(QtGui.QDialog, ui_ma.Ui_MapAppearanceDialog):
             button.setPalette(palette)
 
         # Connect colour button signals.
-        self.connect(self.btWaterColour, QtCore.SIGNAL("clicked()"),
-                     functools.partial(self.setColour, "water"))
-        self.connect(self.btLandColour, QtCore.SIGNAL("clicked()"),
-                     functools.partial(self.setColour, "land"))
-        self.connect(self.btWaypointsColour, QtCore.SIGNAL("clicked()"),
-                     functools.partial(self.setColour, "ft_waypoints"))
-        self.connect(self.btVerticesColour, QtCore.SIGNAL("clicked()"),
-                     functools.partial(self.setColour, "ft_vertices"))
+        self.btWaterColour.clicked.connect(functools.partial(self.setColour, "water"))
+        self.btLandColour.clicked.connect(functools.partial(self.setColour, "land"))
+        self.btWaypointsColour.clicked.connect(functools.partial(self.setColour, "ft_waypoints"))
+        self.btVerticesColour.clicked.connect(functools.partial(self.setColour, "ft_vertices"))
 
     def getSettings(self):
         """
@@ -158,12 +144,7 @@ class MSS_TV_MapAppearanceDialog(QtGui.QDialog, ui_ma.Ui_MapAppearanceDialog):
             button.setPalette(palette)
 
 
-#
-# CLASS FlightPlanTopView
-#
-
-
-class MSSTopViewWindow(mss_qt.MSSMplViewWindow, ui.Ui_TopViewWindow):
+class MSSTopViewWindow(MSSMplViewWindow, ui.Ui_TopViewWindow):
     """PyQt4 window implementing a MapCanvas as an interactive flight track
        editor.
     """
@@ -174,11 +155,12 @@ class MSSTopViewWindow(mss_qt.MSSMplViewWindow, ui.Ui_TopViewWindow):
         """
         super(MSSTopViewWindow, self).__init__(parent, model)
         self.setupUi(self)
+        self.setWindowIcon(QtGui.QIcon(icons('64x64')))
 
         # Dock windows [WMS, Satellite, Trajectories, Remote Sensing, KML Overlay]:
         self.docks = [None, None, None, None, None]
 
-        self.settingsfile = os.path.join(constants.MSS_CONFIG_PATH, "mss.topview.cfg")
+        self.settings_tag = "topview"
         self.loadSettings()
 
         # Initialise the GUI elements (map view, items of combo boxes etc.).
@@ -188,30 +170,23 @@ class MSSTopViewWindow(mss_qt.MSSMplViewWindow, ui.Ui_TopViewWindow):
         # ==========================
 
         # Map controls.
-        self.connect(self.btMapRedraw, QtCore.SIGNAL("clicked()"),
-                     self.mpl.canvas.redrawMap)
-        self.connect(self.cbChangeMapSection, QtCore.SIGNAL("activated(int)"),
-                     self.changeMapSection)
+        self.btMapRedraw.clicked.connect(self.mpl.canvas.redrawMap)
+        self.cbChangeMapSection.activated.connect(self.changeMapSection)
 
         # Settings
-        self.connect(self.btSettings, QtCore.SIGNAL("clicked()"),
-                     self.settingsDlg)
+        self.btSettings.clicked.connect(self.settingsDlg)
 
         # Tool opener.
-        self.connect(self.cbTools, QtCore.SIGNAL("currentIndexChanged(int)"),
-                     self.openTool)
+        self.cbTools.currentIndexChanged.connect(self.openTool)
 
         # Controls to interact with the flight track.
         # (For usage of the functools.partial() function, see Chapter 4 (Section
         # Signals and Slots) of 'Rapid GUI Programming with Python and Qt: The
         # Definitive Guide to PyQt Programming' (Mark Summerfield).)
         wpi = self.mpl.canvas.waypoints_interactor
-        self.connect(self.btMvWaypoint, QtCore.SIGNAL("clicked()"),
-                     functools.partial(wpi.set_edit_mode, mpl_pi.MOVE))
-        self.connect(self.btInsWaypoint, QtCore.SIGNAL("clicked()"),
-                     functools.partial(wpi.set_edit_mode, mpl_pi.INSERT))
-        self.connect(self.btDelWaypoint, QtCore.SIGNAL("clicked()"),
-                     functools.partial(wpi.set_edit_mode, mpl_pi.DELETE))
+        self.btMvWaypoint.clicked.connect(functools.partial(wpi.set_edit_mode, mpl_pi.MOVE))
+        self.btInsWaypoint.clicked.connect(functools.partial(wpi.set_edit_mode, mpl_pi.INSERT))
+        self.btDelWaypoint.clicked.connect(functools.partial(wpi.set_edit_mode, mpl_pi.DELETE))
 
     def __del__(self):
         del self.mpl.canvas.waypoints_interactor
@@ -249,8 +224,8 @@ class MSSTopViewWindow(mss_qt.MSSMplViewWindow, ui.Ui_TopViewWindow):
                 widget = wms.HSecWMSControlWidget(
                     default_WMS=config_loader(dataset="default_WMS", default=mss_default.default_WMS),
                     view=self.mpl.canvas,
-                    wms_cache=config_loader(dataset="wms_cache",
-                                            default=mss_default.wms_cache))
+                    parent=self,
+                    wms_cache=config_loader(dataset="wms_cache", default=mss_default.wms_cache))
             elif index == SATELLITE:
                 title = "Satellite Track Prediction"
                 widget = sat.SatelliteControlWidget(view=self.mpl.canvas)
@@ -270,19 +245,24 @@ class MSSTopViewWindow(mss_qt.MSSMplViewWindow, ui.Ui_TopViewWindow):
         """Change the current map section to one of the predefined regions.
         """
         # Get the initial projection parameters from the tables in mss_settings.
-        current_map_key = str(self.cbChangeMapSection.currentText())
+        current_map_key = unicode(self.cbChangeMapSection.currentText())
         predefined_map_sections = config_loader(dataset="predefined_map_sections",
                                                 default=mss_default.predefined_map_sections)
         current_map = predefined_map_sections[current_map_key]
         crs_to_mpl_basemap_table = config_loader(dataset="crs_to_mpl_basemap_table",
                                                  default=mss_default.crs_to_mpl_basemap_table)
+        proj_params = crs_to_mpl_basemap_table.get(current_map["CRS"])
+        if proj_params is None:
+            proj_params = get_projection_params(current_map["CRS"])
+        if proj_params is None:
+            raise ValueError(u"unknown EPSG code: {:}".format(current_map["CRS"]))
 
         # Create a keyword arguments dictionary for basemap that contains
         # the projection parameters.
         kwargs = current_map["map"]
         kwargs.update({"CRS": current_map["CRS"]})
-        kwargs.update({"BBOX_UNITS": crs_to_mpl_basemap_table[current_map["CRS"]]["bbox"]})
-        kwargs.update(crs_to_mpl_basemap_table[current_map["CRS"]]["basemap"])
+        kwargs.update({"BBOX_UNITS": proj_params["bbox"]})
+        kwargs.update(proj_params["basemap"])
 
         if only_kwargs:
             # Return kwargs dictionary and do NOT redraw the map.
@@ -301,10 +281,11 @@ class MSSTopViewWindow(mss_qt.MSSMplViewWindow, ui.Ui_TopViewWindow):
         settings = self.getView().getMapAppearance()
         dlg = MSS_TV_MapAppearanceDialog(parent=self, settings_dict=settings)
         dlg.setModal(True)
-        if dlg.exec_() == QtGui.QDialog.Accepted:
+        if dlg.exec_() == QtWidgets.QDialog.Accepted:
             settings = dlg.getSettings()
             self.getView().setMapAppearance(settings)
             self.saveSettings()
+            title = "Top View"
             if self.docks[WMS]:
                 wms_widget = self.docks[WMS].widget()
                 layer = wms_widget.getLayer()
@@ -316,13 +297,16 @@ class MSSTopViewWindow(mss_qt.MSSMplViewWindow, ui.Ui_TopViewWindow):
                 level = wms_widget.getLevel()
                 init_time = wms_widget.getInitTime()
                 valid_time = wms_widget.getValidTime()
-                self.mpl.canvas.drawMetadata(title=wms_widget.get_layer_object(layer).title,
+                layer_object = wms_widget.get_layer_object(layer)
+                if layer_object is not None:
+                    title = layer_object.title
+                self.mpl.canvas.drawMetadata(title=title,
                                              init_time=init_time,
                                              valid_time=valid_time,
                                              level=level,
                                              style=style_title)
             else:
-                self.mpl.canvas.drawMetadata(title="Top view")
+                self.mpl.canvas.drawMetadata(title=title)
             self.mpl.canvas.waypoints_interactor.redraw_path()
         dlg.destroy()
 
@@ -333,54 +317,10 @@ class MSSTopViewWindow(mss_qt.MSSMplViewWindow, ui.Ui_TopViewWindow):
         # TODO: ConfigParser and a central configuration file might be the better solution than pickle.
         # http://stackoverflow.com/questions/200599/whats-the-best-way-to-store-simple-user-settings-in-python
         settings = self.getView().getMapAppearance()
-        logging.debug("storing settings to %s", self.settingsfile)
-        fileobj = open(self.settingsfile, "w")
-        pickle.dump(settings, fileobj)
-        fileobj.close()
+        save_settings_pickle(self.settings_tag, settings)
 
     def loadSettings(self):
         """Load settings from the file self.settingsfile.
         """
-        settings = None
-        if os.path.exists(self.settingsfile):
-            logging.debug("loading settings from %s", self.settingsfile)
-            fileobj = open(self.settingsfile, "r")
-            settings = pickle.load(fileobj)
-            fileobj.close()
+        settings = load_settings_pickle(self.settings_tag, {})
         self.getView().setMapAppearance(settings)
-
-
-# Main program to test the window during development. The following code
-# will not be executed if the view is opened from the Mission Support
-# System user interface.
-
-def _main():
-    # Log everything, and send it to stderr.
-    # See http://docs.python.org/library/logging.html for more information
-    # on the Python logging module.
-    # NOTE: http://docs.python.org/library/logging.html#formatter-objects
-    logging.basicConfig(level=logging.DEBUG,
-                        format="%(asctime)s (%(module)s.%(funcName)s): %(message)s",
-                        datefmt="%Y-%m-%d %H:%M:%S")
-
-    import sys
-
-    # Create an initital flight track.
-    initial_waypoints = [ft.Waypoint(48.10, 11.27, 0, comments="take off"),
-                         ft.Waypoint(52.32, 09.21, 200),
-                         ft.Waypoint(55.10, 11.27, 0, comments="landing")]
-    initial_waypoints = [ft.Waypoint(40., 25., 0),
-                         ft.Waypoint(60., -10., 0),
-                         ft.Waypoint(40., 10, 0)]
-
-    waypoints_model = ft.WaypointsTableModel(QtCore.QString(""))
-    waypoints_model.insertRows(0, rows=len(initial_waypoints),
-                               waypoints=initial_waypoints)
-
-    application = QtGui.QApplication(sys.argv)
-    window = MSSTopViewWindow(model=waypoints_model)
-    window.show()
-    sys.exit(application.exec_())
-
-if __name__ == "__main__":
-    _main()
