@@ -1,63 +1,61 @@
-"""Mission Support System tool application that manages trajectories.
+# -*- coding: utf-8 -*-
+"""
 
-********************************************************************************
+    mslib.msui.trajectories_tool
+    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 
-   Copyright 2008-2014 Deutsches Zentrum fuer Luft- und Raumfahrt e.V.
+    Tool application that manages trajectories.
 
-   Licensed under the Apache License, Version 2.0 (the "License");
-   you may not use this file except in compliance with the License.
-   You may obtain a copy of the License at
+    This file is part of mss.
+
+    :copyright: Copyright 2008-2014 Deutsches Zentrum fuer Luft- und Raumfahrt e.V.
+    :copyright: Copyright 2011-2014 Marc Rautenhaus (mr)
+    :copyright: Copyright 2016-2017 by the mss team, see AUTHORS.
+    :license: APACHE-2.0, see LICENSE for details.
+
+    Licensed under the Apache License, Version 2.0 (the "License");
+    you may not use this file except in compliance with the License.
+    You may obtain a copy of the License at
 
        http://www.apache.org/licenses/LICENSE-2.0
 
-   Unless required by applicable law or agreed to in writing, software
-   distributed under the License is distributed on an "AS IS" BASIS,
-   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-   See the License for the specific language governing permissions and
-   limitations under the License.
-
-********************************************************************************
-
-This file is part of the Mission Support System User Interface (MSUI).
-
-AUTHORS:
-========
-
-* Marc Rautenhaus (mr)
-
+    Unless required by applicable law or agreed to in writing, software
+    distributed under the License is distributed on an "AS IS" BASIS,
+    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+    See the License for the specific language governing permissions and
+    limitations under the License.
 """
 
-# standard library imports
 import logging
 import os
 
 # related third party imports
-from PyQt4 import QtGui, QtCore  # Qt4 bindings
+from mslib.msui.mss_qt import QtCore, QtWidgets, QtGui, USE_PYQT5
 
 # local application imports
-from mslib.msui import ui_trajectories_window as ui
+from mslib.msui.mss_qt import ui_trajectories_window as ui
 from mslib.msui import trajectory_item_tree as titree
 # import trajectory_ts
-from mslib.msui import mss_qt
-
-#
-# USER INTERFACE CLASS FlightPlanTableView
-#
+from mslib.msui.viewwindows import MSSViewWindow
+from mslib.msui.icons import icons
 
 
-class MSSTrajectoriesToolWindow(mss_qt.MSSViewWindow, ui.Ui_TrajectoriesWindow):
+class MSSTrajectoriesToolWindow(MSSViewWindow, ui.Ui_TrajectoriesWindow):
     """Implements a trajectory tool.
     """
 
     name = "Trajectories Tool"
+    moduleCloses = QtCore.pyqtSignal(name="moduleCloses")
 
-    def __init__(self, parent=None, listviews=None):
+    def __init__(self, parent=None, listviews=None, listtools=None, viewsChanged=None):
         """
         """
         super(MSSTrajectoriesToolWindow, self).__init__(parent)
         self.setupUi(self)
+        self.setWindowIcon(QtGui.QIcon(icons('64x64')))
 
-        self.actionOpenFlightTrack.setEnabled(titree.hasNAppy)
+        self.actionOpenFlightTrack.setEnabled(titree.HAVE_NAPPY)
+        self.viewsChanged = viewsChanged
 
         # traj_item_tree stores all data corresponding to the map, i.e. flight
         # tracks, trajectories, etc. The design follows Qt's model/view
@@ -78,39 +76,30 @@ class MSSTrajectoriesToolWindow(mss_qt.MSSViewWindow, ui.Ui_TrajectoriesWindow):
         # Pointer to the QListWidget that accomodates the views that are
         # open in the MSUI.
         self.listviews = listviews
+        self.listtools = listtools
 
         # Connect Qt SIGNALs:
         # ===================
 
         # Menu.
-        self.connect(self.actionOpenTrajectories, QtCore.SIGNAL("triggered()"),
-                     self.loadTrajectories)
-        self.connect(self.actionOpenFlightTrack, QtCore.SIGNAL("triggered()"),
-                     self.loadFlightTrack)
+        self.actionOpenTrajectories.triggered.connect(self.loadTrajectories)
+        self.actionOpenFlightTrack.triggered.connect(self.loadFlightTrack)
 
         # Item selection.
-        self.connect(self.btSelectMapElements, QtCore.SIGNAL("clicked()"),
-                     self.selectMapElements)
+        self.btSelectMapElements.clicked.connect(self.selectMapElements)
 
         # Item style.
-        self.connect(self.btColour, QtCore.SIGNAL("clicked()"),
-                     self.setCurrentItemColour)
-        self.connect(self.btLineStyle, QtCore.SIGNAL("clicked()"),
-                     self.setCurrentItemLineStyle)
-        self.connect(self.btLineWidth, QtCore.SIGNAL("clicked()"),
-                     self.setCurrentItemLineWidth)
-        self.connect(self.btTimeMarker, QtCore.SIGNAL("clicked()"),
-                     self.setCurrentItemTimeMarker)
+        self.btColour.clicked.connect(self.setCurrentItemColour)
+        self.btLineStyle.clicked.connect(self.setCurrentItemLineStyle)
+        self.btLineWidth.clicked.connect(self.setCurrentItemLineWidth)
+        self.btTimeMarker.clicked.connect(self.setCurrentItemTimeMarker)
 
         # View control.
-        self.connect(self.btPlotInView, QtCore.SIGNAL("clicked()"),
-                     self.plotCurrentItemInView)
-        self.connect(self.btRemoveFromView, QtCore.SIGNAL("clicked()"),
-                     self.removeCurrentItemFromView)
-        if self.listviews:
-            self.connect(self.listviews, QtCore.SIGNAL("viewsChanged()"),
-                         self.updateViews)
-            self.updateViews()
+        self.btPlotInView.clicked.connect(self.plotCurrentItemInView)
+        self.btRemoveFromView.clicked.connect(self.removeCurrentItemFromView)
+        if self.viewsChanged is not None:
+            self.viewsChanged.connect(self.updateViews)
+        self.updateViews()
 
     def closeEvent(self, event):
         """Ask user if he/she wants to close the window.
@@ -119,7 +108,7 @@ class MSSTrajectoriesToolWindow(mss_qt.MSSViewWindow, ui.Ui_TrajectoriesWindow):
         'moduleCloses()'.
         """
         if super(MSSTrajectoriesToolWindow, self).closeEvent(event):
-            self.emit(QtCore.SIGNAL("moduleCloses()"))
+            self.moduleCloses.emit()
 
     def loadFlightTrack(self):
         """Slot for the 'Open Flight Track..' menu entry. Opens a file dialog
@@ -127,30 +116,34 @@ class MSSTrajectoriesToolWindow(mss_qt.MSSViewWindow, ui.Ui_TrajectoriesWindow):
         """
         # Ask for a file to open, convert the return file name from type
         # QString to str.
-        nas_file = QtGui.QFileDialog.getOpenFileName(self, "Open NASA Ames File",
-                                                     "", "NASA Ames files (*.nas)")
-        if not nas_file.isEmpty():
-            nas_file = unicode(nas_file)
-            logging.debug("Loading flight track data from %s", nas_file)
+        nas_file = QtWidgets.QFileDialog.getOpenFileName(
+            self, "Open NASA Ames File", "", "NASA Ames files (*.nas)")
+        nas_file = nas_file[0] if USE_PYQT5 else unicode(nas_file)
+        if nas_file:
+            logging.debug(u"Loading flight track data from '%s'", nas_file)
 
             # Add the flight track to the data tree.
-            new_item_index = self.traj_item_tree.addFlightTrack(nas_file)
+            try:
+                new_item_index = self.traj_item_tree.addFlightTrack(nas_file)
 
-            # Scroll the tree view so that the new item is visible and resize
-            # the first column to be wide enough for the text.
-            self.tvVisibleElements.scrollTo(new_item_index)
-            self.tvVisibleElements.resizeColumnToContents(0)
+                # Scroll the tree view so that the new item is visible and resize
+                # the first column to be wide enough for the text.
+                self.tvVisibleElements.scrollTo(new_item_index)
+                self.tvVisibleElements.resizeColumnToContents(0)
+            except RuntimeError:
+                QtWidgets.QMessageBox.critical(
+                    self, self.tr("Trajectory Tool"), self.tr(u"ERROR:\nCould not read file\n{}".format(nas_file)))
 
     def loadTrajectories(self):
         """Slot for the 'Open Trajectories..' menu entry. Opens a QFileDialog
            and reads the selected trajectory file using lagrantooutputreader.
         """
         # Ask for a directory to open.
-        traj_dir = QtGui.QFileDialog.getExistingDirectory(self,
-                                                          "Open Lagranto Output Directory", "")
-        if not traj_dir.isEmpty():
-            traj_dir = unicode(traj_dir)
-            logging.debug("Loading trajectory data from %s", traj_dir)
+        traj_dir = QtWidgets.QFileDialog.getExistingDirectory(
+            self, "Open Lagranto Output Directory", "")
+        traj_dir = traj_dir[0] if USE_PYQT5 else unicode(traj_dir)
+        if traj_dir:
+            logging.debug(u"Loading trajectory data from '%s'", traj_dir)
 
             # Test if selected directory contains subdirectories (as is
             # the case with ensemble runs). If yes,
@@ -249,9 +242,8 @@ class MSSTrajectoriesToolWindow(mss_qt.MSSViewWindow, ui.Ui_TrajectoriesWindow):
         else:
             selectedIndexes = self.selectedMapElements()
             if len(selectedIndexes) != 1:
-                QtGui.QMessageBox.warning(self, self.tr("select map elements"),
-                                          self.tr("Please select a single element for this operation."),
-                                          QtGui.QMessageBox.Ok)
+                QtWidgets.QMessageBox.warning(self, self.tr("select map elements"),
+                                              self.tr("Please select a single element for this operation."))
                 return
             else:
                 rootIndex = selectedIndexes[0]
@@ -268,7 +260,7 @@ class MSSTrajectoriesToolWindow(mss_qt.MSSViewWindow, ui.Ui_TrajectoriesWindow):
         # Generally, an instance of this class will contain a list of
         # non-overlapping selection ranges.
         itemSelection = self.traj_item_tree.selectionFromQuery(
-            str(self.leSelectionQuery.text()),
+            unicode(self.leSelectionQuery.text()),
             index=rootIndex)
 
         # Items can be selected in the tree view by using the select()
@@ -283,13 +275,12 @@ class MSSTrajectoriesToolWindow(mss_qt.MSSViewWindow, ui.Ui_TrajectoriesWindow):
         selectionModel = self.tvVisibleElements.selectionModel()
         if itemSelection.isEmpty():
             # If no item matches the query we can't select anything.
-            QtGui.QMessageBox.warning(self, self.tr("select map elements"),
-                                      self.tr("No matching items have been found."),
-                                      QtGui.QMessageBox.Ok)
+            QtWidgets.QMessageBox.warning(self, self.tr("select map elements"),
+                                          self.tr("No matching items have been found."))
         else:
             selectionModel.select(itemSelection,
-                                  QtGui.QItemSelectionModel.ClearAndSelect |
-                                  QtGui.QItemSelectionModel.Rows)
+                                  QtWidgets.QItemSelectionModel.ClearAndSelect |
+                                  QtWidgets.QItemSelectionModel.Rows)
 
     def setCurrentItemColour(self):
         """Informs traj_item_tree to change the colour of the selected elements
@@ -297,10 +288,10 @@ class MSSTrajectoriesToolWindow(mss_qt.MSSViewWindow, ui.Ui_TrajectoriesWindow):
         """
         colour = str(self.cbColour.currentText())
         if colour == 'None':
-            colour = None
+            colour = "blue"
         indices = self.selectedMapElements()
         if len(indices) == 1:
-            logging.debug("Changing colour of element %s",
+            logging.debug(u"Changing colour of element %s",
                           indices[0].internalPointer().getName())
         else:
             logging.debug("Changing colour of selected elements")
@@ -308,8 +299,7 @@ class MSSTrajectoriesToolWindow(mss_qt.MSSViewWindow, ui.Ui_TrajectoriesWindow):
             self.traj_item_tree.changeItemGxProperty_list(indices, 'general',
                                                           'colour', colour)
         except Exception, ex:
-            QtGui.QMessageBox.warning(self, self.tr('item colour'),
-                                      self.tr(str(ex)), QtGui.QMessageBox.Ok)
+            QtWidgets.QMessageBox.warning(self, self.tr('item colour'), self.tr(str(ex)))
 
     def setCurrentItemLineStyle(self):
         """Informs traj_item_tree to change the line style of the selected
@@ -317,19 +307,18 @@ class MSSTrajectoriesToolWindow(mss_qt.MSSViewWindow, ui.Ui_TrajectoriesWindow):
         """
         lineStyle = str(self.cbLineStyle.currentText())
         if lineStyle == 'None':
-            lineStyle = None
+            lineStyle = "-"
         indices = self.selectedMapElements()
         if len(indices) == 1:
-            logging.debug("Changing line style of element %s",
+            logging.debug(u"Changing line style of element %s",
                           indices[0].internalPointer().getName())
         else:
             logging.debug("Changing line style of selected elements")
         try:
-            self.traj_item_tree.changeItemGxProperty_list(indices, 'general',
-                                                          'linestyle', lineStyle)
-        except Exception, e:
-            QtGui.QMessageBox.warning(self, self.tr('item line style'),
-                                      self.tr(str(e)), QtGui.QMessageBox.Ok)
+            self.traj_item_tree.changeItemGxProperty_list(indices, "general",
+                                                          "linestyle", lineStyle)
+        except Exception, ex:
+            QtWidgets.QMessageBox.warning(self, self.tr('item line style'), self.tr(u"{} {}".format(type(ex), ex)))
 
     def setCurrentItemLineWidth(self):
         """Informs traj_item_tree to change the line width of the selected
@@ -338,16 +327,15 @@ class MSSTrajectoriesToolWindow(mss_qt.MSSViewWindow, ui.Ui_TrajectoriesWindow):
         lineWidth = self.sbLineWidth.value()
         indices = self.selectedMapElements()
         if len(indices) == 1:
-            logging.debug("Changing line thickness of element %s",
+            logging.debug(u"Changing line thickness of element %s",
                           indices[0].internalPointer().getName())
         else:
             logging.debug("Changing line thickness of selected elements")
         try:
-            self.traj_item_tree.changeItemGxProperty_list(indices, 'general',
-                                                          'linewidth', lineWidth)
-        except Exception, e:
-            QtGui.QMessageBox.warning(self, self.tr('item line width'),
-                                      self.tr(str(e)), QtGui.QMessageBox.Ok)
+            self.traj_item_tree.changeItemGxProperty_list(indices, "general",
+                                                          "linewidth", lineWidth)
+        except Exception, ex:
+            QtWidgets.QMessageBox.warning(self, self.tr('item line width'), self.tr(u"{} {}".format(type(ex), ex)))
 
     def setCurrentItemTimeMarker(self):
         """Set the given time markers for the selected item and all its
@@ -368,13 +356,13 @@ class MSSTrajectoriesToolWindow(mss_qt.MSSViewWindow, ui.Ui_TrajectoriesWindow):
             # a number of trajectory items) ask the user if he wants to
             # set/change the time marker for all children.
             if isinstance(item, titree.LagrantoOutputItem):
-                ret = QtGui.QMessageBox.warning(self, self.tr("Time Marker"),
-                                                self.tr("Do you want to set the interval " +
-                                                        interval.strftime("%H:%M") +
-                                                        "\nfor all children of " + item.getName()),
-                                                QtGui.QMessageBox.Yes | QtGui.QMessageBox.Default,
-                                                QtGui.QMessageBox.No)
-                if ret == QtGui.QMessageBox.No:
+                ret = QtWidgets.QMessageBox.warning(self, self.tr("Time Marker"),
+                                                    self.tr("Do you want to set the interval " +
+                                                            interval.strftime("%H:%M") +
+                                                            "\nfor all children of " + item.getName()),
+                                                    QtWidgets.QMessageBox.Yes | QtWidgets.QMessageBox.No,
+                                                    QtWidgets.QMessageBox.No)
+                if ret == QtWidgets.QMessageBox.No:
                     #
                     # If the answer is no, continue with the next item on
                     # the stack.
@@ -388,65 +376,57 @@ class MSSTrajectoriesToolWindow(mss_qt.MSSViewWindow, ui.Ui_TrajectoriesWindow):
             #
             # Now set the time marker for the current item.
             try:
-                logging.debug("Setting time marker for element %s",
+                logging.debug(u"Setting time marker for element %s",
                               item.getName())
                 setInterval = self.traj_item_tree.setTimeMarker(index, interval,
                                                                 emit_change=False)
                 if not setInterval:
-                    QtGui.QMessageBox.warning(self, self.tr("Time marker"),
-                                              self.tr("Time markers have been deleted."),
-                                              QtGui.QMessageBox.Ok)
+                    QtWidgets.QMessageBox.warning(self, self.tr("Time marker"),
+                                                  self.tr("Time markers have been deleted."))
                 elif setInterval != interval:
-                    QtGui.QMessageBox.warning(self, self.tr("Time marker"),
-                                              self.tr("Warning: The minimum time interval for the"
-                                                      "selected variable is " +
-                                                      setInterval.strftime("%H:%M") +
-                                                      ".\nThe interval has been set to this value."),
-                                              QtGui.QMessageBox.Ok)
+                    QtWidgets.QMessageBox.warning(self, self.tr("Time marker"),
+                                                  self.tr("Warning: The minimum time interval for the"
+                                                          "selected variable is " +
+                                                          setInterval.strftime("%H:%M") +
+                                                          ".\nThe interval has been set to this value."))
                 if not first_index:
                     first_index = index
                 last_index = index
-            except Exception, e:
-                QtGui.QMessageBox.warning(self, self.tr("Time marker"),
-                                          self.tr(str(e)), QtGui.QMessageBox.Ok)
+            except Exception, ex:
+                QtWidgets.QMessageBox.warning(self, self.tr("Time marker"), self.tr(u"{} {}".format(type(ex), ex)))
 
-        self.traj_item_tree.emitChange(first_index, last_index,
-                                       mode="MARKER_CHANGE")
+        self.traj_item_tree.emitChange(first_index, last_index, mode="MARKER_CHANGE")
 
     def plotCurrentItemInView(self):
         """
         """
         view_name = self.cbPlotInView.currentText()
-        if str(view_name) != "None":
-            view_item = self.listviews.findItems(view_name,
-                                                 QtCore.Qt.MatchContains)[0]
-            logging.debug("Plotting selected elements in view <%s>", view_name)
+        selection = self.selectedMapElements()
+        if unicode(view_name) != "None" and len(selection) > 0:
+            view_item = (self.listviews.findItems(view_name, QtCore.Qt.MatchContains) +
+                         self.listtools.findItems(view_name, QtCore.Qt.MatchContains))[0]
+            logging.debug(u"Plotting selected elements in view <%s>", view_name)
             # Connect the trajectory tree to the view.
             view_window = view_item.window
             view = view_window.getView()
-            if hasattr(view, "setTrajectoryModel"):
-                if view_window not in self.connected_views:
-                    logging.debug("Connecting to view window <%s>",
-                                  view_window.identifier)
-                    self.connected_views.append(view_window)
-                    view.setTrajectoryModel(self.traj_item_tree)
-                self.traj_item_tree.setItemVisibleInView_list(
-                    self.selectedMapElements(), view_item.window, True)
-            else:
-                logging.error("View window <%s> does not support display of trajectories",
+            self.traj_item_tree.setItemVisibleInView_list(
+                selection, view_item.window, True)
+            if view_window not in self.connected_views:
+                logging.debug(u"Connecting to view window <%s>",
                               view_window.identifier)
+                self.connected_views.append(view_window)
+                view.setTrajectoryModel(self.traj_item_tree)
 
     def removeCurrentItemFromView(self):
         """
         """
         view_name = self.cbRemoveFromView.currentText()
-        if str(view_name) != "None":
-            view_item = self.listviews.findItems(view_name,
-                                                 QtCore.Qt.MatchContains)[0]
-            logging.debug("Removing selected elements from view <%s>", view_name)
+        if unicode(view_name) != "None":
+            view_item = (self.listviews.findItems(view_name, QtCore.Qt.MatchContains) +
+                         self.listtools.findItems(view_name, QtCore.Qt.MatchContains))[0]
+            logging.debug(u"Removing selected elements from view <%s>", view_name)
             self.traj_item_tree.setItemVisibleInView_list(
                 self.selectedMapElements(), view_item.window, False)
-            # TODO: Disconnect tree model from view if no item is displayed!! (2010-08-27)
 
     def updateViews(self):
         """Update the list of views in the comboboxes cbPlotInView and
@@ -458,15 +438,20 @@ class MSSTrajectoriesToolWindow(mss_qt.MSSViewWindow, ui.Ui_TrajectoriesWindow):
         # Remember the currently selected views in the comboboxes.
         item_plot = self.cbPlotInView.currentText()
         item_remove = self.cbRemoveFromView.currentText()
+
+        old_views = [self.cbPlotInView.itemText(i) for i in range(self.cbPlotInView.count())]
         # Clear the boxes, add the "None" view.
         self.cbPlotInView.clear()
         self.cbRemoveFromView.clear()
         self.cbPlotInView.addItem("None")
         self.cbRemoveFromView.addItem("None")
         # Add all available views.
-        for i in self.listviews.findItems("(", QtCore.Qt.MatchContains):
-            self.cbPlotInView.addItem(i.text())
-            self.cbRemoveFromView.addItem(i.text())
+        for i in (self.listviews.findItems("(", QtCore.Qt.MatchContains) +
+                  self.listtools.findItems("(", QtCore.Qt.MatchContains)):
+            i_view = i.window.getView()
+            if hasattr(i_view, "setTrajectoryModel"):
+                self.cbPlotInView.addItem(i.text())
+                self.cbRemoveFromView.addItem(i.text())
         # Restore the old selection (set "None" if the selected view was
         # closed).
         index = self.cbPlotInView.findText(item_plot)
@@ -474,29 +459,20 @@ class MSSTrajectoriesToolWindow(mss_qt.MSSViewWindow, ui.Ui_TrajectoriesWindow):
         index = self.cbRemoveFromView.findText(item_remove)
         self.cbRemoveFromView.setCurrentIndex(index if index >= 0 else 0)
 
+        new_views = [self.cbPlotInView.itemText(i) for i in range(self.cbPlotInView.count())]
+        missing_views = [unicode(_x) for _x in old_views if _x not in new_views]
+        stack = [_x for _x in self.traj_item_tree.getRootItem().childItems]
+        while len(stack) > 0:
+            # Downwards traversal of the tree to determine all visible items
+            # below the items that are on the stack.
+            item = stack.pop()
+            if hasattr(item, "views"):
+                item.views = [_x for _x in item.views if _x not in missing_views]
+            if hasattr(item, "childItems"):
+                stack.extend(item.childItems)
+        self.connected_views = [_x for _x in self.connected_views if _x not in missing_views]
+
     def getItemTree(self):
         """
         """
         return self.traj_item_tree
-
-
-################################################################################
-
-def _main():
-    # Log everything, and send it to stderr.
-    # See http://docs.python.org/library/logging.html for more information
-    # on the Python logging module.
-    # NOTE: http://docs.python.org/library/logging.html#formatter-objects
-    logging.basicConfig(level=logging.DEBUG,
-                        format="%(asctime)s (%(module)s.%(funcName)s): %(message)s",
-                        datefmt="%Y-%m-%d %H:%M:%S")
-
-    import sys
-
-    application = QtGui.QApplication(sys.argv)
-    window = MSSTrajectoriesToolWindow()
-    window.show()
-    sys.exit(application.exec_())
-
-if __name__ == "__main__":
-    _main()
