@@ -1,72 +1,58 @@
-"""Tree model for trajectory items loaded into the trajectory tool and drawn
-   on any views.
+# -*- coding: utf-8 -*-
+"""
 
-********************************************************************************
+    mslib.msui.trajectory_item_tree
+    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 
-   Copyright 2008-2014 Deutsches Zentrum fuer Luft- und Raumfahrt e.V.
+    Tree model for trajectory items loaded into the trajectory tool and drawn
+    on any views.
 
-   Licensed under the Apache License, Version 2.0 (the "License");
-   you may not use this file except in compliance with the License.
-   You may obtain a copy of the License at
+    This file is part of mss.
+
+    :copyright: Copyright 2008-2014 Deutsches Zentrum fuer Luft- und Raumfahrt e.V.
+    :copyright: Copyright 2011-2014 Marc Rautenhaus (mr)
+    :copyright: Copyright 2016-2017 by the mss team, see AUTHORS.
+    :license: APACHE-2.0, see LICENSE for details.
+
+    Licensed under the Apache License, Version 2.0 (the "License");
+    you may not use this file except in compliance with the License.
+    You may obtain a copy of the License at
 
        http://www.apache.org/licenses/LICENSE-2.0
 
-   Unless required by applicable law or agreed to in writing, software
-   distributed under the License is distributed on an "AS IS" BASIS,
-   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-   See the License for the specific language governing permissions and
-   limitations under the License.
-
-********************************************************************************
-
-This file is part of the Mission Support System User Interface (MSUI).
-
-AUTHORS:
-========
-
-* Marc Rautenhaus (mr)
-
+    Unless required by applicable law or agreed to in writing, software
+    distributed under the License is distributed on an "AS IS" BASIS,
+    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+    See the License for the specific language governing permissions and
+    limitations under the License.
 """
 
-# standard library imports
 import datetime
 
 import logging
 import os
-from mslib.mss_util import config_loader
+from mslib.utils import config_loader
 from mslib.msui import MissionSupportSystemDefaultConfig as mss_default
 # related third party imports
-from PyQt4 import QtCore, QtGui
+from mslib.msui.mss_qt import QtCore, QtGui
 import numpy
 
 try:
     import nappy
-
-    hasNAppy = True
+    HAVE_NAPPY = True
 except:
-    print "*** NAppy is not available. You will not be able to read NASA Ames files. ***"
-    hasNAppy = False
+    logging.warn("*** NAppy is not available. You will not be able to read NASA Ames files. ***")
+    HAVE_NAPPY = False
 
-# local application imports
 from mslib.msui import lagranto_output_reader
 
 
-#
-# EXCEPTION CLASSES
-#
-
-
 class LagrantoTreeModelUnsupportedOperationError(Exception):
     """Exception class to handle wrong method arguments.
     """
     pass
 
 
-#
-# CLASS AbstractLagrantoDataItem
-#
-
-
 class AbstractLagrantoDataItem:
     """Base class for all trajectory instances that are loaded into a
        trajectory tool.
@@ -95,7 +81,7 @@ class AbstractLagrantoDataItem:
 
         # If this item has a parent, register it as a child in the parent
         # node.
-        if parent:
+        if parent is not None:
             parent.appendChild(self)
 
     def appendChild(self, item):
@@ -123,16 +109,15 @@ class AbstractLagrantoDataItem:
                     self.gxElements['general']['linestyle']) + '/' + str(
                         self.gxElements['general']['linewidth'])
             except Exception, ex:
-                logging.debug("caught a wildcard Exception: %s, %s", type(ex), ex)
+                logging.debug(u"caught a wildcard Exception: {}, {}".format(type(ex), ex))
                 return ''
         elif column == 3:
             # Item markers.
             s = ''
             try:
-                s += 'time(%s)' % self.gxElements['general'][
-                    'timeMarkerInterval'].strftime('%H:%M')
+                s += u'time({})'.format(self.gxElements['general']['timeMarkerInterval'].strftime('%H:%M'))
             except Exception, ex:
-                logging.debug("caught a wildcard Exception: %s, %s", type(ex), ex)
+                logging.debug(u"caught a wildcard Exception: {}, {}".format(type(ex), ex))
             return s
         else:
             return ''
@@ -141,7 +126,7 @@ class AbstractLagrantoDataItem:
         return self.parentItem
 
     def row(self):
-        if self.parentItem:
+        if self.parentItem is not None:
             return self.parentItem.childItems.index(self)
         else:
             return 0
@@ -209,11 +194,6 @@ class AbstractLagrantoDataItem:
         raise NotImplementedError("Abstract AbstractLagrantoDataItem.getMetadataValue called.")
 
 
-#
-# CLASS LagrantoMapItem
-#
-
-
 class LagrantoMapItem(AbstractLagrantoDataItem):
     """
     """
@@ -244,10 +224,10 @@ class LagrantoMapItem(AbstractLagrantoDataItem):
         elif column == 4:
             # Start coordinates from first elements of lon/lat/p variables.
             try:
-                return '%.2f, %.2f, %.1f' % \
-                       (self.lonVariableChild.getVariableData()[0],
-                        self.latVariableChild.getVariableData()[0],
-                        self.pressureVariableChild.getVariableData()[0])
+                return "{:.2f}, {:.2f}, {:.1f}".format(
+                    self.lonVariableChild.getVariableData()[0],
+                    self.latVariableChild.getVariableData()[0],
+                    self.pressureVariableChild.getVariableData()[0])
             except:
                 return ''
         else:
@@ -402,11 +382,6 @@ class LagrantoMapItem(AbstractLagrantoDataItem):
         return eval(qstring)
 
 
-#
-# CLASS FlightTrackItem
-#
-
-
 class FlightTrackItem(LagrantoMapItem):
     """Holds flight track data stored in a NASA Ames file.
     """
@@ -431,7 +406,7 @@ class FlightTrackItem(LagrantoMapItem):
 
         if self.nafile is None:
             logging.error("could not read NASA Ames file")
-            return
+            raise RuntimeError("could not read NASA Ames file")
 
         #
         # Add all variable names contained in self.data as child nodes, so
@@ -445,8 +420,7 @@ class FlightTrackItem(LagrantoMapItem):
             for item in self.childItems:
                 if item.getName().find(identifier) >= 0:
                     self.timeVariableChild = item
-                    logging.debug("identified time variable <%s>"
-                                  % item.getName())
+                    logging.debug(u"identified time variable <{}>".format(item.getName()))
                     #
                     # The time variable has to be modified a bit: convert
                     # seconds to hours and change the name correspondingly.
@@ -463,8 +437,8 @@ class FlightTrackItem(LagrantoMapItem):
             for item in self.childItems:
                 if item.getName().upper().find(identifier) >= 0:
                     self.lonVariableChild = item
-                    logging.debug("identified longitude variable <%s> with "
-                                  "identifier <%s>" % (item.getName(), identifier))
+                    logging.debug(u"identified longitude variable <{}> with "
+                                  u"identifier <{}>".format(item.getName(), identifier))
                     break
             if self.lonVariableChild is not None:
                 break
@@ -474,8 +448,8 @@ class FlightTrackItem(LagrantoMapItem):
             for item in self.childItems:
                 if item.getName().upper().find(identifier) >= 0:
                     self.latVariableChild = item
-                    logging.debug("identified latitude variable <%s> with "
-                                  "identifier <%s>" % (item.getName(), identifier))
+                    logging.debug(u"identified latitude variable <{}> with "
+                                  u"identifier <{}>".format(item.getName(), identifier))
                     break
             if self.latVariableChild is not None:
                 break
@@ -485,8 +459,8 @@ class FlightTrackItem(LagrantoMapItem):
             for item in self.childItems:
                 if item.getName().upper().find(identifier) >= 0:
                     self.pressureVariableChild = item
-                    logging.debug("identified pressure variable <%s> with "
-                                  "identifier <%s>" % (item.getName(), identifier))
+                    logging.debug(u"identified pressure variable <{}> with "
+                                  u"identifier <{}>".format(item.getName(), identifier))
                     break
             if self.pressureVariableChild is not None:
                 break
@@ -494,13 +468,18 @@ class FlightTrackItem(LagrantoMapItem):
     def __readNasaAmesFile(self):
         """Read the NASA Ames file 'self.nasFileName'.
         """
-        if not hasNAppy:
+        if not HAVE_NAPPY:
             self.nafile = None
             return
         #
         # Open the NASA Ames file and read the data.
-        self.nafile = nappy.openNAFile(self.nasFileName)
-        self.nafile.readData()
+        try:
+            self.nafile = nappy.openNAFile(self.nasFileName)
+            self.nafile.readData()
+        except TypeError, ex:  # catch TypeError as nappy itself triggers Exception when raising
+            self.nafile = None
+            logging.error(u"%s %s", type(ex), ex)
+            return
         #
         # Convert variable array of nafile from 'list' to 'NumPy array' for
         # faster access (the array can be large!).
@@ -514,11 +493,11 @@ class FlightTrackItem(LagrantoMapItem):
         for i in range(self.nafile.V.shape[0]):
             Vtemp.append(numpy.ma.masked_values(self.nafile.V[i],
                                                 self.nafile.VMISS[i]))
-            # Append independent variable to list of variables. Independent
-            # variables don't have a missing value, hence the dummy -9999.. here.
-            # TODO: This should be revised; the individual FlightTrackVariableItem shouldn't
-        #      access their parent's 'self.nafile.V' attribute to access the data;
-        #      instead they should own their own data array.
+        # Append independent variable to list of variables. Independent
+        # variables don't have a missing value, hence the dummy -9999.. here.
+        # TODO: This should be revised; the individual FlightTrackVariableItem shouldn't
+        #       access their parent's 'self.nafile.V' attribute to access the data;
+        #       instead they should own their own data array.
         Vtemp.append(numpy.ma.masked_values(self.nafile.X,
                                             -99999999))
         self.nafile.VNAME.append(self.nafile.XNAME[0])
@@ -541,11 +520,6 @@ class FlightTrackItem(LagrantoMapItem):
         return datetime.datetime(nas_date[0], nas_date[1], nas_date[2])
 
 
-#
-# CLASS LagrantoOutputItem
-#
-
-
 class LagrantoOutputItem(LagrantoMapItem):
     """Holds all data stored in a Lagranto output directory.
     """
@@ -577,18 +551,13 @@ class LagrantoOutputItem(LagrantoMapItem):
         # LagrantoOutputReader.
         for i, (trajectory, metadata) in enumerate(zip(self.loutput.data,
                                                        self.loutput.meta)):
-            trname = '%04i ' % i
-            if 'startcoordinates' in metadata.keys():
-                trname += str(['%.2f' % r for r in
-                               metadata['startcoordinates']]).replace('\'', '')
+            trname = "{:04d} ".format(i)
+            if "startcoordinates" in metadata.keys():
+                trname += unicode(
+                    [u"{:.2f}".format(r) for r in metadata["startcoordinates"]]).replace('\'', '')
             TrajectoryItem(trname, True, self, trajectory, metadata)
 
 
-#
-# CLASS TrajectoryItem
-#
-
-
 class TrajectoryItem(LagrantoMapItem):
     """Holds the data from an individual trajectory computed by Lagranto.
 
@@ -650,24 +619,24 @@ class TrajectoryItem(LagrantoMapItem):
         if column < 5:
             return LagrantoMapItem.treeViewData(self, column)
         elif column == 5:
-            return '%s for %i hrs' % \
-                   (self.getStartTime().strftime('%Y-%m-%d %H:%M UTC'),
-                    self.timeVariableChild.getVariableData()[-1])
+            return "{} for {:f} hrs" \
+                .format(self.getStartTime().strftime("%Y-%m-%d %H:%M UTC"),
+                        self.timeVariableChild.getVariableData()[-1])
         elif column == 6:
             s = ''
             for key, value in self.metadata.items():
-                if key not in ['starttime', 'file', 'starttime_filename',
-                               'startcoordinates', 'duration']:
-                    s += '%s = %s, ' % (key, str(value))
+                if key not in ["starttime", "file", "starttime_filename",
+                               "startcoordinates", "duration"]:
+                    s += u"{} = {}, ".format(key, str(value))
             return s
 
     def getStartTime(self):
         """Return the start time of the trajectory as a datetime object.
         """
         try:
-            return self.metadata['starttime']
+            return self.metadata["starttime"]
         except:
-            return self.metadata['starttime_filename']
+            return self.metadata["starttime_filename"]
 
     def getMetadata(self):
         """Return the metadata dictionary.
@@ -688,11 +657,6 @@ class TrajectoryItem(LagrantoMapItem):
             return None
 
 
-#
-# CLASS AbstractVariableItem
-#
-
-
 class AbstractVariableItem(AbstractLagrantoDataItem):
     """Tree node for variables contained in a flight track or trajectory file.
        Such variable can be visible on a time series plot, or they can be used
@@ -747,11 +711,11 @@ class AbstractVariableItem(AbstractLagrantoDataItem):
 
         aboveThreshold = False
         timeWindows = []
-        for i in range(len(myData)):
-            if not aboveThreshold and myData[i] > threshold:
+        for i, data in enumerate(myData):
+            if not aboveThreshold and data > threshold:
                 aboveThreshold = True
                 windowStartTime = timeData[i]
-            elif aboveThreshold and myData[i] <= threshold:
+            elif aboveThreshold and data <= threshold:
                 aboveThreshold = False
                 timeWindows.append([windowStartTime, timeData[i]])
 
@@ -771,11 +735,6 @@ class AbstractVariableItem(AbstractLagrantoDataItem):
                                                           eproperty, value)
 
 
-#
-# CLASS FlightTrackVariableItem
-#
-
-
 class FlightTrackVariableItem(AbstractVariableItem):
     """
     """
@@ -800,11 +759,6 @@ class FlightTrackVariableItem(AbstractVariableItem):
                                                              eproperty)
 
 
-#
-# CLASS TrajectoryVariableItem
-#
-
-
 class TrajectoryVariableItem(AbstractVariableItem):
     """
     """
@@ -817,11 +771,6 @@ class TrajectoryVariableItem(AbstractVariableItem):
         return self.parentItem.data[self.itemName]
 
 
-#
-# CLASS LagrantoMapItemsTreeModel
-#
-
-
 class LagrantoMapItemsTreeModel(QtCore.QAbstractItemModel):
     """
 
@@ -950,9 +899,8 @@ class LagrantoMapItemsTreeModel(QtCore.QAbstractItemModel):
         # in the data structure. self.lastChange is set to "FLIGHT_TRACK_ADDED"
         # so LagrantoMap can query the kind of change that occured.
         self.lastChange = "FLIGHT_TRACK_ADDED"
-        self.emit(QtCore.SIGNAL('dataChanged(QModelIndex,QModelIndex)'),
-                  index, index)
-        self.emit(QtCore.SIGNAL('layoutChanged()'))
+        self.dataChanged.emit(index, index)
+        self.layoutChanged.emit()
 
         return index
 
@@ -977,9 +925,8 @@ class LagrantoMapItemsTreeModel(QtCore.QAbstractItemModel):
         # in the data structure. self.lastChange is set to "FLIGHT_TRACK_ADDED"
         # so LagrantoMap can query the kind of change that occured.
         self.lastChange = "TRAJECTORY_DIR_ADDED"
-        self.emit(QtCore.SIGNAL('dataChanged(QModelIndex,QModelIndex)'),
-                  index, index)
-        self.emit(QtCore.SIGNAL('layoutChanged()'))
+        self.dataChanged.emit(index, index)
+        self.layoutChanged.emit()
 
         return index
 
@@ -992,9 +939,8 @@ class LagrantoMapItemsTreeModel(QtCore.QAbstractItemModel):
                                                      visible)
             if emit_change:
                 self.lastChange = "VISIBILITY_CHANGE"
-                self.emit(QtCore.SIGNAL('dataChanged(QModelIndex,QModelIndex)'),
-                          index, index)
-                self.emit(QtCore.SIGNAL('layoutChanged()'))
+                self.dataChanged.emit(index, index)
+                self.layoutChanged.emit()
 
     def setItemVisibleInView_list(self, indices, view_window, visible,
                                   emit_change=True):
@@ -1008,15 +954,13 @@ class LagrantoMapItemsTreeModel(QtCore.QAbstractItemModel):
                 indices.remove(index)
         if emit_change:
             self.lastChange = "VISIBILITY_CHANGE"
-            self.emit(QtCore.SIGNAL('dataChanged(QModelIndex,QModelIndex)'),
-                      indices[0], indices[-1])
-            self.emit(QtCore.SIGNAL('layoutChanged()'))
+            self.dataChanged.emit(indices[0], indices[-1])
+            self.layoutChanged.emit()
 
     def emitChange(self, index1, index2, mode="GXPROPERTY_CHANGE"):
         self.lastChange = mode
-        self.emit(QtCore.SIGNAL('dataChanged(QModelIndex,QModelIndex)'),
-                  index1, index2)
-        self.emit(QtCore.SIGNAL('layoutChanged()'))
+        self.dataChanged.emit(index1, index2)
+        self.layoutChanged.emit()
 
     def getLastChange(self):
         """Returns a description of the last change that has been made
@@ -1043,9 +987,8 @@ class LagrantoMapItemsTreeModel(QtCore.QAbstractItemModel):
             item = index.internalPointer()
             item.setGxElementProperty(element, eproperty, value)
             self.lastChange = "GXPROPERTY_CHANGE"
-            self.emit(QtCore.SIGNAL('dataChanged(QModelIndex,QModelIndex)'),
-                      index, index)
-            self.emit(QtCore.SIGNAL('layoutChanged()'))
+            self.dataChanged.emit(index, index)
+            self.layoutChanged.emit()
 
     def changeItemGxProperty_list(self, indices, element, eproperty, value):
         """Change a graphics property of the items given by the list indices.
@@ -1061,9 +1004,8 @@ class LagrantoMapItemsTreeModel(QtCore.QAbstractItemModel):
                 indices.remove(index)
         if len(indices) > 0:
             self.lastChange = "GXPROPERTY_CHANGE"
-            self.emit(QtCore.SIGNAL('dataChanged(QModelIndex,QModelIndex)'),
-                      indices[0], indices[-1])
-            self.emit(QtCore.SIGNAL('layoutChanged()'))
+            self.dataChanged.emit(indices[0], indices[-1])
+            self.layoutChanged.emit()
 
     def setTimeMarker(self, index, interval, emit_change=True):
         """Set the time marker interval property of the item given by the
@@ -1078,9 +1020,8 @@ class LagrantoMapItemsTreeModel(QtCore.QAbstractItemModel):
                                       interval)
             if emit_change:
                 self.lastChange = "MARKER_CHANGE"
-                self.emit(QtCore.SIGNAL('dataChanged(QModelIndex,QModelIndex)'),
-                          index, index)
-                self.emit(QtCore.SIGNAL('layoutChanged()'))
+                self.dataChanged.emit(index, index)
+                self.layoutChanged.emit()
 
             return item.getGxElementProperty("general", "timeMarkerInterval")
 
@@ -1133,17 +1074,15 @@ class LagrantoMapItemsTreeModel(QtCore.QAbstractItemModel):
                               if isinstance(child, LagrantoMapItem)])
             #
             # Only flight tracks and trajectories can be queried and selected.
-            if isinstance(item, (FlightTrackItem, TrajectoryItem)):
-                # Check if the item matches the query.
-                if item.query(queryTranslation):
-                    #
-                    # The properties of the current item fulfil the query and
-                    # the item should be selected. Hence create an index.
-                    index = self.createIndex(item.row(), 0, item)
-                    #
-                    # QtGui.QItemSelectionRange.__init__ (self,
-                    #     QModelIndex index)
-                    itemSelectionRange = QtGui.QItemSelectionRange(index)
-                    itemSelection.append(itemSelectionRange)
+            # Check if the item matches the query.
+            if isinstance(item, (FlightTrackItem, TrajectoryItem)) and item.query(queryTranslation):
+                # The properties of the current item fulfil the query and
+                # the item should be selected. Hence create an index.
+                index = self.createIndex(item.row(), 0, item)
+                #
+                # QtGui.QItemSelectionRange.__init__ (self,
+                #     QModelIndex index)
+                itemSelectionRange = QtGui.QItemSelectionRange(index)
+                itemSelection.append(itemSelectionRange)
 
         return itemSelection
