@@ -1,17 +1,37 @@
-# -*- coding: iso-8859-1 -*-
+# -*- coding: utf-8 -*-
 """
-    MSS - some common code for testing
 
-    @copyright: 2017 Reimar Bauer,
+    mslib.mswms._tests.test_dataaccess
+    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 
-    @license: License: Apache-2.0, see LICENSE.txt for details.
+    This module provides pytest functions to tests mswms.dataaccess
+
+    This file is part of mss.
+
+    :copyright: Copyright 2008-2014 Deutsches Zentrum fuer Luft- und Raumfahrt e.V.
+    :copyright: Copyright 2017 Reimar Bauer
+    :copyright: Copyright 2016-2017 by the mss team, see AUTHORS.
+    :license: APACHE-2.0, see LICENSE for details.
+
+    Licensed under the Apache License, Version 2.0 (the "License");
+    you may not use this file except in compliance with the License.
+    You may obtain a copy of the License at
+
+       http://www.apache.org/licenses/LICENSE-2.0
+
+    Unless required by applicable law or agreed to in writing, software
+    distributed under the License is distributed on an "AS IS" BASIS,
+    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+    See the License for the specific language governing permissions and
+    limitations under the License.
 """
+
 import os
 import pytest
 from datetime import datetime
 from mslib.mswms import dataaccess
 from mslib.mswms.dataaccess import ECMWFDataAccess
-from conftest import BASE_DIR, DATA_DIR, VALID_TIME_CACHE
+from mslib._tests.utils import BASE_DIR, DATA_DIR, VALID_TIME_CACHE
 
 # ToDo improve
 dataaccess.valid_time_cache = VALID_TIME_CACHE
@@ -98,6 +118,7 @@ class Test_NWPDataAccess(object):
                  u'Q': u'20121017_12_ecmwf_forecast.Q.EUR_LL015.036.ml.nc',
                  u'SFC': u'20121017_12_ecmwf_forecast.SFC.EUR_LL015.036.sfc.nc',
                  u'T': u'20121017_12_ecmwf_forecast.T.EUR_LL015.036.ml.nc',
+                 u'THETA_LEVELS': '20121017_12_ecmwf_forecast.THETA_LEVELS.EUR_LL015.036.tl.nc',
                  u'U': u'20121017_12_ecmwf_forecast.U.EUR_LL015.036.ml.nc',
                  u'V': u'20121017_12_ecmwf_forecast.V.EUR_LL015.036.ml.nc',
                  u'W': u'20121017_12_ecmwf_forecast.W.EUR_LL015.036.ml.nc'
@@ -121,18 +142,18 @@ class Test_NWPDataAccess(object):
                                datetime(2012, 10, 19, 0, 0)]
 
     def test_get_all_valid_times(self):
-        # Todo why not sorted
         all_valid_times = self.ECMWFDataAccess.get_all_valid_times("air_pressure", "ml")
-        assert all_valid_times == [datetime(2012, 10, 17, 15, 0),
-                                   datetime(2012, 10, 17, 21, 0),
-                                   datetime(2012, 10, 18, 21, 0),
-                                   datetime(2012, 10, 18, 18, 0),
-                                   datetime(2012, 10, 18, 3, 0),
-                                   datetime(2012, 10, 18, 15, 0),
-                                   datetime(2012, 10, 18, 0, 0),
-                                   datetime(2012, 10, 17, 12, 0),
-                                   datetime(2012, 10, 18, 6, 0),
-                                   datetime(2012, 10, 17, 18, 0),
-                                   datetime(2012, 10, 18, 12, 0),
-                                   datetime(2012, 10, 18, 9, 0),
-                                   datetime(2012, 10, 19, 0, 0)]
+        assert sorted(all_valid_times) == \
+            sorted([datetime(2012, 10, 17, 15, 0),
+                    datetime(2012, 10, 17, 21, 0),
+                    datetime(2012, 10, 18, 21, 0),
+                    datetime(2012, 10, 18, 18, 0),
+                    datetime(2012, 10, 18, 3, 0),
+                    datetime(2012, 10, 18, 15, 0),
+                    datetime(2012, 10, 18, 0, 0),
+                    datetime(2012, 10, 17, 12, 0),
+                    datetime(2012, 10, 18, 6, 0),
+                    datetime(2012, 10, 17, 18, 0),
+                    datetime(2012, 10, 18, 12, 0),
+                    datetime(2012, 10, 18, 9, 0),
+                    datetime(2012, 10, 19, 0, 0)])
