@@ -1,6 +1,36 @@
+# -*- coding: utf-8 -*-
+"""
+
+    mslib.mswms.dataaccess
+    ~~~~~~~~~~~~~~~~~~~~~~
+
+    This module provides functions to access data
+
+    This file is part of mss.
+
+    :copyright: Copyright 2008-2014 Deutsches Zentrum fuer Luft- und Raumfahrt e.V.
+    :copyright: Copyright 2011-2014 Marc Rautenhaus (mr)
+    :copyright: Copyright 2016-2017 by the mss team, see AUTHORS.
+    :license: APACHE-2.0, see LICENSE for details.
+
+    Licensed under the Apache License, Version 2.0 (the "License");
+    you may not use this file except in compliance with the License.
+    You may obtain a copy of the License at
+
+       http://www.apache.org/licenses/LICENSE-2.0
+
+    Unless required by applicable law or agreed to in writing, software
+    distributed under the License is distributed on an "AS IS" BASIS,
+    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+    See the License for the specific language governing permissions and
+    limitations under the License.
+"""
+
 from abc import ABCMeta, abstractmethod
+import glob
 import re
 import os
+import logging
 import netCDF4
 from datetime import datetime, timedelta
 import time
@@ -8,6 +38,7 @@ import hashlib
 import pickle
 
 from mslib import netCDF4tools
+from mslib.mswms.msschem import MSSChemTargets
 
 valid_time_cache = None
 # Maximum size of the cache in bytes.
@@ -95,9 +126,13 @@ class NWPDataAccess(object):
         valid_times = None
         filename = os.path.join(valid_time_cache, self.md5_filename(filename))
         if os.path.exists(filename):
-            fileobj = open(filename, "r")
-            valid_times = pickle.load(fileobj)
-            fileobj.close()
+            try:
+                with open(filename, "r") as fileobj:
+                    valid_times = pickle.load(fileobj)
+            except (pickle.UnpicklingError, OSError, IOError), ex:
+                logging.error(u"Error reading cache file '{}': {} - {}".format(filename, type(ex), ex))
+                logging.error(u"os.stat: {}".format(os.stat(filename)))
+
         return valid_times
 
     def save_valid_cache(self, filename, valid_times):
@@ -107,9 +142,11 @@ class NWPDataAccess(object):
             os.makedirs(valid_time_cache)
         filename = os.path.join(valid_time_cache, self.md5_filename(filename))
         if not os.path.exists(filename):
-            fileobj = open(filename, "w")
-            pickle.dump(valid_times, fileobj)
-            fileobj.close()
+            try:
+                with open(filename, "w") as fileobj:
+                    pickle.dump(valid_times, fileobj)
+            except (pickle.PicklingError, OSError, IOError), ex:
+                logging.error(u"Error writing cache file '{}': {} - {}".format(filename, type(ex), ex))
 
     def serviceCache(self):
         """Service the cache: Remove all files older than the maximum file
@@ -119,8 +156,7 @@ class NWPDataAccess(object):
         # logging.debug("servicing cache..")
 
         # Create a list of all files in the cache.
-        files = [os.path.join(valid_time_cache, f)
-                 for f in os.listdir(valid_time_cache)]
+        files = [os.path.join(valid_time_cache, f) for f in os.listdir(valid_time_cache)]
         # Add the ages of the files (via modification times in sec since epoch)
         # and the file sizes.
         # (current time in sec since epoch)
@@ -137,14 +173,17 @@ class NWPDataAccess(object):
         # file age will also be removed.
         cum_size_bytes = 0
         removed_files = 0
-        for f, fsize, fage in files:
-            cum_size_bytes += fsize
-            if (cum_size_bytes > valid_time_cache_max_size_bytes) or \
-                    fage > valid_time_cache_max_age_seconds:
-                os.remove(f)
-                removed_files += 1
-
-                # logging.debug("cache has been cleaned (%i files removed)." % removed_files)
+        for filename, filesize, fileage in files:
+            cum_size_bytes += filesize
+            if (cum_size_bytes > valid_time_cache_max_size_bytes) or fileage > valid_time_cache_max_age_seconds:
+                try:
+                    os.remove(filename)
+                except (OSError, IOError), ex:
+                    logging.error(u"Could not remove {:}: {:} - {:}".format(filename, type(ex), ex))
+                    logging.error(u"os.stat: {}".format(os.stat(filename)))
+                else:
+                    removed_files += 1
+        logging.info(u"cache has been cleaned ({:d} files removed).".format(removed_files))
 
     _mfDatasetArgsDict = {}
 
@@ -164,8 +203,8 @@ class ECMWFDataAccess(NWPDataAccess):
 
     Constructor needs information on domain ID.
     """
-    _file_template = "$Y$m$d_$H_ecmwf_forecast.%s.%s.%03i.%s.nc"
-    _file_regexp = "(\d{8})_(\d{2})_ecmwf_forecast\.(.*)\.%s\.(\d{3}).*\.nc$"
+    _file_template = "$Y$m$d_$H_ecmwf_forecast.{gridtype}.{domain_id}.{fc_step:03d}.{vartype}.nc"
+    _file_regexp = "(?P<date>\d{8})_(?P<time>\d{2})_ecmwf_forecast\.(?P<vartype>.*)\.%s\.(?P<step>\d{3}).*\.nc$"
     _forecast_times = [36, 72, 144, 240]
     _data_organisation_table = {
         "low_cloud_area_fraction": {"sfc": "SFC"},
@@ -180,9 +219,9 @@ class ECMWFDataAccess(NWPDataAccess):
         "surface_geopotential": {"sfc": "SFC"},
         "sea_ice_area_fraction": {"sfc": "SFC"},
         "atmosphere_boundary_layer_thickness": {"sfc": "SFC"},
-        # "": {"": ""},
         "solar_elevation_angle": {"sfc": "SEA"},
         "air_pressure": {"ml": "P_derived",
+                         "tl": "THETA_LEVELS",
                          "pv": "PVU"},
         "geopotential_height": {"ml": "Z_derived",
                                 "pl": "PRESSURE_LEVELS",
@@ -190,14 +229,22 @@ class ECMWFDataAccess(NWPDataAccess):
         "air_temperature": {"ml": "T",
                             "pl": "PRESSURE_LEVELS"},
         "air_potential_temperature": {"ml": "PT_derived",
+                                      "pl": "PRESSURE_LEVELS",
                                       "pv": "PVU"},
-        "ertel_potential_vorticity": {"ml": "PV_derived"},
+        "ertel_potential_vorticity": {"ml": "PV_derived",
+                                      "pl": "PRESSURE_LEVELS",
+                                      "tl": "THETA_LEVELS"},
+        "equivalent_latitude": {"pl": "PRESSURE_LEVELS"},
         "specific_cloud_liquid_water_content": {"ml": "CLWC"},
         "specific_cloud_ice_water_content": {"ml": "CIWC"},
         "specific_humidity": {"ml": "Q",
                               "pl": "PRESSURE_LEVELS"},
         "mass_fraction_of_ozone_in_air": {"ml": "O3",
                                           "pl": "PRESSURE_LEVELS"},
+        "mole_fraction_of_ozone_in_air": {"ml": "O3",
+                                          "pl": "PRESSURE_LEVELS",
+                                          "tl": "THETA_LEVELS"},
+        "msg_brightness_temperature_108": {"sfc": "SFC"},
         "cloud_area_fraction_in_atmosphere_layer": {"ml": "CC"},
         "eastward_wind": {"ml": "U",
                           "pl": "PRESSURE_LEVELS"},
@@ -228,7 +275,7 @@ class ECMWFDataAccess(NWPDataAccess):
            the variable <variable> of the forecast specified by
            init_time and valid_time.
         """
-        variable_dict = self._data_organisation_table[variable]
+        variable_dict = self._data_organisation_table.get(variable, None)
 
         # Compute the time step in hours from the forecast valid time
         # and the initialisation time.
@@ -244,24 +291,26 @@ class ECMWFDataAccess(NWPDataAccess):
                 break
 
         # Substitute variable identifiers in the template filename.
-        if vartype not in variable_dict.keys():
-            raise ValueError("variable type %s not available for variable %s"
-                             % (vartype, variable))
+        if variable_dict is not None and vartype not in variable_dict.keys():
+            raise ValueError(u"variable type {} not available for variable {}"
+                             .format(vartype, variable))
 
-        if type(variable_dict[vartype]) == str:
+        if variable_dict is None:
+            name = self._file_template.format(
+                gridtype=vartype, domain_id=self._domain_id,
+                fc_step=fc_step, vartype=vartype)
+        elif isinstance(variable_dict[vartype], basestring):
             # The string stored for the given variable type contains the
             # variable identifier in the filename.
-            name = self._file_template % (variable_dict[vartype],
-                                          self._domain_id,
-                                          fc_step,
-                                          vartype)
-        elif type(variable_dict[vartype]) == tuple:
+            name = self._file_template.format(
+                gridtype=variable_dict[vartype], domain_id=self._domain_id,
+                fc_step=fc_step, vartype=vartype)
+        elif isinstance(variable_dict[vartype], tuple):
             # The tuple stored for the given variable type contains the
             # variable identifier and an override level identifier.
-            name = self._file_template % (variable_dict[vartype][0],
-                                          self._domain_id,
-                                          fc_step,
-                                          variable_dict[vartype][1])
+            name = self._file_template.format(
+                gridtype=variable_dict[vartype][0], domain_id=self._domain_id,
+                fc_step=fc_step, vartype=variable_dict[vartype][1])
 
         # Substitute init time and time step interval identifiers.
         name = name.replace('$', '%')
@@ -288,12 +337,11 @@ class ECMWFDataAccess(NWPDataAccess):
                 # Extract information from the filename.
                 # date = int(m.group(1))
                 # time = int(m.group(2))
-                var = m.group(3)
-                step = int(m.group(4))
+                var = m.group("vartype")
+                step = int(m.group("step"))
                 # dtime = int(m.group(1)+m.group(2))
-                dtime = m.group(1) + m.group(2)
+                dtime = m.group("date") + m.group("time")
                 dtime = datetime.strptime(dtime, "%Y%m%d%H")
-                # print date, time, step, datetime, var
 
                 # Insert the filename into the tree.
                 if dtime not in filetree.keys():
@@ -326,11 +374,9 @@ class ECMWFDataAccess(NWPDataAccess):
             filename = self.get_filename(variable, vartype,
                                          init_time, check_time,
                                          fullpath=True)
-            # print "checking file %s" % filename
             if os.path.exists(filename):
                 # If the file exists, open the file and read the contained
                 # times. Add the list of times to valid_times.
-                # print "exists."
                 cached_valid_times = self.check_valid_cache(filename)
                 if cached_valid_times is not None:
                     valid_times.extend(cached_valid_times)
@@ -353,8 +399,7 @@ class ECMWFDataAccess(NWPDataAccess):
         for init_time in filetree.keys():
             vtimes = self.get_valid_times(variable, vartype, init_time)
             valid_times.update(vtimes)
-        # valid_times.sort()
-        return list(valid_times)
+        return sorted(list(valid_times))
 
     def get_all_datafiles(self):
         """Return a list of all available data files.
@@ -362,288 +407,50 @@ class ECMWFDataAccess(NWPDataAccess):
         return os.listdir(self._root_path)
 
 
-class CLAMSDataAccess(NWPDataAccess):
-    """Subclass to NWPDataAccess for accessing CLAMS data.
-
-    Constructor needs information on domain ID.
+class CLAMSDataAccess(ECMWFDataAccess):
+    """Subclass to ECMWFDataAccess for accessing CLAMS data.
     """
-    _file_template = "$Y$m$d_$H_clams_forecast.%s.%03i.%s.nc"
-    _file_regexp = "(\d{8})_(\d{2})_clams_forecast\.%s\.(\d{3})\.(.*)\.nc$"
+    _file_template = "$Y$m$d_$H_clams_forecast.{domain_id}.{fc_step:03d}.{vartype}.nc"
+    _file_regexp = "(?P<date>\d{8})_(?P<time>\d{2})_clams_forecast\.%s\.(?P<step>\d{3})\.(?P<vartype>.*)\.nc$"
     _forecast_times = [144]
+    _data_organisation_table = {}
+
     # Workaround for the numerical issue concering the lon dimension in
     # NetCDF files produced by netcdf-java 4.3..
     _mfDatasetArgsDict = {"skipDimCheck": ["lon"]}
 
-    def __init__(self, rootpath, domain_id):
-        NWPDataAccess.__init__(self, rootpath)
-        self._domain_id = domain_id
-        # Compile regular expression to match filenames.
-        self._filename_re = re.compile(self._file_regexp % domain_id)
 
-    def _determine_filename(self, variable, vartype, init_time, valid_time):
-        """Determines the name of the CLAMS data file the contains
-           the variable <variable> of the forecast specified by
-           init_time and valid_time.
-        """
-
-        # Compute the time step in hours from the forecast valid time
-        # and the initialisation time.
-        fc_step = valid_time - init_time
-        fc_step = fc_step.days * 24 + fc_step.seconds / 3600
-
-        # CLAMS forecasts are stored in a series of files containing
-        # different time steps. Determine into which time step interval
-        # the requested valid_time falls.
-        for t in self._forecast_times:
-            if t >= fc_step:
-                fc_step = t
-                break
-
-        name = self._file_template % (
-            self._domain_id,
-            fc_step,
-            vartype)
-
-        # Substitute init time and time step interval identifiers.
-        name = name.replace('$', '%')
-        name = datetime.strftime(init_time, name)
-        return name
-
-    def build_filetree(self):
-        """Build a tree structure with information on the available
-           forecast times and variables.
-
-        The first index of 'filetree' is the forecast date/time, the second the
-        timestep, the third the variable. The names of all available files are
-        parsed corresponding to the above specified regular expression and
-        inserted into the tree.
-        """
-        # Get a list of the available data files.
-        available_files = os.listdir(self._root_path)
-        # Build the tree structure.
-        filetree = {}
-        for filename in available_files:
-            m = self._filename_re.match(filename)
-            if m:
-                # Extract information from the filename.
-                step = int(m.group(3))
-                # dtime = int(m.group(1)+m.group(2))
-                dtime = m.group(1) + m.group(2)
-                dtime = datetime.strptime(dtime, "%Y%m%d%H")
-
-                # print date, time, step, datetime, var
-
-                # Insert the filename into the tree.
-                if dtime not in filetree.keys():
-                    filetree[dtime] = {}
-                if step not in filetree[dtime].keys():
-                    filetree[dtime][step] = {}
-                filetree[dtime][step][m.group(4)] = filename
-
-        return filetree
-
-    def get_init_times(self):
-        """Returns a list of available forecast init times (base times).
-        """
-        filetree = self.build_filetree()
-        init_times = filetree.keys()
-        init_times.sort()
-        return init_times
-
-    def get_valid_times(self, variable, vartype, init_time):
-        """Returns a list of available valid times for the specified
-           variable at the specified init time.
-        """
-        valid_times = []
-        for t in self._forecast_times:
-            # Open each forecast file belonging to this variable. To do this,
-            # query get_filename() with a time one hour less than the upper
-            # limits of the individual files.
-            delta = timedelta(seconds=3600 * (t - 1))
-            check_time = init_time + delta
-            filename = self.get_filename(variable, vartype,
-                                         init_time, check_time,
-                                         fullpath=True)
-            # print "checking file %s" % filename
-            if os.path.exists(filename):
-                # If the file exists, open the file and read the contained
-                # times. Add the list of times to valid_times.
-                # print "exists."
-                cached_valid_times = self.check_valid_cache(filename)
-                if cached_valid_times is not None:
-                    valid_times.extend(cached_valid_times)
-                else:
-                    dataset = netCDF4.Dataset(filename)
-                    timename, timevar = netCDF4tools.identify_CF_time(dataset)
-                    times = netCDF4tools.num2date(timevar[:], timevar.units)
-                    valid_times.extend(times)
-                    dataset.close()
-                    self.save_valid_cache(filename, times)
-
-        return valid_times
-
-    def get_all_valid_times(self, variable, vartype):
-        """Similar to get_valid_times(), but returns the combined valid times
-           of all available init times.
-        """
-        valid_times = set()
-        filetree = self.build_filetree()
-        for init_time in filetree.keys():
-            vtimes = self.get_valid_times(variable, vartype, init_time)
-            valid_times.update(vtimes)
-        # valid_times.sort()
-        return list(valid_times)
-
-    def get_all_datafiles(self):
-        """Return a list of all available data files.
-        """
-        return os.listdir(self._root_path)
-
-
-class GWFCDataAccess(NWPDataAccess):
-    """Subclass to NWPDataAccess for accessing CLAMS data.
-
-    Constructor needs information on domain ID.
+class CLAMSICEDataAccess(ECMWFDataAccess):
+    """Subclass to ECMWFDataAccess for accessing CLAMS data.
     """
-    _file_template = "$Y$m$d_$H_gravity_wave_forecast.%s.%s.%03i.%s.nc"
-    _file_regexp = "(\d{8})_(\d{2})_gravity_wave_forecast\.(.*)\.%s\.(\d{3}).*\.nc$"
+    _file_template = "$Y$m$d_$H_clamsice_forecast.{domain_id}.{fc_step:03d}.{vartype}.nc"
+    _file_regexp = "(?P<date>\d{8})_(?P<time>\d{2})_clamsice_forecast\.%s\.(?P<step>\d{3})\.(?P<vartype>.*)\.nc$"
+    _forecast_times = [144]
+    _data_organisation_table = {}
+
+    # Workaround for the numerical issue concering the lon dimension in
+    # NetCDF files produced by netcdf-java 4.3..
+    _mfDatasetArgsDict = {"skipDimCheck": ["lon"]}
+
+
+class GWFCDataAccess(ECMWFDataAccess):
+    """Subclass to ECMWFDataAccess for accessing gravity wave forecast and related data.
+    """
+    _file_template = "$Y$m$d_$H_gravity_wave_forecast.{gridtype}.{domain_id}.{fc_step:03d}.{vartype}.nc"
+    _file_regexp = "(?P<date>\d{8})_(?P<time>\d{2})_gravity_wave_forecast\.(?P<vartype>.*)\.%s\.(?P<step>\d{3}).*\.nc$"
     _forecast_times = range(0, 150, 6)
     _data_organisation_table = {
         "gravity_wave_temperature_perturbation": {"ml": "ALTITUDE_LEVELS"},
         "air_pressure": {"ml": "ALTITUDE_LEVELS"},
-        "tropopause_altitude": {"sfc": "ALTITUDE_LEVELS"},
+        "air_potential_temperature": {"ml": "ALTITUDE_LEVELS"},
+        "brunt_vaisala_frequency_in_air": {"ml": "ALTITUDE_LEVELS"},
+        "square_of_brunt_vaisala_frequency_in_air": {"ml": "ALTITUDE_LEVELS"},
+        "tropopause_altitude": {"sfc": "SFC"},
+        "tropopause_air_pressure": {"sfc": "SFC"},
+        "max_of_square_of_brunt_vaisala_frequency_above_tropopause_in_air": {"sfc": "SFC"},
+        "mean_of_square_of_brunt_vaisala_frequency_above_tropopause_in_air": {"sfc": "SFC"},
     }
 
-    # Workaround for the numerical issue concering the lon dimension in
-    # NetCDF files produced by netcdf-java 4.3..
-    _mfDatasetArgsDict = {"skipDimCheck": ["lon"]}
-
-    def __init__(self, rootpath, domain_id):
-        NWPDataAccess.__init__(self, rootpath)
-        self._domain_id = domain_id
-        # Compile regular expression to match filenames.
-        self._filename_re = re.compile(self._file_regexp % domain_id)
-
-    def _determine_filename(self, variable, vartype, init_time, valid_time):
-        """Determines the name of the CLAMS data file the contains
-           the variable <variable> of the forecast specified by
-           init_time and valid_time.
-        """
-        variable_dict = self._data_organisation_table[variable]
-
-        # Compute the time step in hours from the forecast valid time
-        # and the initialisation time.
-        fc_step = valid_time - init_time
-        fc_step = fc_step.days * 24 + fc_step.seconds / 3600
-
-        # CLAMS forecasts are stored in a series of files containing
-        # different time steps. Determine into which time step interval
-        # the requested valid_time falls.
-        for t in self._forecast_times:
-            if t >= fc_step:
-                fc_step = t
-                break
-
-        # Substitute variable identifiers in the template filename.
-        if vartype not in variable_dict.keys():
-            raise ValueError("variable type %s not available for variable %s"
-                             % (vartype, variable))
-
-        if type(variable_dict[vartype]) == str:
-            # The string stored for the given variable type contains the
-            # variable identifier in the filename.
-            name = self._file_template % (variable_dict[vartype],
-                                          self._domain_id,
-                                          fc_step,
-                                          "ml")  # vartype)
-        elif type(variable_dict[vartype]) == tuple:
-            # The tuple stored for the given variable type contains the
-            # variable identifier and an override level identifier.
-            name = self._file_template % (variable_dict[vartype][0],
-                                          self._domain_id,
-                                          fc_step,
-                                          "ml")  # variable_dict[vartype][1])
-
-        # Substitute init time and time step interval identifiers.
-        name = name.replace('$', '%')
-        name = datetime.strftime(init_time, name)
-        return name
-
-    def build_filetree(self):
-        """Build a tree structure with information on the available
-           forecast times and variables.
-
-        The first index of 'filetree' is the forecast date/time, the second the
-        timestep, the third the variable. The names of all available files are
-        parsed corresponding to the above specified regular expression and
-        inserted into the tree.
-        """
-        # Get a list of the available data files.
-        available_files = os.listdir(self._root_path)
-
-        # Build the tree structure.
-        filetree = {}
-        for filename in available_files:
-            m = self._filename_re.match(filename)
-            if m:
-                # Extract information from the filename.
-                var = m.group(3)
-                step = int(m.group(4))
-                dtime = m.group(1) + m.group(2)
-                dtime = datetime.strptime(dtime, "%Y%m%d%H")
-
-                # Insert the filename into the tree.
-                if dtime not in filetree.keys():
-                    filetree[dtime] = {}
-                if step not in filetree[dtime].keys():
-                    filetree[dtime][step] = {}
-                filetree[dtime][step][var] = filename
-
-        return filetree
-
-    def get_init_times(self):
-        """Returns a list of available forecast init times (base times).
-        """
-        filetree = self.build_filetree()
-        init_times = filetree.keys()
-        init_times.sort()
-        return init_times
-
-    def get_valid_times(self, variable, vartype, init_time):
-        """Returns a list of available valid times for the specified
-           variable at the specified init time.
-        """
-        valid_times = []
-        for t in self._forecast_times:
-            # Open each forecast file belonging to this variable. To do this,
-            # query get_filename() with a time one hour less than the upper
-            # limits of the individual files.
-            delta = timedelta(seconds=3600 * t)
-            check_time = init_time + delta
-            filename = self.get_filename(variable, vartype,
-                                         init_time, check_time,
-                                         fullpath=True)
-            if os.path.exists(filename):
-                valid_times.append(check_time)
-        return valid_times
-
-    def get_all_valid_times(self, variable, vartype):
-        """Similar to get_valid_times(), but returns the combined valid times
-           of all available init times.
-        """
-        valid_times = set()
-        filetree = self.build_filetree()
-        for init_time in filetree.keys():
-            vtimes = self.get_valid_times(variable, vartype, init_time)
-            valid_times.update(vtimes)
-        # valid_times.sort()
-        return list(valid_times)
-
-    def get_all_datafiles(self):
-        """Return a list of all available data files.
-        """
-        return os.listdir(self._root_path)
-
 
 class EMACDataAccess(NWPDataAccess):
     """Subclass to NWPDataAccess for accessing EMAC datasets.
@@ -707,8 +514,8 @@ class EMACDataAccess(NWPDataAccess):
 
         # Substitute variable identifiers in the template filename.
         if vartype not in variable_dict.keys():
-            raise ValueError("variable type %s not available for variable %s"
-                             % (vartype, variable))
+            raise ValueError(u"variable type {} not available for variable {}"
+                             .format(vartype, variable))
 
         name = self._file_template % variable_dict[vartype]
         if "replace" in variable_dict.keys():
@@ -857,7 +664,6 @@ class MeteosatDataAccess(NWPDataAccess):
 
                 # If the file exists, open the file and read the contained
                 # times. Add the list of times to valid_times.
-                # print "exists."
                 cached_valid_times = self.check_valid_cache(filename)
                 if cached_valid_times is not None:
                     valid_times.extend(cached_valid_times)
@@ -881,3 +687,158 @@ class MeteosatDataAccess(NWPDataAccess):
         """Return a list of all available data files.
         """
         return os.listdir(self._root_path)
+
+
+class MSSChemDataAccess(NWPDataAccess):
+    """Subclass to NWPDataAccess for accessing CTM data prepared by MSS-Chem
+
+    """
+    _file_template = "%s_$Y-$m-$d_%s_%s_%s.nc"
+    _file_regexp = "(?P<model>.*)_(?P<year>\d{4})-(?P<month>\d{2})-(?P<day>\d{2})_(?P<hour>\d{2})_(?P<vartype>.*)_(?P<variable>.*)\.nc$"
+
+    def __init__(self, rootpath, modelname, modelstr, vert):
+        NWPDataAccess.__init__(self, rootpath)
+        # Compile regular expression to match filenames.
+        self._filename_re = re.compile(self._file_regexp)
+        self._modelname = modelname
+        self._modelstr = modelstr
+        self._create_data_table(vert)
+
+    def _create_data_table(self, vert):
+        self._data_organisation_table = {
+            stdname: {vert: shortname.lower()}
+            for stdname, (shortname, _, _, _) in MSSChemTargets.items()}
+        self._data_organisation_table["air_pressure"] = {vert: "p"}
+
+    def _determine_filename(self, variable, vartype, init_time, valid_time):
+        """Determines the name of the CAMS global data file that contains
+           the variable <variable> of the forecast specified by
+           init_time and valid_time.
+
+        This can be simpler than usual, because MSS-Chem will prepare the data
+        such that there is exactly one file per variable, vartype, and
+        init_time, which contains all valid_times.
+
+        """
+        variable_dict = self._data_organisation_table[variable]
+
+        # Substitute variable identifiers in the template filename.
+        if vartype not in variable_dict.keys():
+            raise ValueError("variable type %s not available for variable %s"
+                             % (vartype, variable))
+
+        if type(variable_dict[vartype]) == str:
+            # The string stored for the given variable type contains the
+            # variable identifier in the filename.
+            name = self._file_template % (self._modelstr,
+                                          "{:%H}".format(init_time), vartype,
+                                          variable_dict[vartype])
+
+        # Substitute init time and time step interval identifiers.
+        name = name.replace('$', '%')
+        name = datetime.strftime(init_time, name)
+        return name
+
+    def build_filetree(self):
+        """Build a tree structure with information on the available
+           forecast times and variables.
+
+        The first index of 'filetree' is the forecast date/time, the second the
+        timestep, the third the variable. The names of all available files are
+        parsed corresponding to the above specified regular expression and
+        inserted into the tree.
+        """
+        # Get a list of the available data files.
+        available_files = self.get_all_datafiles()
+
+        # Build the tree structure.
+        filetree = {}
+        for filename in available_files:
+            m = self._filename_re.match(filename)
+            if m:
+                # Extract information from the filename.
+                var = m.group("variable")
+                step = 0
+                initime = datetime(int(m.group("year")), int(m.group("month")),
+                                   int(m.group("day")), int(m.group("hour")))
+
+                # Insert the filename into the tree.
+                if initime not in filetree.keys():
+                    filetree[initime] = {}
+                if step not in filetree[initime].keys():
+                    filetree[initime][step] = {}
+                filetree[initime][step][var] = filename
+
+        return filetree
+
+    def get_init_times(self):
+        """Returns a list of available forecast init times (base times).
+        """
+        filetree = self.build_filetree()
+        init_times = filetree.keys()
+        init_times.sort()
+        return init_times
+
+    def get_valid_times(self, variable, vartype, init_time):
+        """Returns a list of available valid times for the specified
+           variable at the specified init time.
+        """
+        valid_times = []
+        # Open the forecast file belonging to this variable.
+        filename = self.get_filename(variable, vartype, init_time, None,
+                                     fullpath=True)
+        if os.path.exists(filename):
+            # If the file exists, open the file and read the contained
+            # times. Add the list of times to valid_times.
+            cached_valid_times = self.check_valid_cache(filename)
+            if cached_valid_times is not None:
+                valid_times.extend(cached_valid_times)
+            else:
+                dataset = netCDF4.Dataset(filename)
+                timename, timevar = netCDF4tools.identify_CF_time(dataset)
+                times = netCDF4tools.num2date(timevar[:], timevar.units)
+                valid_times.extend(times)
+                dataset.close()
+                self.save_valid_cache(filename, times)
+
+        return valid_times
+
+    def get_all_valid_times(self, variable, vartype):
+        """Similar to get_valid_times(), but returns the combined valid times
+           of all available init times.
+        """
+        valid_times = set()
+        filetree = self.build_filetree()
+        for init_time in filetree.keys():
+            vtimes = self.get_valid_times(variable, vartype, init_time)
+            valid_times.update(vtimes)
+        return sorted(list(valid_times))
+
+    def get_all_datafiles(self):
+        """Return a list of all available data files.
+        """
+        return [os.path.basename(fn) for fn
+                in glob.glob(os.path.join(self._root_path, "*", "*.nc"))]
+
+    def get_filename(self, variable, vartype, init_time, valid_time,
+                     fullpath=False):
+        """Get the filename of the file in which a given variable at
+           a given time can be found.
+
+        Arguments:
+        variable -- string with CF name of variable
+        vartype -- string specifying the type of the variable (model specific).
+                   For example, can be ml (model level), pl (pressure level),
+                   or sfc (surface) for ECMWF data.
+        init_time -- datetime object with initialisation time of forecast run
+        valid_time -- datetime object with valid time of forecast
+        fullpath -- if True, the complete path to the file will be returned.
+                    Default is False, only the filename will be returned.
+        """
+        filename = self._determine_filename(variable, vartype,
+                                            init_time, valid_time)
+        if fullpath:
+            return os.path.join(self._root_path,
+                                "{:%Y-%m-%d_%H}".format(init_time), filename)
+        else:
+            return filename
