@@ -1,11 +1,31 @@
+#!/usr/bin/env python
+# -*- coding: utf-8 -*-
 """
-demodata - creates netCDF test data files and also a mss_wms_settings for accessing this data
 
-:copyright: Jens-Uwe Grooss 2016 - 2017,
-            Reimar Bauer 2016 - 2017
+    mslib.mswms.demodata
+    ~~~~~~~~~~~~~~~~~~~~
 
+    creates netCDF test data files and also a mss_wms_settings for accessing this data
+
+    This file is part of mss.
+
+    :copyright: Copyright 2017 Jens-Uwe Grooss, Joern Ungermann, Reimar Bauer
+    :copyright: Copyright 2017 by the mss team, see AUTHORS.
+    :license: APACHE-2.0, see LICENSE for details.
+
+    Licensed under the Apache License, Version 2.0 (the "License");
+    you may not use this file except in compliance with the License.
+    You may obtain a copy of the License at
+
+       http://www.apache.org/licenses/LICENSE-2.0
+
+    Unless required by applicable law or agreed to in writing, software
+    distributed under the License is distributed on an "AS IS" BASIS,
+    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+    See the License for the specific language governing permissions and
+    limitations under the License.
 """
-import logging
+
 import os
 from StringIO import StringIO
 
@@ -38,7 +58,7 @@ m2.s-2
 time,lat,lon
   2.12e+03   4.74e+03
 Surface_pressure_surface
-surface_air_pressure
+air_pressure_at_sea_level
 Pa
 3
 time,lat,lon
@@ -56,23 +76,59 @@ land_binary_mask
 time,lat,lon
   4.20e-01   4.94e-01
 Low_cloud_cover_surface
-low_type_cloud_area_fraction
+low_cloud_area_fraction
 (0.-.1)
 3
 time,lat,lon
   3.67e-01   3.91e-01
 Medium_cloud_cover_surface
-medium_type_cloud_area_fraction
+medium_cloud_area_fraction
 (0.-.1)
 3
 time,lat,lon
   2.95e-01   3.77e-01
 High_cloud_cover_surface
-high_type_cloud_area_fraction
+high_cloud_area_fraction
 (0.-.1)
 3
 time,lat,lon
   4.07e-01   4.41e-01
+U
+surface_eastward_wind
+m.s-1
+3
+time,lat,lon
+  0 50
+V
+surface_northward_wind
+m.s-1
+3
+time,lat,lon
+  0 50
+SEA
+solar_elevation_angle
+degree
+3
+time,lat,lon
+  45 20
+SIAF
+sea_ice_area_fraction
+(0.-.1)
+3
+time,lat,lon
+  0.5 0.3
+brightness
+msg_brightness_temperature_108
+nW.cm.m-2.sr-1
+3
+time,lat,lon
+  100 20
+atmosphere_boundary_layer_thickness
+atmosphere_boundary_layer_thickness
+m
+3
+time,lat,lon
+  1000 400
 """
         self.forecast_pressure_levels_pl = """\
 Potential_vorticity_isobaric
@@ -128,6 +184,30 @@ air_temperature
 K
 4
 time,isobaric,lat,lon
+18
+  2.15e+02   3.79e+00
+  2.13e+02   2.07e+00
+  2.12e+02   3.10e+00
+  2.12e+02   4.82e+00
+  2.12e+02   6.31e+00
+  2.16e+02   5.91e+00
+  2.18e+02   5.03e+00
+  2.22e+02   3.86e+00
+  2.29e+02   5.00e+00
+  2.44e+02   6.20e+00
+  2.55e+02   6.33e+00
+  2.64e+02   6.30e+00
+  2.71e+02   6.62e+00
+  2.77e+02   7.36e+00
+  2.79e+02   7.67e+00
+  2.81e+02   7.91e+00
+  2.82e+02   7.97e+00
+  2.84e+02   7.93e+00
+Potential_temperature_isobaric
+air_potential_temperature
+K
+4
+time,isobaric,lat,lon
 18
   2.15e+02   3.79e+00
   2.13e+02   2.07e+00
@@ -268,7 +348,7 @@ time,isobaric,lat,lon
   4.93e-08   5.71e-05
   8.47e-08   5.94e-05
 Ozone_mass_mixing_ratio_isobaric
-mass_fraction_of_ozone_in_air
+mole_fraction_of_ozone_in_air
 kg.kg-1
 4
 time,isobaric,lat,lon
@@ -291,7 +371,106 @@ time,isobaric,lat,lon
   6.14e-08   4.60e-09
   6.12e-08   4.64e-09
   6.09e-08   4.65e-09
+EQLAT
+equivalent_latitude
+degree
+4
+time,isobaric,lat,lon
+18
+  80.09   1.21
+  50.70   5.69
+  30.46   4.69
+  20.41   5.23
+  10.44   4.97
+  60.27   3.28
+  20.82   2.03
+  10.37   1.08
+  80.30   4.80
+  60.40   1.39
+  60.23   8.08
+  60.18   6.28
+  60.18   5.47
+  60.17   4.73
+  60.15   4.57
+  60.14   4.60
+  60.12   4.64
+  60.09   4.65
 """
+        self.forecast_theta_levels_pl = """\
+        Pressure
+        air_pressure
+        K.m2.kg-1.s-1
+        4
+        time,isentropic,lat,lon
+        18
+          20   5
+          30   5
+          50   5
+          70   5
+          100   10
+          150   10
+          200   10
+          250   10
+          300   10
+          400   10
+          500   10
+          600   10
+          700   10
+          800   10
+          900   10
+          925   10
+          950   10
+          1000  10
+        Potential_vorticity
+        ertel_potential_vorticity
+        K.m2.kg-1.s-1
+        4
+        time,isentropic,lat,lon
+        18
+          1.04e-04   2.25e-05
+          6.14e-05   1.03e-05
+          2.94e-05   4.86e-06
+          1.88e-05   3.51e-06
+          1.15e-05   3.26e-06
+          6.53e-06   2.85e-06
+          4.71e-06   3.21e-06
+          2.78e-06   3.09e-06
+          1.39e-06   2.23e-06
+          5.58e-07   7.26e-07
+          5.39e-07   4.40e-07
+          5.53e-07   3.95e-07
+          5.66e-07   9.73e-07
+          8.11e-07   3.37e-06
+          8.73e-07   3.62e-06
+          8.96e-07   3.81e-06
+          9.05e-07   3.94e-06
+          8.95e-07   4.09e-06
+        Ozone_mass_mixing_ratio
+        mole_fraction_of_ozone_in_air
+        kg.kg-1
+        4
+        time,isentropic,lat,lon
+        18
+          8.09e-06   1.21e-06
+          5.70e-06   5.69e-07
+          3.46e-06   4.69e-07
+          2.41e-06   5.23e-07
+          1.44e-06   4.97e-07
+          6.27e-07   3.28e-07
+          2.82e-07   2.03e-07
+          1.37e-07   1.08e-07
+          8.30e-08   4.80e-08
+          6.40e-08   1.39e-08
+          6.23e-08   8.08e-09
+          6.18e-08   6.28e-09
+          6.18e-08   5.47e-09
+          6.17e-08   4.73e-09
+          6.15e-08   4.57e-09
+          6.14e-08   4.60e-09
+          6.12e-08   4.64e-09
+          6.09e-08   4.65e-09
+        """
+
         self.forecast_p_derived_ml = """\
 Pressure
 air_pressure
@@ -855,6 +1034,7 @@ time,hybrid,lat,lon
         self.data = {"CC_ml": self.forecast_cc_ml,
                      "P_derived_ml": self.forecast_p_derived_ml,
                      "PRESSURE_LEVELS_pl": self.forecast_pressure_levels_pl,
+                     "THETA_LEVELS_tl": self.forecast_theta_levels_pl,
                      "SFC_sfc": self.forecast_sfc,
                      "T_ml": self.forecast_t_ml,
                      "U_ml": self.forecast_u_ml,
@@ -877,9 +1057,11 @@ class DataFiles(object):
     Jens-Uwe Grooss, IEK-7, Forschungszentrum Juelich, Nov 2016
 
     """
-    def __init__(self, data_dir=None, server_config_dir=None):
+    def __init__(self, data_dir=None, vt_cache=None, server_config_dir=None):
         self.data_dir = data_dir
+        self.vt_cache = vt_cache
         self.server_config_file = os.path.join(server_config_dir, "mss_wms_settings.py")
+        self.server_auth_config_file = os.path.join(server_config_dir, "mss_wms_auth.py")
         self.inidate = '20121017_12'
         self.levtype = 'type'
         self.range_data = RangeData().data
@@ -904,12 +1086,90 @@ class DataFiles(object):
             os.makedirs(self.data_dir)
 
     def create_server_config(self, detailed_information=False):
-        if detailed_information:
-            simple_server_config = '''"""
-
-simple server config for demodata
+        simple_auth_config = '''# -*- coding: utf-8 -*-
 """
+
+    mss_wms_settings
+    ~~~~~~~~~~~~~~~~
+
+    Configuration module for programs accessing data on the MSS server.
+
+    This file is part of mss.
+
+    :copyright: 2008-2014 Deutsches Zentrum fuer Luft- und Raumfahrt e.V.
+    :copyright: 2011-2014 Marc Rautenhaus
+    :copyright: Copyright 2017 Jens-Uwe Grooss, Joern Ungermann, Reimar Bauer
+    :copyright: Copyright 2017 by the mss team, see AUTHORS.
+    :license: APACHE-2.0, see LICENSE for details.
+
+    Licensed under the Apache License, Version 2.0 (the "License");
+    you may not use this file except in compliance with the License.
+    You may obtain a copy of the License at
+
+       http://www.apache.org/licenses/LICENSE-2.0
+
+    Unless required by applicable law or agreed to in writing, software
+    distributed under the License is distributed on an "AS IS" BASIS,
+    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+    See the License for the specific language governing permissions and
+    limitations under the License.
+"""
+#
+# HTTP Authentication                               ###
+#
+#
+# Use the following code to create a new md5 digest of a password (e.g. in
+# ipython):
+#     import hashlib; hashlib.md5("my_new_password").hexdigest()
+allowed_users = [("mswms", "add_md5_digest_of_PASSWORD_here"),
+                 ("add_new_user_here", "add_md5_digest_of_PASSWORD_here")]
+
+'''
+        if detailed_information:
+            simple_server_config = '''# -*- coding: utf-8 -*-
+"""
+
+    mss_wms_settings
+    ~~~~~~~~~~~~~~~~
+
+    Configuration module for programs accessing data on the MSS server.
+
+    This file is part of mss.
+
+    :copyright: 2008-2014 Deutsches Zentrum fuer Luft- und Raumfahrt e.V.
+    :copyright: 2011-2014 Marc Rautenhaus
+    :copyright: Copyright 2017 Jens-Uwe Grooss, Joern Ungermann, Reimar Bauer
+    :copyright: Copyright 2017 by the mss team, see AUTHORS.
+    :license: APACHE-2.0, see LICENSE for details.
+
+    Licensed under the Apache License, Version 2.0 (the "License");
+    you may not use this file except in compliance with the License.
+    You may obtain a copy of the License at
+
+       http://www.apache.org/licenses/LICENSE-2.0
+
+    Unless required by applicable law or agreed to in writing, software
+    distributed under the License is distributed on an "AS IS" BASIS,
+    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+    See the License for the specific language governing permissions and
+    limitations under the License.
+"""
+
 import os
+import sys
+
+# Configuration of Python's code search path
+# If you already have set up the PYTHONPATH environment variable for the
+# stuff you see below, you don't need to do a1) and a2).
+
+# a1) Path of the directory where the mss code package is located.
+# sys.path.insert(0, '/home/mss/miniconda2/lib/python2.7/site-packages')
+
+# a2) Path of the directory where mss_wms_settings.py is located
+#MSSCONFIGPATH = os.path.abspath(os.path.normpath(os.path.dirname(sys.argv[0])))
+#sys.path.insert(0, MSSCONFIGPATH)
+#os.chdir(MSSCONFIGPATH)
+
 import mslib.mswms.dataaccess
 from mslib.mswms import mpl_hsec_styles
 from mslib.mswms import mpl_vsec_styles
@@ -935,44 +1195,36 @@ import mslib.mswms
 #service_access_constraints = "This service is intended for research purposes only."
 
 
-# HTTP Authentication                               ###
 #
-
+# HTTP Authentication
+#
 # If you require basic HTTP authentication, set the following variable
 # to True. Add usernames in the list "allowed:users". Note that the
 # passwords are not specified in plain text but by their md5 digest.
 #enable_basic_http_authentication = False
 
-# Use the following code to create a new md5 digest of a password (e.g. in
-# ipython):
-#     import hashlib; hashlib.md5("my_new_password").hexdigest()
-#allowed_users = [("mswms", "add_md5_digest_of_PASSWORD_here"),
-#                 ("add_new_user_here", "add_md5_digest_of_PASSWORD_here")]
 
-
-
-
-# xml_template directory is a sub directory of mswms
+#xml_template directory is a sub directory of mswms
 #base_dir = os.path.abspath(os.path.dirname(mslib.mswms.__file__))
 #xml_template_location = os.path.join(base_dir, "xml_templates")
 
 
-
-_vt_cache = os.path.join(os.path.expanduser("~"), "mss", "vt_cache")
+_vt_cache = r"{vt_cache}"
 mslib.mswms.dataaccess.valid_time_cache = _vt_cache
 
-_datapath = os.path.join(os.path.expanduser("~"), "mss", "testdata")
-nwpaccess = {
-    "ecmwf_EUR_LL015": mslib.mswms.dataaccess.ECMWFDataAccess(_datapath, "EUR_LL015"),
-}
+_datapath = r"{data_dir}"
 
-epsg_to_mpl_basemap_table = {
+nwpaccess = {{
+    "ecmwf_EUR_LL015": mslib.mswms.dataaccess.ECMWFDataAccess(_datapath, "EUR_LL015"),
+}}
+
+epsg_to_mpl_basemap_table = {{
     # EPSG:4326, the standard cylindrical lat/lon projection.
-    4326: {"projection": "cyl"},
-    77790000: {"projection": "stere", "lat_0": 90., "lon_0": 0.}
-}
+    4326: {{"projection": "cyl"}}
+}}
+
 #
-# Registration of horizontal layers.                     ###
+# Registration of horizontal layers.
 #
 
 # The following list contains tuples of the format (instance of
@@ -981,7 +1233,7 @@ epsg_to_mpl_basemap_table = {
 # visualisation products for which data files are available. The data
 # sets must be defined in mss_config.py. The WMS will only offer
 # products registered here.
-
+register_horizontal_layers = None
 if mpl_hsec_styles is not None:
     register_horizontal_layers = [
         # ECMWF standard pressure level products.
@@ -995,9 +1247,8 @@ if mpl_hsec_styles is not None:
 
 
 #
-# Registration of vertical layers.                       ###
+# Registration of vertical layers.
 #
-
 # The same as above, but for vertical cross-sections.
 register_vertical_layers = None
 if mpl_vsec_styles is not None:
@@ -1011,6 +1262,8 @@ if mpl_vsec_styles is not None:
         (mpl_vsec_styles.VS_TemperatureStyle_01, ["ecmwf_EUR_LL015"])
     ]
 '''
+            simple_server_config = simple_server_config.format(vt_cache=self.vt_cache,
+                                                               data_dir=self.data_dir)
         else:
             simple_server_config = '''"""
 simple server config for demodata
@@ -1026,15 +1279,23 @@ from mslib.mswms.demodata import (nwpaccess, epsg_to_mpl_basemap_table,
             print(u'''
 /!\ existing server config: "{}" for demodata not overwritten!
             '''.format(self.server_config_file))
+        if not os.path.exists(self.server_auth_config_file):
+            fid = open(self.server_auth_config_file, 'w')
+            fid.write(simple_auth_config)
+            fid.close()
+        else:
+            print(u'''
+/!\ existing server auth config: "{}" for demodata not overwritten!
+                '''.format(self.server_auth_config_file))
 
     def hybrid_data(self):
         self.levtype = 'ml'
         labels = ['U', 'V', 'W', 'CC', 'T', 'Q', 'P_derived']
         for label in labels:
             filename_out = os.path.join(self.data_dir,
-                                        "%s_ecmwf_forecast.%s.EUR_LL015.036.%s.nc" % (self.inidate, label,
-                                                                                      self.levtype))
-            text = self.range_data["%s_%s" % (label, self.levtype)]
+                                        u"{}_ecmwf_forecast.{}.EUR_LL015.036.{}.nc".format(
+                                            self.inidate, label, self.levtype))
+            text = self.range_data[u"{}_{}".format(label, self.levtype)]
             rangedata = StringIO(text)
             ecmwf = nc.Dataset(filename_out, 'w', format='NETCDF4_CLASSIC')
             hybrid_default = np.linspace(20, 91, 72)
@@ -1104,7 +1365,7 @@ from mslib.mswms.demodata import (nwpaccess, epsg_to_mpl_basemap_table,
                 ndims = int(rangedata.readline().strip())
                 dims = rangedata.readline().strip().split(",")
                 rangedata.readline().strip()
-                print "hybrid data: %s" % varname
+                print "hybrid data: {}".format(varname)
                 newvar = ecmwf.createVariable(varname, 'f4', dims)
                 newvar.standard_name = standard_name
                 newvar.units = units
@@ -1139,8 +1400,9 @@ from mslib.mswms.demodata import (nwpaccess, epsg_to_mpl_basemap_table,
         self.levtype = 'pl'
         label = 'PRESSURE_LEVELS'
         filename_out = os.path.join(self.data_dir,
-                                    "%s_ecmwf_forecast.%s.EUR_LL015.036.%s.nc" % (self.inidate, label, self.levtype))
-        text = self.range_data["%s_%s" % (label, self.levtype)]
+                                    "{}_ecmwf_forecast.{}.EUR_LL015.036.{}.nc".format(self.inidate, label,
+                                                                                      self.levtype))
+        text = self.range_data["{}_{}".format(label, self.levtype)]
         rangedata = StringIO(text)
         ecmwf = nc.Dataset(filename_out, 'w', format='NETCDF4_CLASSIC')
         press_default = np.array([20, 30, 50, 70, 100, 150, 200, 250, 300, 400,
@@ -1176,7 +1438,79 @@ from mslib.mswms.demodata import (nwpaccess, epsg_to_mpl_basemap_table,
             ndims = int(rangedata.readline().strip())
             dims = rangedata.readline().strip().split(",")
             rangedata.readline()
-            print "pressure data: %s" % varname
+            print "pressure data: {}".format(varname)
+            newvar = ecmwf.createVariable(varname, 'f4', dims)
+            newvar.standard_name = standard_name
+            newvar.units = units
+            if ndims == 4:
+                test_data = np.ndarray(shape=(self.ntimes, self.nlevs, self.nlats, self.nlons), dtype=float, order='F')
+                for ilev in range(self.nlevs):
+                    # read range from file
+                    line = rangedata.readline().strip()
+                    tmean = float(line.split()[0])
+                    tstd = float(line.split()[1])
+                    # create test data arrays
+                    xarr = np.linspace(0., 10. + ilev / 3., self.nlons)
+                    yarr = np.linspace(0., 5. + ilev / 3., self.nlats)
+                    tarr = np.linspace(0, 2., self.ntimes)
+                    datax = xarr[np.newaxis, np.newaxis, :] + tarr[:, np.newaxis, np.newaxis]
+                    datay = yarr[np.newaxis, :, np.newaxis] - tarr[:, np.newaxis, np.newaxis]
+                    test_data[:, ilev, :, :] = tmean + tstd * (np.sin(datax) + np.cos(datay)) / 2
+                if varname == 'Land-sea_mask_surface':
+                    test_data = test_data.round()
+
+                if standard_name not in self.allow_negative:
+                    # let test_data values not be negative
+                    mask = np.where(test_data < 0.)
+                    test_data[mask] = 0.
+
+                newvar[:] = test_data
+                newvar.grid_mapping = 'LatLon_Projection'
+                newvar.missing_value = float('nan')
+        ecmwf.close()
+
+    def theta_data(self):
+        self.levtype = 'tl'
+        label = 'THETA_LEVELS'
+        filename_out = os.path.join(self.data_dir,
+                                    "{}_ecmwf_forecast.{}.EUR_LL015.036.{}.nc".format(self.inidate, label,
+                                                                                      self.levtype))
+        text = self.range_data["{}_{}".format(label, self.levtype)]
+        rangedata = StringIO(text)
+        ecmwf = nc.Dataset(filename_out, 'w', format='NETCDF4_CLASSIC')
+        theta_default = np.linspace(500, 300, 18)
+        self.nlevs = len(theta_default)
+        ecmwf.createDimension('lat', self.nlats)
+        newvar = ecmwf.createVariable('lat', 'f4', 'lat')
+        newvar[:] = self.lats
+        newvar.units = 'degrees_north'
+        newvar.standard_name = "latitude"
+        ecmwf.createDimension('lon', self.nlons)
+        newvar = ecmwf.createVariable('lon', 'f4', 'lon')
+        newvar[:] = self.lons
+        newvar.units = 'degrees_east'
+        newvar.standard_name = "longitude"
+        ecmwf.createDimension('isentropic', self.nlevs)
+        newvar = ecmwf.createVariable('isentropic', 'f4', 'isentropic')
+        newvar[:] = theta_default
+        newvar.units = 'K'
+        newvar.positive = 'down'
+        newvar.standard_name = "atmosphere_potential_temperature_coordinate"
+        ecmwf.createDimension('time', self.ntimes)
+        newvar = ecmwf.createVariable('time', 'i4', 'time')
+        newvar[:] = self.times
+        newvar.units = 'hours since 2012-10-17T12:00:00.000Z'
+        newvar.standard_name = 'time'
+        while True:
+            varname = rangedata.readline().strip()
+            if not varname:
+                break
+            standard_name = rangedata.readline().strip()
+            units = rangedata.readline().strip()
+            ndims = int(rangedata.readline().strip())
+            dims = rangedata.readline().strip().split(",")
+            rangedata.readline()
+            print "theta data: {}".format(varname)
             newvar = ecmwf.createVariable(varname, 'f4', dims)
             newvar.standard_name = standard_name
             newvar.units = units
@@ -1211,8 +1545,9 @@ from mslib.mswms.demodata import (nwpaccess, epsg_to_mpl_basemap_table,
         self.levtype = 'sfc'
         label = 'SFC'
         filename_out = os.path.join(self.data_dir,
-                                    "%s_ecmwf_forecast.%s.EUR_LL015.036.%s.nc" % (self.inidate, label, self.levtype))
-        text = self.range_data["%s_%s" % (label, self.levtype)]
+                                    "{}_ecmwf_forecast.{}.EUR_LL015.036.{}.nc".format(self.inidate, label,
+                                                                                      self.levtype))
+        text = self.range_data["{}_{}".format(label, self.levtype)]
         rangedata = StringIO(text)
         ecmwf = nc.Dataset(filename_out, 'w', format='NETCDF4_CLASSIC')
         ecmwf.createDimension('lat', self.nlats)
@@ -1239,7 +1574,7 @@ from mslib.mswms.demodata import (nwpaccess, epsg_to_mpl_basemap_table,
             ndims = int(rangedata.readline().strip())
             if ndims == 3:
                 dims = rangedata.readline().strip().split(",")
-                print "sfc data: %s" % varname
+                print "sfc data: {}".format(varname)
                 newvar = ecmwf.createVariable(varname, 'f4', dims)
                 newvar.standard_name = standard_name
                 newvar.units = units
@@ -1336,11 +1671,13 @@ def main():
     creates various test data files and also the server configuration
     """
     examples = DataFiles(data_dir=os.path.join(os.path.expanduser("~"), "mss", 'testdata'),
+                         vt_cache=os.path.join(os.path.expanduser("~"), "mss", "vt_cache"),
                          server_config_dir=os.path.join(os.path.expanduser("~"), "mss"))
     examples.create_datadir()
     examples.create_server_config(detailed_information=True)
     examples.hybrid_data()
     examples.pressure_data()
+    examples.theta_data()
     examples.sfc_data()
     print("\nTo use this setup you need the mss_wms_settings.py in your python path e.g. \nexport PYTHONPATH=~/mss")
 
