@@ -1,37 +1,35 @@
-"""Horizontal section style super classes for use with the
-   HorizontalSectionDriver class.
+# -*- coding: utf-8 -*-
+"""
 
-********************************************************************************
+    mslib.mswms.mpl_hsec
+    ~~~~~~~~~~~~~~~~~~~~
 
-   Copyright 2008-2014 Deutsches Zentrum fuer Luft- und Raumfahrt e.V.
+    Horizontal section style super classes for use with the
+    HorizontalSectionDriver class.
 
-   Licensed under the Apache License, Version 2.0 (the "License");
-   you may not use this file except in compliance with the License.
-   You may obtain a copy of the License at
+    This file is part of mss.
+
+    :copyright: Copyright 2008-2014 Deutsches Zentrum fuer Luft- und Raumfahrt e.V.
+    :copyright: Copyright 2011-2014 Marc Rautenhaus (mr)
+    :copyright: Copyright 2016-2017 by the mss team, see AUTHORS.
+    :license: APACHE-2.0, see LICENSE for details.
+
+    Licensed under the Apache License, Version 2.0 (the "License");
+    you may not use this file except in compliance with the License.
+    You may obtain a copy of the License at
 
        http://www.apache.org/licenses/LICENSE-2.0
 
-   Unless required by applicable law or agreed to in writing, software
-   distributed under the License is distributed on an "AS IS" BASIS,
-   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-   See the License for the specific language governing permissions and
-   limitations under the License.
-
-********************************************************************************
-
-This file is part of the DLR/IPA Mission Support System Web Map Service
-(MSS-WMS).
-
-*** style definitions should be put in mpl_hsec_styles.py ***
-
-AUTHORS:
-========
-
-* Marc Rautenhaus (mr)
-
+    Unless required by applicable law or agreed to in writing, software
+    distributed under the License is distributed on an "AS IS" BASIS,
+    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+    See the License for the specific language governing permissions and
+    limitations under the License.
 """
 
-# standard library imports
+
+# style definitions should be put in mpl_hsec_styles.py
+
 import StringIO
 import logging
 from abc import abstractmethod
@@ -46,11 +44,7 @@ import PIL.Image
 
 # local application imports
 from mslib.mswms import mss_2D_sections
-
-
-"""
-Horizontal Section Styles
-"""
+from mslib.utils import get_projection_params
 
 
 class AbstractHorizontalSectionStyle(mss_2D_sections.Abstract2DSectionStyle):
@@ -67,11 +61,6 @@ class AbstractHorizontalSectionStyle(mss_2D_sections.Abstract2DSectionStyle):
         pass
 
 
-"""
-HSEC Matplotlib Super Class
-"""
-
-
 class MPLBasemapHorizontalSectionStyle(AbstractHorizontalSectionStyle):
     """Matplotlib-based super class for all horizontal section styles.
        Sets up the map projection and draws a basemap.
@@ -85,9 +74,13 @@ class MPLBasemapHorizontalSectionStyle(AbstractHorizontalSectionStyle):
         pass
 
     def supported_epsg_codes(self):
+        return mss_wms_settings.epsg_to_mpl_basemap_table.keys()
+
+    def support_epsg_code(self, epsg):
         """Returns a list of supported EPSG codes.
         """
-        return mss_wms_settings.epsg_to_mpl_basemap_table.keys()
+        return (epsg in mss_wms_settings.epsg_to_mpl_basemap_table.keys() or
+                get_projection_params(str(epsg)) is not None)
 
     def supported_crs(self):
         """Returns a list of the coordinate reference systems supported by
@@ -96,7 +89,7 @@ class MPLBasemapHorizontalSectionStyle(AbstractHorizontalSectionStyle):
         crs_list = []
         epsg_codes = self.supported_epsg_codes()
         for code in epsg_codes:
-            crs_list.append("EPSG:%i" % code)
+            crs_list.append("EPSG:{:d}".format(code))
         return crs_list
 
     def _draw_auto_graticule(self, bm):
@@ -202,7 +195,7 @@ class MPLBasemapHorizontalSectionStyle(AbstractHorizontalSectionStyle):
                                             labels=[0, 0, 0, 1],
                                             color='0.5', dashes=[5, 5])
 
-    def plot_hsection(self, data, lats, lons, bbox=[-180, -90, 180, 90],
+    def plot_hsection(self, data, lats, lons, bbox=(-180, -90, 180, 90),
                       level=None, figsize=(960, 640), epsg=None,
                       proj_params={"projection": "cyl"},
                       valid_time=None, init_time=None, style=None,
@@ -213,17 +206,18 @@ class MPLBasemapHorizontalSectionStyle(AbstractHorizontalSectionStyle):
         """
         # Projection parameters from EPSG code.
         if epsg is not None:
-            try:
-                proj_params = mss_wms_settings.epsg_to_mpl_basemap_table[epsg]
-            except:
-                raise ValueError("unknown EPSG code: %i", epsg)
+            proj_params = mss_wms_settings.epsg_to_mpl_basemap_table.get(epsg)
+            if proj_params is None:
+                proj_params = get_projection_params(str(epsg))["basemap"]
+            if proj_params is None:
+                raise ValueError("unknown EPSG code: {:d}".format(epsg))
 
         logging.debug("plotting data..")
 
         # Check if required data is available.
         for datatype, dataitem in self.required_datafields:
             if dataitem not in data.keys():
-                raise KeyError("required data field %s not found" % dataitem)
+                raise KeyError(u"required data field '{}' not found".format(dataitem))
 
         # Copy parameters to properties.
         self.data = data
@@ -344,8 +338,7 @@ class MPLBasemapHorizontalSectionStyle(AbstractHorizontalSectionStyle):
                 facecolor_rgb[i] = int(facecolor_rgb[i] * 255)
             facecolor_index = lut.index(tuple(facecolor_rgb))
 
-            logging.debug("saving figure as transparent PNG with transparency index %i.",
-                          facecolor_index)
+            logging.debug("saving figure as transparent PNG with transparency index {:d}.".format(facecolor_index))
             palette_img.save(output, format="PNG", transparency=facecolor_index)
 
         logging.debug("returning figure..")
@@ -365,8 +358,8 @@ class MPLBasemapHorizontalSectionStyle(AbstractHorizontalSectionStyle):
         axis = self.bm.ax.axis()
         ulcrnrlon, ulcrnrlat = self.bm(axis[0], axis[3], inverse=True)
         left_longitude = min(self.bm.llcrnrlon, ulcrnrlon)
-        logging.debug("shifting data grid to leftmost longitude in map "
-                      "(%.2f).." % left_longitude)
+        logging.debug(u"shifting data grid to leftmost longitude in map "
+                      u"({:.2f})..".format(left_longitude))
 
         # Shift the longitude field such that the data is in the range
         # left_longitude .. left_longitude+360.
