@@ -1,37 +1,35 @@
-"""Vertical section style super classes for use with the
-   VerticalSectionDriver class.
+# -*- coding: utf-8 -*-
+"""
 
-********************************************************************************
+    mslib.mswms.mpl_vsec
+    ~~~~~~~~~~~~~~~~~~~~
 
-   Copyright 2008-2014 Deutsches Zentrum fuer Luft- und Raumfahrt e.V.
+    Vertical section style super classes for use with the
+    VerticalSectionDriver class.
 
-   Licensed under the Apache License, Version 2.0 (the "License");
-   you may not use this file except in compliance with the License.
-   You may obtain a copy of the License at
+    This file is part of mss.
+
+    :copyright: Copyright 2008-2014 Deutsches Zentrum fuer Luft- und Raumfahrt e.V.
+    :copyright: Copyright 2011-2014 Marc Rautenhaus (mr)
+    :copyright: Copyright 2016-2017 by the mss team, see AUTHORS.
+    :license: APACHE-2.0, see LICENSE for details.
+
+    Licensed under the Apache License, Version 2.0 (the "License");
+    you may not use this file except in compliance with the License.
+    You may obtain a copy of the License at
 
        http://www.apache.org/licenses/LICENSE-2.0
 
-   Unless required by applicable law or agreed to in writing, software
-   distributed under the License is distributed on an "AS IS" BASIS,
-   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-   See the License for the specific language governing permissions and
-   limitations under the License.
-
-********************************************************************************
-
-This file is part of the DLR/IPA Mission Support System Web Map Service
-(MSS-WMS).
-
-*** style definitions should be put in mpl_vsec_styles.py ***
-
-AUTHORS:
-========
-
-* Marc Rautenhaus (mr)
-
+    Unless required by applicable law or agreed to in writing, software
+    distributed under the License is distributed on an "AS IS" BASIS,
+    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+    See the License for the specific language governing permissions and
+    limitations under the License.
 """
 
-# standard library imports
+
+# style definitions should be put in mpl_vsec_styles.py
+
 import PIL.Image
 import StringIO
 import logging
@@ -48,13 +46,12 @@ from mslib.mswms import mss_2D_sections
 
 mpl.rcParams['xtick.direction'] = 'out'
 mpl.rcParams['ytick.direction'] = 'out'
-"""
-Abstract Vertical Section Style
-"""
 
 
 class AbstractVerticalSectionStyle(mss_2D_sections.Abstract2DSectionStyle):
-    """Superclass for all Matplotlib-based vertical section styles.
+    """
+    Abstract Vertical Section Style
+    Superclass for all Matplotlib-based vertical section styles.
     """
 
     def __init__(self, driver=None):
@@ -69,15 +66,15 @@ class AbstractVerticalSectionStyle(mss_2D_sections.Abstract2DSectionStyle):
         return ["VERT:LOGP"]
 
     # TODO: the general setup should be a separate class as well
-    def _latlon_logp_setup(self, orography=105000., titlestring=""):
+    def _latlon_logp_setup(self, orography=105000., titlestring=u""):
         """General setup for lat/lon vs. log p vertical cross-sections.
         """
         ax = self.ax
 
         # Set xticks so that they display lat/lon. Plot "numlabels" labels.
-        tick_index_step = len(self.lat_inds) / int(self.numlabels)
+        tick_index_step = max(1, len(self.lat_inds) / int(self.numlabels))
         ax.set_xticks(self.lat_inds[::tick_index_step])
-        ax.set_xticklabels(["%2.1f, %2.1f" % (d[0], d[1])
+        ax.set_xticklabels(["{:2.1f}, {:2.1f}".format(d[0], d[1])
                             for d in zip(self.lats[::tick_index_step],
                                          self.lons[::tick_index_step])],
                            rotation=25, fontsize=10, horizontalalignment='right')
@@ -113,7 +110,7 @@ class AbstractVerticalSectionStyle(mss_2D_sections.Abstract2DSectionStyle):
                                     len_major_ticks + len(major_ticks2) - 1)
             major_ticks[len_major_ticks:] = major_ticks2[1:]
 
-        labels = ['%i' % (l / 100.) for l in major_ticks]
+        labels = ["{:.0f}".format(l / 100.) for l in major_ticks]
 
         # .. the same for the minor ticks ..
         p_top_minor = max(label_distance, self.p_top)
@@ -143,26 +140,10 @@ class AbstractVerticalSectionStyle(mss_2D_sections.Abstract2DSectionStyle):
         # Plot title (either in image axes or above).
         time_step = self.valid_time - self.init_time
         time_step_hrs = (time_step.days * 86400 + time_step.seconds) / 3600
-        titlestring += " [%i..%i hPa]" % (self.p_bot / 100, self.p_top / 100)
-        titlestring = titlestring + '\nValid: %s (step %i hrs from %s)' % \
-                                    (self.valid_time.strftime('%a %Y-%m-%d %H:%M UTC'),
-                                     time_step_hrs,
-                                     self.init_time.strftime('%a %Y-%m-%d %H:%M UTC'))
-
-        if not self.noframe:
-            ax.set_title(titlestring,
-                         horizontalalignment='left', x=0, fontsize=14)
-            ax.set_xlabel('Latitude, longitude (degrees)')
-            ax.set_ylabel('Pressure (hPa)')
-            if self.resolution != (-1, -1):
-                self.fig.text(0.99, 0.01, 'model resolution %.2fx%.2f deg' %
-                              (self.resolution[0], self.resolution[1]),
-                              fontsize=10, horizontalalignment='right',
-                              transform=self.fig.transFigure)
-        else:
-            ax.text(0, self.p_top, titlestring,
-                    fontsize=10, verticalalignment='top',
-                    bbox=dict(facecolor='white', alpha=0.6))
+        titlestring = u"{} [{:.0f}..{:.0f} hPa]\nValid: {} (step {:d} hrs from {})".format(
+            titlestring, self.p_bot / 100, self.p_top / 100,
+            self.valid_time.strftime("%a %Y-%m-%d %H:%M UTC"),
+            time_step_hrs, self.init_time.strftime("%a %Y-%m-%d %H:%M UTC"))
 
     @abstractmethod
     def _plot_style(self):
@@ -181,7 +162,7 @@ class AbstractVerticalSectionStyle(mss_2D_sections.Abstract2DSectionStyle):
         # Check if required data is available.
         for datatype, dataitem in self.required_datafields:
             if dataitem not in data.keys():
-                raise KeyError("required data field %s not found" % dataitem)
+                raise KeyError(u"required data field '{}' not found".format(dataitem))
 
         # Copy parameters to properties.
         self.data = data
@@ -209,7 +190,6 @@ class AbstractVerticalSectionStyle(mss_2D_sections.Abstract2DSectionStyle):
             logging.debug("creating figure..")
             dpi = 80
             figsize = figsize[0] / dpi, figsize[1] / dpi
-            # print figsize
             facecolor = "white"
             self.fig = mpl.figure.Figure(figsize=figsize, dpi=dpi, facecolor=facecolor)
             logging.debug("\twith frame and legends" if not noframe else
@@ -300,7 +280,7 @@ class AbstractVerticalSectionStyle(mss_2D_sections.Abstract2DSectionStyle):
 
             # Longitude data.
             node = xmldoc.createElement("Longitude")
-            node.setAttribute("num_waypoints", "%s" % len(self.lons))
+            node.setAttribute("num_waypoints", "{}".format(len(self.lons)))
 
             data_str = ""
             for value in self.lons:
@@ -312,7 +292,7 @@ class AbstractVerticalSectionStyle(mss_2D_sections.Abstract2DSectionStyle):
 
             # Latitude data.
             node = xmldoc.createElement("Latitude")
-            node.setAttribute("num_waypoints", "%s" % len(self.lats))
+            node.setAttribute("num_waypoints", "{}".format(len(self.lats)))
 
             data_str = ""
             for value in self.lats:
@@ -328,8 +308,8 @@ class AbstractVerticalSectionStyle(mss_2D_sections.Abstract2DSectionStyle):
             for var in self.data.keys():
                 node = xmldoc.createElement(var)
                 data_shape = self.data[var].shape
-                node.setAttribute("num_levels", "%s" % data_shape[0])
-                node.setAttribute("num_waypoints", "%s" % data_shape[1])
+                node.setAttribute("num_levels", u"{}".format(data_shape[0]))
+                node.setAttribute("num_waypoints", u"{}".format(data_shape[1]))
 
                 data_str = ""
                 for data_row in self.data[var]:
