@@ -1,58 +1,52 @@
-"""Matplotlib VERTICAL section styles.
+# -*- coding: utf-8 -*-
+"""
 
-********************************************************************************
+    mslib.mswms.mpl_vsec_styles
+    ~~~~~~~~~~~~~~~~~~~~~~~~~~~
 
-   Copyright 2008-2014 Deutsches Zentrum fuer Luft- und Raumfahrt e.V.
-   Copyright 2011-2014 Marc Rautenhaus
+    Matplotlib VERTICAL section styles.
+    This file corresponds to mpl_hsec_styles.py, but for vertical section styles.
+    Please refer to the introductory docstring in mpl_hsec_styles.py for further
+    information.
 
-   Licensed under the Apache License, Version 2.0 (the "License");
-   you may not use this file except in compliance with the License.
-   You may obtain a copy of the License at
+    This file is part of mss.
+
+    :copyright: Copyright 2008-2014 Deutsches Zentrum fuer Luft- und Raumfahrt e.V.
+    :copyright: Copyright 2011-2014 Marc Rautenhaus (mr)
+    :copyright: Copyright 2016-2017 by the mss team, see AUTHORS.
+    :license: APACHE-2.0, see LICENSE for details.
+
+    Licensed under the Apache License, Version 2.0 (the "License");
+    you may not use this file except in compliance with the License.
+    You may obtain a copy of the License at
 
        http://www.apache.org/licenses/LICENSE-2.0
 
-   Unless required by applicable law or agreed to in writing, software
-   distributed under the License is distributed on an "AS IS" BASIS,
-   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-   See the License for the specific language governing permissions and
-   limitations under the License.
-
-********************************************************************************
-
-This file is part of the DLR/IPA Mission Support System Web Map Service
-(MSS-WMS).
-
-This file corresponds to mpl_hsec_styles.py, but for vertical section styles.
-Please refer to the introductory docstring in mpl_hsec_styles.py for further
-information.
-
-AUTHORS:
-========
-
-* Marc Rautenhaus (mr)
-
+    Unless required by applicable law or agreed to in writing, software
+    distributed under the License is distributed on an "AS IS" BASIS,
+    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+    See the License for the specific language governing permissions and
+    limitations under the License.
 """
 
-# standard library imports
-
-# related third party imports
 import matplotlib
 import matplotlib.pyplot as plt
 import mpl_toolkits.axes_grid1.inset_locator
+from matplotlib import patheffects
 import numpy as np
 
 # local application imports
 from mslib.mswms.mpl_vsec import AbstractVerticalSectionStyle
-from mslib.mswms.utils import Targets, get_log_levels
+from mslib.mswms.utils import Targets, get_style_parameters, get_cbar_label_format
+from mslib.mswms.msschem import MSSChemTargets
 from mslib import thermolib
 
-"""
-TEMPERATURE
-"""
-
 
 class VS_TemperatureStyle_01(AbstractVerticalSectionStyle):
-    """Vertical section of temperature.
+    """
+    Temperature
+    Vertical section of temperature.
+
     """
 
     name = "VS_T01"
@@ -121,38 +115,34 @@ class VS_TemperatureStyle_01(AbstractVerticalSectionStyle):
 
 
 class VS_GenericStyle(AbstractVerticalSectionStyle):
-    """Vertical section of chemical species/other stuff
+    """
+    Vertical section of chemical species/other stuff
     """
     styles = [
         ("auto", "auto colour scale"),
         ("autolog", "auto log colour scale"), ]
 
+    def _prepare_datafields(self):
+        if self.name[-2:] == "pl":
+            self.data["air_pressure"] = np.empty_like(self.data[self.dataname])
+            self.data["air_pressure"][:] = self.driver.vert_data[::-self.driver.vert_order, np.newaxis]
+        elif self.name[-2:] == "tl":
+            self.data["air_potential_temperature"] = np.empty_like(self.data[self.dataname])
+            self.data["air_potential_temperature"][:] = self.driver.vert_data[::-self.driver.vert_order, np.newaxis]
+
     def _plot_style(self):
         ax = self.ax
-        curtain_pv = self.data["ertel_potential_vorticity"]
         curtain_cc = self.data[self.dataname] * self.unit_scale
         curtain_cc = np.ma.masked_invalid(curtain_cc)
-        if self.name[-2:] == "pl":
-            curtain_pt = self.data["air_potential_temperature"]
-            curtain_p = np.empty_like(curtain_cc)
-            curtain_p[:] = self.driver.vert_data[::-self.driver.vert_order, np.newaxis] * 100
-        elif self.name[-2:] == "tl":
-            curtain_p = self.data["air_pressure"] * 100
-            curtain_pt = np.empty_like(curtain_cc)
-            curtain_pt[:] = self.driver.vert_data[::-self.driver.vert_order, np.newaxis]
-        elif self.name[-2:] == "ml":
-            curtain_p = self.data["air_pressure"] * 100
-            curtain_pt = self.data["air_potential_temperature"]
+        curtain_p = self.data["air_pressure"] * 100
 
         numlevel = curtain_p.shape[0]
         numpoints = len(self.lats)
+        curtain_lat = self.lat_inds.repeat(numlevel).reshape((numpoints, numlevel)).transpose()
 
-        delta_pt = 10 if (np.log(self.p_bot) - np.log(self.p_top)) < 2.2 else 20
         # Filled contour plot of cloud cover.
         # INFO on COLORMAPS:
         #    http://matplotlib.sourceforge.net/examples/pylab_examples/show_colormaps.html
-        cmap = plt.cm.rainbow
-        cmin, cmax = Targets.get_range(self.dataname)
         if self.p_bot > self.p_top:
             visible = (curtain_p <= self.p_bot) & (curtain_p >= self.p_top)
         else:
@@ -161,44 +151,24 @@ class VS_GenericStyle(AbstractVerticalSectionStyle):
         if visible.sum() == 0:
             visible = np.ones_like(curtain_cc, dtype=bool)
 
-        if cmin is None or cmax is None:
-            cmin, cmax = curtain_cc[visible].min(), curtain_cc[visible].max()
-            if cmin > 0 and cmin < 0.05 * cmax:
-                cmin = 0.
+        cmin, cmax = Targets.get_range(self.dataname)
+        cmin, cmax, clevs, cmap, norm, ticks = get_style_parameters(
+            self.dataname, self.style, cmin, cmax, curtain_cc[visible])
 
-        if self.style == "default":
-            clev = np.linspace(cmin, cmax, 16)
-        elif self.style == "auto":
-            cmin = curtain_cc[visible].min()
-            cmax = curtain_cc[visible].max()
-            if cmin > 0 and cmin < 0.05 * cmax:
-                cmin = 0.
-            clev = np.linspace(cmin, cmax, 16)
-        elif self.style == "log":
-            clev = get_log_levels(cmin, cmax, 16)
-        elif self.style == "autolog":
-            cmin = curtain_cc[visible].min()
-            cmax = curtain_cc[visible].max()
-            clev = get_log_levels(cmin, cmax, 16)
-        elif self.style == "nonlinear":
-            clev = Targets.get_thresholds(self.dataname)
-        else:
-            raise RuntimeError("Illegal plotting style?!")
-        norm = matplotlib.colors.BoundaryNorm(clev, cmap.N)
+        cs = ax.contourf(curtain_lat, curtain_p, curtain_cc, clevs, cmap=cmap, extend="both", norm=norm)
 
-        cs = ax.contourf(self.lat_inds.repeat(numlevel).reshape((numpoints, numlevel)).transpose(),
-                         curtain_p, curtain_cc, clev, norm=norm,
-                         cmap=cmap)
-        # Contour line plot of PV.
-        cs_pv = ax.contour(self.lat_inds.repeat(numlevel).reshape((numpoints, numlevel)).transpose(),
-                           curtain_p, curtain_pv, [2, 4, 8, 16],
-                           colors='lightgrey', linestyles='dashed', linewidths=2)
-        ax.clabel(cs_pv, fontsize=8, fmt='%i')
-        # Contour line plot of potential temperature.
-        cs_pt = ax.contour(self.lat_inds.repeat(numlevel).reshape((numpoints, numlevel)).transpose(),
-                           curtain_p, curtain_pt, np.arange(200, 700, delta_pt), colors='lightgrey',
-                           linestyles='solid', linewidths=2)
-        ax.clabel(cs_pt, fontsize=8, fmt='%i')
+        # Contour lines
+        for cont_data, cont_levels, cont_colour, cont_label_colour, cont_style, cont_lw, pe in self.contours:
+            if cont_levels is None:
+                pl_cont = ax.plot(self.lat_inds, self.data[cont_data].reshape(-1), "o", color="k", zorder=100)
+                plt.setp(pl_cont, path_effects=[patheffects.withStroke(linewidth=4, foreground="w")])
+            else:
+                cs_pv = ax.contour(curtain_lat, curtain_p, self.data[cont_data], cont_levels,
+                                   colors=cont_colour, linestyles=cont_style, linewidths=cont_lw)
+                plt.setp(cs_pv.collections,
+                         path_effects=[patheffects.withStroke(linewidth=cont_lw + 2, foreground="w")])
+                cs_pv_lab = ax.clabel(cs_pv, colours=cont_label_colour, fontsize=8, fmt='%i')
+                plt.setp(cs_pv_lab, path_effects=[patheffects.withStroke(linewidth=1, foreground="w")])
 
         # Pressure decreases with index, i.e. orography is stored at the
         # zero-p-index (data field is flipped in mss_plot_driver.py if
@@ -206,65 +176,46 @@ class VS_GenericStyle(AbstractVerticalSectionStyle):
         self._latlon_logp_setup(titlestring=self.title)
 
         # Format for colorbar labels
-        clev_format = "%.3g"
-        mclev = np.abs(clev).max()
-        if self.style != "log":
-            if mclev >= 100. and mclev < 10000.:
-                clev_format = "%4i"
-            if mclev >= 10. and mclev < 100.:
-                clev_format = "%.1f"
-            if mclev >= 1. and mclev < 10.:
-                clev_format = "%.2f"
-            if mclev >= .1 and mclev < 1.:
-                clev_format = "%.3f"
-            if mclev >= .01 and mclev < 0.1:
-                clev_format = "%.4f"
+        cbar_format = get_cbar_label_format(self.style, np.abs(clevs).max())
+        cbar_label = self.title
 
-            # Add colorbar.
+        # Add colorbar.
         if not self.noframe:
             self.fig.subplots_adjust(left=0.08, right=0.95, top=0.9, bottom=0.14)
-            cbar = self.fig.colorbar(cs, fraction=0.05, pad=0.01,
-                                     format=clev_format, norm=norm)
-            cbar.set_label(self.cbar_label)
+            self.fig.colorbar(cs, fraction=0.05, pad=0.01, format=cbar_format, label=cbar_label, ticks=ticks)
         else:
-            axins1 = mpl_toolkits.axes_grid1.inset_locator.inset_axes(ax,
-                                                                      width="1%",  # width = % of parent_bbox width
-                                                                      height="45%",  # height : %
-                                                                      loc=1)  # 4 = lr, 3 = ll, 2 = ul, 1 = ur
-            cbar = self.fig.colorbar(cs, cax=axins1, orientation="vertical",
-                                     format=clev_format, norm=norm)
+            axins1 = mpl_toolkits.axes_grid1.inset_locator.inset_axes(
+                ax, width="1%", height="40%", loc=1)
+            self.fig.colorbar(cs, cax=axins1, orientation="vertical", format=cbar_format, ticks=ticks)
 
             # adjust colorbar fontsize to figure height
-            figheight = self.fig.bbox.height
-            fontsize = figheight * 0.035
+            fontsize = self.fig.bbox.height * 0.024
             axins1.yaxis.set_ticks_position("left")
             for x in axins1.yaxis.majorTicks:
-                x.label1.set_backgroundcolor("w")
-                x.label1.set_backgroundcolor("w")
+                x.label1.set_path_effects([patheffects.withStroke(linewidth=4, foreground='w')])
                 x.label1.set_fontsize(fontsize)
 
 
-def make_generic_class(entity, vert):
+def make_generic_class(name, entity, vert, add_data=None, add_contours=None,
+                       fix_styles=None, add_styles=None, add_prepare=None):
+    if add_data is None:
+        add_data = [(vert, "ertel_potential_vorticity")]
+    if add_contours is None:
+        add_contours = [  # ("ertel_potential_vorticity", [2, 4, 8, 16], "dimgrey", "dimgrey", "solid", 2, True)]
+            ("ertel_potential_vorticity", [2, 4, 8, 16], "dimgrey", "dimgrey", "dashed", 2, True),
+            ("air_potential_temperature", np.arange(200, 700, 10), "dimgrey", "dimgrey", "solid", 2, True)]
+
     class fnord(VS_GenericStyle):
-        name = 'VS_' + entity + "_" + vert
+        name = u"VS_{}_{}".format(entity, vert)
         dataname = entity
         units, unit_scale = Targets.get_unit(dataname)
-        title = dataname
+        title = Targets.TITLES.get(entity, entity)
         if units:
-            title += " ({})".format(units)
-        required_datafields = [
-            (vert, entity),
-            (vert, "ertel_potential_vorticity"), ]
-        if vert == "pl":
-            required_datafields.append((vert, "air_potential_temperature"))
-        elif vert == "tl":
-            required_datafields.append((vert, "air_pressure"))
-        elif vert == "ml":
-            required_datafields.append((vert, "air_pressure"))
-            required_datafields.append((vert, "air_potential_temperature"))
-        else:
-            raise ValueError
+            title += u" ({})".format(units)
+        required_datafields = [(vert, entity)] + add_data
+        contours = add_contours
 
+    fnord.__name__ = name
     fnord.styles = list(fnord.styles)
     if Targets.get_thresholds(entity) is not None:
         fnord.styles = fnord.styles + [("nonlinear", "nonlinear colour scale")]
@@ -273,86 +224,98 @@ def make_generic_class(entity, vert):
             ("default", "fixed colour scale"),
             ("log", "fixed logarithmic colour scale")]
 
-    return fnord
+    if add_styles is not None:
+        fnord.styles += add_styles
+    if fix_styles is not None:
+        fnord.styles = fix_styles
+    if add_prepare is not None:
+        fnord._prepare_datafields = add_prepare
 
-for ent in Targets.get_targets():
-    globals()["VS_GenericStyle_PL_" + ent] = make_generic_class(ent, "pl")
-    globals()["VS_GenericStyle_TL_" + ent] = make_generic_class(ent, "tl")
-    globals()["VS_GenericStyle_ML_" + ent] = make_generic_class(ent, "ml")
-
-"""
-GW
-"""
+    globals()[name] = fnord
 
 
-class VS_GravityWaveForecast_ML(AbstractVerticalSectionStyle):
-    """Vertical section of chemical species
-    """
-    name = "GW"
-    title = "Gravity Wave Temperature Residual (K)"
+_ADD_DATA = {
+    "pl": [("pl", "ertel_potential_vorticity"), ("pl", "air_potential_temperature")],
+    "ml": [("ml", "ertel_potential_vorticity"), ("ml", "air_pressure"), ("ml", "air_potential_temperature")],
+    "tl": [("tl", "ertel_potential_vorticity"), ("tl", "air_pressure")]
+}
 
-    required_datafields = [
-        ("ml", "gravity_wave_temperature_perturbation"),
-        ("ml", "air_pressure"),
-        ("sfc", "tropopause_altitude"),
-    ]
+for vert in ["pl", "ml", "tl"]:
+    for ent in Targets.get_targets():
+        make_generic_class(
+            "VS_GenericStyle_{}_{}".format(vert.upper(), ent),
+            ent, vert, add_data=_ADD_DATA[vert])
+    make_generic_class(
+        "VS_GenericStyle_{}_{}".format(vert.upper(), "ertel_potential_vorticity"),
+        "ertel_potential_vorticity", vert, add_data=_ADD_DATA[vert],
+        fix_styles=[("ertel_potential_vorticity_nh", "northern hemisphere"),
+                    ("ertel_potential_vorticity_sh", "southern hemisphere")])
+    make_generic_class(
+        "VS_GenericStyle_{}_{}".format(vert.upper(), "equivalent_latitude"),
+        "equivalent_latitude", vert, add_data=_ADD_DATA[vert],
+        fix_styles=[("equivalent_latitude_nh", "northern hemisphere"),
+                    ("equivalent_latitude_sh", "southern hemisphere")])
+    make_generic_class(
+        "VS_GenericStyle_{}_{}".format(vert.upper(), "square_of_brunt_vaisala_frequency_in_air"),
+        "square_of_brunt_vaisala_frequency_in_air", vert, add_data=_ADD_DATA[vert],
+        fix_styles=[("square_of_brunt_vaisala_frequency_in_air", "")])
+vert = "ml"
+make_generic_class(
+    "VS_GenericStyle_{}_{}".format(vert.upper(), "gravity_wave_temperature_perturbation"),
+    "gravity_wave_temperature_perturbation", vert,
+    add_data=[("sfc", "tropopause_air_pressure"), ("ml", "air_pressure")],
+    add_contours=[("tropopause_air_pressure", None, "dimgrey", "dimgrey", "solid", 2, True)],
+    fix_styles=[("gravity_wave_temperature_perturbation", "")])
+make_generic_class(
+    "VS_GenericStyle_{}_{}".format(vert.upper(), "square_of_brunt_vaisala_frequency_in_air"),
+    "square_of_brunt_vaisala_frequency_in_air", vert,
+    add_data=[("sfc", "tropopause_air_pressure"), ("ml", "air_pressure")],
+    add_contours=[("tropopause_air_pressure", None, "dimgrey", "dimgrey", "solid", 2, True)],
+    fix_styles=[("square_of_brunt_vaisala_frequency_in_air", "")])
 
-    def _plot_style(self):
-        """Make a cloud cover vertical section with temperature/potential
-           temperature overlay.
-        """
-        ax = self.ax
-        curtain_cc = self.data["gravity_wave_temperature_perturbation"]
-        # curtain_p = np.empty_like(curtain_cc)
-        # for i in range(len(self.driver.vert_data)):
-        #    curtain_p[i, :] = 102300 * np.exp(-self.driver.vert_data[i] / 7.9)
-        # curtain_p = curtain_p[::-1, :]
-        curtain_p = self.data["air_pressure"] * 100
-        tropo_p = np.empty_like(self.data["tropopause_altitude"].reshape(-1))
-        for i in range(curtain_p.shape[1]):
-            z = self.data["tropopause_altitude"].reshape(-1)[i]
-            tropo_p[i] = np.interp(z, self.driver.vert_data[:], curtain_p[::-1, i])
-        numlevel = curtain_p.shape[0]
-        numpoints = len(self.lats)
+vert = "pl"
+make_generic_class(
+    "VS_GenericStyle_{}_{}".format(vert.upper(), "cloud_ice_mixing_ratio"),
+    "cloud_ice_mixing_ratio", vert,
+    add_data=[("pl", "maximum_relative_humidity_wrt_ice_on_backtrajectory")],
+    add_contours=[("maximum_relative_humidity_wrt_ice_on_backtrajectory",
+                  [90, 100, 120, 160],
+                  ["dimgrey", "dimgrey", "#443322", "#045FB4"],
+                  ["dimgrey", "dimgrey", "#443322", "#045FB4"],
+                  ["dashed", "solid", "solid", "solid"], 2, True)],
+    fix_styles=[("log_ice_cloud", "iwc")])
 
-        # Filled contour plot of cloud cover.
-        # INFO on COLORMAPS:
-        #    http://matplotlib.sourceforge.net/examples/pylab_examples/show_colormaps.html
-        cmap = plt.cm.Spectral_r
-        norm = matplotlib.colors.BoundaryNorm([-3, -2.5, -2, -1.5, -1, -0.5, 0.5, 1, 1.5, 2, 2.5, 3], cmap.N)
-        cs = ax.pcolormesh(self.lat_inds.repeat(numlevel).reshape((numpoints, numlevel)).transpose(),
-                           curtain_p, curtain_cc, norm=norm,
-                           cmap=cmap)
-        ax.plot(self.lat_inds, tropo_p.reshape(-1), color="gray", zorder=100)
+make_generic_class(
+    "VS_GenericStyle_{}_{}".format(vert.upper(), "number_concentration_of_ice_crystals_in_air"),
+    "number_concentration_of_ice_crystals_in_air", vert,
+    add_data=[("pl", "maximum_relative_humidity_wrt_ice_on_backtrajectory")],
+    add_contours=[("maximum_relative_humidity_wrt_ice_on_backtrajectory",
+                  [90, 100, 120, 160],
+                  ["dimgrey", "dimgrey", "#443322", "#045FB4"],
+                  ["dimgrey", "dimgrey", "#443322", "#045FB4"],
+                  ["dashed", "solid", "solid", "solid"], 2, True)],
+    fix_styles=[("log_ice_cloud", "nice")])
 
-        # Pressure decreases with index, i.e. orography is stored at the
-        # zero-p-index (data field is flipped in mss_plot_driver.py if
-        # pressure increases with index).
-        self._latlon_logp_setup(titlestring=self.title)
+make_generic_class(
+    "VS_GenericStyle_{}_{}".format(vert.upper(), "mean_mass_radius_of_cloud_ice_crystals"),
+    "mean_mass_radius_of_cloud_ice_crystals", vert,
+    add_data=[("pl", "maximum_relative_humidity_wrt_ice_on_backtrajectory")],
+    add_contours=[("maximum_relative_humidity_wrt_ice_on_backtrajectory",
+                  [90, 100, 120, 160],
+                  ["dimgrey", "dimgrey", "#443322", "#045FB4"],
+                  ["dimgrey", "dimgrey", "#443322", "#045FB4"],
+                  ["dashed", "solid", "solid", "solid"], 2, True)],
+    fix_styles=[("ice_cloud", "radius")])
 
-        # Add colorbar.
-        if not self.noframe:
-            self.fig.subplots_adjust(left=0.08, right=0.95, top=0.9, bottom=0.14)
-            cbar = self.fig.colorbar(cs, fraction=0.05, pad=0.01)
-            cbar.set_label(self.cbar_label)
-        else:
-            axins1 = mpl_toolkits.axes_grid1.inset_locator.inset_axes(ax,
-                                                                      width="1%",  # width = % of parent_bbox width
-                                                                      height="30%",  # height : %
-                                                                      loc=1)  # 4 = lr, 3 = ll, 2 = ul, 1 = ur
-            cbar = self.fig.colorbar(cs, cax=axins1, orientation="vertical")
-            axins1.yaxis.set_ticks_position("left")
-            for x in axins1.yaxis.majorTicks:
-                x.label1.set_backgroundcolor("w")
-
-
-"""
-CLOUDS
-"""
+make_generic_class(
+    "VS_GenericStyle_{}_{}".format(vert.upper(), "maximum_pressure_on_backtrajectory"),
+    "maximum_pressure_on_backtrajectory", vert, [], [])
 
 
 class VS_CloudsStyle_01(AbstractVerticalSectionStyle):
-    """Vertical section of cloud cover.
+    """
+    Clouds
+    Vertical section of cloud cover.
     """
 
     name = "VS_CC01"
@@ -437,7 +400,8 @@ class VS_CloudsStyle_01(AbstractVerticalSectionStyle):
 
 
 class VS_CloudsWindStyle_01(AbstractVerticalSectionStyle):
-    """Vertical section of cloud cover and horizontal wind speed.
+    """
+    Vertical section of cloud cover and horizontal wind speed.
     """
 
     name = "VS_CW01"
@@ -518,13 +482,9 @@ class VS_CloudsWindStyle_01(AbstractVerticalSectionStyle):
             axins1.yaxis.set_ticks_position("left")
 
 
-"""
-RELATIVE HUMIDITY
-"""
-
-
 class VS_RelativeHumdityStyle_01(AbstractVerticalSectionStyle):
-    """Vertical sections of relative humidity.
+    """
+    Vertical sections of relative humidity.
     """
 
     name = "VS_RH01"
@@ -625,7 +585,8 @@ class VS_RelativeHumdityStyle_01(AbstractVerticalSectionStyle):
 
 
 class VS_SpecificHumdityStyle_01(AbstractVerticalSectionStyle):
-    """Vertical sections of specific humidity.
+    """
+    Vertical sections of specific humidity.
     """
 
     name = "VS_Q01"
@@ -737,13 +698,9 @@ class VS_SpecificHumdityStyle_01(AbstractVerticalSectionStyle):
             axins1.yaxis.set_ticks_position("left")
 
 
-"""
-VERTICAL VELOCITY
-"""
-
-
 class VS_VerticalVelocityStyle_01(AbstractVerticalSectionStyle):
-    """Vertical sections of vertical velocity.
+    """
+    Vertical sections of vertical velocity.
     """
 
     name = "VS_W01"
@@ -842,13 +799,9 @@ class VS_VerticalVelocityStyle_01(AbstractVerticalSectionStyle):
             axins1.yaxis.set_ticks_position("left")
 
 
-"""
-HORIZONTAL VELOCITY
-"""
-
-
 class VS_HorizontalVelocityStyle_01(AbstractVerticalSectionStyle):
-    """Vertical sections of horizontal velocity.
+    """
+    Vertical sections of horizontal velocity.
     """
 
     name = "VS_HV01"
@@ -951,10 +904,7 @@ class VS_HorizontalVelocityStyle_01(AbstractVerticalSectionStyle):
             axins1.yaxis.set_ticks_position("left")
 
 
-"""
-POTENTIAL VORTICITY
-"""
-
+# POTENTIAL VORTICITY
 pv_cmap_data = [
     [0.0000, 0.6000, 1.0000],
     [0.0980, 0.6039, 1.0000],
@@ -1085,7 +1035,8 @@ pv_eth_cmap_1 = matplotlib.colors.LinearSegmentedColormap.from_list("pv_eth_cmap
 
 
 class VS_PotentialVorticityStyle_01(AbstractVerticalSectionStyle):
-    """Vertical sections of potential vorticity.
+    """
+    Vertical sections of potential vorticity.
     """
 
     name = "VS_PV01"
@@ -1218,14 +1169,11 @@ class VS_PotentialVorticityStyle_01(AbstractVerticalSectionStyle):
             axins1.yaxis.set_ticks_position("left")
 
 
-"""
-PROBABILITY OF WCB
-"""
-
-
 class VS_ProbabilityOfWCBStyle_01(AbstractVerticalSectionStyle):
-    """Vertical sections of probability of WCB trajectory occurence,
-       derived from Lagranto trajectories (TNF 2012 product).
+    """
+    Probability of WCB
+    Vertical sections of probability of WCB trajectory occurence,
+    derived from Lagranto trajectories (TNF 2012 product).
     """
 
     name = "VS_PWCB01"
@@ -1328,8 +1276,9 @@ class VS_ProbabilityOfWCBStyle_01(AbstractVerticalSectionStyle):
 
 
 class VS_LagrantoTrajStyle_PL_01(AbstractVerticalSectionStyle):
-    """Number of Lagranto trajectories per grid box for WCB, MIX, INSITU
-       trajectories (ML-Cirrus 2014 product).
+    """
+    Number of Lagranto trajectories per grid box for WCB, MIX, INSITU
+    trajectories (ML-Cirrus 2014 product).
     """
 
     name = "VS_LGTRAJ01"
@@ -1396,13 +1345,10 @@ class VS_LagrantoTrajStyle_PL_01(AbstractVerticalSectionStyle):
             axins1.yaxis.set_ticks_position("left")
 
 
-"""
-EMAC Eyjafjallajokull
-"""
-
-
 class VS_EMACEyja_Style_01(AbstractVerticalSectionStyle):
-    """EMAC Eyja tracer vertical cross sections.
+    """
+    EMAC Eyjafjallajokull
+    EMAC Eyja tracer vertical cross sections.
     """
 
     name = "VS_EMAC_Eyja_01"
@@ -1488,3 +1434,166 @@ class VS_EMACEyja_Style_01(AbstractVerticalSectionStyle):
                                                                       loc=1)  # 4 = lr, 3 = ll, 2 = ul, 1 = ur
             cbar = self.fig.colorbar(cs, cax=axins1, orientation="vertical")
             axins1.yaxis.set_ticks_position("left")
+
+
+class VS_MSSChemStyle(AbstractVerticalSectionStyle):
+    """ CTM tracer vertical tracer cross sections via MSS-Chem
+    """
+    styles = [
+        ("auto", "auto colour scale"),
+        ("autolog", "auto log colour scale"), ]
+
+    # Variables with the highest number of dimensions first (otherwise
+    # MFDatasetCommonDims will throw an exception)!
+    required_datafields = [("ml", "air_pressure")]
+
+    # In order to use information from the DataAccess class to construct the titles, we override the set_driver to set
+    # self.title.  This cannot happen in __init__, as the WMSServer doesn't initialize the layers with the driver but
+    # rather sets the driver only after initialization.
+    def set_driver(self, driver):
+        super(VS_MSSChemStyle, self).set_driver(driver=driver)
+        self.title = self._title_tpl.format(modelname=self.driver.data_access._modelname)
+        # for altitude level model data, when we don't have air_pressure information, we want to warn users that the
+        # vertical section is only an approximation
+        if (self.name[-2:] == "al") and\
+                ("p" not in self.driver.data_access.build_filetree().values()[0].values()[0].keys()):
+            self.title = self.title.replace(" al)", " al; WARNING: vert. distribution only approximate!)")
+
+    def _prepare_datafields(self):
+        """Computes potential temperature from pressure and temperature if
+        it has not been passed as a data field.
+        """
+        if self.name[-2:] == "pl":
+            self.data["air_pressure"] = np.empty_like(self.data[self.dataname])
+            self.data["air_pressure"][:] = self.driver.vert_data[::-self.driver.vert_order, np.newaxis]
+        elif self.name[-2:] == "tl":
+            self.data["air_potential_temperature"] = np.empty_like(self.data[self.dataname])
+            self.data["air_potential_temperature"][:] = self.driver.vert_data[::-self.driver.vert_order, np.newaxis]
+        elif self.name[-2:] == "al":
+            # CAMS Regional Ensemble doesn't provide any pressure information, but we want to plot vertical sections
+            # anyways, so we do a poor-man's on-the-fly conversion here.
+            if 'air_pressure' not in self.data.keys():
+                self.data["air_pressure"] = np.empty_like(self.data[self.dataname])
+                flightlevel = 3.28083989501 / 100. * self.driver.vert_data[::-self.driver.vert_order, np.newaxis]
+                self.data['air_pressure'][:] = thermolib.flightlevel2pressure_a(flightlevel)
+
+    def _plot_style(self):
+        ax = self.ax
+        curtain_cc = self.data[self.dataname] * self.unit_scale
+        curtain_cc = np.ma.masked_invalid(curtain_cc)
+        curtain_p = self.data["air_pressure"]  # TODO* 100
+
+        numlevel = curtain_p.shape[0]
+        numpoints = len(self.lats)
+        curtain_lat = self.lat_inds.repeat(numlevel).reshape((numpoints, numlevel)).transpose()
+
+        # Filled contour plot of cloud cover.
+        # INFO on COLORMAPS:
+        #    http://matplotlib.sourceforge.net/examples/pylab_examples/show_colormaps.html
+        if self.p_bot > self.p_top:
+            visible = (curtain_p <= self.p_bot) & (curtain_p >= self.p_top)
+        else:
+            visible = (curtain_p >= self.p_bot) & (curtain_p <= self.p_top)
+
+        if visible.sum() == 0:
+            visible = np.ones_like(curtain_cc, dtype=bool)
+
+        cmin, cmax = Targets.get_range(self.dataname)
+        cmin, cmax, clevs, cmap, norm, ticks = get_style_parameters(
+            self.dataname, self.style, cmin, cmax, curtain_cc[visible])
+
+        cs = ax.contourf(curtain_lat, curtain_p, curtain_cc, clevs, cmap=cmap, extend="both", norm=norm)
+
+        # Contour lines
+        for cont_data, cont_levels, cont_colour, cont_label_colour, cont_style, cont_lw, pe in self.contours:
+            if cont_levels is None:
+                pl_cont = ax.plot(self.lat_inds, self.data[cont_data].reshape(-1), "o", color="k", zorder=100)
+                plt.setp(pl_cont, path_effects=[patheffects.withStroke(linewidth=4, foreground="w")])
+            else:
+                cs_pv = ax.contour(curtain_lat, curtain_p, self.data[cont_data], cont_levels,
+                                   colors=cont_colour, linestyles=cont_style, linewidths=cont_lw)
+                plt.setp(cs_pv.collections,
+                         path_effects=[patheffects.withStroke(linewidth=cont_lw + 2, foreground="w")])
+                cs_pv_lab = ax.clabel(cs_pv, colours=cont_label_colour, fontsize=8, fmt='%i')
+                plt.setp(cs_pv_lab, path_effects=[patheffects.withStroke(linewidth=1, foreground="w")])
+
+        # Pressure decreases with index, i.e. orography is stored at the
+        # zero-p-index (data field is flipped in mss_plot_driver.py if
+        # pressure increases with index).
+        self._latlon_logp_setup(titlestring=self.title)
+
+        # Format for colorbar labels
+        cbar_format = get_cbar_label_format(self.style, np.abs(clevs).max())
+        cbar_label = self.title
+
+        # Add colorbar.
+        if not self.noframe:
+            self.fig.subplots_adjust(left=0.08, right=0.95, top=0.9, bottom=0.14)
+            self.fig.colorbar(cs, fraction=0.05, pad=0.01, format=cbar_format, label=cbar_label, ticks=ticks)
+        else:
+            axins1 = mpl_toolkits.axes_grid1.inset_locator.inset_axes(
+                ax, width="1%", height="40%", loc=1)
+            self.fig.colorbar(cs, cax=axins1, orientation="vertical", format=cbar_format, ticks=ticks)
+
+            # adjust colorbar fontsize to figure height
+            fontsize = self.fig.bbox.height * 0.024
+            axins1.yaxis.set_ticks_position("left")
+            for x in axins1.yaxis.majorTicks:
+                x.label1.set_path_effects([patheffects.withStroke(linewidth=4, foreground='w')])
+                x.label1.set_fontsize(fontsize)
+
+
+def make_msschem_class(entity, nam, vert, units, scale, add_data=None,
+                       add_contours=None, fix_styles=None, add_styles=None, add_prepare=None):
+
+    # This is CTM output, so we cannot expect any additional meteorological
+    # parameters except for air_pressure
+    if add_data is None:
+        if vert == "al":
+            # "al" is altitude layer, i.e., for CTM output with no pressure information
+            # at all (e.g., CAMS reg. Ensemble)
+            # In those cases we derive air_pressure from the altitude alone, in the _prepare_datafields() method
+            add_data = []
+        else:
+            # all other layer types need to read air_pressure from the data
+            add_data = [(vert, "air_pressure")]
+    if add_contours is None:
+        add_contours = []
+
+    class fnord(VS_MSSChemStyle):
+        name = "VS_" + entity + "_" + vert
+        dataname = entity
+        # units, unit_scale = Targets.get_unit(dataname)
+        units = units
+        unit_scale = scale
+        _title_tpl = nam + " ({modelname}, " + vert + ")"
+        long_name = entity
+        if units:
+            _title_tpl += u" ({})".format(units)
+        required_datafields = [(vert, entity)] + add_data
+        contours = add_contours if add_contours else []
+
+    fnord.__name__ = name
+    fnord.styles = list(fnord.styles)
+    if Targets.get_thresholds(entity) is not None:
+        fnord.styles = fnord.styles + [("nonlinear", "nonlinear colour scale")]
+    if all(_x is not None for _x in Targets.get_range(entity)):
+        fnord.styles = fnord.styles + [
+            ("default", "fixed colour scale"),
+            ("log", "fixed logarithmic colour scale")]
+
+    if add_styles is not None:
+        fnord.styles += add_styles
+    if fix_styles is not None:
+        fnord.styles = fix_styles
+    if add_prepare is not None:
+        fnord._prepare_datafields = add_prepare
+
+    return fnord
+
+
+for vert in ["ml", "tl", "pl", "al"]:
+    for stdname, props in MSSChemTargets.items():
+        name, qty, units, scale = props
+        key = "VS_MSSChemStyle_" + vert.upper() + "_" + name + "_" + qty
+        globals()[key] = make_msschem_class(stdname, name, vert, units, scale)
