@@ -1,34 +1,31 @@
-"""Driver classes to create plots from ECMWF NetCDF data.
+# -*- coding: utf-8 -*-
+"""
 
-********************************************************************************
+    mslib.mswms.mss_plot_driver
+    ~~~~~~~~~~~~~~~~~~~~~~~~~~~
 
-   Copyright 2008-2014 Deutsches Zentrum fuer Luft- und Raumfahrt e.V.
+    Driver classes to create plots from ECMWF NetCDF data.
 
-   Licensed under the Apache License, Version 2.0 (the "License");
-   you may not use this file except in compliance with the License.
-   You may obtain a copy of the License at
+    This file is part of mss.
+
+    :copyright: Copyright 2008-2014 Deutsches Zentrum fuer Luft- und Raumfahrt e.V.
+    :copyright: Copyright 2011-2014 Marc Rautenhaus (mr)
+    :copyright: Copyright 2016-2017 by the mss team, see AUTHORS.
+    :license: APACHE-2.0, see LICENSE for details.
+
+    Licensed under the Apache License, Version 2.0 (the "License");
+    you may not use this file except in compliance with the License.
+    You may obtain a copy of the License at
 
        http://www.apache.org/licenses/LICENSE-2.0
 
-   Unless required by applicable law or agreed to in writing, software
-   distributed under the License is distributed on an "AS IS" BASIS,
-   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-   See the License for the specific language governing permissions and
-   limitations under the License.
-
-********************************************************************************
-
-This file is part of the DLR/IPA Mission Support System Web Map Service
-(MSS-WMS).
-
-AUTHORS:
-========
-
-* Marc Rautenhaus (mr)
-
+    Unless required by applicable law or agreed to in writing, software
+    distributed under the License is distributed on an "AS IS" BASIS,
+    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+    See the License for the specific language governing permissions and
+    limitations under the License.
 """
 
-# standard library imports
 from datetime import datetime
 
 import logging
@@ -40,16 +37,13 @@ import numpy as np
 
 # local application imports
 from mslib import netCDF4tools
-from mslib import mss_util
-
-"""
-MSS Plot Driver
-"""
+from mslib import utils
 
 
 class MSSPlotDriver(object):
-    """Abstract super class for implementing driver classes that provide
-       access to the MSS data server.
+    """
+    Abstract super class for implementing driver classes that provide
+    access to the MSS data server.
 
     The idea of a driver class is to encapsulate all methods related to
     loading data fields into memory. A driver can control objects from
@@ -78,8 +72,7 @@ class MSSPlotDriver(object):
     def __del__(self):
         """Closes the open NetCDF dataset, if existing.
         """
-        logging.debug("closing plot driver")
-        if self.dataset:
+        if self.dataset is not None:
             self.dataset.close()
 
     def _set_time(self, init_time, fc_time):
@@ -115,19 +108,18 @@ class MSSPlotDriver(object):
         fc_step = fc_time - init_time
         fc_step = fc_step.days * 24 + fc_step.seconds / 3600
         self.fc_time = fc_time
-        logging.debug("\trequested initialisation time %s", init_time)
-        logging.debug("\trequested forecast valid time %s (step %i hrs)",
-                      fc_time, fc_step)
+        logging.debug(u"\trequested initialisation time {}".format(init_time))
+        logging.debug(u"\trequested forecast valid time {} (step {} hrs)".format(fc_time, fc_step))
 
         # Check if a dataset is open and if it contains the requested times.
         # (a dataset will only be open if the used layer has not changed,
         # i.e. the required variables have not changed as well).
-        if self.dataset:
+        if self.dataset is not None:
             logging.debug("checking on open dataset.")
             if self.init_time == init_time:
-                logging.debug("\tinitialisation time ok (%s).", init_time)
+                logging.debug(u"\tinitialisation time ok ({}).".format(init_time))
                 if fc_time in self.times:
-                    logging.debug("\tforecast valid time contained (%s).", fc_time)
+                    logging.debug(u"\tforecast valid time contained ({}).".format(fc_time))
                     return
             logging.debug("need to re-open input files.")
             self.dataset.close()
@@ -149,11 +141,10 @@ class MSSPlotDriver(object):
             short_filename = os.path.basename(filename)
             if filename not in filenames:
                 filenames.append(filename)
-            logging.debug("\tvariable %s requires input file %s", var,
-                          short_filename)
+            logging.debug(u"\tvariable '{}' requires input file '{}'".format(var, short_filename))
             if short_filename not in available_files:
-                logging.error("ERROR: file %s does not exist" % short_filename)
-                raise IOError("file %s does not exist" % short_filename)
+                logging.error(u"ERROR: file '{}' does not exist".format(short_filename))
+                raise IOError(u"file '{}' does not exist".format(short_filename))
 
         if len(filenames) == 0:
             raise ValueError("no files found that correspond to the specified "
@@ -170,12 +161,14 @@ class MSSPlotDriver(object):
         # if an Exception is raised here.
         timename, timevar = netCDF4tools.identify_CF_time(dataset)
         times = netCDF4tools.num2date(timevar[:], timevar.units)
-        if init_time != netCDF4tools.num2date(0, timevar.units):
-            dataset.close()
-            raise ValueError("wrong initialisation time in input")
+        # removed after discussion, see
+        # https://mss-devel.slack.com/archives/emerge/p1486658769000007
+        # if init_time != netCDF4tools.num2date(0, timevar.units):
+        #     dataset.close()
+        #     raise ValueError("wrong initialisation time in input")
 
         if fc_time not in times:
-            msg = "Forecast valid time %s is not available." % fc_time
+            msg = u"Forecast valid time {} is not available.".format(fc_time)
             logging.error(msg)
             dataset.close()
             raise ValueError(msg)
@@ -184,10 +177,10 @@ class MSSPlotDriver(object):
         try:
             lat_data, lon_data, lat_order = \
                 netCDF4tools.get_latlon_data(dataset)
-        except Exception as e:
-            logging.error("ERROR: %s" % e)
+        except Exception as ex:
+            logging.error("ERROR: {} {}".format(type(ex), ex))
             dataset.close()
-            raise e
+            raise
 
         # Try to load vertical hybrid coordinate (model levels), isopressure
         # coordinate (pressure levels) or iso-potential-vorticity (pv levels).
@@ -199,7 +192,7 @@ class MSSPlotDriver(object):
 
         hybrid_name, hybrid_var, hybrid_orientation = \
             netCDF4tools.identify_CF_hybrid(dataset)
-        if hybrid_var:
+        if hybrid_var is not None:
             vert_data = hybrid_var[:]
             vert_orientation = hybrid_orientation
             vert_units = "model_level"
@@ -207,7 +200,7 @@ class MSSPlotDriver(object):
         if vert_data is None:
             isopressure_name, isopressure_var, isopressure_orientation = \
                 netCDF4tools.identify_CF_isopressure(dataset)
-            if isopressure_var:
+            if isopressure_var is not None:
                 vert_data = isopressure_var[:]
                 vert_orientation = isopressure_orientation
                 vert_units = "hPa"
@@ -215,7 +208,7 @@ class MSSPlotDriver(object):
         if vert_data is None:
             isopotvort_name, isopotvort_var, isopotvort_orientation = \
                 netCDF4tools.identify_CF_isopotvort(dataset)
-            if isopotvort_var:
+            if isopotvort_var is not None:
                 vert_data = isopotvort_var[:]
                 vert_orientation = isopotvort_orientation
                 vert_units = "10^-3 PVU"
@@ -223,7 +216,7 @@ class MSSPlotDriver(object):
         if vert_data is None:
             isoalt_name, isoalt_var, isoalt_orientation = \
                 netCDF4tools.identify_CF_isoaltitude(dataset)
-            if isoalt_var:
+            if isoalt_var is not None:
                 vert_data = isoalt_var[:]
                 vert_orientation = isoalt_orientation
                 vert_units = isoalt_var.units
@@ -231,7 +224,7 @@ class MSSPlotDriver(object):
         if vert_data is None:
             isoalt_name, isoalt_var, isoalt_orientation = \
                 netCDF4tools.identify_CF_isopottemp(dataset)
-            if isoalt_var:
+            if isoalt_var is not None:
                 vert_data = isoalt_var[:]
                 vert_orientation = isoalt_orientation
                 vert_units = isoalt_var.units
@@ -259,8 +252,8 @@ class MSSPlotDriver(object):
         """
         self.data_vars = {}
         for df_type, df_name in self.plot_object.required_datafields:
-            varname, var = netCDF4tools.identify_variable(self.dataset, df_name)
-            logging.debug("\tidentified variable <%s> for field <%s>", varname, df_name)
+            varname, var = netCDF4tools.identify_variable(self.dataset, df_name, check=True)
+            logging.debug("\tidentified variable <{}> for field <{}>".format(varname, df_name))
             self.data_vars[df_name] = var
 
     @abstractmethod
@@ -278,17 +271,16 @@ class MSSPlotDriver(object):
         Derived methods need to call the super method before all other
         statements.
         """
-        logging.debug("using plot object '%s'", plot_object.name)
-        logging.debug("\tfigure size %s in pixels", figsize)
+        logging.debug("using plot object '{}'".format(plot_object.name))
+        logging.debug("\tfigure size {} in pixels".format(figsize))
 
         # If the plot object has been changed, the dataset needs to be reloaded
         # (the required variables could have changed).
-        if self.plot_object:
+        if self.plot_object is not None:
             require_reload = require_reload or (self.plot_object != plot_object)
-        if require_reload:
-            if self.dataset:
-                self.dataset.close()
-                self.dataset = None
+        if require_reload and self.dataset is not None:
+            self.dataset.close()
+            self.dataset = None
 
         self.plot_object = plot_object
         self.figsize = figsize
@@ -315,13 +307,13 @@ class MSSPlotDriver(object):
         Derived methods need to call the super method before all other
         statements.
         """
-        plot_object = plot_object if plot_object else self.plot_object
-        figsize = figsize if figsize else self.figsize
+        plot_object = plot_object if plot_object is not None else self.plot_object
+        figsize = figsize if figsize is not None else self.figsize
         noframe = noframe if noframe is not None else self.noframe
-        init_time = init_time if init_time else self.init_time
-        valid_time = valid_time if valid_time else self.valid_time
-        style = style if style else self.style
-        bbox = bbox if bbox else self.bbox
+        init_time = init_time if init_time is not None else self.init_time
+        valid_time = valid_time if valid_time is not None else self.valid_time
+        style = style if style is not None else self.style
+        bbox = bbox if bbox is not None else self.bbox
         transparent = transparent if transparent is not None else self.transparent
         return_format = return_format if return_format is not None else self.return_format
         # Explicitly call MSSPlotDriver's set_plot_parameters(). A "self.--"
@@ -363,11 +355,6 @@ class MSSPlotDriver(object):
         return self.data_access.get_valid_times(variable, vartype, init_time)
 
 
-"""
-Vertical Section Driver
-"""
-
-
 class VerticalSectionDriver(MSSPlotDriver):
     """The vertical section driver is responsible for loading the data that
        is to be plotted and for calling the plotting routines (that have
@@ -404,17 +391,17 @@ class VerticalSectionDriver(MSSPlotDriver):
                                transparent=None, return_format=None):
         """
         """
-        plot_object = plot_object if plot_object else self.plot_object
-        figsize = figsize if figsize else self.figsize
+        plot_object = plot_object if plot_object is not None else self.plot_object
+        figsize = figsize if figsize is not None else self.figsize
         noframe = noframe if noframe is not None else self.noframe
-        init_time = init_time if init_time else self.init_time
-        valid_time = valid_time if valid_time else self.valid_time
-        style = style if style else self.style
-        bbox = bbox if bbox else self.bbox
-        vsec_path = vsec_path if vsec_path else self.vsec_path
-        vsec_numpoints = vsec_numpoints if vsec_numpoints else self.vsec_numpoints
-        vsec_numlabels = vsec_numlabels if vsec_numlabels else self.vsec_numlabels
-        vsec_path_connection = vsec_path_connection if vsec_path_connection \
+        init_time = init_time if init_time is not None else self.init_time
+        valid_time = valid_time if valid_time is not None else self.valid_time
+        style = style if style is not None else self.style
+        bbox = bbox if bbox is not None else self.bbox
+        vsec_path = vsec_path if vsec_path is not None else self.vsec_path
+        vsec_numpoints = vsec_numpoints if vsec_numpoints is not None else self.vsec_numpoints
+        vsec_numlabels = vsec_numlabels if vsec_numlabels is not None else self.vsec_numlabels
+        vsec_path_connection = vsec_path_connection if vsec_path_connection is not None \
             else self.vsec_path_connection
         show = show if show else self.show
         transparent = transparent if transparent is not None else self.transparent
@@ -438,11 +425,11 @@ class VerticalSectionDriver(MSSPlotDriver):
                                    vsec_path_connection='linear'):
         """
         """
-        logging.debug("computing %i interpolation points, connection: %s" %
-                      (vsec_numpoints, vsec_path_connection))
-        self.lats, self.lons = mss_util.path_points(vsec_path,
-                                                    numpoints=vsec_numpoints,
-                                                    connection=vsec_path_connection)
+        logging.debug("computing {:} interpolation points, connection: {}"
+                      .format(vsec_numpoints, vsec_path_connection))
+        self.lats, self.lons = utils.path_points(vsec_path,
+                                                 numpoints=vsec_numpoints,
+                                                 connection=vsec_path_connection)
         self.vsec_path = vsec_path
         self.vsec_numpoints = vsec_numpoints
         self.vsec_path_connection = vsec_path_connection
@@ -460,45 +447,43 @@ class VerticalSectionDriver(MSSPlotDriver):
         cross section crosses the data longitude boundaries (e.g. data is
         stored on a 0..360 grid, but the path is in the range -10..+20).
         """
-        if not self.dataset:
+        if self.dataset is None:
             return {}
         data = {}
         timestep = self.times.searchsorted(self.fc_time)
-        logging.debug("loading data for time step %i (%s)", timestep,
-                      self.fc_time)
+        logging.debug("loading data for time step {} ({})".format(timestep, self.fc_time))
 
         # Determine the westmost longitude in the cross-section path. Subtract
         # one gridbox size to obtain "left_longitude".
         dlon = self.lon_data[1] - self.lon_data[0]
         left_longitude = self.lons.min() - dlon
         logging.debug("shifting data grid to gridpoint west of westmost "
-                      "longitude in path: %.2f (path %.2f).."
-                      % (left_longitude, self.lons.min()))
+                      "longitude in path: {:.2f} (path {:.2f}).."
+                      .format(left_longitude, self.lons.min()))
 
         # Shift the longitude field such that the data is in the range
         # left_longitude .. left_longitude+360.
         # NOTE: This does not overwrite self.lon_data (which is required
         # in its original form in case other data is loaded while this
         # file is open).
-        # print self.lon_data
         lon_data = ((self.lon_data - left_longitude) % 360) + left_longitude
-        # print lon_data
         lon_indices = lon_data.argsort()
         lon_data = lon_data[lon_indices]
-        # print lon_data
 
         for name, var in self.data_vars.items():
-            var_data = var[timestep, ::-self.vert_order, ::self.lat_order, :]
-            logging.debug("\tLoaded %.2f Mbytes from data field <%s> at timestep %i." %
-                          (var_data.nbytes / 1048576., name, timestep))
-            logging.debug("\tVertical dimension direction is %s." %
-                          ("up" if self.vert_order == 1 else "down"))
+            if len(var.shape) == 4:
+                var_data = var[timestep, ::-self.vert_order, ::self.lat_order, :]
+            else:
+                var_data = var[:][timestep, np.newaxis, ::self.lat_order, :]
+            logging.debug("\tLoaded {:.2f} Mbytes from data field <{:}> at timestep {:}.".format(
+                          var_data.nbytes / 1048576., name, timestep))
+            logging.debug("\tVertical dimension direction is {}.".format(
+                          "up" if self.vert_order == 1 else "down"))
             logging.debug("\tInterpolating to cross-section path.")
             # Re-arange longitude dimension in the data field.
             var_data = var_data[:, :, lon_indices]
-            data[name] = mss_util.interpolate_vertsec3(var_data, self.lat_data, lon_data,
-                                                       self.lats, self.lons)
-            # print data[name][:,30]
+            data[name] = utils.interpolate_vertsec3(var_data, self.lat_data, lon_data,
+                                                    self.lats, self.lons)
             # Free memory.
             del var_data
 
@@ -516,16 +501,14 @@ class VerticalSectionDriver(MSSPlotDriver):
         """
         # Determine the leftmost longitude in the plot.
         left_longitude = self.lons.min()
-        logging.debug("shifting data grid to leftmost longitude in path "
-                      "(%.2f).." % left_longitude)
+        logging.debug(u"shifting data grid to leftmost longitude in path "
+                      u"({:.2f})..".format(left_longitude))
 
         # Shift the longitude field such that the data is in the range
         # left_longitude .. left_longitude+360.
-        # print self.lons
         self.lons = ((self.lons - left_longitude) % 360) + left_longitude
         lon_indices = self.lons.argsort()
         self.lons = self.lons[lon_indices]
-        # print self.lons
 
         # Shift data fields correspondingly.
         for key in self.data.keys():
@@ -543,7 +526,7 @@ class VerticalSectionDriver(MSSPlotDriver):
         data = self._load_interpolate_timestep()
 
         d2 = datetime.now()
-        logging.debug("Loaded and interpolated data (required time %s)." % (d2 - d1))
+        logging.debug("Loaded and interpolated data (required time {}).".format(d2 - d1))
         logging.debug("Plotting interpolated curtain.")
 
         if len(self.lat_data) > 1 and len(self.lon_data) > 1:
@@ -570,17 +553,12 @@ class VerticalSectionDriver(MSSPlotDriver):
         del data
 
         d3 = datetime.now()
-        logging.debug("Finished plotting (required time %s; total "
-                      "time %s).\n" % (d3 - d2, d3 - d1))
+        logging.debug("Finished plotting (required time {}; total "
+                      "time {}).\n".format(d3 - d2, d3 - d1))
 
         return image
 
 
-"""
-Horizontal Section Driver
-"""
-
-
 class HorizontalSectionDriver(MSSPlotDriver):
     """The horizontal section driver is responsible for loading the data that
        is to be plotted and for calling the plotting routines (that have
@@ -614,16 +592,16 @@ class HorizontalSectionDriver(MSSPlotDriver):
                                transparent=None, return_format=None):
         """
         """
-        plot_object = plot_object if plot_object else self.plot_object
-        figsize = figsize if figsize else self.figsize
-        noframe = noframe if noframe else self.noframe
-        init_time = init_time if init_time else self.init_time
-        valid_time = valid_time if valid_time else self.valid_time
-        style = style if style else self.style
-        bbox = bbox if bbox else self.bbox
-        level = level if level else self.level
-        epsg = epsg if epsg else self.epsg
-        show = show if show else self.show
+        plot_object = plot_object if plot_object is not None else self.plot_object
+        figsize = figsize if figsize is not None else self.figsize
+        noframe = noframe if noframe is not None else self.noframe
+        init_time = init_time if init_time is not None else self.init_time
+        valid_time = valid_time if valid_time is not None else self.valid_time
+        style = style if style is not None else self.style
+        bbox = bbox if bbox is not None else self.bbox
+        level = level if level is not None else self.level
+        epsg = epsg if epsg is not None else self.epsg
+        show = show if show is not None else self.show
         transparent = transparent if transparent is not None else self.transparent
         return_format = return_format if return_format is not None else self.return_format
         self.set_plot_parameters(plot_object=plot_object,
@@ -643,7 +621,7 @@ class HorizontalSectionDriver(MSSPlotDriver):
         """Load the data fields as required by the horizontal section style
            instance at the current timestep.
         """
-        if not self.dataset:
+        if self.dataset is None:
             return {}
         data = {}
         timestep = self.times.searchsorted(self.fc_time)
@@ -653,18 +631,17 @@ class HorizontalSectionDriver(MSSPlotDriver):
             if self.vert_order == -1:
                 level = len(self.vert_data) - 1 - level
             self.actual_level = self.vert_data[level]
-        logging.debug("loading data for time step %i (%s), level index "
-                      "%s (level %s)",
-                      timestep, self.fc_time, level, self.level)
+        logging.debug("loading data for time step {} ({}), level index {} (level {})".format(
+                      timestep, self.fc_time, level, self.level))
         for name, var in self.data_vars.items():
-            if level is None:
+            if level is None or len(var.shape) == 3:
                 # 2D fields: time, lat, lon.
                 var_data = var[timestep, ::self.lat_order, :]
             else:
                 # 3D fields: time, level, lat, lon.
                 var_data = var[timestep, level, ::self.lat_order, :]
-            logging.debug("\tLoaded %.2f Mbytes from data field <%s>." %
-                          (var_data.nbytes / 1048576., name))
+            logging.debug("\tLoaded {:.2f} Mbytes from data field <{}>."
+                          .format(var_data.nbytes / 1048576., name))
             data[name] = var_data
             # Free memory.
             del var_data
@@ -710,145 +687,7 @@ class HorizontalSectionDriver(MSSPlotDriver):
         del data
 
         d3 = datetime.now()
-        logging.debug("Finished plotting (required time %s; total "
-                      "time %s).\n" % (d3 - d2, d3 - d1))
+        logging.debug("Finished plotting (required time {}; total "
+                      "time {}).\n".format(d3 - d2, d3 - d1))
 
         return image
-
-
-"""
-Module TESTING
-"""
-
-
-def test_vsec_clouds_path():
-    """TEST: Create a vertical section of the CLOUDS style.
-    """
-    # Define cross-section path (great circle interpolation between two points).
-    p1 = [45.00, 8.]
-    p2 = [50.00, 12.]
-    p3 = [51.00, 15.]
-    p4 = [48.00, 11.]
-
-    import mss_wms_settings
-    nwpaccess = mss_wms_settings.nwpaccess["ecmwf_EUR_LL015"]
-
-    init_time = datetime(2010, 12, 14, 00)
-    valid_time = datetime(2010, 12, 14, 15)
-
-    import mpl_vsec_styles
-    # plot_object = mpl_vsec_styles.VSCloudsStyle01(p_top=20000.)
-    plot_object = mpl_vsec_styles.VSTemperatureStyle01(p_top=2000.)
-
-    vsec = VerticalSectionDriver(nwpaccess)
-    vsec.set_plot_parameters(plot_object=plot_object,
-                             vsec_path=[p1, p2, p3, p4],
-                             vsec_numpoints=101,
-                             vsec_path_connection='greatcircle',
-                             init_time=init_time,
-                             valid_time=valid_time,
-                             noframe=False,
-                             show=True)
-    vsec.plot()
-
-
-def test_hsec_clouds_total():
-    """TEST: Create a horizontal section of the CLOUDS style.
-    """
-    # Define a bounding box for the map.
-    #    bbox = [0,30,30,60]
-    bbox = [-22.5, 27.5, 55, 62.5]
-
-    import mss_wms_settings
-    nwpaccess = mss_wms_settings.nwpaccess["ecmwf_EUR_LL015"]
-
-    init_time = datetime(2010, 12, 14, 00)
-    valid_time = datetime(2010, 12, 14, 15)
-
-    import mpl_hsec_styles
-    plot_object = mpl_hsec_styles.HSCloudsStyle01()
-
-    hsec = HorizontalSectionDriver(nwpaccess)
-    hsec.set_plot_parameters(plot_object=plot_object,
-                             bbox=bbox,
-                             # epsg=4326,
-                             epsg=77790010,
-                             init_time=init_time,
-                             valid_time=valid_time,
-                             noframe=False,
-                             show=True)
-    hsec.plot()
-
-
-def test_hsec_temp():
-    """TEST: Create a horizontal section of the TEMPERATURE style.
-    """
-    # Define a bounding box for the map.
-    #    bbox = [0,30,30,60]
-    bbox = [-22.5, 27.5, 55, 62.5]
-
-    import mss_wms_settings
-    nwpaccess = mss_wms_settings.nwpaccess["ecmwf_EUR_LL015"]
-
-    init_time = datetime(2010, 12, 16, 00)
-    valid_time = datetime(2010, 12, 16, 15)
-
-    import mpl_hsec_styles
-    #    plot_object = mpl_hsec_styles.HS_TemperatureStyle_ML_01()
-    #    level = 50
-    plot_object = mpl_hsec_styles.HS_TemperatureStyle_PL_01()
-    level = 925
-
-    hsec = HorizontalSectionDriver(nwpaccess)
-    hsec.set_plot_parameters(plot_object=plot_object,
-                             bbox=bbox,
-                             level=level,
-                             # epsg=4326,
-                             epsg=77790010,
-                             init_time=init_time,
-                             valid_time=valid_time,
-                             noframe=True,
-                             show=True)
-    hsec.plot()
-
-
-def test_hsec_geopwind():
-    """TEST: Create a horizontal section.
-    """
-    # Define a bounding box for the map.
-    bbox = [-22.5, 27.5, 55, 62.5]
-
-    import mss_wms_settings
-    nwpaccess = mss_wms_settings.nwpaccess["ecmwf_EUR_LL015"]
-
-    init_time = datetime(2010, 12, 16, 00)
-    valid_time = datetime(2010, 12, 16, 15)
-
-    import mpl_hsec_styles
-    plot_object = mpl_hsec_styles.HS_GeopotentialWindStyle_PL()
-    level = 300
-
-    hsec = HorizontalSectionDriver(nwpaccess)
-    hsec.set_plot_parameters(plot_object=plot_object,
-                             bbox=bbox,
-                             level=level,
-                             # epsg=4326,
-                             epsg=77790010,
-                             init_time=init_time,
-                             valid_time=valid_time,
-                             noframe=True,
-                             show=True)
-    hsec.plot()
-
-
-if __name__ == "__main__":
-    # Log everything, and send it to stderr.
-    # See http://docs.python.org/library/logging.html for more information
-    # on the Python logging module.
-    logging.basicConfig(level=logging.DEBUG,
-                        format="%(levelname)s %(asctime)s %(funcName)s || %(message)s")
-
-    # test_vsec_clouds_path()
-    # test_hsec_clouds_total()
-    test_hsec_temp()
-    # test_hsec_geopwind()
