@@ -1,18 +1,44 @@
 #!/usr/bin/env python
-
+# -*- coding: utf-8 -*-
 """
-The module can be run with the Python PASTE framework as a stand-alone
-server (simply execute this file with Python).
+
+    mslib.mswms.mswms
+    ~~~~~~~~~~~~~~~~~
+
+    The module can be run with the Python PASTE framework as a stand-alone
+    server (simply execute this file with Python).
+
+    This file is part of mss.
+
+    :copyright: Copyright 2016 Reimar Bauer
+    :copyright: Copyright 2016-2017 by the mss team, see AUTHORS.
+    :license: APACHE-2.0, see LICENSE for details.
+
+    Licensed under the Apache License, Version 2.0 (the "License");
+    you may not use this file except in compliance with the License.
+    You may obtain a copy of the License at
+
+       http://www.apache.org/licenses/LICENSE-2.0
+
+    Unless required by applicable law or agreed to in writing, software
+    distributed under the License is distributed on an "AS IS" BASIS,
+    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+    See the License for the specific language governing permissions and
+    limitations under the License.
 """
 
 import paste.httpserver
 import argparse
 import logging
-from wms import mss_wms_settings
+import sys
+
+from mslib import __version__
+from wms import mss_wms_settings, mss_wms_auth
 
 
 def main():
     parser = argparse.ArgumentParser()
+    parser.add_argument("-v", "--version", help="show version", action="store_true", default=False)
     parser.add_argument("--host", help="hostname",
                         default="127.0.0.1", dest="host")
     parser.add_argument("--port", help="port", dest="port", default="8081")
@@ -21,6 +47,14 @@ def main():
                         default=None)
     args = parser.parse_args()
 
+    if args.version:
+        print "***********************************************************************"
+        print "\n            Mission Support System (mss)\n"
+        print "***********************************************************************"
+        print "Documentation: http://mss.rtfd.io"
+        print "Version:", __version__
+        sys.exit()
+
     if args.logfile is not None:
         # Log everything to "logfile".
         # TODO: Change this to write to a rotating log handler (so that the file size
@@ -49,13 +83,13 @@ def main():
         realm = 'Mission Support Web Map Service'
 
         def authfunc(environ, username, password):
-            for u, p in mss_wms_settings.allowed_users:
+            for u, p in mss_wms_auth.allowed_users:
                 if (u == username) and (p == hashlib.md5(password).hexdigest()):
                     return True
             return False
 
         application = AuthBasicHandler(application, realm, authfunc)
-    logging.info("Configuration File: %s" % mss_wms_settings.__file__)
+    logging.info(u"Configuration File: '{}'".format(mss_wms_settings.__file__))
     paste.httpserver.serve(application, host=args.host, port=args.port, use_threadpool=args.use_threadpool)
 
 if __name__ == '__main__':
