@@ -1,65 +1,46 @@
-"""WSGI module for MSS WMS server that provides access to ECMWF forecast data.
+# -*- coding: utf-8 -*-
+"""
 
-********************************************************************************
+    mslib.mswms.wms
+    ~~~~~~~~~~~~~~~
 
-   Copyright 2008-2014 Deutsches Zentrum fuer Luft- und Raumfahrt e.V.
+    WSGI module for MSS WMS server that provides access to ECMWF forecast data.
 
-   Licensed under the Apache License, Version 2.0 (the "License");
-   you may not use this file except in compliance with the License.
-   You may obtain a copy of the License at
+    The module implements a Web Map Service 1.1.1 interface to provide forecast data
+    from numerical weather predictions to the Mission Support User Interface.
+    Supported operations are GetCapabilities and GetMap for (WMS 1.1.1 compliant)
+    maps and (non-compliant) vertical sections.
+
+    1) Configure the WMS server by modifying the settings in mss_wms_settings.py
+    (address, products that shall be offered, ..).
+
+    2) If you want to define new visualisation styles, the files to put them
+    are mpl_hsec_styles.py and mpl_vsec_styles for maps and vertical sections,
+    respectively.
+
+    For more information on WMS, see http://www.opengeospatial.org/standards/wms
+
+    This file is part of mss.
+
+    :copyright: Copyright 2008-2014 Deutsches Zentrum fuer Luft- und Raumfahrt e.V.
+    :copyright: Copyright 2011-2014 Marc Rautenhaus (mr), Omar Qunsul (oq)
+    :copyright: Copyright 2016-2017 Reimar Bauer
+    :copyright: Copyright 2016-2017 by the mss team, see AUTHORS.
+    :license: APACHE-2.0, see LICENSE for details.
+
+    Licensed under the Apache License, Version 2.0 (the "License");
+    you may not use this file except in compliance with the License.
+    You may obtain a copy of the License at
 
        http://www.apache.org/licenses/LICENSE-2.0
 
-   Unless required by applicable law or agreed to in writing, software
-   distributed under the License is distributed on an "AS IS" BASIS,
-   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-   See the License for the specific language governing permissions and
-   limitations under the License.
-
-********************************************************************************
-
-   When using this software, please acknowledge its use by citing the
-   reference documentation in any publication, presentation, report,
-   etc. you create:
-
-   Rautenhaus, M., Bauer, G., and Doernbrack, A.: A web service based tool
-   to plan atmospheric research flights, Geosci. Model Dev., 5, 55-71,
-   doi:10.5194/gmd-5-55-2012, 2012.
-
-********************************************************************************
-
-The module implements a Web Map Service 1.1.1 interface to provide forecast data
-from numerical weather predictions to the Mission Support User Interface.
-Supported operations are GetCapabilities and GetMap for (WMS 1.1.1 compliant)
-maps and (non-compliant) vertical sections.
-
-For more information on WMS, see
-       http://www.opengeospatial.org/standards/wms
-
-
-For an introduction on WSGI and Paste, see
-       http://pythonpaste.org/do-it-yourself-framework.html
-
-A good starting point to understand the program is to look at the main
-program at the end of the file.
-
-USAGE:
-======
-
-1) Configure the WMS server by modifying the settings in mss_wms_settings.py
-(address, products that shall be offered, ..).
-
-2) If you want to define new visualisation styles, the files to put them
-are mpl_hsec_styles.py and mpl_vsec_styles for maps and vertical sections,
-respectively.
-
-AUTHORS:
-========
-
-* Marc Rautenhaus (mr)
-* Omar Qunsul (oq)
-
+    Unless required by applicable law or agreed to in writing, software
+    distributed under the License is distributed on an "AS IS" BASIS,
+    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+    See the License for the specific language governing permissions and
+    limitations under the License.
 """
+
 import os
 import logging
 from datetime import datetime
@@ -73,8 +54,8 @@ from chameleon import PageTemplateLoader
 
 try:
     import mss_wms_settings
-except ImportError, e:
-    logging.warning("Couldn't import mss_wms_settings (ImportError:'{}'), creating dummy config.".format(e))
+except ImportError, ex:
+    logging.warning(u"Couldn't import mss_wms_settings (ImportError:'{}'), creating dummy config.".format(ex))
 
     class mss_wms_settings(object):
         base_dir = os.path.abspath(os.path.dirname(__file__))
@@ -94,15 +75,25 @@ except ImportError, e:
         service_access_constraints = "This service is intended for research purposes only."
         register_horizontal_layers = []
         register_vertical_layers = []
-        enable_basic_http_authentication = False
         nwpaccess = {
             "ecmwf_NH_LL05": os.path.join(tempfile.gettempdir(), "mss/grid/ecmwf/netcdf")
         }
+        enable_basic_http_authentication = False
+        __file__ = None
+
+try:
+    import mss_wms_auth
+except ImportError, ex:
+    logging.warning(u"Couldn't import mss_wms_auth (ImportError:'{}'), creating dummy config.".format(ex))
+
+    class mss_wms_auth(object):
+        allowed_users = [("mswms", "add_md5_digest_of_PASSWORD_here"),
+                         ("add_new_user_here", "add_md5_digest_of_PASSWORD_here")]
         __file__ = None
 
 from mslib.mswms import mss_plot_driver
 
-from mslib.mswms.case_insensitive_multi_dict import CaseInsensitiveMultiDict
+from mslib.utils import CaseInsensitiveMultiDict
 
 # Logging the Standard Output, which will be added to the Apache Log Files
 logging.basicConfig(level=logging.DEBUG,
@@ -143,15 +134,15 @@ class WMSServer(object):
         # instances with the datasets.
         for dataset in datasets:
             layer = layer_class()
-            logging.debug("registering horizontal section layer %s with "
-                          "dataset %s", layer.name, dataset)
+            logging.debug("registering horizontal section layer '{}' with "
+                          "dataset '{}'".format(layer.name, dataset))
             # Check if the current dataset has already been registered. If
             # not, check whether a suitable driver is available.
             if dataset not in self.hsec_layer_registry.keys():
                 if dataset in self.hsec_drivers.keys():
                     self.hsec_layer_registry[dataset] = {}
                 else:
-                    raise ValueError("dataset %s not available" % dataset)
+                    raise ValueError(u"dataset '{}' not available".format(dataset))
             layer.set_driver(self.hsec_drivers[dataset])
             self.hsec_layer_registry[dataset][layer.name] = layer
 
@@ -165,15 +156,14 @@ class WMSServer(object):
         # instances with the datasets.
         for dataset in datasets:
             layer = layer_class()
-            logging.debug("registering vertical section layer %s with "
-                          "dataset %s", layer.name, dataset)
+            logging.debug(u"registering vertical section layer '{}' with dataset '{}'".format(layer.name, dataset))
             # Check if the current dataset has already been registered. If
             # not, check whether a suitable driver is available.
             if dataset not in self.vsec_layer_registry.keys():
                 if dataset in self.vsec_drivers.keys():
                     self.vsec_layer_registry[dataset] = {}
                 else:
-                    raise ValueError("dataset %s not available" % dataset)
+                    raise ValueError(u"dataset '{}' not available".format(dataset))
             layer.set_driver(self.vsec_drivers[dataset])
             self.vsec_layer_registry[dataset][layer.name] = layer
 
@@ -181,9 +171,8 @@ class WMSServer(object):
         """Returns True if the given parameter is an XML that contains
            a service exception.
         """
-        if isinstance(var, str) or isinstance(var, unicode):
-            if var.find("<ServiceException") > 1:
-                return True
+        if isinstance(var, basestring) and var.find("<ServiceException") > 1:
+            return True
         return False
 
     def service_exception(self, code=None, text=""):
@@ -203,19 +192,30 @@ class WMSServer(object):
 
     def get_capabilities(self, server_url=None):
         template = templates['get_capabilities.pt']
-        logging.debug("server-url {0}".format(server_url))
+        logging.debug("server-url '{0}'".format(server_url))
 
         # Horizontal Layers
         hsec_layers = []
         for dataset in self.hsec_layer_registry.keys():
             for layer in self.hsec_layer_registry[dataset].values():
+                if layer.uses_time_dimensions() and len(layer.get_init_times()) == 0:
+                    logging.error("layer %s/%s has no init times!", layer, dataset)
+                    continue
+                if layer.uses_time_dimensions() and len(layer.get_all_valid_times()) == 0:
+                    logging.error("layer %s/%s has no valid times!", layer, dataset)
+                    continue
                 hsec_layers.append((dataset, layer))
 
         # Vertical Layers
         vsec_layers = []
-
         for dataset in self.vsec_layer_registry.keys():
             for layer in self.vsec_layer_registry[dataset].values():
+                if layer.uses_time_dimensions() and len(layer.get_init_times()) == 0:
+                    logging.error("layer %s/%s has no init times!", layer, dataset)
+                    continue
+                if layer.uses_time_dimensions() and len(layer.get_all_valid_times()) == 0:
+                    logging.error("layer %s/%s has no valid times!", layer, dataset)
+                    continue
                 vsec_layers.append((dataset, layer))
 
         # return_format = 'text/xml'
@@ -228,6 +228,8 @@ class WMSServer(object):
                                service_contact_person=mss_wms_settings.__dict__.get("service_contact_person", ""),
                                service_contact_organisation=mss_wms_settings.__dict__.get(
                                    "service_contact_organisation", ""),
+                               service_contact_position=mss_wms_settings.__dict__.get("service_contact_position", ""),
+                               service_email=mss_wms_settings.__dict__.get("service_email", ""),
                                service_address_type=mss_wms_settings.__dict__.get("service_address_type", ""),
                                service_address=mss_wms_settings.__dict__.get("service_address", ""),
                                service_city=mss_wms_settings.__dict__.get("service_city", ""),
@@ -241,8 +243,9 @@ class WMSServer(object):
         return return_data
 
     def produce_plot(self, environ, mode):
-        """Handler for a GetMap and GetVSec requests. Produces a plot with
-           the parameters specified in the URL.
+        """
+        Handler for a GetMap and GetVSec requests. Produces a plot with
+        the parameters specified in the URL.
 
         # TODO: Handle multiple layers. (mr, 2010-06-09)
         # TODO: Cache the produced images: Check whether an image with the given
@@ -259,7 +262,7 @@ class WMSServer(object):
 
         # Image size.
         figsize = float(query.get('WIDTH', 900)), float(query.get('HEIGHT', 600))
-        logging.debug("  requested image size = %ix%i" % (figsize[0], figsize[1]))
+        logging.debug("  requested image size = {:}x{:}".format(figsize[0], figsize[1]))
 
         # Requested layers.
         layers = [layer for layer in query.get('LAYERS', '').strip().split(',') if layer]
@@ -268,15 +271,15 @@ class WMSServer(object):
             dataset, layer = layer.split(".")
         else:
             dataset = None
-        logging.debug("  requested dataset = %s, layer = %s" % (dataset, layer))
+        logging.debug(u"  requested dataset = '{}', layer = '{}'".format(dataset, layer))
 
         # Requested style(s).
         styles = [style for style in query.get('STYLES', 'default').strip().split(',') if style]
         style = styles[0] if len(styles) > 0 else None
-        logging.debug("  requested style = %s" % style)
+        logging.debug(u"  requested style = '{}'".format(style))
 
         # Forecast initialisation time.
-        init_time = query.get('DIM_INIT_TIME', None)
+        init_time = query.get('DIM_INIT_TIME')
         if init_time is not None:
             try:
                 init_time = datetime.strptime(init_time, "%Y-%m-%dT%H:%M:%SZ")
@@ -284,10 +287,10 @@ class WMSServer(object):
                 return self.service_exception(code="InvalidDimensionValue",
                                               text="DIM_INIT_TIME has wrong format "
                                               "(needs to be 2005-08-29T13:00:00Z)")
-        logging.debug("  requested initialisation time = %s" % init_time)
+        logging.debug(u"  requested initialisation time = '{}'".format(init_time))
 
         # Forecast valid time.
-        valid_time = query.get('TIME', None)
+        valid_time = query.get('TIME')
         if valid_time is not None:
             try:
                 valid_time = datetime.strptime(valid_time, "%Y-%m-%dT%H:%M:%SZ")
@@ -295,7 +298,7 @@ class WMSServer(object):
                 return self.service_exception(code="InvalidDimensionValue",
                                               text="TIME has wrong format "
                                               "(needs to be 2005-08-29T13:00:00Z)")
-        logging.debug("  requested (valid) time = %s" % valid_time)
+        logging.debug(u"  requested (valid) time = '{}'".format(valid_time))
 
         # Coordinate reference system.
         crs = query.get('SRS', 'EPSG:4326')
@@ -308,15 +311,15 @@ class WMSServer(object):
             try:
                 epsg = int(crs[5:])
             except ValueError:
-                msg = "The requested CRS %s is not supported." % crs
+                msg = u"The requested CRS '{}' is not supported.".format(crs)
         else:
-            msg = "The requested CRS %s is not supported." % crs
+            msg = u"The requested CRS '{}' is not supported.".format(crs)
 
         if msg is not None:
             logging.error(msg)
             return self.service_exception(code="InvalidSRS", text=msg)
         else:
-            logging.debug("  requested coordinate reference system = %s" % crs)
+            logging.debug(u"  requested coordinate reference system = {}".format(crs))
 
         # Create a frameless figure (WMS) or one with title and legend
         # (MSS specific)? Default is WMS mode (frameless).
@@ -329,9 +332,9 @@ class WMSServer(object):
 
         # Return format (image/png, text/xml, etc.).
         return_format = query.get('FORMAT', 'IMAGE/PNG').lower()
-        logging.debug("  requested return format = %s" % return_format)
+        logging.debug(u"  requested return format = '{}'".format(return_format))
         if return_format not in ["image/png", "text/xml"]:
-            msg = "unsupported FORMAT: %s" % return_format
+            msg = u"unsupported FORMAT: {}".format(return_format)
             logging.error(msg)
             return self.service_exception(code="InvalidFORMAT", text=msg)
 
@@ -344,7 +347,7 @@ class WMSServer(object):
             # Check requested layer.
             if (dataset not in self.hsec_layer_registry.keys()) or \
                (layer not in self.hsec_layer_registry[dataset].keys()):
-                msg = "Invalid LAYER '%s.%s' requested" % (dataset, layer)
+                msg = u"Invalid LAYER '{}.{}' requested".format(dataset, layer)
                 logging.error(msg)
                 return self.service_exception(code="LayerNotDefined", text=msg)
 
@@ -361,8 +364,8 @@ class WMSServer(object):
                     return self.service_exception(code="MissingDimensionValue", text=msg)
 
             # Check if the requested coordinate system is supported.
-            if epsg not in self.hsec_layer_registry[dataset][layer].supported_epsg_codes():
-                msg = "The requested CRS EPSG:%i is not supported." % epsg
+            if not self.hsec_layer_registry[dataset][layer].support_epsg_code(epsg):
+                msg = u"The requested CRS 'EPSG:{:d}' is not supported.".format(epsg)
                 logging.error(msg)
                 return self.service_exception(code="InvalidSRS", text=msg)
 
@@ -371,12 +374,12 @@ class WMSServer(object):
                 bbox = [float(v) for v in query.get('BBOX',
                                                     '-180,-90,180,90').split(',')]
             except ValueError:
-                msg = "Invalid BBOX: %s" % query.get("BBOX")
+                msg = u"Invalid BBOX: {}".format(query.get("BBOX"))
                 logging.error(msg)
                 return self.service_exception(text=msg)
 
             # Vertical level, if applicable.
-            level = query.get('ELEVATION', None)
+            level = query.get('ELEVATION')
             level = int(level) if level else None
             layer_datatypes = self.hsec_layer_registry[dataset][layer].required_datatypes()
             if (("ml" in layer_datatypes) or ("pl" in layer_datatypes)) \
@@ -385,8 +388,8 @@ class WMSServer(object):
                 level = -1
             elif ("sfc" in layer_datatypes) and ("ml" not in layer_datatypes) \
                     and ("pl" not in layer_datatypes) and level:
-                msg = "ELEVATION argument not applicable for layer "\
-                      "%s. Please omit this argument." % layer
+                msg = u"ELEVATION argument not applicable for layer " \
+                      u"'{}'. Please omit this argument.".format(layer)
                 logging.error(msg)
                 return self.service_exception(text=msg)
 
@@ -403,33 +406,33 @@ class WMSServer(object):
                                                 noframe=noframe,
                                                 transparent=transparent,
                                                 return_format=return_format)
-            except (IOError, ValueError) as e:
-                logging.error("ERROR: %s" % e)
-                msg = "The data corresponding to your request "\
-                      "is not available. Please check the "\
-                      "times and/or levels you have specified."\
-                      "\n\nError message: %s" % e
+            except (IOError, ValueError) as ex:
+                logging.error(u"ERROR: {}".format(ex))
+                msg = u"The data corresponding to your request " \
+                      u"is not available. Please check the " \
+                      u"times and/or levels you have specified." \
+                      u"\n\nError message: '{}'".format(ex)
                 return self.service_exception(text=msg)
 
         elif mode == "GetVSec":
 
             # Vertical secton path.
-            path = query.get("PATH", None)
+            path = query.get("PATH")
             if not path:
                 return self.service_exception(text="PATH not specified")
             try:
                 path = [float(v) for v in path.split(',')]
                 path = [[lat, lon] for lat, lon in zip(path[0::2], path[1::2])]
             except ValueError:
-                msg = "Invalid PATH: %s" % path
+                msg = u"Invalid PATH: {}".format(path)
                 logging.error(msg)
                 return self.service_exception(text=msg)
-            logging.debug("VSEC PATH: %s", path)
+            logging.debug(u"VSEC PATH: {}".format(path))
 
             # Check requested layers.
             if (dataset not in self.vsec_layer_registry.keys()) or \
                     (layer not in self.vsec_layer_registry[dataset].keys()):
-                msg = "Invalid LAYER '%s.%s' requested" % (dataset, layer)
+                msg = u"Invalid LAYER '{}.{}' requested".format(dataset, layer)
                 return self.service_exception(code="LayerNotDefined", text=msg)
 
             # Check if the layer requires time information and if they are given.
@@ -448,7 +451,7 @@ class WMSServer(object):
             try:
                 bbox = [float(v) for v in query.get("BBOX", "101,1050,10,180").split(",")]
             except ValueError:
-                msg = "Invalid BBOX: %s" % query.get("BBOX")
+                msg = u"Invalid BBOX: {}".format(query.get("BBOX"))
                 logging.error(msg)
                 return self.service_exception(text=msg)
 
@@ -467,12 +470,12 @@ class WMSServer(object):
                                                 noframe=noframe,
                                                 transparent=transparent,
                                                 return_format=return_format)
-            except (IOError, ValueError) as e:
-                logging.error("ERROR: %s" % e)
-                msg = "The data corresponding to your request "\
-                      "is not available. Please check the "\
-                      "times and/or path you have specified."\
-                      "\n\nError message: %s" % e
+            except (IOError, ValueError) as ex:
+                logging.error(u"ERROR: {}".format(ex))
+                msg = u"The data corresponding to your request " \
+                      u"is not available. Please check the " \
+                      u"times and/or path you have specified." \
+                      u"\n\nError message: {}".format(ex)
                 return self.service_exception(text=msg)
 
         # Produce the image.
@@ -500,10 +503,10 @@ def application(environ, start_response):
     try:
         # Request info
         query = CaseInsensitiveMultiDict(paste.request.parse_dict_querystring(environ))
-        logging.debug("ENVIRON: %s", environ)
+        logging.debug(u"ENVIRON: {}".format(environ))
 
         # Processing
-        request = str(query.get('request', None))
+        request = str(query.get('request'))
         return_data = ""
         return_format = 'text/plain'
 
@@ -534,9 +537,9 @@ def application(environ, start_response):
 
         return [output]
 
-    except Exception as e:
+    except Exception as ex:
         status = '404 NOT FOUND'
-        error_message = str(type(e)) + ": " + str(e)
+        error_message = u"{}: {}".format(type(ex), ex)
         # ToDo add a config var to disable output, replace by standard text, "Internal Server error"
         error_message = error_message + "\n" + traceback.format_exc()
         logging.error(error_message)
