@@ -1,43 +1,36 @@
-"""Some useful functions for handling NetCDF files with the netCDF4 library.
+# -*- coding: utf-8 -*-
+"""
 
-********************************************************************************
+    mslib.netCDF4tools
+    ~~~~~~~~~~~~~~~~~~
 
-   Copyright 2008-2014 Deutsches Zentrum fuer Luft- und Raumfahrt e.V.
-   Copyright 2011-2014 Marc Rautenhaus
+    Some useful functions for handling NetCDF files with the netCDF4 library.
 
-   Licensed under the Apache License, Version 2.0 (the "License");
-   you may not use this file except in compliance with the License.
-   You may obtain a copy of the License at
+    This file is part of mss.
+
+    :copyright: Copyright 2008-2014 Deutsches Zentrum fuer Luft- und Raumfahrt e.V.
+    :copyright: Copyright 2011-2014 Marc Rautenhaus (mr)
+    :copyright: Copyright 2016-2017 by the mss team, see AUTHORS.
+    :license: APACHE-2.0, see LICENSE for details.
+
+    Licensed under the Apache License, Version 2.0 (the "License");
+    you may not use this file except in compliance with the License.
+    You may obtain a copy of the License at
 
        http://www.apache.org/licenses/LICENSE-2.0
 
-   Unless required by applicable law or agreed to in writing, software
-   distributed under the License is distributed on an "AS IS" BASIS,
-   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-   See the License for the specific language governing permissions and
-   limitations under the License.
-
-********************************************************************************
-
-AUTHORS:
-========
-
-* Marc Rautenhaus (mr)
-
+    Unless required by applicable law or agreed to in writing, software
+    distributed under the License is distributed on an "AS IS" BASIS,
+    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+    See the License for the specific language governing permissions and
+    limitations under the License.
 """
 
-# standard library imports
 import glob
 import re
 
 # related third party imports
 import netCDF4
-import numpy as np
-
-
-"""
-EXCEPTION CLASSES
-"""
 
 
 class NetCDFVariableError(Exception):
@@ -46,10 +39,6 @@ class NetCDFVariableError(Exception):
     pass
 
 
-"""
-CONSTANTS
-"""
-
 NO_FIELDS = 1
 
 CHECK_NONE = 512
@@ -62,13 +51,13 @@ re_datetime = re.compile("(\d{4})-(\d{2})-(\d{2})[ T](\d{2}):(\d{2}):(\d{2})Z")
 # Expression used to identify the time variable by its units string.
 re_timeunits = re.compile("\w+ since \d{4}-\d{1,2}-\d{1,2}.\d{1,2}")
 
-"""
-NETCDF FILE TOOLS
-"""
+
+# NETCDF FILE TOOLS
 
 
 def identify_variable(ncfile, standard_name, check=False):
-    """Identify the variable in ncfile that is described by specified rules.
+    """
+    Identify the variable in ncfile that is described by specified rules.
 
     Arguments:
     ncfile -- Handle to open netCDF4.Dataset().
@@ -82,14 +71,15 @@ def identify_variable(ncfile, standard_name, check=False):
         if "standard_name" in variable.ncattrs() and variable.standard_name == standard_name:
             return var_name, variable
     if check:
-        raise NetCDFVariableError("cannot identify NetCDF variable "
-                                  "specified by <%s>" % standard_name)
+        raise NetCDFVariableError(u"cannot identify NetCDF variable "
+                                  u"specified by <{}>".format(standard_name))
     return None, None
 
 
 def identify_CF_coordhybrid(ncfile, check=CHECK_NONE):
-    """Identify variables representing longitude, latitude, hybrid model levels
-       in ECMWF files.
+    """
+    Identify variables representing longitude, latitude, hybrid model levels
+    in ECMWF files.
 
 
     Returns:
@@ -112,8 +102,9 @@ def identify_CF_coordhybrid(ncfile, check=CHECK_NONE):
 
 
 def hybrid_orientation(hybrid_var):
-    """Returns -1 if orientation of hybrid_var is down (largest value first),
-       otherwise 1.
+    """
+    Returns -1 if orientation of hybrid_var is down (largest value first),
+    otherwise 1.
     """
     hybrid_levels = hybrid_var[:]
     if hybrid_levels[0] > hybrid_levels[-1]:
@@ -127,7 +118,8 @@ def hybrid_orientation(hybrid_var):
 
 
 def identify_CF_hybrid(ncfile):
-    """Identify the vertical variable from a CF-compliant NetCDF file.
+    """
+    Identify the vertical variable from a CF-compliant NetCDF file.
 
     Returns: hybrid_name, hybrid_var, hybrid_orientation
     """
@@ -140,7 +132,8 @@ def identify_CF_hybrid(ncfile):
 
 
 def identify_CF_isopressure(ncfile):
-    """Identify the vertical variable from a CF-compliant NetCDF file.
+    """
+    Identify the vertical variable from a CF-compliant NetCDF file.
 
     Returns: isopressure_name, isopressure_var, isopressure_orientation
     """
@@ -154,7 +147,8 @@ def identify_CF_isopressure(ncfile):
 
 
 def identify_CF_isopotvort(ncfile, isopotvortname_override=None):
-    """Identify the vertical variable from a CF-compliant NetCDF file.
+    """
+    Identify the vertical variable from a CF-compliant NetCDF file.
 
     Returns: isopotvort_name, isopotvort_var, isopotvort_orientation
     """
@@ -167,7 +161,8 @@ def identify_CF_isopotvort(ncfile, isopotvortname_override=None):
 
 
 def identify_CF_isoaltitude(ncfile):
-    """Identify the vertical variable from a CF-compliant NetCDF file.
+    """
+    Identify the vertical variable from a CF-compliant NetCDF file.
 
     Returns: isoaltitude_name, isoaltiude_var, isoalttiude_orientation
     """
@@ -179,7 +174,8 @@ def identify_CF_isoaltitude(ncfile):
 
 
 def identify_CF_isopottemp(ncfile):
-    """Identify the vertical variable from a CF-compliant NetCDF file.
+    """
+    Identify the vertical variable from a CF-compliant NetCDF file.
 
     Returns: isoaltitude_name, isoaltiude_var, isoalttiude_orientation
     """
@@ -192,7 +188,8 @@ def identify_CF_isopottemp(ncfile):
 
 
 def identify_CF_time(ncfile):
-    """Identify the time variable from a CF-compliant NetCDF file.
+    """
+    Identify the time variable from a CF-compliant NetCDF file.
 
     From the CF-conventions document: 'A time coordinate is identifiable
     from its units string alone'. This method tries to match identify the
@@ -207,7 +204,8 @@ def identify_CF_time(ncfile):
 
 
 def identify_CF_ensemble(ncfile, ensname_override=None):
-    """Identify the ensemble dimension from a CF-compliant NetCDF file.
+    """
+    Identify the ensemble dimension from a CF-compliant NetCDF file.
 
     Returns: ens_name, ens_var
     """
@@ -221,8 +219,9 @@ def identify_CF_ensemble(ncfile, ensname_override=None):
 
 
 def num2date(times, units, calendar='standard'):
-    """Extension to the netCDF4.num2date() function to correctly handle
-       time strings of format '2010-01-01T00:00:00Z', as used by netcdf-java.
+    """
+    Extension to the netCDF4.num2date() function to correctly handle
+    time strings of format '2010-01-01T00:00:00Z', as used by netcdf-java.
 
     Simply replaces an occurence of '2010-01-01T00:00:00Z' in units by
     '2010-01-01 00:00:00 0:00', which is the CF-compliant format.
@@ -236,7 +235,8 @@ def num2date(times, units, calendar='standard'):
 
 
 def get_latlon_data(ncfile, autoreverse=True):
-    """Get data arrays of latitude and longitude in a NetCDF file.
+    """
+    Get data arrays of latitude and longitude in a NetCDF file.
 
     ncfile needs to be an open netCDF4.Dataset.
 
@@ -274,185 +274,6 @@ def get_latlon_data(ncfile, autoreverse=True):
     return lat_data, lon_data, lat_order
 
 
-"""
-COPY NETCDF FILES
-
-# THE FOLLOWING CODE INCLUDES SUBSTANTIAL PARTS TAKEN FROM THE NETCDF4-PYTHON
-# LIBRARY (<http://netcdf4-python.googlecode.com/>).
-#
-# NETCDF4-PYTHON COPYRIGHT INFORMATION:
-#
-# Contact:  Jeffrey Whitaker <jeffrey.s.whitaker@noaa.gov>
-# Copyright: 2008 by Jeffrey Whitaker.
-#
-# License: Permission to use, copy, modify, and distribute this software
-# and its documentation for any purpose and without fee is hereby granted,
-# provided that the above copyright notice appear in all copies and that
-# both the copyright notice and this permission notice appear in supporting
-# documentation. THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
-# SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
-# IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, INDIRECT OR
-# CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
-# DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
-# TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE
-# OF THIS SOFTWARE.
-"""
-
-
-# The following method is taken from the "nc3tonc4" script from the
-# netcdf4-python package. Small adjustments have been made.
-def nc_to_nc4(filename3, filename4, unpackshort=True, zlib=True,
-              complevel=6, shuffle=True, fletcher32=False, clobber=False,
-              lsd_dict=None, nchunk=1000, quiet=False, classic=0,
-              exclude_vars=[], exclude_dims=[]):
-    """convert a netcdf 3 file (filename3) to a netcdf 4 file
-
-    The default format is 'NETCDF4', but can be set
-    to NETCDF4_CLASSIC if classic=1.
-    If unpackshort=True, variables stored as short
-    integers with a scale and offset are unpacked to floats.
-    in the netcdf 4 file.  If the lsd_dict is not None, variable names
-    corresponding to the keys of the dict will be truncated to the decimal place
-    specified by the values of the dict.  This improves compression by
-    making it 'lossy'..
-    The zlib, complevel and shuffle keywords control
-    how the compression is done.
-
-    exclude_vars can be a list of variables that should NOT be copied.
-    """
-
-    ncfile3 = netCDF4.Dataset(filename3, 'r')
-    if classic:
-        ncfile4 = netCDF4.Dataset(filename4, 'w', clobber=clobber, format='NETCDF4_CLASSIC')
-    else:
-        ncfile4 = netCDF4.Dataset(filename4, 'w', clobber=clobber, format='NETCDF4')
-    mval = 1.e30  # missing value if unpackshort=True
-    # create dimensions. Check for unlimited dim.
-    unlimdimname = False
-    unlimdim = None
-    # create global attributes.
-    if not quiet:
-        print 'copying global attributes ..'
-    for attname in ncfile3.ncattrs():
-        if attname == 'file_format':
-            # Attribute 'file_format' fails as it is a reserved keyword.
-            # Workaround 2010/01/26 Marc Rautenhaus
-            setattr(ncfile4, 'orig_file_format', getattr(ncfile3, attname))
-        else:
-            setattr(ncfile4, attname, getattr(ncfile3, attname))
-    if not quiet:
-        print 'copying dimensions ..'
-    for dimname, dim in ncfile3.dimensions.iteritems():
-        if dimname in exclude_dims:
-            if not quiet:
-                print 'skipping dimension', dimname
-            continue
-        if dim.isunlimited():
-            unlimdimname = dimname
-            unlimdim = dim
-            ncfile4.createDimension(dimname, None)
-        else:
-            ncfile4.createDimension(dimname, len(dim))
-    # create variables.
-    for varname, ncvar in ncfile3.variables.iteritems():
-        # Check whether variable should be skipped. (mr, 2010/01/26)
-        if exclude_vars == NO_FIELDS:
-            # Skip all variables that have two or more dimensions.
-            if len(ncvar.dimensions) > 1:
-                if not quiet:
-                    print 'skipping variable', varname
-                continue
-        if (isinstance(exclude_vars, list) and varname in exclude_vars) \
-                or varname in exclude_dims:
-            if not quiet:
-                print 'skipping variable', varname
-            continue
-        if not quiet:
-            print 'copying variable', varname
-        # quantize data?
-        if lsd_dict is not None and varname in lsd_dict:
-            lsd = lsd_dict[varname]
-            if not quiet:
-                print 'truncating to least_significant_digit =', lsd
-        else:
-            lsd = None  # no quantization.
-        # unpack short integers to floats?
-        if unpackshort and hasattr(ncvar, 'scale_factor') and hasattr(ncvar, 'add_offset'):
-            dounpackshort = True
-            datatype = 'f4'
-        else:
-            dounpackshort = False
-            datatype = ncvar.dtype
-        # is there an unlimited dimension?
-        if unlimdimname and unlimdimname in ncvar.dimensions:
-            hasunlimdim = True
-        else:
-            hasunlimdim = False
-        if dounpackshort:
-            if not quiet:
-                print 'unpacking short integers to floats ...'
-        var = ncfile4.createVariable(varname, datatype, ncvar.dimensions,
-                                     least_significant_digit=lsd,
-                                     zlib=zlib, complevel=complevel,
-                                     shuffle=shuffle, fletcher32=fletcher32)
-        # fill variable attributes.
-        for attname in ncvar.ncattrs():
-            if dounpackshort and attname in ['add_offset', 'scale_factor']:
-                continue
-            if dounpackshort and attname == 'missing_value':
-                setattr(var, attname, mval)
-            else:
-                setattr(var, attname, getattr(ncvar, attname))
-        # fill variables with data.
-        if hasunlimdim:  # has an unlim dim, loop over unlim dim index.
-            # range to copy
-            if nchunk:
-                start = 0
-                stop = len(unlimdim)
-                step = nchunk
-                if step < 1:
-                    step = 1
-                for n in range(start, stop, step):
-                    nmax = n + nchunk
-                    if nmax > len(unlimdim):
-                        nmax = len(unlimdim)
-                    idata = ncvar[n:nmax]
-                    if dounpackshort:
-                        tmpdata = (ncvar.scale_factor * idata.astype('f') + ncvar.add_offset).astype('f')
-                        if hasattr(ncvar, 'missing_value'):
-                            tmpdata = np.where(idata == ncvar.missing_value, mval, tmpdata)
-                    else:
-                        tmpdata = idata
-                    var[n:nmax] = tmpdata
-            else:
-                idata = ncvar[:]
-                if dounpackshort:
-                    tmpdata = (ncvar.scale_factor * idata.astype('f') + ncvar.add_offset).astype('f')
-                    if hasattr(ncvar, 'missing_value'):
-                        tmpdata = np.where(idata == ncvar.missing_value, mval, tmpdata)
-                else:
-                    tmpdata = idata
-                var[0:len(unlimdim)] = tmpdata
-        else:  # no unlim dim or 1-d variable, just copy all data at once.
-            idata = ncvar[:]
-            if dounpackshort:
-                tmpdata = (ncvar.scale_factor * idata.astype('f') + ncvar.add_offset).astype('f')
-                if hasattr(ncvar, 'missing_value'):
-                    tmpdata = np.where(idata == ncvar.missing_value, mval, tmpdata)
-            else:
-                tmpdata = idata
-            var[:] = tmpdata
-        ncfile4.sync()  # flush data to disk
-    # close files.
-    ncfile3.close()
-    ncfile4.close()
-
-
-"""
-MFDataset_CommonDims
-"""
-
-
 class MFDatasetCommonDims(netCDF4.MFDataset):
     """MFDatasetCommonDims(self, files, exclude=[], requireDimNum=False)
 
@@ -468,7 +289,8 @@ class MFDatasetCommonDims(netCDF4.MFDataset):
 
     def __init__(self, files, exclude=[], skipDimCheck=[],
                  requireDimNum=False):
-        """Open a Dataset spanning multiple files sharing common dimensions but
+        """
+        Open a Dataset spanning multiple files sharing common dimensions but
         containing different record variables, making it look as if it was a
         single file.
 
@@ -532,7 +354,7 @@ class MFDatasetCommonDims(netCDF4.MFDataset):
             cdfVar[vName] = v
             cdfOrigin[vName] = (master, cdfm)
         if len(cdfVar) == 0:
-            raise IOError("master dataset %s does not have any variable" % master)
+            raise IOError(u"master dataset '{}' does not have any variable".format(master))
 
         # Open each remaining file in read-only mode.
         # Make sure each file defines the same record variables as the master
@@ -544,16 +366,16 @@ class MFDatasetCommonDims(netCDF4.MFDataset):
                 # (..except those that shall not be tested..)
                 if dimName not in skipDimCheck:
                     if dimName not in masterDims:
-                        raise IOError("dimension %s not defined in master %s" % (dimName, master))
+                        raise IOError(u"dimension '{}' not defined in master '{}'".format(dimName, master))
                     if len(part.dimensions[dimName]) != len(cdfm.dimensions[dimName]) or \
                             (part.variables[dimName][:] != cdfm.variables[dimName][:]).any():
-                        raise IOError("dimension %s differs in master %s and "
-                                      "file %s" % (dimName, master, f))
+                        raise IOError(u"dimension '{}' differs in master '{}' and "
+                                      u"file '{}'".format(dimName, master, f))
 
             if requireDimNum:
                 if len(part.dimensions) != len(masterDims):
-                    raise IOError("number of dimensions not consistent in master "
-                                  "%s and %s" % (master, f))
+                    raise IOError(u"number of dimensions not consistent in master "
+                                  u"'{}' and '{}'".format(master, f))
 
             for vName, v in part.variables.items():
                 # Exclude dimension variables.
@@ -573,10 +395,10 @@ class MFDatasetCommonDims(netCDF4.MFDataset):
 
         self._file_format = []
         for dset in self._cdf:
-            if dset.file_format == 'NETCDF4':
-                raise ValueError('MFNetCDF4 only works with NETCDF3_CLASSIC, '
-                                 'NETCDF3_64BIT and NETCDF4_CLASSIC '
-                                 'formatted files, not NETCDF4')
+            if dset.file_format == "NETCDF4":
+                raise ValueError("MFNetCDF4 only works with NETCDF3_CLASSIC, "
+                                 "NETCDF3_64BIT and NETCDF4_CLASSIC "
+                                 "formatted files, not NETCDF4")
             self._file_format.append(dset.file_format)
 
     def getOriginFile(self, varname):
