@@ -28,6 +28,7 @@ import cgi
 from urllib import urlencode
 from etree import etree
 from .util import openURL
+from collections import OrderedDict
 
 
 class ServiceException(Exception):
@@ -127,7 +128,7 @@ class WebMapService(object):
             self.contents[cm.id]=cm       
             for subelem in elem.findall('Layer'):
                 subcm=ContentMetadata(subelem, cm)
-                self.contents[subcm.id]=subcm 
+                self.contents[subcm.id]=subcm
         
         #exceptions
         self.exceptions = [f.text for f \
@@ -276,6 +277,10 @@ class ServiceIdentification(object):
         fees = self._root.find('Fees')
         if fees is not None:
             self.fees = fees.text
+        # (mss) Always set a default value similar to abstract.
+        else:
+            self.fees = None
+        # (mss)
              
 class ServiceProvider(object):
     ''' Implements IServiceProviderMetatdata '''
@@ -323,7 +328,8 @@ class ContentMetadata:
         # (mss) Added "Abstract".
         for key in ('Name', 'Title', 'Abstract'):
             val = elem.find(key)
-            if val is not None:
+            # (mss) Added " and val.text is not None".
+            if val is not None and val.text is not None:
                 setattr(self, key.lower(), val.text.strip())
             else:
                 setattr(self, key.lower(), None)
@@ -404,14 +410,15 @@ class ContentMetadata:
             self.crsOptions=None
             
         #Styles
-        self.styles = {}
+        self.styles = OrderedDict()
         
         #Copy any parent styles (they are inheritable properties)
         if self.parent:
             self.styles = self.parent.styles.copy()
- 
+
         #Get the styles for this layer (items with the same name are replaced)
         for s in elem.findall('Style'):
+
             name = s.find('Name')
             title = s.find('Title')
             if name is None or title is None:
