@@ -1,3 +1,31 @@
+# -*- coding: utf-8 -*-
+"""
+
+    mslib.plugins.io.csv
+    ~~~~~~~~~~~~~~~~~~~~
+
+    plugin for csv format flight track export
+
+    This file is part of mss.
+
+    :copyright: Copyright 2008-2014 Deutsches Zentrum fuer Luft- und Raumfahrt e.V.
+    :copyright: Copyright 2011-2014 Marc Rautenhaus (mr)
+    :copyright: Copyright 2016-2017 by the mss team, see AUTHORS.
+    :license: APACHE-2.0, see LICENSE for details.
+
+    Licensed under the Apache License, Version 2.0 (the "License");
+    you may not use this file except in compliance with the License.
+    You may obtain a copy of the License at
+
+       http://www.apache.org/licenses/LICENSE-2.0
+
+    Unless required by applicable law or agreed to in writing, software
+    distributed under the License is distributed on an "AS IS" BASIS,
+    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+    See the License for the specific language governing permissions and
+    limitations under the License.
+"""
+
 from __future__ import absolute_import
 import csv
 import mslib.msui.flighttrack as ft
@@ -8,18 +36,18 @@ def save_to_csv(filename, name, waypoints):
         raise ValueError("filename to save flight track cannot be None")
     with open(filename, "w") as out_file:
         csv_writer = csv.writer(out_file, dialect='excel', delimiter=";", lineterminator="\n")
-        csv_writer.writerow([name])
+        csv_writer.writerow([name.encode("ascii", "replace")])
         csv_writer.writerow(["Index", "Location", "Lat (+-90)", "Lon (+-180)", "Flightlevel", "Pressure (hPa)",
                              "Leg dist. (km)", "Cum. dist. (km)", "Comments"])
         for i, wp in enumerate(waypoints):
-            loc = str(wp.location)
+            loc = unicode(wp.location).encode("ascii", "replace")
             lat = "{:.3f}".format(wp.lat)
             lon = "{:.3f}".format(wp.lon)
             lvl = "{:.3f}".format(wp.flightlevel)
             pre = "{:.3f}".format(wp.pressure / 100.)
             leg = "{:.3f}".format(wp.distance_to_prev)
             cum = "{:.3f}".format(wp.distance_total)
-            com = str(wp.comments)
+            com = unicode(wp.comments).encode("ascii", "replace")
             csv_writer.writerow([i, loc, lat, lon, lvl, pre, leg, cum, com])
 
 
@@ -27,6 +55,8 @@ def load_from_csv(filename):
     waypoints = []
     with open(filename, "r") as in_file:
         lines = in_file.readlines()
+    if len(lines) < 4:
+        raise SyntaxError("CSV file requires at least 4 lines!")
     dialect = csv.Sniffer().sniff(lines[-1])
     csv_reader = csv.reader(lines, dialect=dialect)
     name = csv_reader.next()[0]
