@@ -1,44 +1,37 @@
-"""Collection of thermodynamic functions.
+# -*- coding: utf-8 -*-
+"""
 
-********************************************************************************
+    mslib.thermolib
+    ~~~~~~~~~~~~~~~~
 
-   Copyright 2008-2014 Deutsches Zentrum fuer Luft- und Raumfahrt e.V.
-   Copyright 2011-2014 Marc Rautenhaus
+    Collection of thermodynamic functions.
 
-   Licensed under the Apache License, Version 2.0 (the "License");
-   you may not use this file except in compliance with the License.
-   You may obtain a copy of the License at
+    This file is part of mss.
+
+    :copyright: Copyright 2008-2014 Deutsches Zentrum fuer Luft- und Raumfahrt e.V.
+    :copyright: Copyright 2011-2014 Marc Rautenhaus (mr)
+    :copyright: Copyright 2016-2017 by the mss team, see AUTHORS.
+    :license: APACHE-2.0, see LICENSE for details.
+
+    Licensed under the Apache License, Version 2.0 (the "License");
+    you may not use this file except in compliance with the License.
+    You may obtain a copy of the License at
 
        http://www.apache.org/licenses/LICENSE-2.0
 
-   Unless required by applicable law or agreed to in writing, software
-   distributed under the License is distributed on an "AS IS" BASIS,
-   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-   See the License for the specific language governing permissions and
-   limitations under the License.
-
-********************************************************************************
-
-The function sat_vapour_pressure() has been ported from the IDL function
-'VaporPressure' by Holger Voemel, available at
-              http://cires.colorado.edu/~voemel/vp.html.
-
-AUTHORS:
-========
-
-* Marc Rautenhaus (mr)
-
+    Unless required by applicable law or agreed to in writing, software
+    distributed under the License is distributed on an "AS IS" BASIS,
+    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+    See the License for the specific language governing permissions and
+    limitations under the License.
 """
 
-# related third party imports
+# The function sat_vapour_pressure() has been ported from the IDL function
+# 'VaporPressure' by Holger Voemel, available at http://cires.colorado.edu/~voemel/vp.html.
+
 import numpy
-import pylab
 import scipy.integrate
 
-"""
-EXCEPTION CLASSES
-"""
-
 
 class VapourPressureError(Exception):
     """Exception class to handle error arising during the computation of vapour
@@ -47,15 +40,11 @@ class VapourPressureError(Exception):
     pass
 
 
-"""
-Vapour Pressure
-"""
-
-
 def sat_vapour_pressure(t, liquid='HylandWexler', ice='GoffGratch',
                         force_phase='None'):
-    """Compute the saturation vapour pressure over liquid water and over ice
-       with a variety of formulations.
+    """
+    Compute the saturation vapour pressure over liquid water and over ice
+    with a variety of formulations.
 
     This function is a direct port from the IDL function 'VaporPressure' by
     Holger Voemel, available at http://cires.colorado.edu/~voemel/vp.html.
@@ -103,18 +92,18 @@ def sat_vapour_pressure(t, liquid='HylandWexler', ice='GoffGratch',
 
     # Get indexes of input temperatures above and below freezing, to select
     # the appropriate method for each temperature.
-    if force_phase == 'ice':
+    if force_phase == "ice":
         idx_ice = ()  # numpy.where(t is not None)
         idx_liq = None
-    elif force_phase == 'liquid':
+    elif force_phase == "liquid":
         idx_liq = ()  # numpy.where(t is not None)
         idx_ice = None
-    elif force_phase == 'None':
+    elif force_phase == "None":
         idx_ice = numpy.where(t <= 273.15)
         idx_liq = numpy.where(t > 273.15)
     else:
-        raise VapourPressureError("Cannot recognize the force_phase "
-                                  "keyword: %s (valid are ice, liquid, None)" % force_phase)
+        raise VapourPressureError(u"Cannot recognize the force_phase "
+                                  u"keyword: '{}' (valid are ice, liquid, None)".format(force_phase))
 
     # Initialise output field.
     e_sat = numpy.zeros(numpy.shape(t))
@@ -293,8 +282,8 @@ def sat_vapour_pressure(t, liquid='HylandWexler', ice='GoffGratch',
                                        0.014025 * t[idx_liq])) / 100.
 
         else:
-            raise VapourPressureError("Unkown method for computing "
-                                      "the vapour pressure curve over liquid: %s" % liquid)
+            raise VapourPressureError(u"Unkown method for computing "
+                                      u"the vapour pressure curve over liquid: {}".format(liquid))
 
     # =============================================================================
     #  Calculate saturation pressure over ice -------------------------------------
@@ -303,7 +292,7 @@ def sat_vapour_pressure(t, liquid='HylandWexler', ice='GoffGratch',
         if ice == 'WMO2000':
             ice = 'WMO_Goff'
 
-        elif ice == 'IAWPS':
+        if ice == 'IAWPS':
             raise VapourPressureError("IAPWS does not provide a vapour "
                                       "pressure formulation over ice")
 
@@ -390,127 +379,12 @@ def sat_vapour_pressure(t, liquid='HylandWexler', ice='GoffGratch',
 
         else:
             raise VapourPressureError("Unkown method for computing "
-                                      "the vapour pressure curve over ice: %s" % ice)
+                                      "the vapour pressure curve over ice: {}".format(ice))
 
     # Convert return value units from hPa to Pa.
     return e_sat * 100. if not input_scalar else e_sat[0] * 100.
 
 
-def test_vapour_pressure():
-    """Make test plots of the saturation vapour pressure curves.
-    """
-
-    # Specify a temperature range in [K].
-    t = numpy.arange(173, 313, 0.1)
-
-    # Compute saturation pressure over liquid water.
-    e_sat_liq_HylandWexler = sat_vapour_pressure(t, liquid='HylandWexler', force_phase='liquid')
-    e_sat_liq_GoffGratch = sat_vapour_pressure(t, liquid='GoffGratch', force_phase='liquid')
-    e_sat_liq_Wexler = sat_vapour_pressure(t, liquid='Wexler', force_phase='liquid')
-    e_sat_liq_MagnusTeten = sat_vapour_pressure(t, liquid='MagnusTeten', force_phase='liquid')
-    e_sat_liq_Buck_original = sat_vapour_pressure(t, liquid='Buck_original', force_phase='liquid')
-    e_sat_liq_Buck_manual = sat_vapour_pressure(t, liquid='Buck_manual', force_phase='liquid')
-    e_sat_liq_WMO_Goff = sat_vapour_pressure(t, liquid='WMO_Goff', force_phase='liquid')
-    e_sat_liq_WMO2000 = sat_vapour_pressure(t, liquid='WMO2000', force_phase='liquid')
-    e_sat_liq_Sonntag = sat_vapour_pressure(t, liquid='Sonntag', force_phase='liquid')
-    e_sat_liq_Bolton = sat_vapour_pressure(t, liquid='Bolton', force_phase='liquid')
-    # e_sat_liq_Fukuta = sat_vapour_pressure(t, liquid='Fukuta', force_phase='liquid')
-    e_sat_liq_IAPWS = sat_vapour_pressure(t, liquid='IAPWS', force_phase='liquid')
-    e_sat_liq_MurphyKoop = sat_vapour_pressure(t, liquid='MurphyKoop', force_phase='liquid')
-
-    # Compute saturation pressure over ice.
-    e_sat_ice_MartiMauersberger = sat_vapour_pressure(t, ice='MartiMauersberger', force_phase='ice')
-    e_sat_ice_HylandWexler = sat_vapour_pressure(t, ice='HylandWexler', force_phase='ice')
-    e_sat_ice_GoffGratch = sat_vapour_pressure(t, ice='GoffGratch', force_phase='ice')
-    e_sat_ice_MagnusTeten = sat_vapour_pressure(t, ice='MagnusTeten', force_phase='ice')
-    e_sat_ice_Buck_original = sat_vapour_pressure(t, ice='Buck_original', force_phase='ice')
-    e_sat_ice_Buck_manual = sat_vapour_pressure(t, ice='Buck_manual', force_phase='ice')
-    e_sat_ice_WMO_Goff = sat_vapour_pressure(t, ice='WMO_Goff', force_phase='ice')
-    e_sat_ice_Sonntag = sat_vapour_pressure(t, ice='Sonntag', force_phase='ice')
-    e_sat_ice_MurphyKoop = sat_vapour_pressure(t, ice='MurphyKoop', force_phase='ice')
-
-    # Plot saturation pressure curves over liquid water.
-    pylab.figure()
-    pylab.plot(t, e_sat_liq_HylandWexler, 'b-')
-    pylab.plot(t, e_sat_liq_GoffGratch, 'b--')
-    pylab.plot(t, e_sat_liq_Wexler, 'g-')
-    pylab.plot(t, e_sat_liq_MagnusTeten, 'g--')
-    pylab.plot(t, e_sat_liq_Buck_original, 'r-')
-    pylab.plot(t, e_sat_liq_Buck_manual, 'r-.')
-    pylab.plot(t, e_sat_liq_WMO_Goff, 'c-')
-    pylab.plot(t, e_sat_liq_WMO2000, 'c:')
-    pylab.plot(t, e_sat_liq_Sonntag, 'm-')
-    pylab.plot(t, e_sat_liq_Bolton, 'm:')
-    # pylab.plot(t, e_sat_liq_Fukuta, 'k-')
-    pylab.plot(t, e_sat_liq_IAPWS, 'k-.')
-    pylab.plot(t, e_sat_liq_MurphyKoop, 'k:')
-    pylab.title("Saturation vapour pressure curves over liquid water.")
-    pylab.xlabel("Temperature [K]")
-    pylab.ylabel("Pressure [Pa]")
-
-    # Plot saturation pressure curves over ice.
-    pylab.figure()
-    pylab.plot(t, e_sat_ice_MartiMauersberger, 'b-')
-    pylab.plot(t, e_sat_ice_HylandWexler, 'b--')
-    pylab.plot(t, e_sat_ice_GoffGratch, 'g-')
-    pylab.plot(t, e_sat_ice_MagnusTeten, 'g:')
-    pylab.plot(t, e_sat_ice_Buck_original, 'r-')
-    pylab.plot(t, e_sat_ice_Buck_manual, 'r-.')
-    pylab.plot(t, e_sat_ice_WMO_Goff, 'c-')
-    pylab.plot(t, e_sat_ice_Sonntag, 'm-')
-    pylab.plot(t, e_sat_ice_MurphyKoop, 'k-')
-    pylab.title("Saturation vapour pressure curves over ice.")
-    pylab.xlabel("Temperature [K]")
-    pylab.ylabel("Pressure [Pa]")
-
-    # Plot deviation in [%] of the liquid water saturation curves to the
-    # GoffGratch curve.
-    def deviation_from_GoffGratch_liquid(e_sat):
-        return 100. * (e_sat - e_sat_liq_GoffGratch) / e_sat_liq_GoffGratch
-
-    pylab.figure()
-    pylab.plot(t, deviation_from_GoffGratch_liquid(e_sat_liq_HylandWexler), 'b-')
-    pylab.plot(t, deviation_from_GoffGratch_liquid(e_sat_liq_GoffGratch), 'b--')
-    pylab.plot(t, deviation_from_GoffGratch_liquid(e_sat_liq_Wexler), 'g-')
-    pylab.plot(t, deviation_from_GoffGratch_liquid(e_sat_liq_MagnusTeten), 'g--')
-    pylab.plot(t, deviation_from_GoffGratch_liquid(e_sat_liq_Buck_original), 'r-')
-    pylab.plot(t, deviation_from_GoffGratch_liquid(e_sat_liq_Buck_manual), 'r-.')
-    pylab.plot(t, deviation_from_GoffGratch_liquid(e_sat_liq_WMO_Goff), 'c-')
-    pylab.plot(t, deviation_from_GoffGratch_liquid(e_sat_liq_WMO2000), 'c:')
-    pylab.plot(t, deviation_from_GoffGratch_liquid(e_sat_liq_Sonntag), 'm-')
-    pylab.plot(t, deviation_from_GoffGratch_liquid(e_sat_liq_Bolton), 'm:')
-    # pylab.plot(t, deviation_from_GoffGratch_liquid(e_sat_liq_Fukuta), 'k-')
-    pylab.plot(t, deviation_from_GoffGratch_liquid(e_sat_liq_IAPWS), 'k-.')
-    pylab.plot(t, deviation_from_GoffGratch_liquid(e_sat_liq_MurphyKoop), 'k:')
-    pylab.title("Saturation vapour pressure curves over liquid water.")
-    pylab.xlabel("Temperature [K]")
-    pylab.ylabel("Deviation from Goff Gratch [%]")
-
-    # Plot deviation in [%] of the ice water saturation curves to the
-    # GoffGratch curve.
-    def deviation_from_GoffGratch_ice(e_sat):
-        return 100. * (e_sat - e_sat_ice_GoffGratch) / e_sat_ice_GoffGratch
-
-    pylab.figure()
-    pylab.plot(t, deviation_from_GoffGratch_ice(e_sat_ice_MartiMauersberger), 'b-')
-    pylab.plot(t, deviation_from_GoffGratch_ice(e_sat_ice_HylandWexler), 'b--')
-    pylab.plot(t, deviation_from_GoffGratch_ice(e_sat_ice_GoffGratch), 'g-')
-    pylab.plot(t, deviation_from_GoffGratch_ice(e_sat_ice_MagnusTeten), 'g:')
-    pylab.plot(t, deviation_from_GoffGratch_ice(e_sat_ice_Buck_original), 'r-')
-    pylab.plot(t, deviation_from_GoffGratch_ice(e_sat_ice_Buck_manual), 'r-.')
-    pylab.plot(t, deviation_from_GoffGratch_ice(e_sat_ice_WMO_Goff), 'c-')
-    pylab.plot(t, deviation_from_GoffGratch_ice(e_sat_ice_Sonntag), 'm-')
-    pylab.plot(t, deviation_from_GoffGratch_ice(e_sat_ice_MurphyKoop), 'k-')
-    pylab.title("Saturation vapour pressure curves over ice.")
-    pylab.xlabel("Temperature [K]")
-    pylab.ylabel("Deviation from Goff Gratch [%]")
-
-
-"""
-Relative Humidity
-"""
-
-
 def rel_hum(p, t, q, liquid='HylandWexler', ice='GoffGratch',
             force_phase='None'):
     """Compute relative humidity in [%] from pressure, temperature, and
@@ -553,14 +427,10 @@ def rel_hum(p, t, q, liquid='HylandWexler', ice='GoffGratch',
     return 100. * w / w_sat
 
 
-"""
-Virtual Temperature
-"""
-
-
 def virt_temp(t, q, method='exact'):
-    """Compute virtual temperature in [K] from temperature and
-       specific humidity.
+    """
+    Compute virtual temperature in [K] from temperature and
+    specific humidity.
 
     Arguments:
     t -- temperature in [K]
@@ -598,11 +468,6 @@ def virt_temp(t, q, method='exact'):
         raise TypeError('virtual temperature method not understood')
 
 
-"""
-Geopotential
-"""
-
-
 def geop_difference(p, t, method='trapz', axis=-1):
     """Compute geopotential difference in [m**2 s**-2] between the pressure
        levels given by the first and last element in p (= pressure).
@@ -654,8 +519,9 @@ def geop_difference(p, t, method='trapz', axis=-1):
 
 
 def geop_thickness(p, t, q=None, cumulative=False, axis=-1):
-    """Compute the geopotential thickness in [m] between the pressure levels
-       given by the first and last element in p (= pressure).
+    """
+    Compute the geopotential thickness in [m] between the pressure levels
+    given by the first and last element in p (= pressure).
 
     Implements the hypsometric equation (1.18) from Holton, 3rd edition (or
     alternatively (3.24) in Wallace and Hobbs, 2nd ed.).
@@ -700,75 +566,10 @@ def geop_thickness(p, t, q=None, cumulative=False, axis=-1):
     return -1. / 9.80665 * geop_difference(p, tv, method='cumtrapz' if cumulative else 'trapz', axis=axis)
 
 
-def test_geop_thickness():
-    """Test geop_thickness() with some values from the 1976 US standard
-       atmosphere.
-    """
-    # Define some std. atmosphere values (height in m, T in K, p in Pa).
-    std_atm_76 = numpy.array([[0, 288.15, 101325],
-                              [500, 284.9, 95460.839342],
-                              [1000, 281.65, 89874.570502],
-                              [1500, 278.4, 84556.004841],
-                              [2000, 275.15, 79495.215511],
-                              [2500, 271.9, 74682.533661],
-                              [3000, 268.65, 70108.54467],
-                              [3500, 265.4, 65764.084371],
-                              [4000, 262.15, 61640.235304],
-                              [4500, 258.9, 57728.32297],
-                              [5000, 255.65, 54019.912104],
-                              [5500, 252.4, 50506.802952],
-                              [6000, 249.15, 47181.027568],
-                              [6500, 245.9, 44034.846117],
-                              [7000, 242.65, 41060.743191],
-                              [7500, 239.4, 38251.424142],
-                              [8000, 236.15, 35599.811423],
-                              [8500, 232.9, 33099.040939],
-                              [9000, 229.65, 30742.45842],
-                              [9500, 226.4, 28523.615797],
-                              [10000, 223.15, 26436.267594],
-                              [10500, 219.9, 24474.367338],
-                              [11000, 216.65, 22632.063973],
-                              [11500, 216.65, 20916.189034],
-                              [12000, 216.65, 19330.405049],
-                              [12500, 216.65, 17864.849029],
-                              [13000, 216.65, 16510.405758],
-                              [13500, 216.65, 15258.6511],
-                              [14000, 216.65, 14101.799606],
-                              [14500, 216.65, 13032.656085],
-                              [15000, 216.65, 12044.570862],
-                              [15500, 216.65, 11131.398413],
-                              [16000, 216.65, 10287.459141],
-                              [16500, 216.65, 9507.504058],
-                              [17000, 216.65, 8786.682132],
-                              [17500, 216.65, 8120.510116],
-                              [18000, 216.65, 7504.844668],
-                              [18500, 216.65, 6935.856576],
-                              [19000, 216.65, 6410.006945],
-                              [19500, 216.65, 5924.025185],
-                              [20000, 216.65, 5474.88867]])
-
-    # Extract p and T arrays.
-    p = std_atm_76[:, 2]
-    print p
-    t = std_atm_76[:, 1]
-    print t
-
-    # Compute geopotential difference and layer thickness. Layer thickness
-    # should be similar to the actual altitude given above.
-    geop = geop_difference(p, t, method='cumtrapz')
-    print geop
-    geop = geop_thickness(p, t, cumulative=True)
-    print geop
-
-
-"""
-Specific Humidity
-"""
-
-
 def spec_hum_from_pTd(p, td, liquid='HylandWexler'):
-    """Computes specific humidity in [kg/kg] from pressure and dew point
-       temperature.
+    """
+    Computes specific humidity in [kg/kg] from pressure and dew point
+    temperature.
 
     Arguments:
     p -- pressure in [Pa]
@@ -798,13 +599,9 @@ def spec_hum_from_pTd(p, td, liquid='HylandWexler'):
     return 0.622 * e_sat / (p + e_sat * (0.622 - 1.))
 
 
-"""
-Dew Point
-"""
-
-
 def dewpoint_approx(p, q, method='Bolton'):
-    """Computes dew point in [K] from pressure and specific humidity.
+    """
+    Computes dew point in [K] from pressure and specific humidity.
 
     Arguments:
     p -- pressure in [Pa]
@@ -839,18 +636,14 @@ def dewpoint_approx(p, q, method='Bolton'):
     if method == 'Bolton':
         td = 243.5 / (17.67 / numpy.log(e_q / 100. / 6.112) - 1) + 273.15
     else:
-        raise ValueError("invalid dew point method <%s>" % method)
+        raise ValueError(u"invalid dew point method '{}'".format(method))
 
     return td
 
 
-"""
-Potential Temperature
-"""
-
-
 def pot_temp(p, t):
-    """Computes potential temperature in [K] from pressure and temperature.
+    """
+    Computes potential temperature in [K] from pressure and temperature.
 
     Arguments:
     p -- pressure in [Pa]
@@ -870,8 +663,9 @@ def pot_temp(p, t):
 
 def eqpt_approx(p, t, q, liquid='HylandWexler', ice='GoffGratch',
                 force_phase='None'):
-    """Computes equivalent potential temperature in [K] from pressure,
-       temperature and specific humidity.
+    """
+    Computes equivalent potential temperature in [K] from pressure,
+    temperature and specific humidity.
 
     Arguments:
     p -- pressure in [Pa]
@@ -910,7 +704,8 @@ def eqpt_approx(p, t, q, liquid='HylandWexler', ice='GoffGratch',
 
 
 def omega_to_w(omega, p, t):
-    """Convert pressure vertical velocity to geometric vertical velocity.
+    """
+    Convert pressure vertical velocity to geometric vertical velocity.
 
     Arguments:
     omega -- vertical velocity in pressure coordinates, in [Pa/s]
@@ -934,11 +729,6 @@ def omega_to_w(omega, p, t):
     return omega / (-9.80665 * rho)
 
 
-"""
-Flight Level / Pressure Conversion
-"""
-
-
 def flightlevel2pressure(flightlevel):
     """Conversion of flight level (given in hft) to pressure (Pa) with
        hydrostatic equation, according to the profile of the ICAO
@@ -1061,9 +851,10 @@ def pressure2flightlevel(p):
 
 
 def flightlevel2pressure_a(flightlevel):
-    """Conversion of flight level (given in hft) to pressure (Pa) with
-       hydrostatic equation, according to the profile of the ICAO
-       standard atmosphere.
+    """
+    Conversion of flight level (given in hft) to pressure (Pa) with
+    hydrostatic equation, according to the profile of the ICAO
+    standard atmosphere.
 
     Array version, the argument "p" must be a numpy array.
 
@@ -1097,7 +888,6 @@ def flightlevel2pressure_a(flightlevel):
     # ICAO standard atmosphere between 0 and 11 km: T(z=0km) = 15 degC,
     # p(z=0km) = 1013.25 hPa. Temperature gradient is 6.5 K/km.
     indices = numpy.where(z <= 11000.)
-    # print "0..11km", indices
     z0 = 0
     T0 = 288.15
     gamma = 6.5e-3
@@ -1109,7 +899,6 @@ def flightlevel2pressure_a(flightlevel):
     # ICAO standard atmosphere between 11 and 20 km: T(z=11km) = -56.5 degC,
     # p(z=11km) = 226.32 hPa. Temperature is constant at -56.5 degC.
     indices = numpy.where((z > 11000.) & (z <= 20000.))
-    # print "11..20km", indices
     z0 = 11000.
     p0 = 22632.
     T = 216.65
@@ -1120,7 +909,6 @@ def flightlevel2pressure_a(flightlevel):
     # ICAO standard atmosphere between 20 and 32 km: T(z=20km) = -56.5 degC,
     # p(z=20km) = 54.75 hPa. Temperature gradient is -1.0 K/km.
     indices = numpy.where((z > 20000.) & (z <= 32000.))
-    # print "20..32km", indices
     z0 = 20000.
     T0 = 216.65
     gamma = -1.0e-3
@@ -1133,9 +921,10 @@ def flightlevel2pressure_a(flightlevel):
 
 
 def pressure2flightlevel_a(p, fake_above_32km=False):
-    """Conversion of pressure (Pa) to flight level (hft) with
-       hydrostatic equation, according to the profile of the ICAO
-       standard atmosphere.
+    """
+    Conversion of pressure (Pa) to flight level (hft) with
+    hydrostatic equation, according to the profile of the ICAO
+    standard atmosphere.
 
     Array version, the argument "p" must be a numpy array.
 
@@ -1206,7 +995,8 @@ def pressure2flightlevel_a(p, fake_above_32km=False):
 
 
 def isa_temperature(flightlevel):
-    """International standard atmosphere temperature at the given flight level.
+    """
+    International standard atmosphere temperature at the given flight level.
 
     Reference:
         For example, H. Kraus, Die Atmosphaere der Erde, Springer, 2001,
@@ -1244,231 +1034,3 @@ def isa_temperature(flightlevel):
     else:
         raise ValueError("ISA temperature from flight level not "
                          "implemented for z > 32km")
-
-
-"""
-Schmidt/Appleman
-"""
-
-
-def schmidt_appleman_tdiff(p_Pa, T_K, rh_01):
-    """Schmidt/Appleman criterion folded by relative humidity of 80%.
-
-    Reference: U. Schumann, "On conditions for contrail formation from
-        aircraft exhausts", Met.Z. (1996) 4-23.
-
-    Arguments:
-    p_Pa  -- pressure in Pa
-    T_K   -- temperature in K
-    rh_01 -- relative humidity mapped to 0..1
-
-    Returns:
-    Temperature difference T_K - Schm./Appl.Threshold. Of interest are
-    return values below 0.
-
-    NOTE: The current implementation returns a value of 1. if relative
-    humidity is below 80% (as used, e.g. during CONCERT and ML-Cirrus).
-
-    NOTE: This function is a port of U. Schumann's FORTRAN77 function in
-    the old Metview3 scripts.
-
-    mr/18Mar2014
-    """
-
-    # Original comments from the FORTRAN program are in CAPITAL LETTERS:
-
-    # PROGRAMMED BY:
-    # ULRICH SCHUMANN; DLR, INSTITUT FUER PHYSIK DER ATMOSPHAERE
-    # OBERPFAFFENHOFEN, POSTFACH 1116, 82230 WESSLING, GERMANY
-    # REFERENCE: U.SCHUMANN,
-    #     ON CONDITIONS FOR CONTRAIL FORMATION FROM AIRCRAFT EXHAUSTS.
-    #     METEOROL. Z., 5 (1996) 4-23.
-    # VERSION OF FEBRUARY 7, 2000.
-
-    # G IN PA/K, SLOPE OF MIXING LINE,
-    #     G=EI*CP*R1*P/(Q*(1.-ETA)*R0)
-    # WITH  DATA R1/461.5/,R0/287.04/,CP/1004./
-    #     EI = EMISSION INDEX OF WATER VAPOR, E.G. 1.23 FOR KEROSENE
-    #     Q =  COMBUSTION HEAT, E.G. Q = 43.2E6 FOR KEROSENE
-    #     ETA= OVERALL EFFICIENCY, E.G. 0.3 FOR A SUBSONIC JET AIRCRAFT
-    #     P =  AMBIENT PRESSURE IN PA
-
-    EI = 1.23
-    CP = 1004.
-    R0 = 287.04
-    R1 = 461.5
-    Q = 43.2e6
-    ETA = 0.3
-
-    G = EI * CP * R1 * p_Pa / (Q * (1. - ETA) * R0)
-
-    # SATURATION PRESSURE FROM SONNTAG, TT IN K, PSAT IN PA
-    #    METEOROL. Z., 3 (1994) 51-66.
-    def PSAT(TT):
-        return 100. * numpy.exp(
-            -6096.9385 / TT + 16.635794 - 2.711193E-2 * TT + 1.673952E-5 * TT * TT + 2.433502 * numpy.log(TT))
-
-    def DPSAT(TT):
-        return (PSAT(TT + 1.) - PSAT(TT - 1.)) / 2.
-
-    def DDPSAT(TT):
-        return PSAT(TT + 1.) + PSAT(TT - 1.) - 2. * PSAT(TT)
-
-    if (rh_01 < 0.):
-        rh_01 = 0.
-
-    # TLM: THRESHOLD TEMPERATURE FOR U=1., IN K
-    # COMPUTATION OF TLM BY APPROXIMATION ACCORDING TO SCHUMANN (1996)
-    TLM = 273.15 - 46.46 + 9.43 * numpy.log(G - 0.053) + 0.720 * (numpy.log(G - 0.053)) ** 2
-
-    for ITER in range(1, 11):
-        F = DPSAT(TLM) - G
-        DF = DDPSAT(TLM)
-        DX = F / DF
-        TLM = TLM - DX
-        if (numpy.abs(DX) < 1.E-3):
-            break
-
-    # PRINT *,' ITER NOT LARGE ENOUGH FOR TLM, TLM,DX=',TLM,DX
-
-    # TLC: THRESHOLD TEMPERATURE FOR GIVEN U, IN K
-    # COMPUTATION OF TLC BY NEWTON ITERATION
-    if (rh_01 < 1.E-6):
-        TLC = TLM - PSAT(TLM) / G
-    else:
-        TLC = TLM
-        if (rh_01 < 0.99999):
-            EM = PSAT(TLM)
-            T0 = TLM - EM / G
-            E0 = PSAT(T0)
-            C = 1. - numpy.sqrt(1. + 4. * rh_01 * E0 / ((1. - rh_01) * EM))
-            TLC = TLM + (1. - rh_01) * (EM * (TLM - T0) / (2. * rh_01 * E0)) * C
-            for ITER in range(1, 11):
-                DELT = TLM - TLC
-                F = PSAT(TLM) - G * DELT - rh_01 * PSAT(TLC)
-                DF = G - rh_01 * DPSAT(TLC)
-                DX = F / DF
-                TLC = TLC - DX
-                if (numpy.abs(DX) < 1.E-3):
-                    break
-
-    # Filter values with rel.hum. < 80%.
-    TMKDIFF = 1.
-    if (rh_01 > 0.8):
-        TMKDIFF = T_K - TLC
-
-    return TMKDIFF
-
-
-def schmidt_appleman_tdiff_q(p, t, q, liquid='HylandWexler', ice='GoffGratch',
-                             force_phase='None'):
-    """Same as schmidt_appleman_tdiff(), but with specific humidity q
-    instead of relative humidity.
-
-    Arguments for q, liquid, ice, force_phase the same as for rel_hum().
-    """
-    rh01 = rel_hum(p, t, q, liquid=liquid, ice=ice, force_phase=force_phase)
-    rh01 /= 100.
-    return schmidt_appleman_tdiff(p, t, rh01)
-
-
-def schmidt_appleman_tdiff_q_a_slow(p, t, q, liquid='HylandWexler', ice='GoffGratch',
-                                    force_phase='None'):
-    """Same as schmidt_appleman_tdiff_q(), but takes arrays as arguments.
-
-    NOTE: This is the slow version, using Python loops to process the
-    array elements.  Use schmidt_appleman_tdiff_q_a() instead.
-    """
-    rh01 = rel_hum(p, t, q, liquid=liquid, ice=ice, force_phase=force_phase)
-    rh01 /= 100.
-
-    # Remember array shape.
-    shp = p.shape
-
-    # Transform arrays into 1D fields.
-    p_r = p.ravel()
-    t_r = t.ravel()
-    rh01_r = rh01.ravel()
-
-    # Compute Schmidt-Applement threshold difference for each value.
-    result = numpy.zeros(p_r.shape)
-    for i in range(len(p_r)):
-        result[i] = schmidt_appleman_tdiff(p_r[i], t_r[i], rh01_r[i])
-
-    # Return the result in the shape of the input arrays.
-    result.reshape(shp)
-    return result
-
-
-def schmidt_appleman_tdiff_q_a(p_Pa, T_K, q, liquid='HylandWexler', ice='GoffGratch',
-                               force_phase='None'):
-    """Same as schmidt_appleman_tdiff_q(), but takes arrays as arguments.
-    """
-
-    # For comments see schmidt_appleman_tdiff().
-    # ==========================================
-
-    rh_01 = rel_hum(p_Pa, T_K, q, liquid=liquid, ice=ice, force_phase=force_phase)
-    rh_01 /= 100.
-
-    EI = 1.23
-    CP = 1004.
-    R0 = 287.04
-    R1 = 461.5
-    Q = 43.2e6
-    ETA = 0.3
-
-    G = EI * CP * R1 * p_Pa / (Q * (1. - ETA) * R0)
-
-    def PSAT(TT):
-        return 100. * numpy.exp(
-            -6096.9385 / TT + 16.635794 - 2.711193E-2 * TT + 1.673952E-5 * TT * TT + 2.433502 * numpy.log(TT))
-
-    def DPSAT(TT):
-        return (PSAT(TT + 1.) - PSAT(TT - 1.)) / 2.
-
-    def DDPSAT(TT):
-        return PSAT(TT + 1.) + PSAT(TT - 1.) - 2. * PSAT(TT)
-
-    rh_01[numpy.where(rh_01 < 0.)] = 0.
-
-    TLM = 273.15 - 46.46 + 9.43 * numpy.log(G - 0.053) + 0.720 * (numpy.log(G - 0.053)) ** 2
-
-    for ITER in range(1, 11):
-        F = DPSAT(TLM) - G
-        DF = DDPSAT(TLM)
-        DX = F / DF
-        TLM = TLM - DX
-        if (numpy.abs(DX) < 1.E-3).all():
-            break
-
-    TLC = numpy.zeros(TLM.shape)
-
-    i_rh_verysmall = numpy.where(rh_01 < 1.E-6)
-    TLC[i_rh_verysmall] = TLM[i_rh_verysmall] - PSAT(TLM[i_rh_verysmall]) / G[i_rh_verysmall]
-
-    i_rh = numpy.where(rh_01 >= 1.E-6)
-    TLC[i_rh] = TLM[i_rh]
-
-    i_rhLT1 = numpy.where(rh_01 < 0.99999)
-
-    EM = PSAT(TLM[i_rhLT1])
-    T0 = TLM[i_rhLT1] - EM / G[i_rhLT1]
-    E0 = PSAT(T0)
-    C = 1. - numpy.sqrt(1. + 4. * rh_01[i_rhLT1] * E0 / ((1. - rh_01[i_rhLT1]) * EM))
-    TLC[i_rhLT1] = TLM[i_rhLT1] + (1. - rh_01[i_rhLT1]) * (EM * (TLM[i_rhLT1] - T0) / (2. * rh_01[i_rhLT1] * E0)) * C
-    for ITER in range(1, 11):
-        DELT = TLM[i_rhLT1] - TLC[i_rhLT1]
-        F = PSAT(TLM[i_rhLT1]) - G[i_rhLT1] * DELT - rh_01[i_rhLT1] * PSAT(TLC[i_rhLT1])
-        DF = G[i_rhLT1] - rh_01[i_rhLT1] * DPSAT(TLC[i_rhLT1])
-        DX = F / DF
-        TLC[i_rhLT1] = TLC[i_rhLT1] - DX
-        if (numpy.abs(DX) < 1.E-3).all():
-            break
-
-    TMKDIFF = T_K - TLC
-
-    # Filter values with rel.hum. < 80%.
-    TMKDIFF[numpy.where(rh_01 < 0.8)] = 1.
-
-    return TMKDIFF
